/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_base.h
*
* @author    r50233
* 
* @version   0.0.101.0
* 
* @date      Aug-9-2012
* 
* @brief     D4D Driver core and base functions header file
*
*******************************************************************************/

#ifndef __D4D_BASE_H
#define __D4D_BASE_H

/******************************************************************************
* Includes
******************************************************************************/

/******************************************************************************
* Constants
******************************************************************************/

#define D4D_VERSION "v2.9X"  // to check release history - read the release_notes.txt file

/*********************************************************
*
* D4D constants 
*
*********************************************************/

#define D4D_LINE_THIN           0x00
#define D4D_LINE_THICK          0x01

/*********************************************************
*
* general helper constants
*
*********************************************************/

#ifndef D4D_MCU_TYPE
  #define D4D_MCU_TYPE D4D_MCF51
  #warning "There is no definition for D4D_MCU_TYPE in configuration file. The driver is using as default D4D_MCF51."
#endif    

#ifndef D4D_SCREEN_HISTORY
  #define D4D_SCREEN_HISTORY 5
#endif 

#ifdef D4D_ENABLE_AUTOSIZE
  #if D4D_ENABLE_AUTOSIZE != D4D_FALSE
    #error The D4D_ENABLE_AUTOSIZE capability had removed from eGUI since version 3.00. For update your eGUI project just define size for all objects.
  #endif
#endif

#ifndef D4D_ROUND_CORNER_ENABLE
  #define D4D_ROUND_CORNER_ENABLE D4D_FALSE
#endif  
  
#ifndef D4D_FONT_TABLE_DISABLED
  #define D4D_FONT_TABLE_DISABLED D4D_FALSE
  #warning "Font table is disabled by default, set the D4D_FONT_TABLE_DISABLED to D4D_TRUE in config file."
#endif 

#ifndef D4D_SCREEN_SIZE_LONGER_SIDE
	#define D4D_SCREEN_SIZE_LONGER_SIDE 320
	#warning "The D4D configuration file doesn't contains the resolution information for longer axis. The D4D sets this value to default: 320 pixels."
#endif	

#ifndef D4D_SCREEN_SIZE_SHORTER_SIDE
	#define D4D_SCREEN_SIZE_SHORTER_SIDE 240
	#warning "The D4D configuration file doesn't contains the resolution information for shorter axis. The D4D sets this value to default: 240 pixels."
#endif	

#if D4D_SCREEN_SIZE_SHORTER_SIDE > D4D_SCREEN_SIZE_LONGER_SIDE
	#warning "The D4D_SCREEN_SIZE_SHORTER_SIDE macro has greater value then D4D_SCREEN_SIZE_LONGER_SIDE in d4d_cfg.h file. Please check!"
#endif

#ifndef D4D_MCU_BUS_CLOCK 
  #define D4D_MCU_BUS_CLOCK 24000000L
#endif

#ifndef D4D_COLOR_SYSTEM_FORE 
  #define D4D_COLOR_SYSTEM_FORE D4D_COLOR_YELLOW
#endif

#ifndef D4D_COLOR_SYSTEM_BCKG 
  #define D4D_COLOR_SYSTEM_BCKG D4D_COLOR_BLUE
#endif

#ifndef D4D_FONT_SYSTEM_DEFAULT 
  #define D4D_FONT_SYSTEM_DEFAULT 0
#endif  

#ifndef D4D_BMP_EXTSRC_SUPPORT_D4D
  #define D4D_BMP_EXTSRC_SUPPORT_D4D D4D_FALSE
#endif  

#ifndef D4D_BMP_EXTSRC_SUPPORT_BMP
  #define D4D_BMP_EXTSRC_SUPPORT_BMP D4D_FALSE
#endif  

#ifndef D4D_EXTSRC_TEXT_ENABLE 
	#define D4D_EXTSRC_TEXT_ENABLE D4D_FALSE
#endif  

#ifndef D4D_EXTSRC_FILE_ENABLE 
	#if (D4D_BMP_EXTSRC_SUPPORT_BMP != D4D_FALSE) || (D4D_BMP_EXTSRC_SUPPORT_D4D != D4D_FALSE) || (D4D_FNT_EXTSRC_SUPPORT != D4D_FALSE)
		#define D4D_EXTSRC_FILE_ENABLE D4D_TRUE
	#else
		#define D4D_EXTSRC_FILE_ENABLE D4D_FALSE
	#endif
#endif 



#ifndef D4D_EXTSRC_TEXT_FLAGCHAR 
  #define D4D_EXTSRC_TEXT_FLAGCHAR '^'
#endif  

#ifndef D4D_EXTSRC_BUFF_SIZE 
  #define D4D_EXTSRC_BUFF_SIZE 64
#endif  


#ifndef D4D_FILENAME_MAX_SIZE 
  #define D4D_FILENAME_MAX_SIZE 64
#endif  


#define D4D_BEVEL_WIDTH 2

#ifndef D4D_BEVEL_FORE_INTENSITY
  #define D4D_BEVEL_FORE_INTENSITY 33
#endif

#ifndef D4D_BEVEL_BCKG_INTENSITY
  #define D4D_BEVEL_BCKG_INTENSITY -33
#endif

#ifndef D4D_LLD_FLUSH_ELEMENT
  #define D4D_LLD_FLUSH_ELEMENT D4D_FALSE
#endif


#define D4D_DEBUG D4D_FALSE  

/*********************************************************
*
* D4D align constants
*
*********************************************************/
#define D4D_ALIGN_MASK                  (0x0F)

#define D4D_ALIGN_H_MASK                (0x03)
#define D4D_ALIGN_H_LEFT_MASK           (0x00)
#define D4D_ALIGN_H_RIGHT_MASK          (0x01)
#define D4D_ALIGN_H_CENTER_MASK         (0x02)

#define D4D_ALIGN_V_MASK                (0x0C)
#define D4D_ALIGN_V_TOP_MASK            (0x00)
#define D4D_ALIGN_V_BOTTOM_MASK         (0x04)
#define D4D_ALIGN_V_CENTER_MASK         (0x08)
  
/*********************************************************
*
* keys setting  constants
*
*********************************************************/



#ifndef D4D_KEY_UP
  #define D4D_KEY_UP     0x01
#endif
#ifndef D4D_KEY_DOWN
  #define D4D_KEY_DOWN   0x02
#endif
#ifndef D4D_KEY_LEFT  
  #define D4D_KEY_LEFT   0x04
#endif
#ifndef D4D_KEY_RIGHT  
  #define D4D_KEY_RIGHT  0x08
#endif
#ifndef D4D_KEY_ENTER  
  #define D4D_KEY_ENTER  0x10
#endif
#ifndef D4D_KEY_ESC
  #define D4D_KEY_ESC    0x20
#endif


#ifndef D4D_KEY_SCANCODE_UP
  #define D4D_KEY_SCANCODE_UP     0x51
#endif

#ifndef D4D_KEY_SCANCODE_DOWN
  #define D4D_KEY_SCANCODE_DOWN   0x50
#endif

#ifndef D4D_KEY_SCANCODE_LEFT  
  #define D4D_KEY_SCANCODE_LEFT   0x4B
#endif

#ifndef D4D_KEY_SCANCODE_RIGHT  
  #define D4D_KEY_SCANCODE_RIGHT  0x4D
#endif

#ifndef D4D_KEY_SCANCODE_ENTER  
  #define D4D_KEY_SCANCODE_ENTER  0x1C
#endif

#ifndef D4D_KEY_SCANCODE_ESC
  #define D4D_KEY_SCANCODE_ESC    0x01
#endif

#define D4D_KEY_SCANCODE_KEYMASK  0x7F
#define D4D_KEY_SCANCODE_RELEASEMASK  0x80 

#ifndef D4D_KEY_FUNC_FOCUS_NEXT
  #define D4D_KEY_FUNC_FOCUS_NEXT    D4D_KEY_SCANCODE_DOWN
#endif 

#ifndef D4D_KEY_FUNC_FOCUS_PREV
  #define D4D_KEY_FUNC_FOCUS_PREV    D4D_KEY_SCANCODE_UP
#endif 


#ifndef D4D_KEYS_BUFF_LENGTH
  #define D4D_KEYS_BUFF_LENGTH 4
#endif 

/*********************************************************
*
* touch screen constants
*
*********************************************************/


#ifndef D4DTCH_CALIB_CROSS_OFFSET
  #define D4DTCH_CALIB_CROSS_OFFSET 30
#endif

/******************************************************************************
* Types
******************************************************************************/

/*********************************************************
*
* general helper types
*
*********************************************************/

#define D4D_ENDIAN_BIG 0
#define D4D_ENDIAN_LITTLE 1
   

#ifndef D4D_ENDIAN
  #if (D4D_MCU_TYPE == D4D_MK)
    #define D4D_ENDIAN D4D_ENDIAN_LITTLE
  #else  // for other cases (this should not work for every cases)   
    #define D4D_ENDIAN D4D_ENDIAN_BIG
  #endif  
#endif

#define D4D_SWAP_WORD(x) (((x) << 8) | ((x) >> 8))

#if D4D_ENDIAN == D4D_ENDIAN_BIG
  #define D4D_MODIFY_WORD(x) (x)
#else
  #define D4D_MODIFY_WORD(x) D4D_SWAP_WORD(x)
#endif 

#define D4D_LOG_EXOR(a,b) (!!(a) ^ !!(b))

typedef struct
{
  D4D_KEY_SCANCODE buff[D4D_KEYS_BUFF_LENGTH];
  Byte readPos;
  Byte writePos;
} D4D_KEYS_BUFFER;

typedef Byte D4D_TOUCHSCREEN_STATUS;

#define D4D_TCHSCR_STATUS_TOUCHED       (0x01) // The new touch has to be 0x01
#define D4D_TCHSCR_STATUS_RESERVED      (0x02)
#define D4D_TCHSCR_STATUS_NEWTOUCH      (0x04)
#define D4D_TCHSCR_STATUS_RESERVED1     (0x08)
#define D4D_TCHSCR_STATUS_LOSTTOUCH     (0x10)
#define D4D_TCHSCR_STATUS_NEWRAWEVENT   (0x20)
#define D4D_TCHSCR_STATUS_NEWEVENT      (0x40)
#define D4D_TCHSCR_STATUS_CHECKTOUCH    (0x80)

#define D4D_TCHSCR_STATUS_EVENTS        (0xE0)

typedef Byte D4D_SYSTEM_FLAGS;

#define D4D_SYSTEM_F_TIMETICK     (0x01)

typedef Byte D4D_OBJECT_DRAWFLAGS;

#define D4D_OBJECT_DRAWFLAGS_COMPLETE   (0x01)
#define D4D_OBJECT_DRAWFLAGS_FOCUSED    (0x02)
#define D4D_OBJECT_DRAWFLAGS_CAPTURING  (0x04)


/*********************************************************
*
* object messages
*
*********************************************************/

typedef enum 
{
    D4D_MSG_ONINIT,         // no parameter         // allways
    D4D_MSG_DRAW,           // parameter wprm_DRAW  // visible
    D4D_MSG_DRAWDONE,       // parameter wprm_DRAW  // visible
    D4D_MSG_SETFOCUS,       // no parameter         // visible + enable 
    D4D_MSG_KILLFOCUS,      // no parameter         // visible + enable
    D4D_MSG_SETCAPTURE,     // no parameter         // visible + enable
    D4D_MSG_KILLCAPTURE,    // no parameter         // visible + enable
    D4D_MSG_KEYUP,          // parameter wprm_KEYS  // visible + tabstop + enable
    D4D_MSG_KEYDOWN,        // parameter wprm_KEYS  // visible + tabstop + enable
    D4D_MSG_TOUCHED,        // no parameter         // visible + enable + touch enable
    D4D_MSG_TOUCH_AUTO,     // no parameter         // visible + enable + touch enable
    D4D_MSG_TOUCH_LOST,     // no parameter         // visible + enable + touch enable
    D4D_MSG_UNTOUCHED,      // no parameter         // visible + enable + touch enable
    D4D_MSG_TIMETICK        // no parameter         // visible
} D4D_MSGID;

/* the message object */

struct D4D_OBJECT_S;

typedef struct D4D_MESSAGE_S
{
    struct D4D_OBJECT_S* pObject;
    struct D4D_SCREEN_S* pScreen;
    D4D_MSGID   nMsgId;  
     
    union
    {
        D4D_OBJECT_DRAWFLAGS draw;
        D4D_KEY_SCANCODE key;      // there will be stored only code of key without release / press information
                                   // this information is handlad by message ID 
        
    } prm;
    
} D4D_MESSAGE;

#define D4D_MSG_NOSKIP 0
#define D4D_MSG_SKIP 1

/*********************************************************
*
* global events
*
*********************************************************/

typedef enum 
{
    D4D_EVENT_ONCLICK,          // On Click Event
    D4D_EVENT_ONDOUBLECLICK,    // On Double Click Event
    D4D_EVENT_ONCHANGE,         // On Change Event
    D4D_EVENT_ONRELOAD,         // On Reload
    D4D_EVENT_CNT               // Keep it on end of list (there is stored count of events)
} D4D_EVENTID;

/******************************************************************************
* Macros 
******************************************************************************/

/*********************************************************
*
* internal macros
*
*********************************************************/

#define D4D_UNUSED(x) (void)(x); 
#define D4D_CONST const
#define D4D_NO_CONST

#define D4D_TEXT_LEN(txt) (sizeof((txt)) / sizeof(D4D_TCHAR))

/*********************************************************
*
* external macros
*
*********************************************************/

#define D4D_DrawTextTab(ppt, buffText, pTab, colorText, colorBkgd) D4D_DrawTextTabXY((ppt)->x, (ppt)->y, buffText, pTab, colorText, colorBkgd)
#define D4D_DrawTextXY(x, y, buffText, colorText, colorBkgd) D4D_DrawTextTabXY(x, y, buffText, NULL, colorText, colorBkgd)
#define D4D_DrawText(ppt, buffText, colorText, colorBkgd) D4D_DrawTextTab(ppt, buffText, NULL, colorText, colorBkgd)

#define D4D_DrawTextRect(ppt, psz, buffText, colorText, colorBkgd) D4D_DrawTextRectTab(ppt, psz, buffText, NULL, colorText, colorBkgd)
#define D4D_DrawTextRectTo(ppt, buffText, colorText, colorBkgd) D4D_DrawTextRectTabTo(ppt, buffText, NULL, colorText, colorBkgd)
#define D4D_DrawTextRectXY(x1, y1, x2, y2, buffText, colorText, colorBkgd) D4D_DrawTextRectTabXY(x1, y1, x2, y2, buffText, NULL, colorText, colorBkgd)
#define D4D_DrawTextRectToXY(x, y, buffText, colorText, colorBkgd) D4D_DrawTextRectTabToXY(x, y, buffText, NULL, colorText, colorBkgd)

#define D4D_DrawTextRRect(ppt, psz, buffText, colorText, colorBkgd, radius) D4D_DrawTextRRectTab(ppt, psz, buffText, NULL, colorText, colorBkgd, radius)
#define D4D_DrawTextRRectTo(ppt, buffText, colorText, colorBkgd, radius) D4D_DrawTextRRectTabTo(ppt, buffText, NULL, colorText, colorBkgd, radius)
#define D4D_DrawTextRRectXY(x1, y1, x2, y2, buffText, colorText, colorBkgd, radius) D4D_DrawTextRRectTabXY(x1, y1, x2, y2, buffText, NULL, colorText, colorBkgd, radius)
#define D4D_DrawTextRRectToXY(x, y, buffText, colorText, colorBkgd, radius) D4D_DrawTextRRectTabToXY(x, y, buffText, NULL, colorText, colorBkgd, radius)

#define D4D_GetTextBuffWidthTabAdj(text_buffer, pTab) D4D_GetTextBuffWidthTab(text_buffer, pTab) 

#define D4D_GetTextBuffWidth(text_buffer) D4D_GetTextBuffWidthTab(text_buffer, NULL)



/******************************************************************************
* Global variables
******************************************************************************/
   
// global shared memory used by multiple modules as temporary storage
#define D4D_SCRATCHPAD_SIZE 36
extern Byte d4d_scratchPad[D4D_SCRATCHPAD_SIZE];

// assert macro
#define D4D_ASSERT(cond) if(!(cond)) for(;;)

/******************************************************************************
* Global functions
******************************************************************************/

#ifdef D4D_INPUT_EVENT_CALLBACK
   extern void D4D_INPUT_EVENT_CALLBACK(void);
#endif

D4D_COOR D4D_RndCornerGetNonPrintedPxl(D4D_COOR radius, D4D_COOR line);

/*********************************************************
*
* internal API
*
*********************************************************/


#endif /* __D4D_BASE_H */










