/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_bmp.c
*
* @author    b01119
* 
* @version   0.0.31.0
* 
* @date      Jun-29-2012
* 
* @brief     D4D driver bmp decoder functions c file 
*
******************************************************************************/

#include "d4d.h"
#include "common_files/d4d_lldapi.h"
#include "common_files/d4d_private.h"

#if D4D_EXTSRC_FILE_ENABLE != D4D_FALSE
  static const D4D_CHAR imgDfltPath[] = D4D_IMG_WORK_PATH_DEFAULT;
  static D4D_CHAR* pImgWorkPath = (D4D_CHAR*)imgDfltPath;
#endif
  
static const D4D_EXTIMG_FUNC d4d_extImgFunc[D4D_EXTIMG_ITEMS_CNT] =
{
  { NULL, NULL, NULL, NULL, NULL },
  { D4D_ImgDecIntD4DDraw, D4D_ImgDecIntD4DGetWidth, D4D_ImgDecIntD4DGetHeight, D4D_ImgDecIntD4DGetSize, D4D_ImgDecIntD4DGetHeader },
  { D4D_ImgDecExtD4DDraw, D4D_ImgDecExtD4DGetWidth, D4D_ImgDecExtD4DGetHeight, D4D_ImgDecExtD4DGetSize, D4D_ImgDecExtD4DGetHeader },
  { D4D_ImgDecBmpDraw, D4D_ImgDecBmpGetWidth, D4D_ImgDecBmpGetHeight, D4D_ImgDecBmpGetSize, D4D_ImgDecBmpGetHeader }
};



/**************************************************************//*!
*
* External Bitmap support
*
******************************************************************/

#if D4D_EXTSRC_FILE_ENABLE != D4D_FALSE
  /**************************************************************//*!
  *
  * Set Image external source files work directory
  *
  ******************************************************************/
  void D4D_ExtImgSetWorkPath(D4D_CHAR* pPath)
  {
    if(pPath)
      pImgWorkPath = pPath;  
  }

  /**************************************************************//*!
  *
  * Get Image external source files work directory
  *
  ******************************************************************/
  D4D_CHAR* D4D_ExtImgGetWorkPath(void)
  {
    return pImgWorkPath;  
  }

  /**************************************************************//*!
  *
  * Get Image external source files work directory
  *
  ******************************************************************/
  D4D_FILEPTR D4D_OpenImgFile(D4D_CHAR* pFileName, D4D_CHAR* mode)
  {
    // create the file name (if needed)
    if(D4D_FileIsAbsolutePath(pFileName))
    {
      // the input path is absolute and can't be changed
      return D4D_FileOpen(pFileName,mode);
    }
    else
    {
      // the input path is relative and must be updated with preselected image work paths
      if((D4D_StrLen(pImgWorkPath) + D4D_StrLen(pFileName)) >= D4D_EXTSRC_BUFF_SIZE)
        return NULL;
      
      D4D_StrCopy((D4D_CHAR*)d4d_extsrcBuffer, pImgWorkPath);
      D4D_StrCat((D4D_CHAR*)d4d_extsrcBuffer, pFileName);
      return D4D_FileOpen((D4D_CHAR*)d4d_extsrcBuffer,mode);
    }
    
  }
#endif

/**************************************************************//*!
*
* Get the type of image source
*
******************************************************************/
D4D_EXTIMG_TYPE D4D_GetImgSrcType(D4D_BMP* pBmp)
{
  if(pBmp == NULL)
    return D4D_EXTIMG_UNKNOWN;
  
  #if D4D_EXTSRC_FILE_ENABLE != D4D_FALSE  
    if(pBmp->pData == NULL)
    {
      D4D_CHAR* pExt;
      
      // run the external data dekoder
      if(pBmp->pParam == NULL)
        return D4D_EXTIMG_UNKNOWN;
      
      // 1. get File extension
      pExt = D4D_GetFileExtension((D4D_CHAR*)pBmp->pParam);
      
      // Convert extension to upper format
      D4D_ToUpper(pExt);
      
      if(pExt == NULL)
        return D4D_EXTIMG_UNKNOWN;
      
      #if (D4D_BMP_EXTSRC_SUPPORT_D4D == D4D_TRUE) && (D4D_EXTSRC_FILE_ENABLE == D4D_TRUE)  
        if(!D4D_CompareStrings(pExt, D4D_BMP_EXTSRC_SUPPORT_D4D_EXT))        // big or little endian of the egui picture format
          return D4D_EXTIMG_D4D; // d4d bitmap binary files
        
      #endif

      #if (D4D_BMP_EXTSRC_SUPPORT_BMP == D4D_TRUE) && (D4D_EXTSRC_FILE_ENABLE == D4D_TRUE)
        if(!D4D_CompareStrings(pExt, "BMP"))
          return D4D_EXTIMG_BMP; // standard bitmap BMP files
      #endif  
      
      
    }else
    
  #endif  
    if((pBmp->pData != NULL))
      return D4D_EXTIMG_INTD4D;  
    
  return D4D_EXTIMG_UNKNOWN;      
}




#ifdef __CWCC__
  #pragma mark -
  #pragma mark Interface Bitmap Functions
  #pragma mark -
#endif

/**************************************************************//*!
*
* Draw bitmap 
*
******************************************************************/

void D4D_DrawRBmpXY(D4D_COOR x, D4D_COOR y, const D4D_BMP* pBmp, D4D_BOOL greyScale, D4D_COOR radius)
{
  D4D_EXTIMG_TYPE imgType = D4D_GetImgSrcType((D4D_BMP*)pBmp);
  
  if(imgType == D4D_EXTIMG_UNKNOWN)
    return;
  
  if(imgType == D4D_EXTIMG_INTD4D)
  {
    if(d4d_extImgFunc[imgType].D4D_ImgDecDraw)
      d4d_extImgFunc[imgType].D4D_ImgDecDraw(x, y, (D4D_CHAR*)pBmp, greyScale, radius);    
  
  }
#if D4D_EXTSRC_FILE_ENABLE != D4D_FALSE    
  else
  {
    if(d4d_extImgFunc[imgType].D4D_ImgDecDraw)
      d4d_extImgFunc[imgType].D4D_ImgDecDraw(x, y, (D4D_CHAR*)(pBmp->pParam), greyScale, radius);    
  }
#endif
}


/**************************************************************//*!
*
* Draw bitmap in rectangle
*
******************************************************************/

void D4D_DrawBmpRect(D4D_POINT* ppt, D4D_SIZE* psz, const D4D_BMP* pBmp, D4D_BMP_PROPERTIES bmpProp, D4D_COLOR colorBkgd, D4D_BOOL greyScale)
{
  D4D_SIZE tmp_size = *psz;
  D4D_POINT tmp_pos = *ppt;
  
  D4D_COOR tmp_bmpWidth = (D4D_COOR)D4D_GetBmpWidth(pBmp);
  D4D_COOR tmp_bmpHeight = (D4D_COOR)D4D_GetBmpHeight(pBmp);
  
  if(!tmp_size.cx)
    tmp_size.cx = tmp_bmpWidth;
  
  if(!tmp_size.cy)
    tmp_size.cy = tmp_bmpHeight;
  
  tmp_pos.y = ppt->y;
  tmp_pos.x = ppt->x; 
  
  // Compute the text position in rectangle in Y axis
  switch(bmpProp & D4D_ALIGN_V_MASK)
  {
    case D4D_ALIGN_V_BOTTOM_MASK:
      tmp_pos.y += (D4D_COOR)(tmp_size.cy - tmp_bmpHeight);
      break;
    
    case D4D_ALIGN_V_CENTER_MASK:
      tmp_pos.y += (D4D_COOR)((tmp_size.cy - tmp_bmpHeight) / 2);
      break;        
  }
  
  // Compute the text position in rectangle in X axis
  switch(bmpProp & D4D_ALIGN_H_MASK)
  {
    case D4D_ALIGN_H_RIGHT_MASK:
      tmp_pos.x += (D4D_COOR)(tmp_size.cx - tmp_bmpWidth);
      break;
    
    case D4D_ALIGN_H_CENTER_MASK:
      tmp_pos.x += (D4D_COOR)((tmp_size.cx - tmp_bmpWidth) / 2);
      break;       
  }

    if(tmp_size.cx > tmp_bmpWidth)
    {
      if((bmpProp & D4D_ALIGN_H_MASK) != D4D_ALIGN_H_LEFT_MASK)
      {
        // Fill the part of rect in front of the text in full height of rect
          D4D_FillRectXY(ppt->x, ppt->y, tmp_pos.x, (D4D_COOR)(ppt->y + tmp_size.cy - 1), colorBkgd);
      }
      
      if((bmpProp & D4D_ALIGN_H_MASK) != D4D_ALIGN_H_RIGHT_MASK)
      {
        // Fill the part of rect behind the text in full height of rect
          D4D_FillRectXY((D4D_COOR)(tmp_pos.x + tmp_bmpWidth - 1), ppt->y, (D4D_COOR)(ppt->x + tmp_size.cx - 1), (D4D_COOR)(ppt->y + tmp_size.cy - 1), colorBkgd);
      }
    }

    if(tmp_size.cy > tmp_bmpHeight)
    {  
      if((bmpProp & D4D_ALIGN_V_MASK) != D4D_ALIGN_V_TOP_MASK)
      {
        // Fill the part of rect above the text in lenght of text
        D4D_FillRectXY(tmp_pos.x, ppt->y, (D4D_COOR)(tmp_pos.x + tmp_bmpWidth - 1), tmp_pos.y, colorBkgd);
      }
      
      if((bmpProp & D4D_ALIGN_V_MASK) != D4D_ALIGN_V_BOTTOM_MASK)
      {
        // Fill the part of rect under the text in lenght of text
        D4D_FillRectXY(tmp_pos.x, (D4D_COOR)(tmp_pos.y + tmp_bmpHeight - 1), (D4D_COOR)(tmp_pos.x + tmp_bmpWidth - 1), (D4D_COOR)(ppt->y + tmp_size.cy - 1), colorBkgd);
      }
    }
  
  // Print the string itself
  D4D_DrawRBmpXY(tmp_pos.x, tmp_pos.y, pBmp, greyScale, 0); // radius, grayscale?
  
  
}

#if D4D_ROUND_CORNER_ENABLE != D4D_FALSE

void D4D_DrawRBmpRect(D4D_POINT* ppt, D4D_SIZE* psz, const D4D_BMP* pBmp, D4D_BMP_PROPERTIES bmpProp, D4D_COLOR colorBkgd, D4D_BOOL greyScale, D4D_COOR radius)
{
  D4D_POINT tmp_pos = *ppt;
  D4D_SIZE  tmp_size = *psz;
  
  D4D_COOR tmp_bmpWidth;
  D4D_COOR tmp_bmpHeight;
  
  if(!radius)
  {
    D4D_DrawBmpRect(ppt, psz, pBmp, bmpProp, colorBkgd, greyScale);
    return;
  }
  
  
  tmp_bmpWidth = (D4D_COOR)D4D_GetBmpWidth(pBmp);
  tmp_bmpHeight = (D4D_COOR)D4D_GetBmpHeight(pBmp);

   
  // Compute the text position in rectangle in Y axis
  switch(bmpProp & D4D_ALIGN_V_MASK)
  {
    case D4D_ALIGN_V_BOTTOM_MASK:
      tmp_pos.y = (D4D_COOR)(ppt->y + (tmp_size.cy - tmp_bmpHeight));
      break;
    
    case D4D_ALIGN_V_CENTER_MASK:
      tmp_pos.y = (D4D_COOR)(ppt->y + ((tmp_size.cy - tmp_bmpHeight) / 2));
      break;
    
    default:
    case D4D_ALIGN_V_TOP_MASK:
      tmp_pos.y = ppt->y;
      break;        
  }
  
  // Compute the text position in rectangle in X axis
  switch(bmpProp & D4D_ALIGN_H_MASK)
  {
    case D4D_ALIGN_H_RIGHT_MASK:
      tmp_pos.x = (D4D_COOR)(ppt->x + (tmp_size.cx - tmp_bmpWidth));
      break;
    
    case D4D_ALIGN_H_CENTER_MASK:
      tmp_pos.x = (D4D_COOR)(ppt->x + ((tmp_size.cx - tmp_bmpWidth) / 2));
      break;
    
    default:
    case D4D_ALIGN_H_LEFT_MASK:
      tmp_pos.x = ppt->x;
      break;        
  }
  
  
  // Fill the part of rect in front of the text in full height of rect
  if((psz->cx != tmp_bmpWidth) || (psz->cy != tmp_bmpHeight))
    D4D_FillRRect(ppt, psz, colorBkgd, radius);
   
  // Print the bmp itself
  D4D_DrawRBmpXY(tmp_pos.x, tmp_pos.y, pBmp, greyScale, radius); // radius, grayscale?
  
  
}

#endif

/**************************************************************//*!
*
* Get bitmap Size
*
******************************************************************/

D4D_SIZE D4D_GetBmpSize(const D4D_BMP* pBmp)
{
  D4D_EXTIMG_TYPE imgType = D4D_GetImgSrcType((D4D_BMP*)pBmp);
  
  if(imgType == D4D_EXTIMG_UNKNOWN)
    return *(D4D_SIZE*)&d4d_size_zero;
  
  if(imgType == D4D_EXTIMG_INTD4D)
  {
    if(d4d_extImgFunc[imgType].D4D_ImgDecGetSize)
      return d4d_extImgFunc[imgType].D4D_ImgDecGetSize((D4D_CHAR*)pBmp);    
  
  }
#if D4D_EXTSRC_FILE_ENABLE != D4D_FALSE   
  else
  {
    if(d4d_extImgFunc[imgType].D4D_ImgDecGetSize)
      return d4d_extImgFunc[imgType].D4D_ImgDecGetSize((D4D_CHAR*)(pBmp->pParam));    
  }
#endif

  return *(D4D_SIZE*)&d4d_size_zero;  

}

/**************************************************************//*!
*
* Get bitmap width
*
******************************************************************/

D4D_COOR D4D_GetBmpWidth(const D4D_BMP* pBmp)
{
  D4D_EXTIMG_TYPE imgType = D4D_GetImgSrcType((D4D_BMP*)pBmp);
  
  if(imgType == D4D_EXTIMG_UNKNOWN)
    return 0;
  
  if(imgType == D4D_EXTIMG_INTD4D)
  {
    if(d4d_extImgFunc[imgType].D4D_ImgDecGetWidth)
      return d4d_extImgFunc[imgType].D4D_ImgDecGetWidth((D4D_CHAR*)pBmp);    
  
  }
#if D4D_EXTSRC_FILE_ENABLE != D4D_FALSE    
  else
  {
    if(d4d_extImgFunc[imgType].D4D_ImgDecGetWidth)
      return d4d_extImgFunc[imgType].D4D_ImgDecGetWidth((D4D_CHAR*)(pBmp->pParam));    
  }
#endif

  return 0;
}

/**************************************************************//*!
*
* Get bitmap height
*
******************************************************************/

D4D_COOR D4D_GetBmpHeight(const D4D_BMP* pBmp)
{
  D4D_EXTIMG_TYPE imgType = D4D_GetImgSrcType((D4D_BMP*)pBmp);
  
  if(imgType == D4D_EXTIMG_UNKNOWN)
    return 0;
  
  if(imgType == D4D_EXTIMG_INTD4D)
  {
    if(d4d_extImgFunc[imgType].D4D_ImgDecGetHeight)
      return d4d_extImgFunc[imgType].D4D_ImgDecGetHeight((D4D_CHAR*)pBmp);    
  
  }
#if D4D_EXTSRC_FILE_ENABLE != D4D_FALSE    
  else
  {
    if(d4d_extImgFunc[imgType].D4D_ImgDecGetHeight)
      return d4d_extImgFunc[imgType].D4D_ImgDecGetHeight((D4D_CHAR*)(pBmp->pParam));    
  }
#endif

  return 0;
}


/**************************************************************//*!
*
* Draw bitmap 
*
******************************************************************/

int D4D_GetBmpHeader(D4D_BMP* pBmp, void* pBuff, int max_size)
{
  D4D_EXTIMG_TYPE imgType = D4D_GetImgSrcType((D4D_BMP*)pBmp);
     
  if(imgType == D4D_EXTIMG_INTD4D)
  {
    if(d4d_extImgFunc[imgType].D4D_ImgDecGetHeader)
      return d4d_extImgFunc[imgType].D4D_ImgDecGetHeader((D4D_CHAR*)pBmp, pBuff, max_size);    
  
  }
#if D4D_EXTSRC_FILE_ENABLE != D4D_FALSE    
  else
  {
    if(d4d_extImgFunc[imgType].D4D_ImgDecGetHeader)
      return d4d_extImgFunc[imgType].D4D_ImgDecGetHeader((D4D_CHAR*)(pBmp->pParam), pBuff, max_size);    
  }
#endif

  return 0;
}
   




