/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_bmp.h
*
* @author    b01119
* 
* @version   0.0.20.0
* 
* @date      Jun-29-2012
* 
* @brief     D4D driver bmp decoder header file 
*
******************************************************************************/

#ifndef __D4D_BMP_H
#define __D4D_BMP_H

/******************************************************************************
* Include
******************************************************************************/

#include "common_files/d4d_imgdec_d4dint.h"
#include "common_files/d4d_imgdec_d4d.h"
#include "common_files/d4d_imgdec_bmp.h"

/******************************************************************************
* Types
******************************************************************************/

typedef enum
{
  D4D_EXTIMG_UNKNOWN = 0,
  D4D_EXTIMG_INTD4D,
  D4D_EXTIMG_D4D,
  D4D_EXTIMG_BMP,
  D4D_EXTIMG_ITEMS_CNT
}D4D_EXTIMG_TYPE;


typedef struct
{
  void (*D4D_ImgDecDraw)(D4D_COOR x, D4D_COOR y, D4D_CHAR* pFileName, D4D_BOOL greyScale, D4D_COOR radius);
  D4D_COOR (*D4D_ImgDecGetWidth)(D4D_CHAR* pFileName);
  D4D_COOR (*D4D_ImgDecGetHeight)(D4D_CHAR* pFileName);
  D4D_SIZE (*D4D_ImgDecGetSize)(D4D_CHAR* pFileName);
  int (*D4D_ImgDecGetHeader)(D4D_CHAR* pFileName, void* pBuff, int max_size);
}D4D_EXTIMG_FUNC;

typedef Byte D4D_BMP_PROPERTIES;

/******************************************************************************
* Constants
******************************************************************************/

#if D4D_ENDIAN == D4D_ENDIAN_LITTLE
  #define D4D_BMP_EXTSRC_SUPPORT_D4D_EXT "D4L"
#else
  #define D4D_BMP_EXTSRC_SUPPORT_D4D_EXT "D4B"
#endif

/******************************************************************************
* Static declaration macros
******************************************************************************/

#define D4D_DECLARE_BMP(name, pbmp, ppal) \
    const D4D_BMP name = \
    { \
        pbmp, ppal \
    };


#define D4D_DECLARE_BMPFILE(name, filename) \
    const D4D_BMP name = \
    { \
        NULL, filename \
    };


#define D4D_EXTERN_BMP(name) \
    extern const D4D_BMP name

/******************************************************************************
* External source constants
******************************************************************************/



/******************************************************************************
* Global D4D bitmap drawing level functions
******************************************************************************/


#if D4D_EXTSRC_FILE_ENABLE != D4D_FALSE
  void D4D_ExtImgSetWorkPath(D4D_CHAR* pPath);
  D4D_CHAR* D4D_ExtImgGetWorkPath(void);
  D4D_FILEPTR D4D_OpenImgFile(D4D_CHAR* pFileName, D4D_CHAR* mode);
#endif

D4D_EXTIMG_TYPE D4D_GetImgSrcType(D4D_BMP* pBmp);



#endif /* __D4D_BMP_H */
