/******************************************************************************
* 
* Copyright (c) 2011 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_extsrc.c
*
* @author    r50233
* 
* @version   0.0.11.0
* 
* @date      Aug-6-2012
* 
* @brief     D4D driver extscr functions c file 
*
******************************************************************************/

#include "d4d.h"
#include "common_files/d4d_private.h"
#include "common_files/d4d_lldapi.h"

#ifdef D4D_STD_STRING_H_INCLUDE
  #include D4D_STD_STRING_H_INCLUDE
#endif

#if (D4D_EXTSRC_TEXT_ENABLE != D4D_FALSE) || (D4D_EXTSRC_FILE_ENABLE != D4D_FALSE)
  Byte d4d_extsrcBuffer[D4D_EXTSRC_BUFF_SIZE];
#endif

#if (D4D_EXTSRC_FILE_ENABLE != D4D_FALSE)
/**************************************************************//*!
*
* The function returns the pointer on file extension (if exists)
*
******************************************************************/
D4D_CHAR* D4D_GetFileExtension(D4D_CHAR* pFileName)
{
  int i;
  
  if(pFileName == NULL)
    return NULL;

  
  i = D4D_StrLen(pFileName);
  
  if(i == 0)
    return NULL;
  
  i--; // decrement index to point on last char
  
  while((pFileName[i] >= ' ') && (pFileName[i] != '.'))
  {
    i--;
  };
  
  if(pFileName[i] == '.')
    return &pFileName[i+1];
  else
    return NULL;
}

/**************************************************************//*!
*
* The function find out if the path is absolute or relative
*
******************************************************************/
D4D_BOOL D4D_FileIsAbsolutePath(D4D_CHAR* pFileName)
{
  if(pFileName[1] == ':')
    return D4D_TRUE;
  
  return D4D_FALSE;  
}

/**************************************************************//*!
*
* The function convert screen into BMP file 
*
******************************************************************/
D4D_BOOL D4D_PrintScreenToBMP(D4D_CHAR* pFileName)
{  
#if D4D_EXTSRC_BUFF_SIZE < 64
  D4D_UNUSED(pFileName);
  return D4D_FALSE;
#else  
  D4D_FILEPTR pFile = NULL;
  D4D_COLOR clr;
  D4D_COOR x, y, x_max, y_max;
  LWord     dataIx = 0;
  
  if((D4D_LCD_GetOrientation() == D4D_ORIENT_PORTRAIT) || (D4D_LCD_GetOrientation() == D4D_ORIENT_PORTRAIT180))
  {
    x_max = D4D_SCREEN_SIZE_SHORTER_SIDE;
    y_max = D4D_SCREEN_SIZE_LONGER_SIDE;    
  }else
  {
    x_max = D4D_SCREEN_SIZE_LONGER_SIDE;
    y_max = D4D_SCREEN_SIZE_SHORTER_SIDE;
  }
  
  // 1. Open File for Write
  pFile = D4D_OpenImgFile(pFileName, "w");
  
  if(pFile == NULL)
    return D4D_FALSE;
   
  // 2. Create BMP Header
  d4d_extsrcBuffer[0] = 'B';
  d4d_extsrcBuffer[1] = 'M';
  // File Size 54B header + 24bits color (3Bytes) * width * height
  *((LWord*)&(d4d_extsrcBuffer[2])) = D4D_WRITE32L(54 + 3 * D4D_SCREEN_SIZE_LONGER_SIDE * D4D_SCREEN_SIZE_SHORTER_SIDE);
  // Bitmap header reserved bytes
  *((LWord*)&(d4d_extsrcBuffer[6])) = D4D_WRITE32L(0);  
  // Bitmap bitmap data offset 
  *((LWord*)&(d4d_extsrcBuffer[10])) = D4D_WRITE32L(54);
  
  // 3. Create BMP Information Data BID
  //size of this information 
  *((LWord*)&(d4d_extsrcBuffer[14])) = D4D_WRITE32L(40);
  // Width of bitmap 
  *((LWord*)&(d4d_extsrcBuffer[18])) = D4D_WRITE32L(x_max);
  // Height of bitmap 
  *((LWord*)&(d4d_extsrcBuffer[22])) = D4D_WRITE32L(y_max);
  // Number of color planes being used. 
  *((Word*)&(d4d_extsrcBuffer[26])) = D4D_WRITE16L(1);
  // The number of bits/pixel. 
  *((Word*)&(d4d_extsrcBuffer[28])) = D4D_WRITE16L(24);
  //BI_RGB, No compression used
  *((LWord*)&(d4d_extsrcBuffer[30])) = D4D_WRITE32L(0);
  // Bitmap data size  24bits color (3Bytes) * width * height
  *((LWord*)&(d4d_extsrcBuffer[34])) = D4D_WRITE32L( 3 * D4D_SCREEN_SIZE_LONGER_SIDE * D4D_SCREEN_SIZE_SHORTER_SIDE);
  //The horizontal resolution of the image   2,835 pixels/meter
  *((LWord*)&(d4d_extsrcBuffer[38])) = D4D_WRITE32L(2835);
  //The vertical resolution of the image     2,835 pixels/meter
  *((LWord*)&(d4d_extsrcBuffer[42])) = D4D_WRITE32L(2835);
  //Number of colors in the palette
  *((LWord*)&(d4d_extsrcBuffer[46])) = D4D_WRITE32L(0);
  //	Means all colors are important
  *((LWord*)&(d4d_extsrcBuffer[50])) = D4D_WRITE32L(0);
  
  if(D4D_FileWrite(pFile, d4d_extsrcBuffer, 54) != 54)
  {
    D4D_FileClose(pFile);
    return D4D_FALSE;    
  }
  
  // 4. Write the Bitmap Data
  
  for(y = 0; y < y_max; y++)
  {
    
    (void)D4D_LLD_LCD.D4DLCD_SetWindow(0, (D4D_COOR)(y_max - 1 - y), (D4D_COOR)(x_max - 1), (D4D_COOR)(y_max - 1 - y));
    for(x = 0; x < x_max; x++)
    {
      clr = D4D_LLD_LCD.D4DLCD_Read_Word();
      d4d_extsrcBuffer[dataIx++] = D4D_COLOR_GET_B(clr);
      d4d_extsrcBuffer[dataIx++] = D4D_COLOR_GET_G(clr);
      d4d_extsrcBuffer[dataIx++] = D4D_COLOR_GET_R(clr);
      
      // Check if data buffer is full
      if(dataIx > (D4D_EXTSRC_BUFF_SIZE - 3))
      {
        // If it full, flush data to file
        if(D4D_FileWrite(pFile, d4d_extsrcBuffer, dataIx) != dataIx)
        {
          D4D_FileClose(pFile);
          return D4D_FALSE;    
        }        
        dataIx = 0;
      }      
    }
  }
  
  // check the last chunk of data
  if(dataIx)
  {
    if(D4D_FileWrite(pFile, d4d_extsrcBuffer, dataIx) != dataIx)
    {
      D4D_FileClose(pFile);
      return D4D_FALSE;    
    }
  }
    
  D4D_FileClose(pFile);
  return D4D_TRUE;    
#endif
}

#endif

/**************************************************************//*!
*
* The function returns length of string terminated by zero
*
******************************************************************/
int D4D_StrLen(register const D4D_CHAR *s)
{
#ifndef D4D_STD_STRING_H_INCLUDE
  register int n;
  
  if(!s)
    return 0;
  
  n = 0;
  while (*s++)
          n++;
  return(n);
#else
  // Standard system implementation
  return strlen(s);
#endif
}

/**************************************************************//*!
*
* The function copy string one string to other one (terminated by zero)
*
******************************************************************/
D4D_CHAR* D4D_StrCopy(register D4D_CHAR *sd, register const D4D_CHAR *ss)
{
#ifndef D4D_STD_STRING_H_INCLUDE  
  register D4D_CHAR *s = sd;

  while (*sd++ = *ss++)
    ;

  return s;
#else
  // Standard system implementation
  return strcpy(sd, ss);
#endif
}

/**************************************************************//*!
*
* The function copy string one string to other one (terminated by zero)
*
******************************************************************/
D4D_CHAR* D4D_StrCopyUnicode2Ascii(register D4D_CHAR *sd, register const D4D_WCHAR *ss)
{
  register D4D_CHAR *s = sd;

  while (*sd++ = (D4D_CHAR)*ss++)
    ;

  return s;
}

/**************************************************************//*!
*
* The function copy string one string to other one (terminated by zero)
*
******************************************************************/
D4D_WCHAR* D4D_StrCopyAscii2Unicode(register D4D_WCHAR *sd, register const D4D_CHAR *ss)
{
  register D4D_WCHAR *s = sd;

  while (*sd++ = (D4D_WCHAR)*ss++)
    ;

  return s;
}
/**************************************************************//*!
*
* The function concatenate strings (terminated by zero)
*
******************************************************************/
D4D_CHAR* D4D_StrCat(register D4D_CHAR *sd, register const D4D_CHAR *ss)
{
#ifndef D4D_STD_STRING_H_INCLUDE  
  register D4D_CHAR *s = sd;

  while (*sd)
    sd++;
  
  while ((*sd++ = *ss++))
    ;

  return s;
#else
  // Standard system implementation
  return strcat(sd, ss);
#endif
}

/**************************************************************//*!
*
* The function concatenate strings (terminated by zero)
*
******************************************************************/
D4D_WCHAR* D4D_StrCatUnicode(register D4D_WCHAR *sd, register const D4D_WCHAR *ss)
{
  register D4D_WCHAR *s = sd;

  while (*sd)
    sd++;
  
  while (*sd++ = *ss++)
    ;

  return s;
}


/**************************************************************//*!
*
* The function copare two strings
*
******************************************************************/
int D4D_CompareStrings(const D4D_CHAR* s1, const D4D_CHAR* s2)
{
#ifndef D4D_STD_STRING_H_INCLUDE  
  D4D_CHAR val2 = *s2++;
    while(val2)
    {
        if (*s1++ != val2)
        {
            if (s1[-1] < val2)
                return -1;
            
            return 1;
        }
        val2 = *s2++;
    }

    if (*s1)        // if we get to here, then s2 terminated
      return 1;   // s1 is still going
    
    return 0;       // they both terminated  
#else
  // Standard system implementation
  return strcmp(s1, s2);
#endif    
}

/**************************************************************//*!
*
* The function convert all small alpha char to upper
*
******************************************************************/
void D4D_ToUpper(D4D_CHAR* s)
{
  while(s && *s)
  {
    if((*s >= 'a') && (*s <= 'z'))
      *s -= 'a' - 'A';
    
    s++;
  }
}



/**************************************************************//*!
*
* The function swap bytes in Long
*
******************************************************************/
LWord D4D_SwapLong(LWord n)
{
  Byte temp[4];

  temp[3] = *((Byte *)&n);
  temp[2] = ((Byte *)&n)[1];
  temp[1] = ((Byte *)&n)[2];
  temp[0] = ((Byte *)&n)[3];
  return (*(LWord *)temp);
}

/**************************************************************//*!
*
* The function swap bytes in Word
*
******************************************************************/

Word D4D_SwapWord(Word n)
{
  Byte temp[2];

  temp[1] = *((Byte *)&n);
  temp[0] = ((Byte *)&n)[1];
  return (*(Word *)temp);
}