/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_font.c
*
* @author    b01119
* 
* @version   0.0.45.0
* 
* @date      Aug-22-2012
* 
* @brief     D4D driver font c file 
*
******************************************************************************/

#include "d4d.h"
#include "common_files/d4d_lldapi.h"
#include "common_files/d4d_private.h"

#if (D4D_EXTSRC_FILE_ENABLE != D4D_FALSE) && (D4D_FNT_EXTSRC_SUPPORT != D4D_FALSE)
  static D4D_CHAR* pFntWorkPath = D4D_FNT_WORK_PATH_DEFAULT;
  static D4D_FONT_TYPE * d4d_FontTableRam = NULL;
  static Byte D4D_GetFontTableEntries(D4D_FONT_TYPE* fontTable);
#endif

 typedef struct 
  {
    LWord row;
    LWord clm;
    LWord char_width;
    D4D_FONT_DATA* pFntData;
    D4D_FONT_DESCRIPTOR* pFontDescriptor;
  }CHAR_PXL_DESC;
  
  //(Byte row, Byte clm, D4D_FONT_SIZE char_width, D4D_FONT_DATA* pFntData, D4D_FONT_DESCRIPTOR* pFontDescriptor)

static D4D_COOR D4D_LCD_PrintChr(D4D_TCHAR ch, D4D_PRINT_STRING* p_CharDes);
static D4D_FONT_IX D4D_GetCharIndex(D4D_FONT_DESCRIPTOR* pFontDescriptor, D4D_TCHAR ch);

#if (D4D_FONT_TABLE_DISABLED == D4D_FALSE)
  static D4D_FONT_TYPE* d4d_pFontTable =  (D4D_FONT_TYPE*)&d4d_FontTable;
#endif

#if (D4D_EXTSRC_FILE_ENABLE != D4D_FALSE) && (D4D_FNT_EXTSRC_SUPPORT != D4D_FALSE)

  /**************************************************************//*!
  *
  * Init external font table
  *
  ******************************************************************/  
  static Byte D4D_GetFontTableEntries(D4D_FONT_TYPE* fontTable)
  {
    Byte i = 0;
    
    if(fontTable != NULL)
      
    
    while(fontTable[i].pFontDescriptor || fontTable[i].fileName)
      i++;
      
    return i;
  }

  /**************************************************************//*!
  *
  * Init external font table
  *
  ******************************************************************/
  D4D_BOOL D4D_ExtFntInit(void)
  {
    LWord i,j;
    
    // Free possible previous font table
    if(d4d_FontTableRam)
      D4D_MemFree(d4d_FontTableRam);
    
    j = (sizeof(D4D_FONT_TYPE) * (D4D_GetFontTableEntries(d4d_pFontTable) + 1));
    
    // Allocate the new place for the font table
    d4d_FontTableRam = D4D_MemAlloc(j);
    
    if(d4d_FontTableRam == NULL)
      return D4D_FALSE;
    
    //Clear up the d4d_FontTableRam structure
    for(i = 0; i < j; i++)
      ((Byte*)d4d_FontTableRam)[i] = 0;
    
    // Load to RAM the font tables
    D4D_ExtFntSetWorkPath(pFntWorkPath);   
   
    d4d_pFontTable = d4d_FontTableRam;
    
    return D4D_TRUE;
  }

  /**************************************************************//*!
  *
  * Set Font external source files work directory
  *
  ******************************************************************/
  void D4D_ExtFntSetWorkPath(D4D_CHAR* pPath)
  {
    Byte i=0;
    D4D_FILEPTR pFile;
    LWord fileSize;
    D4D_FONT_DESCRIPTOR* pFntDesc;
    if(pPath)
      pFntWorkPath = pPath; 
    
    // create the new font table from template and also copy the new fonts to the RAM
    
    // to do Endiniasm
    
    while(d4d_pFontTable[i].pFontDescriptor || d4d_pFontTable[i].fileName)
    {
      // Standard local memory based font?
      if(d4d_pFontTable[i].fileName == NULL)
      {
        // just copy to RAM version of font table
        d4d_FontTableRam[i] = d4d_pFontTable[i];
      }else
      {
        // external file that should be loaded into RAM to be able to use it
        
        // 1. clear possible previous allocated RAM
        if(d4d_FontTableRam[i].pFontDescriptor)
          D4D_MemFree((void*)d4d_FontTableRam[i].pFontDescriptor);
        
        // 2. just copy to RAM version of font table
        d4d_FontTableRam[i] = d4d_pFontTable[i];
        
        // 3. try to open the font file
        pFile = D4D_OpenFntFile((void*)d4d_pFontTable[i].fileName);
        
        // 4. if the file can't be opened, just go to next font
        if(pFile == NULL)
        {
          i++;
          continue;
        }
        // 5. Get the size of file 
        if(D4D_FileSeek(pFile, 0, D4D_SEEK_END) != 0)
        {
          D4D_FileClose(pFile);
          i++;
          continue;
        }
        
        fileSize = D4D_FileTell(pFile);
        
        // Return the file location to start
        if(D4D_FileSeek(pFile, 0, D4D_SEEK_SET) != 0)
        {
          D4D_FileClose(pFile);
          i++;
          continue;
        }
        
        // 6. Alloc the memory on heap for the data
        d4d_FontTableRam[i].pFontDescriptor = D4D_MemAlloc(fileSize);
        
        if(d4d_FontTableRam[i].pFontDescriptor == NULL)
        {
          D4D_FileClose(pFile);
          i++;
          continue;
        }
        
        // 7. Copy the content of file to RAM 
        if(D4D_FileRead(pFile, d4d_FontTableRam[i].pFontDescriptor, fileSize) != fileSize)
        {
          D4D_MemFree(d4d_FontTableRam[i].pFontDescriptor);
          d4d_FontTableRam[i].pFontDescriptor = NULL;
          D4D_FileClose(pFile);
          i++;
          continue;
        }  
        
        // 8. Adjust the used pointers in all loaded structures
        pFntDesc = d4d_FontTableRam[i].pFontDescriptor;
#ifdef D4D_UNICODE        
        while((pFntDesc != NULL) && (pFntDesc - d4d_FontTableRam[i].pFontDescriptor) <= fileSize)
        {
#endif
          if(pFntDesc->pIxTable)
            *(LWord*)&(pFntDesc->pIxTable) += (LWord)pFntDesc;
          
          if(pFntDesc->pOffTable)
            *(LWord*)&(pFntDesc->pOffTable) += (LWord)pFntDesc;
          
          if(pFntDesc->pSizeTable)
            *(LWord*)&(pFntDesc->pSizeTable) += (LWord)pFntDesc;
          
          if(pFntDesc->pFontData)
            *(LWord*)&(pFntDesc->pFontData) += (LWord)pFntDesc;
#ifdef D4D_UNICODE          
          if(pFntDesc->pNext)
          {
            *(LWord*)&(pFntDesc->pNext) += (LWord)pFntDesc;
            pFntDesc = (D4D_FONT_DESCRIPTOR*)pFntDesc->pNext;
          }
          else
            pFntDesc = NULL;
        }
#endif                
                
      }
      
      i++;  
    }
  }

  /**************************************************************//*!
  *
  * Set Font external source files work directory
  *
  ******************************************************************/
  void D4D_ExtFntSetDfltWorkPath(void)
  {
    D4D_ExtFntSetWorkPath(D4D_FNT_WORK_PATH_DEFAULT);
  }
  /**************************************************************//*!
  *
  * Get Font external source files work directory
  *
  ******************************************************************/
  D4D_CHAR* D4D_ExtFntGetWorkPath(void)
  {
    return pFntWorkPath;  
  }

  /**************************************************************//*!
  *
  * Open font external source file
  *
  ******************************************************************/
  D4D_FILEPTR D4D_OpenFntFile(D4D_CHAR* pFileName)
  {
    // create the file name (if needed)
    if(D4D_FileIsAbsolutePath(pFileName))
    {
      // the input path is absolute and can't be changed
      return D4D_FileOpen(pFileName,"r");
    }
    else
    {
      // the input path is relative and must be updated with preselected image work paths
      if(D4D_StrLen(pFntWorkPath) + D4D_StrLen(pFileName) >= D4D_EXTSRC_BUFF_SIZE)
        return NULL;
      
      D4D_StrCopy((D4D_CHAR*)d4d_extsrcBuffer, pFntWorkPath);
      D4D_StrCat((D4D_CHAR*)d4d_extsrcBuffer, pFileName);
      return D4D_FileOpen((D4D_CHAR*)d4d_extsrcBuffer,"r");
    }
    
  }
#endif

/**************************************************************//*!
*
* Change the Font Table
*
******************************************************************/
void D4D_SetFontTable(D4D_FONT_TYPE* pFontTable)
{
  if(pFontTable == d4d_pFontTable)
    return;
  
  if(pFontTable == NULL)
    d4d_pFontTable = (D4D_FONT_TYPE*)&d4d_FontTable;
  else  
    d4d_pFontTable = pFontTable;  
  
#if (D4D_EXTSRC_FILE_ENABLE != D4D_FALSE) && (D4D_FNT_EXTSRC_SUPPORT != D4D_FALSE)
  D4D_ExtFntInit();
#endif
  
}

D4D_FONT_TYPE* D4D_GetFontTable(void)
{
  return d4d_pFontTable;
}
/**************************************************************//*!
*
* Get selected font information
*
******************************************************************/

D4D_FONT_TYPE* D4D_GetFont(D4D_FONT ix)
{        

#if (D4D_FONT_TABLE_DISABLED == D4D_TRUE)
    D4D_UNUSED(ix);
    return NULL;
#else      
    Byte i=0;
    
    while(d4d_pFontTable[i].pFontDescriptor != NULL)
    {
      if(d4d_pFontTable[i].ix_font == ix)
        return (D4D_FONT_TYPE*) &(d4d_pFontTable[i]);  
      i++;
    }           
    return NULL;
#endif
}

/**************************************************************//*!
*
* Get selected font size information
*
******************************************************************/

D4D_FONT_SIZES D4D_GetFontSize(D4D_FONT ix)
{    
    D4D_FONT_SIZES tmpFontSize;

    tmpFontSize.height = D4D_GetFontHeight(ix);
    tmpFontSize.width = D4D_GetFontWidth(ix);       
    return tmpFontSize;
}


/**************************************************************//*!
*
* Get selected font height information
*
******************************************************************/

D4D_FONT_SIZE D4D_GetFontHeight(D4D_FONT ix)
{    
    D4D_FONT_TYPE* pFontType = D4D_GetFont(ix);   
    D4D_FONT_DESCRIPTOR* pFontDescriptor = (D4D_FONT_DESCRIPTOR*)(pFontType->pFontDescriptor);                                                              
    
    if(pFontType != NULL)
      return (D4D_FONT_SIZE)((pFontType->scale.height * pFontDescriptor->charFullSize.height) + pFontType->charSpacing.height);
           
    return (D4D_FONT_SIZE) 0;
}

/**************************************************************//*!
*
* Get selected font width information
*
******************************************************************/

D4D_FONT_SIZE D4D_GetFontWidth(D4D_FONT ix)
{    
    D4D_FONT_TYPE* pFontType = D4D_GetFont(ix);
    D4D_FONT_DESCRIPTOR* pFontDescriptor = (D4D_FONT_DESCRIPTOR*)(pFontType->pFontDescriptor);                                                              
    
    if(pFontType != NULL)
      return (D4D_FONT_SIZE)((pFontType->scale.width * pFontDescriptor->charFullSize.width) + pFontType->charSpacing.width);
           
    return (D4D_FONT_SIZE) 0;
}

/**************************************************************//*!
*
* Get selected font width information
*
******************************************************************/

D4D_FONT_SIZE D4D_GetCharWidth(D4D_FONT ix, D4D_TCHAR ch)
{    
    D4D_FONT_TYPE* pFontType = D4D_GetFont(ix);
    const D4D_FONT_DESCRIPTOR* pFontDescriptor;
    D4D_FONT_SIZE tmp_width = 0;
    
    if(NULL == pFontType)
      return 0;
    
    if(ch < ' ')
      return 0;
            
    pFontDescriptor = (D4D_FONT_DESCRIPTOR*)(pFontType->pFontDescriptor);
    
#ifdef D4D_UNICODE
    while (pFontDescriptor)
    {
      if((ch & 0xff00) == (pFontDescriptor->startChar & 0xff00))
        break;
      
      pFontDescriptor = pFontDescriptor->pNext;
    }    
#endif
    
    if((pFontType != NULL) && (pFontDescriptor != NULL))
    {
      // Get the raw width of char
      if(pFontDescriptor->flags & D4D_FONT_FLAGS_FNT_WIDTH_PROPORTIONAL) {  
        tmp_width = pFontDescriptor->pSizeTable[D4D_GetCharIndex((D4D_FONT_DESCRIPTOR*)pFontDescriptor, ch)];
      }else {
        tmp_width = pFontDescriptor->charFullSize.width;        
      }
      // Apply the scale for the char width
      tmp_width *= pFontType->scale.width;
      // Add the char spacing 
      tmp_width += pFontType->charSpacing.width;
    }
    
    // return the char width
    return tmp_width;
}



/**************************************************************//*!
*
* Get "real" index in data font table of used font
*
******************************************************************/

static D4D_FONT_IX D4D_GetCharIndex(D4D_FONT_DESCRIPTOR* pFontDescriptor, D4D_TCHAR ch) 
{ 
  D4D_FONT_IX char_ix = (D4D_FONT_IX)(ch -  pFontDescriptor->startChar);

  // is the indes table linear or nonlinear?
  if(pFontDescriptor->flags & D4D_FONT_FLAGS_IX_STYLE_NONLINEAR) {    
    // Index table is used? 
    if(pFontDescriptor->pIxTable != NULL) {
      if(pFontDescriptor->flags & D4D_FONT_FLAGS_IX_TYPE_MAP) {
        // Map style of Index Table
        D4D_FONT_IX i;
        // Look for the used char in index map table
        for(i = 0; i < pFontDescriptor->charCnt; i++) {
          if(ch == pFontDescriptor->pIxTable[i])
            return i;
        }
      }else {
        
         // check if the char is set into range of used font
        if(char_ix < pFontDescriptor->charCnt)
          // Look up style of Index Table
          return pFontDescriptor->pIxTable[char_ix];      
      }
    }
  }else 
  {
     // check if the char is set into range of used font
    if(char_ix < pFontDescriptor->charCnt)    
      return char_ix;
  }
  
  // return the "real table index" of used car 
  return pFontDescriptor->charDefault; 
}


/**************************************************************//*!
*
* Get pointer on bitmap of selected char of used font
*
******************************************************************/

D4D_FONT_DATA* D4D_GetCharData(D4D_FONT_TYPE* pFontType, D4D_TCHAR ch) {
  
  const D4D_FONT_DESCRIPTOR* pFontDescriptor = (D4D_FONT_DESCRIPTOR*)(pFontType->pFontDescriptor);
  D4D_FONT_IX char_ix;

#ifdef D4D_UNICODE
  while (pFontDescriptor)
  {
    if((ch & 0xff00) == (pFontDescriptor->startChar & 0xff00))
      break;
    
    pFontDescriptor = pFontDescriptor->pNext;
  }    
#endif
  
  if(!pFontDescriptor)
    return NULL;
  
  char_ix = D4D_GetCharIndex((D4D_FONT_DESCRIPTOR*)pFontDescriptor, ch);
  
  // check the style of font monospace / proportional
  if(pFontDescriptor->flags & D4D_FONT_FLAGS_FNT_WIDTH_PROPORTIONAL) {
    // Proportinal font is used
    
    // Check if Offset table exist
    if(pFontDescriptor->pOffTable == NULL)
      return NULL;
      
    return (D4D_FONT_DATA*)(pFontDescriptor->pFontData + pFontDescriptor->pOffTable[char_ix]);
    
      
  }else {
    // Monospace font is used
    return (D4D_FONT_DATA*)(pFontDescriptor->pFontData + char_ix * pFontDescriptor->charBmpSize);
  }
}

/**************************************************************//*!
*
* Get "real" index in data font table of used font
*
******************************************************************/



static D4D_BOOL D4D_LCD_GetFntBit(CHAR_PXL_DESC * pPxlDesc) {
  
  Word bit_cnt;
  Byte data;
  
  // This is decoder only for fonts that are row scan based and preffered !!!
  if(pPxlDesc->pFontDescriptor->pack & D4D_FONT_PACK_SCANPREFERRED_COLUMN)
    return 0;
  
  if(pPxlDesc->pFontDescriptor->pack & D4D_FONT_PACK_SCANBASED_COLUMN)
    return 0;
  
  
  if(pPxlDesc->pFontDescriptor->pack & D4D_FONT_PACK_COMPRESSED_ON) {
    // data are packed in bit stream
    bit_cnt = (Word)(pPxlDesc->char_width * pPxlDesc->row + pPxlDesc->clm);
    data = pPxlDesc->pFntData[bit_cnt / 8];
  }else 
  {
    // data aren't packed
    Word byte_cnt; 
    byte_cnt = (Word)((((pPxlDesc->char_width - 1) / 8) + 1) * pPxlDesc->row);  // get the start of row index of data
    byte_cnt += (pPxlDesc->clm / 8);  // add the collumn index move
    data = pPxlDesc->pFntData[byte_cnt];  // get the data byte
    bit_cnt = (Byte)(pPxlDesc->clm % 8); // get position of bit in byte for this pixel
  }
  
  if(pPxlDesc->pFontDescriptor->pack & D4D_FONT_PACK_BITORDER_LITTLEEND)
    return (D4D_BOOL)((data >> (bit_cnt % 8)) & 0x01);   // little endian decoding
  else
    return (D4D_BOOL)((data << (bit_cnt % 8)) & 0x80);   // big endian decoding
}

//static D4D_BOOL D4D_LCD_GetFntBit(Byte row, Byte clm, D4D_FONT_SIZE char_width, D4D_FONT_DATA* pFntData, D4D_FONT_DESCRIPTOR* pFontDescriptor) {
//  
//  Word bit_cnt;
//  Byte data;
//  
//  // This is decoder only for fonts that are row scan based and preffered !!!
//  if(pFontDescriptor->pack & D4D_FONT_PACK_SCANPREFERRED_COLUMN)
//    return 0;
//  
//  if(pFontDescriptor->pack & D4D_FONT_PACK_SCANBASED_COLUMN)
//    return 0;
//  
//  
//  if(pFontDescriptor->pack & D4D_FONT_PACK_COMPRESSED_ON) {
//    // data are packed in bit stream
//    bit_cnt = (Word)(char_width * row + clm);
//    data = pFntData[bit_cnt / 8];
//  }else 
//  {
//    // data aren't packed
//    Word byte_cnt; 
//    byte_cnt = (Word)((((char_width - 1) / 8) + 1) * row);  // get the start of row index of data
//    byte_cnt += (clm / 8);  // add the collumn index move
//    data = pFntData[byte_cnt];  // get the data byte
//    bit_cnt = (Byte)(clm % 8); // get position of bit in byte for this pixel
//  }
//  
//  if(pFontDescriptor->pack & D4D_FONT_PACK_BITORDER_LITTLEEND)
//    return (D4D_BOOL)((data >> (bit_cnt % 8)) & 0x01);   // little endian decoding
//  else
//    return (D4D_BOOL)((data << (bit_cnt % 8)) & 0x80);   // big endian decoding
//}

D4D_COOR D4D_GetNextTab(D4D_TAB* pTab, D4D_COOR pos)
{
  D4D_INDEX ix = 0;

  if(pTab == NULL)
    return 0;
  
  if(pTab->pTabTable == NULL)
    return 0;

  while(pTab->pTabTable[ix])
  {
    if((pos + pTab->tabOffset) < pTab->pTabTable[ix])
      return (D4D_COOR)(pTab->pTabTable[ix] /*- pTab->tabOffset*/);
    
    ix++;
  }

  return 0;

}

/**************************************************************//*!
*
* Print the Char in ASCII in simple format up to 8 columns
*
******************************************************************/
static D4D_COOR D4D_LCD_PrintChr(D4D_TCHAR ch, D4D_PRINT_STRING* p_CharDes)
{
  D4D_FONT_SIZE xScale, yScale;               
  LWord row_cnt;
  LWord bit_cnt;
  LWord px_cnt;
  LWord yScale_cnt;
 // LWord char_width;
  D4D_COLOR clr;
  D4D_FONT_TYPE* pFontType = (D4D_FONT_TYPE*)p_CharDes->pFontType;
  CHAR_PXL_DESC pxlDesc;
  
  if(pFontType == NULL)
    return 0;
  
  pxlDesc.pFontDescriptor = (D4D_FONT_DESCRIPTOR*)(pFontType->pFontDescriptor);
  
  if(ch < ' ')
  {
    if(ch == '\t')
    {
      D4D_COOR nextTab;
      // print tabulator
      nextTab = D4D_GetNextTab(p_CharDes->pTab, p_CharDes->x);
      
      if(nextTab)
      {
        D4D_LCD_Box(p_CharDes->x, p_CharDes->y, nextTab,\
                       (D4D_COOR)(p_CharDes->y + D4D_GetFontHeight(p_CharDes->pFontType->ix_font) - 1),\
                       p_CharDes->colorBack);
        return (D4D_COOR)(nextTab - p_CharDes->x);
      }
    }else
      return 0;  // none printable char  
  }  
  xScale = p_CharDes->pFontType->scale.width;
  yScale = p_CharDes->pFontType->scale.height;

#ifdef D4D_UNICODE
  while (pxlDesc.pFontDescriptor)
  {
    if((ch & 0xff00) == (pxlDesc.pFontDescriptor->startChar & 0xff00))
      break;
    
    pxlDesc.pFontDescriptor = (D4D_FONT_DESCRIPTOR*)pxlDesc.pFontDescriptor->pNext;
  }    
#endif      
        
  if(!pxlDesc.pFontDescriptor)
    return 0;  
  
  pxlDesc.pFntData = D4D_GetCharData( pFontType, ch);	 // Get pointer to char bmp data
	
  if(pxlDesc.pFontDescriptor->flags & D4D_FONT_FLAGS_FNT_WIDTH_PROPORTIONAL)  
    pxlDesc.char_width = pxlDesc.pFontDescriptor->pSizeTable[D4D_GetCharIndex(pxlDesc.pFontDescriptor, ch)];
  else
    pxlDesc.char_width = pxlDesc.pFontDescriptor->charFullSize.width;
	
  
  
#ifndef D4D_COLOR_TRANSPARENT  
  
  if(p_CharDes->properties & D4D_FNT_PRTY_TRANSPARENT_MASK)
  {
    for (pxlDesc.row=0; pxlDesc.row<pxlDesc.pFontDescriptor->charFullSize.height; ++pxlDesc.row)			// For each vertical row of this character  
    {
      for(pxlDesc.clm = 0; pxlDesc.clm < pxlDesc.char_width; pxlDesc.clm++)
      {
        if(D4D_LCD_GetFntBit(&pxlDesc))
        {
          D4D_LCD_Box((D4D_COOR)(p_CharDes->x + pFontType->charSpacing.width + pxlDesc.clm * xScale),\
            (D4D_COOR)(p_CharDes->y + pFontType->charSpacing.height + (pxlDesc.row * yScale)),\
            (D4D_COOR)(p_CharDes->x + pFontType->charSpacing.width + pxlDesc.clm * xScale + xScale - 1),\
            (D4D_COOR)(p_CharDes->y + pFontType->charSpacing.height + (pxlDesc.row * yScale) + yScale - 1),\
            p_CharDes->colorText);
        } 
      }									     
    }											  
  } 
  else  // if(Transparent)
#else
  if(p_CharDes->properties & D4D_FNT_PRTY_TRANSPARENT_MASK)
    p_CharDes->colorBack = D4D_COLOR_TRANSPARENT;
    
#endif   
  {
    (void)D4D_LLD_LCD.D4DLCD_SetWindow(p_CharDes->x, p_CharDes->y, 
      (D4D_COOR)(p_CharDes->x + D4D_GetCharWidth(p_CharDes->pFontType->ix_font, ch) - 1), 
      (D4D_COOR)(p_CharDes->y + D4D_GetFontHeight(p_CharDes->pFontType->ix_font) + 1)); // Set character window
    
    // add on background box on bottom of character to provide line spacing
    //char_width = D4D_GetCharWidth(p_CharDes->pFontType->ix_font, ch);
    
    for(row_cnt=0; row_cnt < pFontType->charSpacing.height; ++row_cnt) 
    {
      for(bit_cnt = 0; bit_cnt < (D4D_COOR)(pxlDesc.char_width * xScale + pFontType->charSpacing.width); ++bit_cnt)
        D4D_LLD_LCD.D4DLCD_Send_PixelColor(p_CharDes->colorBack);	  
    }
    
    
    for (pxlDesc.row=0; pxlDesc.row < pxlDesc.pFontDescriptor->charFullSize.height; ++pxlDesc.row)			// For each vertical row of this character  
    {
      for (yScale_cnt=1; yScale_cnt<=yScale; ++yScale_cnt)		// As often as needed by the Y-scale factor (e.g. draws complete line twice if double height font is requested)  
      {
        for (pxlDesc.clm=1; pxlDesc.clm<=pFontType->charSpacing.width; ++pxlDesc.clm)     
        {
          // empty column = always background color
          D4D_LLD_LCD.D4DLCD_Send_PixelColor(p_CharDes->colorBack);								// draw pixel
        }
        
        for(pxlDesc.clm = 0; pxlDesc.clm < pxlDesc.char_width; ++pxlDesc.clm)
        {
          if(D4D_LCD_GetFntBit(&pxlDesc))
            clr = p_CharDes->colorText;
          else
            clr = p_CharDes->colorBack;
        
          for (px_cnt=1; px_cnt<=xScale; ++px_cnt)			// as often as needed by the horizotal scale factor (e.g. draws pixel twice if double wide font is requested)  
            D4D_LLD_LCD.D4DLCD_Send_PixelColor(clr);			// draw pixel
        }
      }											// draw this line again if needed for scaling  
    }	 	
  }// next row of this character
  
  return (D4D_COOR)(pxlDesc.char_width * xScale + pFontType->charSpacing.width);
}

/**************************************************************//*!
*
* Print the String in ASCII in simple format up to 8 columns
*
******************************************************************/
void D4D_LCD_PrintStr(D4D_PRINT_STRING* p_StrDes)
{
  D4D_INDEX str_len, ch_ix;
  
  D4D_COOR orig_x = p_StrDes->x;
  D4D_COOR base_y, base_x;
  D4D_FONT_TYPE* pFontType = (D4D_FONT_TYPE*)p_StrDes->pFontType;
  D4D_FONT_DESCRIPTOR* pFontDescriptor = (D4D_FONT_DESCRIPTOR*)(pFontType->pFontDescriptor);	
  const D4D_TCHAR pLongTextEnd[] = D4D_DEFSTR("...");
  D4D_BOOL longText = D4D_FALSE;
  D4D_COOR tmp_MaxLen = p_StrDes->maxWidth;
  D4D_COOR longTextLen;
  // check if fontdescriptor exist and check the font scale value
  if((pFontDescriptor == NULL) || (pFontType->scale.width == 0) || (pFontType->scale.height == 0))
      return;      
  
  // long strings restriction
  if(tmp_MaxLen)
  {
    // check if the text fits into enabled width
    D4D_STRING tmpString;
    D4D_COOR tmp_txtLen;
    
    tmpString.pText = p_StrDes->pText;
    tmpString.buffSize = 0;
    tmpString.fontId = p_StrDes->pFontType->ix_font;
    tmpString.printLen = p_StrDes->textLength;
    tmpString.printOff = p_StrDes->textOffset;    
    tmp_txtLen = D4D_GetTextBuffWidthTabAdj(&tmpString, p_StrDes->pTab);
    
    if(p_StrDes->pTab)
    {
      if(tmp_txtLen > p_StrDes->pTab->tabOffset)
        tmp_txtLen -= p_StrDes->pTab->tabOffset;
      else
        return;
    }
    
    longTextLen = D4D_GetTextWidth(tmpString.fontId, (D4D_TCHAR*)pLongTextEnd);
    
    if(tmp_txtLen > p_StrDes->maxWidth)  
    {
      if(tmp_txtLen < longTextLen)
        return;
      
      tmp_MaxLen -= longTextLen;
      longText = D4D_TRUE;       
    }
  }
  
  
  
  
#if D4D_EXTSRC_TEXT_ENABLE == D4D_TRUE    

  if(*(p_StrDes->pText) == d4d_extsrcLocale.strTranslatePrefix)
  {
    D4D_INDEX charCnt = 0;
    D4D_INDEX charOff = p_StrDes->textOffset;
    D4D_INDEX charCntTmp; 

    do
    {
      charOff += charCnt;
      
      charCnt = (D4D_INDEX)d4d_extsrcLocale.D4DLOCALE_TranslateStr(p_StrDes->pText, (D4D_TCHAR*)d4d_extsrcBuffer, charOff, D4D_EXTSRC_BUFF_SIZE);
    
        // get the lenght of string 
      str_len = charCnt;     

      if(p_StrDes->textLength)
        if(str_len > p_StrDes->textLength)
          str_len = p_StrDes->textLength;
    
      str_len += (charOff - 1);
    
      for(charCntTmp = 0; charCntTmp < charCnt; charCntTmp++)
      {
        if((!((D4D_TCHAR*)d4d_extsrcBuffer)[charCntTmp]) || (str_len < (charCntTmp + charOff)))
        {
          charCnt = 0;
          break;
        }
        if(longText)
        {
          D4D_COOR tmp_Len;
          tmp_Len = D4D_GetCharWidth(p_StrDes->pFontType->ix_font, ((D4D_TCHAR*)d4d_extsrcBuffer)[charCntTmp]);
          
          if(tmp_MaxLen - tmp_Len <= 0)
          {
            charCnt = 0;
            break;
          }
          tmp_MaxLen -= tmp_Len;        
        }        
        
        p_StrDes->x += D4D_LCD_PrintChr(((D4D_TCHAR*)d4d_extsrcBuffer)[charCntTmp], p_StrDes); // print the char and automatically update the coordintaion of next char  
      }
    }while(charCnt == D4D_EXTSRC_BUFF_SIZE);
    
  
  }else
#endif        
  {
    D4D_TCHAR* pText;
    
    pText = &p_StrDes->pText[p_StrDes->textOffset];
  				
    // get the lenght of string 
    str_len = D4D_GetTextLength(p_StrDes->pText);     

    if(p_StrDes->textLength)
      if(str_len > p_StrDes->textLength)
        str_len = p_StrDes->textLength;
    
    // Draw the text
    for (ch_ix=0; ch_ix<str_len; ++ch_ix)	// For each character in the string 	  
    {
      if(longText)
      {
        D4D_COOR tmp_Len;
        tmp_Len = D4D_GetCharWidth(p_StrDes->pFontType->ix_font, *pText);  
        
        if(tmp_MaxLen - tmp_Len <= 0)
        {
          break;
        }
        tmp_MaxLen -= tmp_Len;        
      }
      
      p_StrDes->x += D4D_LCD_PrintChr(*pText++, p_StrDes); // print the char and automatically update the coordintaion of next char         
    }
  }



  if(longText)
  {
    ch_ix=0;  
    while(pLongTextEnd[ch_ix])	
      p_StrDes->x += D4D_LCD_PrintChr(pLongTextEnd[ch_ix++], p_StrDes); // print the char and automatically update the coordintaion of next char   
    
    if(!(p_StrDes->properties & D4D_FNT_PRTY_TRANSPARENT_MASK))
      D4D_FillRectXY(p_StrDes->x, p_StrDes->y, (D4D_COOR)(orig_x + p_StrDes->maxWidth - 1), (D4D_COOR)(p_StrDes->y + D4D_GetFontHeight(p_StrDes->pFontType->ix_font) - 1), p_StrDes->colorBack);
    
    
  }
	
	// Draw the Underline option	
	switch(p_StrDes->properties & D4D_FNT_PRTY_UNDERLINE_MASK)
	{
	  case D4D_FNT_PRTY_UNDERLINE_LINE_MASK:  // Draw the full line
	    base_y = (D4D_COOR)(p_StrDes->y + pFontDescriptor->charBaseLine * pFontType->scale.height);
	    base_x = p_StrDes->x;
	    D4D_FillRectXY(orig_x, base_y, (D4D_COOR)(base_x - 1), (D4D_COOR)(base_y + pFontType->scale.height - 1), p_StrDes->colorText);
	    break;
	    
	  case D4D_FNT_PRTY_UNDERLINE_DOT_MASK:  // Draw the dot line
	    base_y = (D4D_COOR)(p_StrDes->y + pFontDescriptor->charBaseLine * pFontType->scale.height);
	    
	    ch_ix = 0;
	    for(base_x = orig_x; base_x < p_StrDes->x; base_x += pFontType->scale.height)
	    {
	      if(++ch_ix % 2)
	        D4D_FillRectXY(base_x, base_y, (D4D_COOR)(base_x + pFontType->scale.width - 1), (D4D_COOR)(base_y + pFontType->scale.height - 1), p_StrDes->colorText);	    
	    }
	    break;
	  default:
	    break;	    
	}
	
	// Draw the Strike Through option	
	if(p_StrDes->properties & D4D_FNT_PRTY_STRIKETHROUGH_MASK)
	{
	    Byte line_cnt = (Byte)((p_StrDes->properties & D4D_FNT_PRTY_STRIKETHROUGH_MASK) >> D4D_FNT_PRTY_STRIKETHROUGH_SHIFT);
      
      base_y = (D4D_COOR)(p_StrDes->y + (pFontDescriptor->charBaseLine - (pFontDescriptor->charSize / 3)) * pFontType->scale.height);
	    base_x = p_StrDes->x;
	    // Draw Strike Through line as many as specified by parameter
	    while(line_cnt)
	    {	      
	      D4D_FillRectXY(orig_x, base_y, (D4D_COOR)(base_x - 1), (D4D_COOR)(base_y - 1), p_StrDes->colorText);
	      base_y -=2;
              line_cnt--;
	    }
	      
	}
#if D4D_LLD_FLUSH_ELEMENT != D4D_FALSE
	D4D_LLD_LCD.D4DLCD_FlushBuffer(D4DLCD_FLSH_ELEMENT);
#endif
				
}


