/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_imgdec_d4d.h
*
* @author    b01119
* 
* @version   0.0.6.0
* 
* @date      Jun-29-2012
* 
* @brief     D4D driver external pictures d4d decoder functions header file 
*
******************************************************************************/

#ifndef __D4D_IMGDEC_D4D_H
#define __D4D_IMGDEC_D4D_H

/******************************************************************************
* Types
******************************************************************************/

#pragma pack (push)
#pragma pack (1)

typedef struct D4D_EXTSRC_HEADER_D4D_S
{
  char            ident[7];     // identification string of file "D4DBMP" include termination char
  unsigned char   version;      // version of d4d bitmap header version (at the moment 0)
  unsigned short  width;        // width of bitmap
  unsigned short  height;       // height of bitmap
  unsigned short  colorMode;    // color mode of stored bitmap
  unsigned short  reserved;     // allign  2 bytes to 32 bit shape
  unsigned long   dataOffset;   // offset of first byte of bitmap data
  unsigned long   dataSize;     // size of bitmap data
  unsigned long   palleteOffset;// offset of first byte of pallete data. If 0 there is no pallete
  unsigned long   palleteSize;  // size of pallete data
}D4D_EXTSRC_HEADER_D4D;

#pragma pack (pop)

/******************************************************************************
* Constants
******************************************************************************/



/******************************************************************************
* External source constants
******************************************************************************/


#define D4D_EXTSRC_HEADER_D4D_SIZE sizeof(D4D_EXTSRC_HEADER_D4D)

  

/******************************************************************************
* Global D4D bitmap drawing level functions
******************************************************************************/

#if (D4D_BMP_EXTSRC_SUPPORT_D4D == D4D_TRUE) && (D4D_EXTSRC_FILE_ENABLE == D4D_TRUE) 
  void D4D_ImgDecExtD4DDraw(D4D_COOR x, D4D_COOR y, D4D_CHAR* pFileName, D4D_BOOL greyScale, D4D_COOR radius);
  D4D_COOR D4D_ImgDecExtD4DGetWidth(D4D_CHAR* pFileName);
  D4D_COOR D4D_ImgDecExtD4DGetHeight(D4D_CHAR* pFileName);
  D4D_SIZE D4D_ImgDecExtD4DGetSize(D4D_CHAR* pFileName); 
  int D4D_ImgDecExtD4DGetHeader(D4D_CHAR* pFileName, void* pBuff, int max_size);

  void D4D_LCD_Bmp65kNoPalExt(D4D_COOR x1, D4D_COOR y1, D4D_COOR x2, D4D_COOR y2, 
                       D4D_FILEPTR pFile, LWord dataOffset, 
                       D4D_BOOL Compressed, D4D_BOOL GreyScale);
#else

  #define D4D_ImgDecExtD4DDraw       NULL
  #define D4D_ImgDecExtD4DGetWidth   NULL
  #define D4D_ImgDecExtD4DGetHeight  NULL
  #define D4D_ImgDecExtD4DGetSize    NULL
  #define D4D_ImgDecExtD4DGetHeader  NULL
  
#endif                     
                     
#endif /* __D4D_IMGDEC_D4D_H */