/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_object.c
*
* @author    b01119
* 
* @version   0.0.10.0
* 
* @date      Jun-29-2012
* 
* @brief     D4D driver object function c file 
*
******************************************************************************/

#include "d4d.h"
#include "common_files/d4d_private.h"

/**************************************************************//*!
*
* Global variables
*
******************************************************************/

/**************************************************************//*!
*
* invalidate object
*
******************************************************************/

void D4D_InvalidateObject(D4D_OBJECT_PTR pObject, D4D_BOOL bComplete)
{
  D4D_OBJECT_FLAGS flags = D4D_OBJECT_F_REDRAW;

  if(bComplete)
    flags |= D4D_OBJECT_F_REDRAWC;
  
  D4D_SetObjectFlags((D4D_OBJECT*)pObject,  flags);
  
  if(pObject->pData->pScreen)
    (pObject->pData->pScreen)->pData->flags |= D4D_SCR_FINT_CHECKOBJECTS;
}

/**************************************************************//*!
*
* Show / Hide object
*
******************************************************************/

void D4D_ShowObject(D4D_OBJECT_PTR pObject, D4D_BOOL bShow)
{
  if(bShow)
  {
    pObject->pData->flags |= D4D_OBJECT_F_VISIBLE;
    D4D_InvalidateObject(pObject, D4D_FALSE);
  }
  else
  {
    pObject->pData->flags &= ~D4D_OBJECT_F_VISIBLE;
    
    if((D4D_OBJECT*)pObject == D4D_GetFocusedObject(pObject->pData->pScreen))
      D4D_FocusNextObject(pObject->pData->pScreen, D4D_FALSE);
    
    if(pObject->pRelations)
    {
      D4D_OBJECT* pParent = D4D_GetParentObject((D4D_OBJECT*)pObject);
      
      if(pParent)
        D4D_InvalidateObject(pParent, D4D_TRUE);
      else
        D4D_InvalidateScreen(pObject->pData->pScreen, D4D_TRUE);
      
    }else
      D4D_InvalidateScreen(pObject->pData->pScreen, D4D_TRUE);        
  }
}

/**************************************************************//*!
*
* Get the user pointer / data of of object
*
******************************************************************/

void* D4D_GetUserPointer(D4D_OBJECT *pThis)
{
  return pThis->userData;
}


/**************************************************************//*!
*
* enable or diasble touch pad capatibilities for an OBJECT
*
******************************************************************/

void D4D_EnableTouchScreen(D4D_OBJECT_PTR pObj, D4D_BOOL bEnable, D4D_BOOL bFastTouch)
{
  if(bEnable)  
    pObj->pData->flags |= D4D_OBJECT_F_TOUCHENABLE;
  else
    pObj->pData->flags &= ~D4D_OBJECT_F_TOUCHENABLE;
    
  if(bFastTouch)  
    pObj->pData->flags |= D4D_OBJECT_F_FASTTOUCH;
  else
    pObj->pData->flags &= ~D4D_OBJECT_F_FASTTOUCH;  

}


/**************************************************************//*!
*
* enable or diasble receiving timeticks for an OBJECT
*
******************************************************************/

void D4D_EnableTimeTicks(D4D_OBJECT_PTR pObj, D4D_BOOL bEnable)
{
  if(bEnable)  
    pObj->pData->flags |= D4D_OBJECT_F_TIMETICK;
  else
    pObj->pData->flags &= ~D4D_OBJECT_F_TIMETICK;
}

/**************************************************************//*!
*
* Capturing keys
*
******************************************************************/

void D4D_CaptureKeys(D4D_OBJECT_PTR  pObj)
{
    // NOTE: we need to send message, but we may just be in the middle of
    //       message processing (very likely). This may cause problem with
    //       the global d4d_msg object as we are changing it...
    //       We better use the temporary memory to instanitate the message
    
    D4D_MESSAGE* pMsg = (D4D_MESSAGE*) d4d_scratchPad;

    if(d4d_pKeysCapturer == pObj)
      return;

    pMsg->pScreen = D4D_GetActiveScreen();
    
    if(d4d_pKeysCapturer)
    {
        pMsg->nMsgId = D4D_MSG_KILLCAPTURE;
        pMsg->pObject = d4d_pKeysCapturer;
        D4D_SendMessage(pMsg); 
        
        D4D_InvalidateObject(d4d_pKeysCapturer, D4D_FALSE);
        d4d_pKeysCapturer = NULL;
    }
    
    if(pObj != NULL)
    {
      if((pObj->pData->flags & (D4D_OBJECT_F_VISIBLE | D4D_OBJECT_F_ENABLED | D4D_OBJECT_F_TABSTOP)) == (D4D_OBJECT_F_VISIBLE | D4D_OBJECT_F_ENABLED | D4D_OBJECT_F_TABSTOP))
      {
        if(D4D_IsEnabled((D4D_OBJECT*)pObj))
        {
          d4d_pKeysCapturer = (D4D_OBJECT*) pObj;  
          pMsg->nMsgId = D4D_MSG_SETCAPTURE;
          pMsg->pObject = d4d_pKeysCapturer;               
          D4D_SendMessage(pMsg);        
          D4D_InvalidateObject(d4d_pKeysCapturer, D4D_FALSE);                
        }
      }
    }
}

/**************************************************************//*!
*
* Get Object that is capturing keys
*
******************************************************************/


D4D_OBJECT* D4D_GetCapturedObject(void)
{
  return d4d_pKeysCapturer;
}


/**************************************************************//*!
*
* enable or diasble tab stop for an OBJECT
*
******************************************************************/

void D4D_EnableTabStop(D4D_OBJECT_PTR pObj, D4D_BOOL bEnable)
{
  if(bEnable)  
    pObj->pData->flags |= D4D_OBJECT_F_TABSTOP;
  else
    pObj->pData->flags &= ~D4D_OBJECT_F_TABSTOP;
}

/**************************************************************//*!
*
* enable or diasble  an OBJECT
*
******************************************************************/

void D4D_EnableObject(D4D_OBJECT_PTR pObj, D4D_BOOL bEnable)
{
//  if(((pObj->pData->flags & D4D_OBJECT_F_ENABLED) && !bEnable) || (!pObj->pData->flags.bits.bEnabled && bEnable))
  if((pObj->pData->flags & D4D_OBJECT_F_ENABLED) ^ bEnable)
  {      
    if(bEnable)
      pObj->pData->flags |= D4D_OBJECT_F_ENABLED;
    else
    {
      pObj->pData->flags &= ~D4D_OBJECT_F_ENABLED;
      
      if((D4D_OBJECT*)pObj == d4d_pKeysCapturer)
        D4D_CaptureKeys(NULL);
      
      if((D4D_OBJECT*)pObj == D4D_GetFocusedObject(pObj->pData->pScreen))
          D4D_FocusNextObject(pObj->pData->pScreen, D4D_FALSE);
    }
    
    D4D_InvalidateObject(pObj, D4D_TRUE);
  }
}


/**************************************************************//*!
*
* defualt message handler for all OBJECTS
*
******************************************************************/

void D4D_ObjOnMessage(D4D_MESSAGE* pMsg)
{
  if(pMsg->pObject)
  {
    if(pMsg->nMsgId == D4D_MSG_ONINIT)
      pMsg->pObject->pData->flags &= ~D4D_OBJECT_F_NOTINIT; // just clear the non init flags for objects which doeasn't handle this message
  }
  
}

/*******************************************************
*
* The object default check coordinators  handler 
*
*******************************************************/

D4D_BOOL D4D_ObjectCheckCoordinates(D4D_OBJECT* pThis, D4D_POINT point)
{

#ifndef D4D_LLD_TCH
  D4D_UNUSED(pThis);
  D4D_UNUSED(point);  
#else  

  if((point.x >= pThis->position.x) && (point.y >= pThis->position.y))
  { 
    if((point.x <= (pThis->position.x + pThis->size.cx)) && (point.y <= (pThis->position.y + pThis->size.cy)))
      return D4D_TRUE;
  }  
#endif   
  return D4D_FALSE;
}

/**************************************************************//*!
*
* Function returns the parent object 
*
******************************************************************/
D4D_OBJECT* D4D_GetParentObject(D4D_OBJECT* pObject)
{
  return (D4D_OBJECT*)pObject->pRelations[D4D_OBJECT_USR_DATA_PARENT_IX];    
}

/**************************************************************//*!
*
* Function returns the next object defined in screen (top level)
*
******************************************************************/
D4D_OBJECT* D4D_FindNextObjectOnScreen(D4D_OBJECT* pObject)
{
  D4D_SCREEN* pScreen = pObject->pData->pScreen;
  const D4D_OBJECT* const* pObjects = pScreen->pObjects;
  LWord i = 0;
  while(pObjects[i])
  {
    if(pObjects[i] == pObject)
      if(pObjects[i+1])
        return (D4D_OBJECT*)pObjects[i+1];
      else
        return (D4D_OBJECT*)pObjects[0];
      
    i++;  
  }    
  // Error case
  return NULL;
}

/**************************************************************//*!
*
* Function returns the next parent sibling object 
*
******************************************************************/
D4D_OBJECT* D4D_FindNextParentSiblingObject(D4D_OBJECT* pObject)
{
  D4D_OBJECT* pParent;
 // D4D_OBJECT* pSibling;
  
  pParent = D4D_GetParentObject(pObject);
  
  if(!pParent)
    return D4D_FindNextObjectOnScreen(pObject);
  
  // return pointer on parent sibling object or null if there is no parent next sibling
  return D4D_FindNextSiblingObject(pParent);
}

/**************************************************************//*!
*
* Function returns the next sibling object 
*
******************************************************************/
D4D_OBJECT* D4D_FindNextSiblingObject(D4D_OBJECT* pObject)
{
  D4D_OBJECT* pParent;
  LWord i = D4D_OBJECT_USR_DATA_CHILD_IX;
  pParent = D4D_GetParentObject(pObject);
  
  if(!pParent)
    return D4D_FindNextObjectOnScreen(pObject);
  
  while(pParent->pRelations[i])
  {
    if(pParent->pRelations[i] == pObject)
    {
      if(pParent->pRelations[i+1])
        return (D4D_OBJECT*)pParent->pRelations[i+1];
      else
        return NULL;
    }
    i++;
  }
      
  return NULL;  
}

/**************************************************************//*!
*
* Function returns the next object (looking for any object in object tree) 
*
******************************************************************/
D4D_OBJECT* D4D_FindNextObject(D4D_OBJECT* pObject, D4D_BOOL childrenAlso)
{
  D4D_OBJECT* pLocObject;

  
  // Compatible mode - just only screen top level objects
  if(pObject->pRelations == NULL)
    return D4D_FindNextObjectOnScreen(pObject);
  
  if(childrenAlso)
    if(pObject->pRelations[D4D_OBJECT_USR_DATA_CHILD_IX])    // Does I have childrens?
      return (D4D_OBJECT*)pObject->pRelations[D4D_OBJECT_USR_DATA_CHILD_IX];
  
  // Does I have siblings?
  pLocObject = D4D_FindNextSiblingObject(pObject);
  if(pLocObject)
    return pLocObject;
  
  // So my parent has to have siblings
  do
  {
    pLocObject = D4D_FindNextParentSiblingObject(pObject);  
    if(pLocObject)
      return pLocObject;
    
    pObject = D4D_GetParentObject(pObject);
    
  }while(1);
  
  /*
  if(pLocObject)
    return pLocObject;      
  
  return D4D_FindNextObjectOnScreen(pObject);        
  */
}

/**************************************************************//*!
*
* Function returns the previous object defined in screen (top level)
*
******************************************************************/
D4D_OBJECT* D4D_FindPreviousObjectOnScreen(D4D_OBJECT* pObject)
{
  D4D_SCREEN* pScreen = pObject->pData->pScreen;
  const D4D_OBJECT* const* pObjects = pScreen->pObjects;
  LWord i = 0;
  
  while(pObjects[i])
  {
    if(pObjects[i] == pObject)
      if(i)
        return (D4D_OBJECT*)pObjects[i-1];
      else
        return D4D_GetLastObject(pObject->pData->pScreen);
      
    i++;  
  }    
  // Error case
  return NULL;
}

/**************************************************************//*!
*
* Function returns the previous parent sibling object 
*
******************************************************************/
D4D_OBJECT* D4D_FindPreviousParentSiblingObject(D4D_OBJECT* pObject)
{
  D4D_OBJECT* pParent;
 // D4D_OBJECT* pSibling;
  
  pParent = D4D_GetParentObject(pObject);
  
  if(!pParent)
    return  D4D_FindPreviousObjectOnScreen(pObject);
  
  // return pointer on parent sibling object or null if there is no parent previous sibling
  return D4D_FindPreviousSiblingObject(pParent);  
}

/*
{
  D4D_OBJECT* pParent;
 // D4D_OBJECT* pSibling;
  
  pParent = D4D_GetParentObject(pObject);
  
  if(!pParent)
    return D4D_FindNextObjectOnScreen(pObject);
  
  // return pointer on parent sibling object or null if there is no parent next sibling
  return D4D_FindNextSiblingObject(pParent);
*/

/**************************************************************//*!
*
* Function returns the next sibling object 
*
******************************************************************/
D4D_OBJECT* D4D_FindPreviousSiblingObject(D4D_OBJECT* pObject)
{
  D4D_OBJECT* pParent;
  LWord i = D4D_OBJECT_USR_DATA_CHILD_IX;
  pParent = D4D_GetParentObject(pObject);
  
  if(!pParent)
    return D4D_FindPreviousObjectOnScreen(pObject);
  
  while(pParent->pRelations[i])
  {
    if(pParent->pRelations[i] == pObject)
    {
      if(i>D4D_OBJECT_USR_DATA_CHILD_IX)
        return (D4D_OBJECT*)pParent->pRelations[i-1];
      else
        return NULL;
    }
    i++;
  }
      
  return NULL;  
}


/**************************************************************//*!
*
* Function returns the next object (looking for any object in object tree) 
*
******************************************************************/
D4D_OBJECT* D4D_FindPreviousObject(D4D_OBJECT* pObject, D4D_BOOL childrenAlso)
{
  D4D_OBJECT* pLocObject;

  
  // Compatible mode - just only screen top level objects
  if(pObject->pRelations  == NULL)
    return D4D_FindPreviousObjectOnScreen(pObject);
  
  // find out the "latest child" of my previous sibling
  
  // Does I have siblings?
  pLocObject = D4D_FindPreviousSiblingObject(pObject);
  
  if(childrenAlso)
  {
    while(pLocObject && pLocObject->pRelations)
    {
      D4D_OBJECT** pChild = (D4D_OBJECT**)&pLocObject->pRelations[D4D_OBJECT_USR_DATA_CHILD_IX];
      //Check the children
      if(*pChild)
      {  while(*(pChild + 1)) //Find out the last child
        {
          pChild++;
        }      
        pLocObject = *pChild;
      }else
        return pLocObject;    
    }
  }else if(pLocObject)
    return pLocObject;
  
  pLocObject = D4D_GetParentObject(pObject);
  
  if(pLocObject)
    return pLocObject;
  
  return D4D_FindPreviousObjectOnScreen(pObject);
}

/**************************************************************//*!
*
* Function find outs if the object is enabled or not
*
******************************************************************/
D4D_BOOL D4D_IsEnabled(D4D_OBJECT* pObject)
{
  if(!(pObject->pData->flags & D4D_OBJECT_F_ENABLED))
    return D4D_FALSE;
  
  if(pObject->pRelations)
  {
    D4D_OBJECT* pParent = D4D_GetParentObject(pObject);
    if(pParent)
      return D4D_IsEnabled(pParent);    
  } 
  
  return D4D_TRUE; 
}

/**************************************************************//*!
*
* Function find outs if the object is visible or not
*
******************************************************************/
D4D_BOOL D4D_IsVisible(D4D_OBJECT* pObject)
{
  if(!(pObject->pData->flags & D4D_OBJECT_F_VISIBLE))
    return D4D_FALSE;
  
  if(pObject->pRelations)
  {
    D4D_OBJECT* pParent = D4D_GetParentObject(pObject);
    if(pParent)
      return D4D_IsVisible(pParent);    
  } 
  
  return D4D_TRUE; 
}

/**************************************************************//*!
*
* Function sets the flags of object and all it's childs
*
******************************************************************/
void D4D_SetObjectFlags(D4D_OBJECT* pObject, D4D_OBJECT_FLAGS flagsMask)
{
  pObject->pData->flags |= flagsMask;
  
  if(pObject->pRelations)
  {
    D4D_OBJECT** pChild = (D4D_OBJECT**)&(pObject->pRelations[D4D_OBJECT_USR_DATA_CHILD_IX]);
    
    while(*pChild)
    {
      D4D_SetObjectFlags(*pChild, flagsMask);
      pChild++;
    }    
  }  
}

/**************************************************************//*!
*
* Function sets the screen pointer on object and its children
*
******************************************************************/
void D4D_SetObjectScreenPointer(D4D_OBJECT* pObject, D4D_SCREEN* pScreen)
{
  pObject->pData->pScreen = pScreen;
  
  if(pObject->pRelations)
  {
    D4D_OBJECT** pChild = (D4D_OBJECT**)&(pObject->pRelations[D4D_OBJECT_USR_DATA_CHILD_IX]);
    
    while(*pChild)
    {
      D4D_SetObjectScreenPointer(*pChild, pScreen);
      pChild++;
    }    
  }  
}
