/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_object.h
*
* @author    b01119
* 
* @version   0.0.16.0
* 
* @date      Jun-29-2012
* 
* @brief     D4D Driver object functions header file
*
*******************************************************************************/

#ifndef __D4D_OBJECT_H
#define __D4D_OBJECT_H

/******************************************************************************
* Includes
******************************************************************************/

typedef Word D4D_OBJECT_FLAGS;

typedef LWord D4D_OBJECT_INITFLAGS;

// Common flags as for initial as for runtime  
#define D4D_OBJECT_F_VISIBLE        0x0001
#define D4D_OBJECT_F_ENABLED        0x0002
#define D4D_OBJECT_F_TABSTOP        0x0004
#define D4D_OBJECT_F_TOUCHENABLE    0x0008
#define D4D_OBJECT_F_FASTTOUCH      0x0010
#define D4D_OBJECT_F_FOCUSRECT      0x0020
#define D4D_OBJECT_F_TIMETICK       0x0040
#define D4D_OBJECT_F_RESERVED1      0x0080

#define D4D_OBJECT_F_BEVEL_MASK     0x0300
#define D4D_OBJECT_F_BEVEL_SHIFT    8

#define D4D_OBJECT_F_BEVEL_RAISED   0x0100
#define D4D_OBJECT_F_BEVEL_DENTED   0x0200


#define D4D_OBJECT_FLAGS2BEVEL(pObject) ((D4D_BEVEL)(((pObject)->initFlags & D4D_OBJECT_F_BEVEL_MASK) >> D4D_OBJECT_F_BEVEL_SHIFT))



// just runtime flags
#define D4D_OBJECT_F_REDRAW         0x2000
#define D4D_OBJECT_F_REDRAWC        0x4000
#define D4D_OBJECT_F_NOTINIT        0x8000

#define D4D_OBJECT_F_SYSTEM_MASK        0x0000FFFF
#define D4D_OBJECT_F_OBJECT_MASK        0xFFFF0000 
#define D4D_OBJECT_F_OBJECT_SHIFT       16

struct D4D_OBJECT_S;
struct D4D_SCREEN_S;
struct D4D_MESSAGE_S;
struct D4D_CLR_SCHEME_S;

typedef struct D4D_OBJECT_SYS_FUNCTION_S
{
  D4D_TCHAR* strName;
  void (*OnSysMessage)(struct D4D_MESSAGE_S* pMsg);
  D4D_BOOL (*CheckCoordinates)(struct D4D_OBJECT_S* pObj, D4D_POINT point);
  struct D4D_STRING_S* (*GetTextBuffer)(struct D4D_OBJECT_S* pObj);            
}D4D_OBJECT_SYS_FUNCTION;


typedef struct D4D_OBJECT_DATA_S
{
  D4D_OBJECT_FLAGS flags;
  struct D4D_SCREEN_S* pScreen;
}D4D_OBJECT_DATA, *D4D_OBJECT_DATA_PTR;




#define D4D_OBJECT_USR_DATA_PARENT_IX   0
#define D4D_OBJECT_USR_DATA_CHILD_IX    1


typedef void* D4D_OBJECT_USR_DATA;


typedef const struct D4D_OBJECT_S* const * D4D_OBJECT_RELATIONS;



// this is allocated in ROM always
typedef struct D4D_OBJECT_S
{
  D4D_POINT                             position;             // position on the screen/object 
  D4D_SIZE                              size;                 // size of the object
  D4D_COOR                              radius;               // object corners radius  
  D4D_MARGIN*                           pMargin;              // object inner margin
  void*                                 pParam;               // the object depends parameters
  D4D_OBJECT_SYS_FUNCTION*              pObjFunc;             // the pointer on object system functions
  Byte (*OnUsrMessage)(struct D4D_MESSAGE_S* pMsg); // the pointer on user message   
  D4D_OBJECT_USR_DATA                   userData;             // the pointer on user data container (user pointer and optionaly parent/children)
  D4D_OBJECT_RELATIONS                  pRelations;              // relationship between the objects
  D4D_OBJECT_INITFLAGS                  initFlags;            // the initializations object flags
  struct D4D_CLR_SCHEME_S*              clrScheme;            // pointer on used color scheme
  D4D_OBJECT_DATA_PTR                   pData;                // pointer on runtime object data
} D4D_OBJECT;

typedef const D4D_OBJECT* D4D_OBJECT_PTR;




/******************************************************************************
* Macros
******************************************************************************/
    
#define D4D_EXTERN_OBJECT(name) \
    extern const D4D_OBJECT name;

#define D4D_GET_OBJECT_DATA(pObject) ((pObject)->pData)
/*********************************************************
*                  
* static declaration macros
*
*********************************************************/
#define D4D_DECLARE_OBJECT(type, name, x, y, cx, cy, rad, margin, relations, onusrmsg, sysFunc , param, initFlags, usrData, pScheme) \
    D4D_OBJECT_DATA name##_Data = {(D4D_OBJECT_FLAGS)((initFlags | D4D_OBJECT_F_NOTINIT) & D4D_OBJECT_F_SYSTEM_MASK), NULL}; \
    type D4D_OBJECT name = \
    { \
        {x, y},\
        {cx, cy},\
        rad,\
        margin,\
        (void*)param, \
        (D4D_OBJECT_SYS_FUNCTION*)sysFunc, \
        onusrmsg, \
        (D4D_OBJECT_USR_DATA)usrData, \
        relations,\
        initFlags, \
        pScheme, \
        &(name##_Data) \
    };     
 
 
 // First way how to define object relation array   
 #define D4D_DECLARE_OBJECT_RELATIONS(name, pParent, ...)\
    const D4D_OBJECT * const name[] =  {pParent, __VA_ARGS__ , NULL};
 
 // next way how to define object relation array   
 #define D4D_DECLARE_OBJECT_RELATIONS_BEGIN(name, pParent)\
    const D4D_OBJECT * const name[] = \
    {\
     pParent,
     
 #define D4D_DECLARE_OBJECT_RELATIONS_CHILD(child) &child,
 
 #define D4D_DECLARE_OBJECT_RELATIONS_END NULL };
  
#if D4D_EXTSRC_TEXT_ENABLE != D4D_FALSE 
  #define D4D_OBJECT_MAX_TEXT_LEN(str) 1000
#else
  #define D4D_OBJECT_MAX_TEXT_LEN(str) D4D_TEXT_LEN(str)
#endif
    
/******************************************************************************
* Public functions
******************************************************************************/    
D4D_BOOL D4D_ObjectCheckCoordinates(D4D_OBJECT* pThis, D4D_POINT point);    
D4D_BOOL D4D_IsEnabled(D4D_OBJECT* pObject);
D4D_BOOL D4D_IsVisible(D4D_OBJECT* pObject);
#endif  /* __D4D_OBJECT_H */