/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_button.c
*
* @author    r50233
* 
* @version   0.0.57.0
* 
* @date      Aug-6-2012
* 
* @brief     D4D driver button object c file 
*
******************************************************************************/

#include "d4d.h"
#include "common_files/d4d_private.h"

//Internal API
void D4D_BtnOnMessage(D4D_MESSAGE* pMsg);
D4D_STRING* D4D_BtnGetTextBuffer(D4D_OBJECT* pThis);

const D4D_OBJECT_SYS_FUNCTION d4d_btnSysFunc = 
{
  D4D_DEFSTR("Button"),
  D4D_BtnOnMessage,
  NULL,
  D4D_BtnGetTextBuffer
};

typedef struct 
{
    D4D_POINT Position;
    D4D_POINT ButPos; 
    D4D_SIZE ButSize;
    D4D_COOR ButRadius;
    D4D_POINT CorPos1; 
    D4D_POINT CorPos2;
    D4D_SIZE CorSize1;
    D4D_SIZE CorSize2;
} D4D_BTN_TMP_VAL;

#define _calc (*((D4D_BTN_TMP_VAL*)d4d_scratchPad))

static void D4D_BtnValue2Coor(D4D_OBJECT* pThis)
{
  #if (D4D_BTN_3D_ENABLE != D4D_FALSE)
    D4D_BUTTON* pBtn = D4D_GET_BUTTON(pThis);
  #endif
  _calc.Position = D4D_GetClientToScreenPoint(pThis, &pThis->position);  
  
  //---------------- Get Max size of content  -----------------
  // 1. compare bitmap sizes
  // Get normal bitmap size if exists 

  _calc.ButSize =pThis->size;
 
  #if (D4D_BTN_3D_ENABLE != D4D_FALSE)        
    if(pThis->initFlags & D4D_BTN_F_3D)
    {
      _calc.ButSize.cx -= (2*D4D_BTN_3D_FRAME_WIDTH +D4D_BTN_3D_SHIFT);
      _calc.ButSize.cy -= (2*D4D_BTN_3D_FRAME_WIDTH +D4D_BTN_3D_SHIFT);
    }else
  #endif
    if(pThis->initFlags & D4D_OBJECT_F_FOCUSRECT)
    {
      _calc.ButSize.cx -= 2;
      _calc.ButSize.cy -= 2;
    }
  
// 3D Buttons Enabled  
#if (D4D_BTN_3D_ENABLE != D4D_FALSE)
  if(pThis->initFlags & D4D_BTN_F_3D)
  {
    _calc.ButPos.x = (D4D_COOR) (_calc.Position.x + D4D_BTN_3D_FRAME_WIDTH);
    _calc.ButPos.y = (D4D_COOR) (_calc.Position.y + D4D_BTN_3D_FRAME_WIDTH);
    
    _calc.CorSize1.cy = (D4D_COOR) D4D_BTN_3D_SHIFT;
    
    _calc.CorSize2.cx = (D4D_COOR) D4D_BTN_3D_SHIFT;
    
    _calc.CorPos1 = _calc.ButPos; 
    _calc.CorPos2 = _calc.ButPos; 
  
  #if (D4D_ROUND_CORNER_ENABLE != D4D_FALSE) //if Round corners
    
    _calc.ButRadius = 0;
    
    if(pThis->radius)
    {
      if(pThis->radius > D4D_BTN_3D_FRAME_WIDTH)
        _calc.ButRadius = (D4D_COOR)(pThis->radius - D4D_BTN_3D_FRAME_WIDTH);
      
      _calc.CorSize1.cx = (D4D_COOR)(_calc.ButSize.cx - _calc.ButRadius * 2);
      _calc.CorSize2.cy = (D4D_COOR)(_calc.ButSize.cy - _calc.ButRadius * 2);
      
      _calc.CorPos1.x += _calc.ButRadius;
      _calc.CorPos2.y += _calc.ButRadius;
      
      
    }else
  #endif
    {
      _calc.CorSize1.cx = (D4D_COOR)(_calc.ButSize.cx + D4D_BTN_3D_SHIFT);
      _calc.CorSize2.cy = (D4D_COOR)(_calc.ButSize.cy + D4D_BTN_3D_SHIFT);
    }
    
    
    if(*(pBtn->pStatus) & D4D_BUTTON_STATUS_PRESSED_MASK)
    {
      _calc.ButPos.x +=  D4D_BTN_3D_SHIFT;
      _calc.ButPos.y +=  D4D_BTN_3D_SHIFT;    
    }else
    {
      _calc.CorPos1.y += _calc.ButSize.cy;
      _calc.CorPos2.x += _calc.ButSize.cx;
      
    }
  }else 
#endif // NO 3D Buttons
  { 
    #if (D4D_ROUND_CORNER_ENABLE != D4D_FALSE) //if Round corners
      _calc.ButRadius = pThis->radius;  
      if(_calc.ButRadius)
        if(pThis->initFlags & D4D_OBJECT_F_FOCUSRECT)
          _calc.ButRadius--;  
    #endif

    _calc.ButPos = _calc.Position;
    if(pThis->initFlags & D4D_OBJECT_F_FOCUSRECT)
    {
      _calc.ButPos.x++;
      _calc.ButPos.y++;
    }
  }   
  //D4D_BTN_3D_ENABLE
  
}


/*******************************************************
*
* BUTTON Drawing routine
*
*******************************************************/

static void D4D_BtnOnDraw(D4D_MESSAGE* pMsg)
{
    D4D_OBJECT* pThis = pMsg->pObject;
    D4D_BUTTON* pBtn = D4D_GET_BUTTON(pThis);
    D4D_OBJECT_DRAWFLAGS draw = pMsg->prm.draw;
    D4D_COLOR clrT, clrB;    
    const D4D_BMP* pBmp = pBtn->pBmpNormal;
   
//-------- 3D Buttons Enabled --------
#if (D4D_BTN_3D_ENABLE != D4D_FALSE)    
    D4D_POINT shPos;
    D4D_SIZE shSize;
    
    D4D_COLOR shadeColFore;
    D4D_COLOR shadeColBack;
#endif    
    D4D_BMP_PROPERTIES bmpProp;
    
    bmpProp = (D4D_ALIGN_V_CENTER_MASK | D4D_ALIGN_H_CENTER_MASK);
    
    D4D_BtnValue2Coor(pThis);
    
    // focused bitmap (may be also NULL)

    clrT = D4D_ObjectGetForeColor(pThis, draw);
    clrB = D4D_ObjectGetBckgColor(pThis, draw);  
     
    if((draw & D4D_OBJECT_DRAWFLAGS_FOCUSED) && (pBtn->pBmpFocus))
      pBmp = pBtn->pBmpFocus;  
    
//-------- 3D Buttons Enabled --------
#if (D4D_BTN_3D_ENABLE != D4D_FALSE)
    
    if(pThis->initFlags & D4D_BTN_F_3D)
    {
      // If NOT FOCUS RECT
   /*   if(!(pThis->initFlags & D4D_OBJECT_F_FOCUSRECT))
      {          
        clrT = D4D_ObjectGetForeFillColor(pThis);
        clrB = D4D_ObjectGetBckgFillColor(pThis);  
      }
     */ 
      // draw the shading        
      if(*(pBtn->pStatus) & D4D_BUTTON_STATUS_PRESSED_MASK)
      {
        shadeColFore = D4D_ChangeColorIntesity(clrB, D4D_BUTTON_3D_FORE_INTENSITY);
        shadeColBack = D4D_ChangeColorIntesity(clrB, D4D_BUTTON_3D_BCKG_INTENSITY); 
      }
      else
      { 
        shadeColFore = D4D_ChangeColorIntesity(clrB, D4D_BUTTON_3D_BCKG_INTENSITY);
        shadeColBack = D4D_ChangeColorIntesity(clrB, D4D_BUTTON_3D_FORE_INTENSITY); 
      }        
  
      // Draw 3D Shading    
      shPos = _calc.Position;    
      shSize.cx = pThis->size.cx;
      shSize.cy = (D4D_COOR)(D4D_BTN_3D_FRAME_WIDTH);
  
  #if (D4D_ROUND_CORNER_ENABLE != D4D_FALSE) //if Round corners    
      shPos.x += pThis->radius;
      shPos.y += pThis->radius;
      D4D_FillQuadrant(&shPos, pThis->radius, shadeColFore, D4D_QUAD_4);    
      shPos.x += pThis->size.cx - pThis->radius * 2 - 1;
      D4D_FillQuadrant(&shPos, pThis->radius, shadeColFore, D4D_QUAD_1);
      shPos.y += pThis->size.cy - pThis->radius * 2 - 1;
      D4D_FillQuadrant(&shPos, pThis->radius, shadeColBack, D4D_QUAD_2);
      shPos.x -= (pThis->size.cx - pThis->radius * 2 - 1);
      D4D_FillQuadrant(&shPos, pThis->radius, shadeColBack, D4D_QUAD_3);
      
      shPos = _calc.Position;
      shPos.x += pThis->radius;
      shSize.cx -= pThis->radius * 2;
  #endif    
      D4D_FillRect(&shPos, &shSize, shadeColFore);
      
      shPos.y += (_calc.ButSize.cy + D4D_BTN_3D_SHIFT + D4D_BTN_3D_FRAME_WIDTH);     
      D4D_FillRect(&shPos, &shSize, shadeColBack);
      
      shPos = _calc.Position;
      
      // change pos
      shSize.cx = (D4D_COOR)(D4D_BTN_3D_FRAME_WIDTH);
      shSize.cy = pThis->size.cy; 
  #if (D4D_ROUND_CORNER_ENABLE != D4D_FALSE) //if Round corners
      shPos.y += pThis->radius;
      shSize.cy -= pThis->radius * 2; 
  #endif     
      D4D_FillRect(&shPos, &shSize, shadeColFore);
      
      shPos.x += (_calc.ButSize.cx + D4D_BTN_3D_SHIFT + D4D_BTN_3D_FRAME_WIDTH);
      
      D4D_FillRect(&shPos, &shSize, shadeColBack);
         
      // Draw Button Corners
      D4D_FillRect(&_calc.CorPos1, &_calc.CorSize1,clrB);
      D4D_FillRect(&_calc.CorPos2, &_calc.CorSize2,clrB); 
    }    
//-------- NO 3D Buttons ------------

 
#endif 
  if(pThis->initFlags & D4D_OBJECT_F_FOCUSRECT)
    D4D_RRect(&_calc.Position, &pThis->size, D4D_LINE_THIN, clrT, pThis->radius);    
 
  // Draw the content of button
  
  pBtn->textBuff.str_properties->font_properties &= ~D4D_FNT_PRTY_TRANSPARENT_MASK;
  
  clrT = D4D_ObjectGetForeColor(pThis, draw);
  clrB = D4D_ObjectGetBckgColor(pThis, draw);  
    
  // Draw the bitmap
  if(pBmp != NULL)
  { 
    D4D_DrawRBmpRect(&_calc.ButPos,&_calc.ButSize, pBmp, bmpProp, clrB, (D4D_BOOL)!D4D_IsEnabled(pThis), pThis->radius);    
    pBtn->textBuff.str_properties->font_properties |= D4D_FNT_PRTY_TRANSPARENT_MASK;
  }
  
  // draw the text
  if(pBtn->textBuff.pText)
    D4D_DrawTextRRectTab(&_calc.ButPos, &_calc.ButSize,&pBtn->textBuff, NULL, clrT, clrB, _calc.ButRadius); 
      
  if(!pBtn->textBuff.pText && !pBmp)
    D4D_FillRRect(&_calc.ButPos, &_calc.ButSize, clrB, pThis->radius);    
    
   if(pThis->initFlags & D4D_BTN_F_CONT_RECT)
    D4D_RRect(&_calc.ButPos, &_calc.ButSize, D4D_LINE_THIN, clrT, _calc.ButRadius);    
   
}

/*******************************************************
*
* BUTTON key handling routine
*
*******************************************************/

static void D4D_BtnOnKeyDown(D4D_MESSAGE* pMsg)    
{                
  // invoke "Click" event when enter key is pressed
  if(pMsg->prm.key == D4D_KEY_SCANCODE_ENTER)
  {      
    D4D_BUTTON_STATUS* pStatus = D4D_GET_BUTTON_STATUS(pMsg->pObject);
    *pStatus |= D4D_BUTTON_STATUS_PRESSED_MASK;
    D4D_InvalidateObject(pMsg->pObject, D4D_FALSE); 
    
  }  
}

/*******************************************************
*
* BUTTON key handling routine
*
*******************************************************/

static void D4D_BtnOnKeyUp(D4D_MESSAGE* pMsg)    
{
                 
  // invoke "Click" event when enter key is pressed
  if(pMsg->prm.key == D4D_KEY_SCANCODE_ENTER)
  {
    D4D_BUTTON* pBtn = D4D_GET_BUTTON(pMsg->pObject);
    D4D_BUTTON_STATUS* pStatus = D4D_GET_BUTTON_STATUS(pMsg->pObject);
    
    if(*pStatus & D4D_BUTTON_STATUS_PRESSED_MASK)
    {
      *pStatus &= ~D4D_BUTTON_STATUS_PRESSED_MASK;
      if(pBtn->OnClicked)
          pBtn->OnClicked(pMsg->pObject);
    }
    
    D4D_InvalidateObject(pMsg->pObject, D4D_FALSE);       
  }
}

/*******************************************************
*
* BUTTON focus handling routine
*
*******************************************************/

static void D4D_BtnKillFocus(D4D_MESSAGE* pMsg)
{
  D4D_BUTTON_STATUS* pStatus = D4D_GET_BUTTON_STATUS(pMsg->pObject);
  
  if(*pStatus & D4D_BUTTON_STATUS_PRESSED_MASK)
  {    
    *pStatus &= ~D4D_BUTTON_STATUS_PRESSED_MASK;
    D4D_InvalidateObject(pMsg->pObject, D4D_FALSE);
  }
  
}

/*******************************************************
*
* BUTTON Touched Button handling routine
*
*******************************************************/
#ifdef D4D_LLD_TCH
static void D4D_BtnTouched(D4D_MESSAGE* pMsg)
{ 
  if(pMsg->pObject->pData->flags & (D4D_OBJECT_F_TABSTOP | D4D_OBJECT_F_FASTTOUCH))
  {    
	D4D_BUTTON* pBtn = D4D_GET_BUTTON(pMsg->pObject);
	
	if((D4D_GetFocusedObject(pMsg->pScreen) == pMsg->pObject) || (pMsg->pObject->pData->flags & D4D_OBJECT_F_FASTTOUCH))
    {    
       *(pBtn->pStatus) |= D4D_BUTTON_STATUS_PRESSED_MASK;
       *(pBtn->pStatus) &= ~D4D_BUTTON_STATUS_1ST_TOUCH_MASK;
    }else
       *(pBtn->pStatus) |= D4D_BUTTON_STATUS_1ST_TOUCH_MASK;
    
    D4D_FocusSet(pMsg->pScreen, pMsg->pObject);

    D4D_InvalidateObject(pMsg->pObject, D4D_FALSE);
  }
}

/*******************************************************
*
* BUTTON UnTouched Button handling routine
*
*******************************************************/
static void D4D_BtnUnTouched(D4D_MESSAGE* pMsg)    
{
    D4D_BUTTON* pBtn = D4D_GET_BUTTON(pMsg->pObject);
    D4D_BUTTON_STATUS* pStatus = D4D_GET_BUTTON_STATUS(pMsg->pObject);
    
    if(*pStatus & D4D_BUTTON_STATUS_PRESSED_MASK)
    {
      *pStatus &= ~D4D_BUTTON_STATUS_PRESSED_MASK;
      if(pBtn->OnClicked)
        pBtn->OnClicked(pMsg->pObject);  
      
      D4D_InvalidateObject(pMsg->pObject, D4D_FALSE);
    }
             
}


#endif

/**************************************************************//*!
*
*
*
*     THE INTERNAL API FUNCTIONS
*
*
*
******************************************************************/

/*******************************************************
*
* The main BUTTON message handler 
*
*******************************************************/

void D4D_BtnOnMessage(D4D_MESSAGE* pMsg)
{
#ifdef D4D_LLD_TCH
        D4D_OBJECT* pThis = pMsg->pObject; 	
        D4D_BUTTON* pBtn = D4D_GET_BUTTON(pThis);
#endif
	
	switch(pMsg->nMsgId)
    {
    case D4D_MSG_DRAW:
        D4D_BtnOnDraw(pMsg);
        break;
        
    case D4D_MSG_KEYDOWN:
        D4D_BtnOnKeyDown(pMsg);
        break;

    case D4D_MSG_KEYUP:
        D4D_BtnOnKeyUp(pMsg);
        break;
        
    case D4D_MSG_KILLFOCUS:
        D4D_BtnKillFocus(pMsg);
        break;
   
   #ifdef D4D_LLD_TCH 
    case D4D_MSG_TOUCHED:     
        D4D_BtnTouched(pMsg);
      break; 
      
    case D4D_MSG_TOUCH_AUTO:
        if((*(pBtn->pStatus) & (D4D_BUTTON_STATUS_PRESSED_MASK | D4D_BUTTON_STATUS_1ST_TOUCH_MASK)) == 0)
        {
          *(pBtn->pStatus) |= D4D_BUTTON_STATUS_PRESSED_MASK;
          D4D_InvalidateObject(pThis, D4D_FALSE);
        }    		
    	break;
    
    case D4D_MSG_TOUCH_LOST:
        if((*(pBtn->pStatus) & (D4D_BUTTON_STATUS_PRESSED_MASK | D4D_BUTTON_STATUS_1ST_TOUCH_MASK)) == D4D_BUTTON_STATUS_PRESSED_MASK)
        {
          *(pBtn->pStatus) &= ~D4D_BUTTON_STATUS_PRESSED_MASK;
          D4D_InvalidateObject(pThis, D4D_FALSE);
        }    		
      	break;	
	   
    case D4D_MSG_UNTOUCHED:     
        D4D_BtnUnTouched(pMsg);
      break;
   #endif
    default:
        // call the default behavior
        D4D_ObjOnMessage(pMsg);
    }
}

/**************************************************************//*!
*
* Get object text buffer
*
******************************************************************/
D4D_STRING* D4D_BtnGetTextBuffer(D4D_OBJECT* pThis)
{
  return &(D4D_GET_BUTTON(pThis)->textBuff);  
}
