/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_check_box.c
*
* @author    b01119
* 
* @version   0.0.40.0
* 
* @date      Jun-29-2012
* 
* @brief     D4D driver check box object c file 
*
******************************************************************************/

#include "d4d.h"
#include "common_files/d4d_private.h"

//Internal API
void D4D_CheckBoxOnMessage(D4D_MESSAGE* pMsg);
D4D_STRING* D4D_CheckBoxGetTextBuffer(D4D_OBJECT* pThis);

const D4D_OBJECT_SYS_FUNCTION d4d_checkBoxSysFunc = 
{
  D4D_DEFSTR("Check Box"),
  D4D_CheckBoxOnMessage,
  NULL,
  D4D_CheckBoxGetTextBuffer
};

typedef struct 
{
    D4D_POINT Position;
    D4D_POINT TxtPos;
    D4D_SIZE TxtSize;
    D4D_POINT BmpPos;
    D4D_SIZE  SizeBmpEnabled;
    D4D_POINT iconCentre;
    D4D_COOR iconRadius;    
} D4D_CHECKBOX_TMP_VAL;

#define _calc (*((D4D_CHECKBOX_TMP_VAL*)d4d_scratchPad))

    
static void D4D_CheckBoxValue2Coor(D4D_OBJECT* pThis)
{
  D4D_CHECKBOX* pCheckB = D4D_GET_CHECKBOX(pThis);
  D4D_COOR tmp_BorderOffset;
  
  _calc.Position = D4D_GetClientToScreenPoint(pThis, &pThis->position);
  
  _calc.SizeBmpEnabled = D4D_GetBmpSize(pCheckB->pBmpChecked);
  
  if(!_calc.SizeBmpEnabled.cx || !_calc.SizeBmpEnabled.cy)
  {
    if(pThis->size.cy > D4D_CHECKBOX_BORDER_OFFSET * 2)
    {
      _calc.SizeBmpEnabled = pThis->size;    
      _calc.SizeBmpEnabled.cy -= D4D_CHECKBOX_BORDER_OFFSET * 2;    
      _calc.SizeBmpEnabled.cx = _calc.SizeBmpEnabled.cy;
    }
  }
  
  tmp_BorderOffset = (D4D_COOR)((pThis->size.cy - _calc.SizeBmpEnabled.cy) / 2);
  
  _calc.BmpPos.x = (D4D_COOR)(_calc.Position.x + tmp_BorderOffset);
  _calc.BmpPos.y = (D4D_COOR)(_calc.Position.y + tmp_BorderOffset);

  if(pCheckB->textBuff.pText/* && pCheckB->textBuff.buffSize*/)
  {
    _calc.TxtPos.x = (D4D_COOR) (_calc.BmpPos.x + _calc.SizeBmpEnabled.cx + D4D_CHECKBOX_TEXT_OFFSET);      
    _calc.TxtPos.y = (D4D_COOR) (_calc.Position.y + 1);
    _calc.TxtSize.cx = (D4D_COOR)(pThis->size.cx - (_calc.TxtPos.x - _calc.Position.x + 1));      
    _calc.TxtSize.cy = (D4D_COOR) (pThis->size.cy - 2);
    
  }
     
#if D4D_ROUND_CORNER_ENABLE != D4D_FALSE  
    _calc.iconRadius = pThis->radius;
#endif

}


/*******************************************************
*
* CHECK BOX Drawing routine
*
*******************************************************/

static void D4D_CheckBoxOnDraw(D4D_MESSAGE* pMsg)
{
    D4D_OBJECT* pThis = pMsg->pObject;
    D4D_CHECKBOX* pCheckB = D4D_GET_CHECKBOX(pThis);
    D4D_CHECKBOX_STATUS* pStatus = D4D_GET_CHECKBOX_STATUS(pThis);    
    D4D_OBJECT_DRAWFLAGS draw = pMsg->prm.draw;    
    D4D_COLOR clrT, clrB;
    
    D4D_CheckBoxValue2Coor(pThis);
    
    clrT = D4D_ObjectGetForeColor(pThis, draw);
    clrB = D4D_ObjectGetBckgFillColor(pThis);

    if(draw & D4D_OBJECT_DRAWFLAGS_COMPLETE)
      D4D_FillRRect(&_calc.Position, &pThis->size, clrB, pThis->radius);  
   
 
    // normal bitmap (also may be NULL if simple rect button is needed  
    if(*pStatus & D4D_CHECKBOX_STATUS_CHECKED_MASK)
    {
      if(pCheckB->pBmpChecked != NULL)         
        D4D_DrawRBmp(&_calc.BmpPos, pCheckB->pBmpChecked, (D4D_BOOL)!D4D_IsEnabled(pThis), _calc.iconRadius);  // draw the bitmap            
      else
      {
        D4D_POINT tmp_point = _calc.BmpPos;
        D4D_SIZE tmp_size = _calc.SizeBmpEnabled;
        if(tmp_size.cy - 2 > 0)
        {
          tmp_point.x++;
          tmp_point.y++;
          
          tmp_size.cx -= 2;
          tmp_size.cy -= 2;
          
          D4D_RBox(&tmp_point, &tmp_size, D4D_LINE_THICK, clrB, clrT, _calc.iconRadius);
        }
      }
    }    
    // no bitmap
    else
    {     
      if(pCheckB->pBmpUnChecked)
      {
        // draw the bitmap
        D4D_DrawRBmp(&_calc.BmpPos, pCheckB->pBmpUnChecked, (D4D_BOOL)!D4D_IsEnabled(pThis), _calc.iconRadius);   
      }else
      {        
        // draw just a color rectangle instead of bitmap
        D4D_CLR_SCHEME* pScheme = D4D_ObjectGetScheme(pThis);
        D4D_FillRRect(&_calc.BmpPos, &_calc.SizeBmpEnabled, pScheme->objectDepend.checkBox.iconBckg, _calc.iconRadius);        
      }
    }
    
    if(pThis->initFlags & D4D_CHECKBOX_F_ICON_RECTANGLE)
      D4D_RRect(&_calc.BmpPos, &_calc.SizeBmpEnabled, D4D_LINE_THIN, clrT, _calc.iconRadius);        
    
    if(pThis->initFlags & D4D_OBJECT_F_FOCUSRECT)          
      D4D_RRect(&_calc.Position, &pThis->size, D4D_LINE_THIN, clrT, pThis->radius);
    
    // draw the text
    if(pCheckB->textBuff.pText)
      D4D_DrawTextRect(&_calc.TxtPos, &_calc.TxtSize, &pCheckB->textBuff, clrT, D4D_ObjectGetBckgColor(pThis, draw));
    
}

/*******************************************************
*
* CHECK BOX key handling routine
*
*******************************************************/

static void D4D_CheckBoxOnKeyDown(D4D_MESSAGE* pMsg)    
{
    D4D_CHECKBOX_STATUS* pStatus = D4D_GET_CHECKBOX_STATUS(pMsg->pObject);
    
    // invoke "Click" event when enter key is pressed
    if(pMsg->prm.key == D4D_KEY_SCANCODE_ENTER)
    {      
      *pStatus |= D4D_CHECKBOX_STATUS_PRESSED_MASK;
      D4D_InvalidateObject(pMsg->pObject, D4D_FALSE);
    }
}

/*******************************************************
*
* CHECK BOX key handling routine
*
*******************************************************/

static void D4D_CheckBoxOnKeyUp(D4D_MESSAGE* pMsg)    
{
  D4D_CHECKBOX_STATUS* pStatus = D4D_GET_CHECKBOX_STATUS(pMsg->pObject);
  
  // invoke "Click" event when enter key is pressed
  if(pMsg->prm.key == D4D_KEY_SCANCODE_ENTER)
  {
    if(*pStatus & D4D_CHECKBOX_STATUS_PRESSED_MASK)
    {
      D4D_CHECKBOX* pCheckB = D4D_GET_CHECKBOX(pMsg->pObject);
      *pStatus &= ~D4D_CHECKBOX_STATUS_PRESSED_MASK;      
  
      if(*pStatus & D4D_CHECKBOX_STATUS_CHECKED_MASK)
        *pStatus &= ~D4D_CHECKBOX_STATUS_CHECKED_MASK;
      else
        *pStatus |= D4D_CHECKBOX_STATUS_CHECKED_MASK;
      
      if(pCheckB->OnChange)
        pCheckB->OnChange(pMsg->pObject);            
    }
    D4D_InvalidateObject(pMsg->pObject, D4D_FALSE); 
  } 
}

/*******************************************************
*
* CHECK BOX focus handling routine
*
*******************************************************/

static void D4D_CheckBoxKillFocus(D4D_MESSAGE* pMsg)
{
  D4D_CHECKBOX_STATUS* pStatus = D4D_GET_CHECKBOX_STATUS(pMsg->pObject);

  *pStatus &= ~D4D_CHECKBOX_STATUS_PRESSED_MASK;
}

/*******************************************************
*
* CHECK BOX Touched Button handling routine
*
*******************************************************/
#ifdef D4D_LLD_TCH
  static void D4D_CheckBoxTouched(D4D_MESSAGE* pMsg)
  {  
    if(pMsg->pObject->pData->flags & (D4D_OBJECT_F_TABSTOP | D4D_OBJECT_F_FASTTOUCH))
    {
      if((D4D_GetFocusedObject(pMsg->pScreen) == pMsg->pObject) || (pMsg->pObject->pData->flags & D4D_OBJECT_F_FASTTOUCH))
      {               
        D4D_CHECKBOX_STATUS* pStatus = D4D_GET_CHECKBOX_STATUS(pMsg->pObject);
        D4D_CHECKBOX* pCheckB = D4D_GET_CHECKBOX(pMsg->pObject);
        
        if(*pStatus & D4D_CHECKBOX_STATUS_CHECKED_MASK)
          *pStatus &= ~D4D_CHECKBOX_STATUS_CHECKED_MASK;
        else
          *pStatus |= D4D_CHECKBOX_STATUS_CHECKED_MASK;
        
        if(pCheckB->OnChange)
          pCheckB->OnChange(pMsg->pObject);
        
        D4D_InvalidateObject(pMsg->pObject, D4D_FALSE);
      }                    
      D4D_FocusSet(pMsg->pScreen, pMsg->pObject);
    }
  }
#endif
/*******************************************************
*
* Set CHECK BOX value
*
*******************************************************/

void D4D_CheckBoxSetValue(D4D_OBJECT_PTR pThis, D4D_BOOL value)
{
  D4D_CHECKBOX_STATUS* pStatus = D4D_GET_CHECKBOX_STATUS(pThis);
  D4D_CHECKBOX* pCheckB = D4D_GET_CHECKBOX(pThis);
  
  if(!((*pStatus & D4D_CHECKBOX_STATUS_CHECKED_MASK) ^ value))
    return ;
  
  // remember new value
  if(value)
    *pStatus |= D4D_CHECKBOX_STATUS_CHECKED_MASK;
  else
    *pStatus &= ~D4D_CHECKBOX_STATUS_CHECKED_MASK;
  
  D4D_InvalidateObject(pThis, D4D_TRUE);

  // notify user that the value has changed        
  if(pCheckB->OnChange)
    pCheckB->OnChange((D4D_OBJECT*)pThis);
}


D4D_BOOL D4D_CheckBoxGetValue(D4D_OBJECT_PTR pThis)
{
  D4D_CHECKBOX_STATUS* pStatus = D4D_GET_CHECKBOX_STATUS(pThis);
  
  return (D4D_BOOL)(*pStatus & D4D_CHECKBOX_STATUS_CHECKED_MASK);
}


/**************************************************************//*!
*
*
*
*     THE INTERNAL API FUNCTIONS
*
*
*
******************************************************************/

/*******************************************************
*
* The main CHECK BOX message handler 
*
*******************************************************/

void D4D_CheckBoxOnMessage(D4D_MESSAGE* pMsg)
{
  switch(pMsg->nMsgId)
  {
  case D4D_MSG_DRAW:
      D4D_CheckBoxOnDraw(pMsg);
      break;
      
  case D4D_MSG_KEYDOWN:
      D4D_CheckBoxOnKeyDown(pMsg);
      break;

  case D4D_MSG_KEYUP:
      D4D_CheckBoxOnKeyUp(pMsg);
      break;
      
  case D4D_MSG_KILLFOCUS:
      D4D_CheckBoxKillFocus(pMsg);
      break;
    
    
#ifdef D4D_LLD_TCH     
    case D4D_MSG_TOUCHED:     
      D4D_CheckBoxTouched(pMsg);
      break;
#endif      

    default:
        // call the default behavior
        D4D_ObjOnMessage(pMsg);
    }
}

/**************************************************************//*!
*
* Get object text buffer
*
******************************************************************/
D4D_STRING* D4D_CheckBoxGetTextBuffer(D4D_OBJECT* pThis)
{
  return &(D4D_GET_CHECKBOX(pThis)->textBuff);  
}