/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_edit_box.c
*
* @author    b01119
* 
* @version   0.0.1.0
* 
* @date      Jul-11-2012
* 
* @brief     D4D driver editBox object c file 
*
******************************************************************************/

#include "d4d.h"
#include "common_files/d4d_private.h"

//Internal API
void D4D_EditBoxOnMessage(D4D_MESSAGE* pMsg);
static D4D_BOOL D4D_EditBoxPutCharInt(D4D_OBJECT_PTR pObj, D4D_TCHAR ch);
static void D4D_EditBoxTimeTick(D4D_MESSAGE* pMsg);

static void D4D_EditBoxOnInit(D4D_MESSAGE* pMsg);

const D4D_OBJECT_SYS_FUNCTION d4d_editBoxSysFunc = 
{
  D4D_DEFSTR("Edit Box"),
  D4D_EditBoxOnMessage,
  NULL,
  NULL
};

const D4D_MARGIN editBox_marginDefault = 
{
  D4D_BEVEL_WIDTH, 
  D4D_BEVEL_WIDTH,
  D4D_BEVEL_WIDTH,
  D4D_BEVEL_WIDTH
};

// temporary structure for editBox temporary value calculation
typedef struct 
{
    D4D_POINT position;
    D4D_POINT contentPos;
    D4D_SIZE  contentSize;        
} D4D_EDIT_BOX_TMP_VAL;

#define _calc (*((D4D_EDIT_BOX_TMP_VAL*)d4d_scratchPad))




/*******************************************************
*
* EDIT_BOX Helper computing routines
*
*******************************************************/

static void D4D_EditBoxValue2Coor(D4D_OBJECT* pThis)
{
  //D4D_EDIT_BOX* pEditBox = D4D_GET_EDIT_BOX(pThis);
  _calc.position = D4D_GetClientToScreenPoint(pThis, &pThis->position);
  _calc.contentPos = _calc.position;
  _calc.contentSize = pThis->size;
    
  if(pThis->pMargin)
  {
    _calc.contentPos.x += pThis->pMargin->left;
    _calc.contentPos.y += pThis->pMargin->top;
    _calc.contentSize.cx -= (pThis->pMargin->left + pThis->pMargin->right);
    _calc.contentSize.cy -= (pThis->pMargin->top + pThis->pMargin->bottom);
    
  }else
  {
    if(pThis->initFlags & D4D_OBJECT_F_FOCUSRECT)
    {
      _calc.contentPos.x++;
      _calc.contentPos.y++;
      _calc.contentSize.cx -= 2;
      _calc.contentSize.cy -= 2;
    }
  }        
}

/*******************************************************
*
* EDIT_BOX Drawing routine
*
*******************************************************/

static void D4D_EditBoxOnDraw(D4D_MESSAGE* pMsg)
{
    D4D_OBJECT* pThis = pMsg->pObject;
    D4D_EDIT_BOX* pEditBox = D4D_GET_EDIT_BOX(pThis);
    D4D_EDIT_BOX_DATA* pEditBoxData = D4D_GET_EDIT_BOX_DATA(pEditBox);
    D4D_OBJECT_DRAWFLAGS draw = pMsg->prm.draw;
    D4D_COLOR clrT, clrB;     
        
    // Get background simple color 
    clrT = D4D_ObjectGetForeColor(pThis, draw);
    clrB = D4D_ObjectGetScheme(pThis)->bckgFocus;
    
    #ifdef D4D_DEBUG
      // sanity check
      D4D_ASSERT(sizeof(_calc) <= D4D_SCRATCHPAD_SIZE);
    #endif
    
    // Compute all tempoarary values
    D4D_EditBoxValue2Coor(pThis);
    
    // draw just a color rectangle instead of bitmap
    if(draw & D4D_OBJECT_DRAWFLAGS_COMPLETE)  
    {      
      // Draw the background
      #if D4D_ROUND_CORNER_ENABLE != D4D_FALSE
      if(pThis->radius)
        D4D_FillRRect(&_calc.position, &pThis->size, clrB, pThis->radius);
      else
      #endif
        D4D_FillRect(&_calc.position, &pThis->size, clrB);                                
      
      pEditBoxData->flags |= (D4D_EDIT_BOX_FLAGS_REDRAWBEVEL | D4D_EDIT_BOX_FLAGS_REDRAWTEXT | D4D_EDIT_BOX_FLAGS_REDRAWCURSOR);
    }

    if(pEditBoxData->flags & D4D_EDIT_BOX_FLAGS_REDRAWBEVEL)
    {
      pEditBoxData->flags &= ~D4D_EDIT_BOX_FLAGS_REDRAWBEVEL;
      
      // Draw the frame
      if(pThis->initFlags & D4D_OBJECT_F_BEVEL_MASK)
      {
        D4D_COLOR clrBevel;
        // In case that focus rect is enabled and also BEVEL, just change the color of bevel
        if(pThis->initFlags & D4D_OBJECT_F_FOCUSRECT)
          clrBevel = clrT;
        else
          clrBevel = D4D_ObjectGetBckgFillColor(pThis);
        
        D4D_Bevel(&_calc.position, &pThis->size, clrBevel, D4D_OBJECT_FLAGS2BEVEL(pThis), pThis->radius);
      }else
      {
        if(pThis->initFlags & D4D_OBJECT_F_FOCUSRECT)
          D4D_Rect(&_calc.position, &pThis->size, D4D_LINE_THIN, clrT);              
      }
    }    
    
    // Draw text
    if(pEditBoxData->flags & D4D_EDIT_BOX_FLAGS_REDRAWTEXT)
    {
      pEditBoxData->flags &= ~D4D_EDIT_BOX_FLAGS_REDRAWTEXT;
      // draw the text
      #if D4D_ROUND_CORNER_ENABLE != D4D_FALSE
      if(pThis->radius)
        D4D_DrawTextRRect(&_calc.contentPos, &_calc.contentSize, &pEditBoxData->string, clrT, clrB, pThis->radius);
      else
      #endif
        D4D_DrawTextRect(&_calc.contentPos, &_calc.contentSize, &pEditBoxData->string, clrT, clrB);
    }
    
    // Draw cursor
    if(pEditBoxData->flags & D4D_EDIT_BOX_FLAGS_REDRAWCURSOR)
    {
      D4D_STRING tmp_textbuff = pEditBoxData->string;
      D4D_POINT tmp_point = _calc.contentPos;
      D4D_SIZE tmp_size = _calc.contentSize;
      D4D_COOR widthChar, widthText;
      pEditBoxData->flags &= ~D4D_EDIT_BOX_FLAGS_REDRAWCURSOR;
      
      tmp_textbuff.printLen = (D4D_INDEX)(pEditBoxData->curPos - tmp_textbuff.printOff);
      widthText = D4D_GetTextBuffWidthTab(&tmp_textbuff, NULL);
      tmp_point.x += widthText;

      widthChar = D4D_GetCharWidth(tmp_textbuff.fontId, *(tmp_textbuff.pText + pEditBoxData->curPos));
      
      if((widthText + widthChar) > _calc.contentSize.cx)
        widthChar = 0;  
      
      if(((pEditBoxData->flags & D4D_EDIT_BOX_FLAGS_CURSORSTATE) && !(pEditBoxData->flags & D4D_EDIT_BOX_FLAGS_MOVECURSOR)) || (widthChar == 0))
      {
        
        //Draw the cursor
        tmp_size.cy = D4D_GetFontHeight(tmp_textbuff.fontId);
        tmp_size.cx = 2;
        
        // moed to cursor to right position in Y
        if (_calc.contentSize.cy > tmp_size.cy)  
          tmp_point.y += ( _calc.contentSize.cy - tmp_size.cy)/2;
                
        if (pThis->radius > 0)
        {
          // if rounded box we need to move the cursor to the right slightly if it will fit
          if ((tmp_point.x + pThis->radius/2) < (_calc.contentPos.x + _calc.contentSize.cx - 1))
            tmp_point.x += pThis->radius/2;
        }

        
        if(pEditBoxData->flags & D4D_EDIT_BOX_FLAGS_CURSORSTATE)
          D4D_FillRect(&tmp_point, &tmp_size, clrT);
        else
          D4D_FillRect(&tmp_point, &tmp_size, clrB);
      }else
      {
        // Hide the Cursor
        pEditBoxData->flags &= ~D4D_EDIT_BOX_FLAGS_MOVECURSOR;
        
        tmp_size.cx = widthChar;
        tmp_textbuff.printLen = 1;
        tmp_textbuff.printOff = pEditBoxData->curPos;
        D4D_DrawTextRect(&tmp_point, &tmp_size, &tmp_textbuff, clrT, clrB);        
      }
    }
    
          
}


/*******************************************************
*
* EDIT_BOX key handling routine
*
*******************************************************/

static void D4D_EditBoxOnKeyUp(D4D_MESSAGE* pMsg)    
{
  D4D_OBJECT* pThis = pMsg->pObject;
  D4D_EDIT_BOX* pEditBox = D4D_GET_EDIT_BOX(pThis);
  D4D_EDIT_BOX_DATA* pEditBoxData = D4D_GET_EDIT_BOX_DATA(pEditBox);
  
  if(D4D_GetCapturedObject() == pMsg->pObject)
  {
    switch(pMsg->prm.key)
    {
      case D4D_KEY_SCANCODE_LEFT:
        // move cursor on new char position
        if(pEditBoxData->curPos)
          D4D_EditBoxSetCursor(pThis, (D4D_INDEX)(pEditBoxData->curPos - 1));  
      break;
      
      case D4D_KEY_SCANCODE_RIGHT:
        D4D_EditBoxSetCursor(pThis, (D4D_INDEX)(pEditBoxData->curPos + 1));
      break;
      
      case D4D_KEY_SCANCODE_ESC:
        D4D_CaptureKeys(NULL);
      break;
      
      case D4D_KEY_SCANCODE_ENTER:
      case D4D_KEY_SCANCODE_UP:
      case D4D_KEY_SCANCODE_DOWN:
      break;
      
      default:
        // Standard keys input
        D4D_EditBoxPutChar(pThis, pMsg->prm.key);
        break;
    }    
  }else 
    if(pMsg->prm.key == D4D_KEY_SCANCODE_ENTER)
    D4D_CaptureKeys(pMsg->pObject);                          
  
  
}

/*******************************************************
*
* EDIT_BOX touch screen handling routine
*
*******************************************************/
#ifdef D4D_LLD_TCH
static void D4D_EditBoxOnTouch(D4D_MESSAGE* pMsg)
{
  D4D_OBJECT* pThis = pMsg->pObject;
  
  if(D4D_GetFocusedObject(pMsg->pScreen) != pThis)
  {    
    if(pMsg->nMsgId == D4D_MSG_TOUCHED)
      D4D_FocusSet(pMsg->pScreen, pThis);
  }
}
#endif

static void D4D_EditBoxOnInit(D4D_MESSAGE* pMsg)
{
  D4D_OBJECT* pThis = pMsg->pObject;
  D4D_EDIT_BOX* pEditBox = D4D_GET_EDIT_BOX(pThis);
  D4D_EDIT_BOX_DATA* pEditBoxData = D4D_GET_EDIT_BOX_DATA(pEditBox);
  
  
  // If there is any init text just copy it into work buffer
  if(pEditBox->pInitText)
  {
    D4D_INDEX i = (D4D_INDEX)(pEditBoxData->string.buffSize - 1);
    D4D_TCHAR* s = pEditBox->pInitText;
    D4D_TCHAR* d = pEditBoxData->string.pText;
    D4D_INDEX cnt = 0;  
    D4D_COOR maxWidth = (D4D_COOR)(pThis->size.cx - 1);
    
    if(pThis->pMargin)
    {
      maxWidth -= pThis->pMargin->left;
      maxWidth -= pThis->pMargin->right;            
    }
    
    while((i--) && *s)
    {
      *d++ = *s++;
      cnt++;
    }
      
    *d = '\0'; // And terminate string
    pEditBoxData->string.printLen = cnt;
    pEditBoxData->string.printLen = D4D_GetTextMaxLength(&(pEditBoxData->string), maxWidth);      
    
    if(pEditBoxData->string.printLen != cnt)
      pEditBoxData->flags |= D4D_EDIT_BOX_FLAGS_FULL;  
    
    D4D_EditBoxSetCursor(pThis, cnt);
      
  }
  
  // set flag to redraw screen
  D4D_InvalidateObject(pMsg->pObject, D4D_TRUE);
}


static void D4D_EditBoxTimeTick(D4D_MESSAGE* pMsg)
{
  D4D_OBJECT* pThis = pMsg->pObject;
  D4D_EDIT_BOX* pEditBox = D4D_GET_EDIT_BOX(pThis);
  D4D_EDIT_BOX_DATA* pEditBoxData = D4D_GET_EDIT_BOX_DATA(pEditBox);
  
  pEditBoxData->tickCounter++;
  if(pEditBoxData->tickCounter >= D4D_EDIT_BOX_CURSOR_BLINK_TICK_COUNTER)
  {
    pEditBoxData->tickCounter = 0;
    pEditBoxData->flags |= D4D_EDIT_BOX_FLAGS_REDRAWCURSOR;
    D4D_InvalidateObject(pMsg->pObject, D4D_FALSE);
    
    if(pEditBoxData->flags & D4D_EDIT_BOX_FLAGS_CURSORSTATE)
      pEditBoxData->flags &= ~D4D_EDIT_BOX_FLAGS_CURSORSTATE;
    else
      pEditBoxData->flags |= D4D_EDIT_BOX_FLAGS_CURSORSTATE;
  }
}

/**************************************************************//*!
*
*
*
*     THE INTERNAL HELP FUNCTIONS
*
*
*
******************************************************************/

static D4D_BOOL D4D_EditBoxPutCharInt(D4D_OBJECT_PTR pObj, D4D_TCHAR ch)
{
  D4D_EDIT_BOX* pEditBox = D4D_GET_EDIT_BOX(pObj);
  D4D_EDIT_BOX_DATA* pData = pEditBox->pData;
  
  D4D_INDEX max_size = (D4D_INDEX)(pData->string.buffSize - 1);
  D4D_COOR maxWidth = (D4D_COOR)(pObj->size.cx - 1);     
  
  if(pData->curPos >= max_size)
    return D4D_FALSE;
  
  // Place new character
  pData->string.pText[pData->curPos] = ch;
  
  if(pObj->pMargin)
  {
    maxWidth -= pObj->pMargin->left;
    maxWidth -= pObj->pMargin->right;            
  }
  // just for case
  pData->string.printLen++;
  pData->string.printLen = D4D_GetTextMaxLength(&(pData->string), maxWidth);      
          
  // move cursor on new char position
  D4D_EditBoxSetCursor(pObj, (D4D_INDEX)(pData->curPos + 1));
  
  if(pEditBox->pOnChange)
     pEditBox->pOnChange((D4D_OBJECT*)pObj);
    
  return D4D_TRUE;
}

//-----------------------------------------------------------------------------
// FUNCTION:    D4D_EditBoxPutChar
// SCOPE:       Edit Box object external API function
// DESCRIPTION: The function puts one char on cursor position and updates cursor
//              
// PARAMETERS:  D4D_OBJECT_PTR pObj - pointer on Edit Box object
//              D4D_TCHAR ch - char that will be puts on edit box
//              
// RETURNS:     result
//-----------------------------------------------------------------------------
D4D_BOOL D4D_EditBoxPutChar(D4D_OBJECT_PTR pObj, D4D_TCHAR ch) 
{  
  D4D_EDIT_BOX* pEditBox = D4D_GET_EDIT_BOX(pObj);
  D4D_EDIT_BOX_DATA* pData = pEditBox->pData;
  D4D_BOOL res = D4D_TRUE;
  
  if(ch >= ' ') {
    res = D4D_EditBoxPutCharInt(pObj, ch);   //Put New char
  }else {      
    switch(ch) {      
      case '\r': // return carrier
        D4D_EditBoxSetCursor(pObj, 0);
        break;
      
      case '\b': // backspace
        if(pData->curPos)
        {
          D4D_TCHAR* pText = pData->string.pText;
          D4D_COOR maxWidth = (D4D_COOR)(pObj->size.cx - 1);     
          D4D_INDEX i = pData->curPos; //Go back one char 
                      
          i--;
          
          while(pText[i])
          {
            pText[i] = pText[i+1];
            i++;
          }

          if(pObj->pMargin)
          {
            maxWidth -= pObj->pMargin->left;
            maxWidth -= pObj->pMargin->right;            
          }
          
          pData->string.printLen = D4D_GetTextMaxLength(&(pData->string), maxWidth);
 
          // move cursor on new char position
          D4D_EditBoxSetCursor(pObj, (D4D_INDEX)(pData->curPos - 1));
          
          if(pEditBox->pOnChange)
            pEditBox->pOnChange((D4D_OBJECT*)pObj);
    
        }else
          res = D4D_FALSE;        
        break;
    }
  }
  pData->flags |= (D4D_EDIT_BOX_FLAGS_REDRAWTEXT | D4D_EDIT_BOX_FLAGS_REDRAWCURSOR);
  D4D_InvalidateObject(pObj, D4D_FALSE);
  return res;
}

//-----------------------------------------------------------------------------
// FUNCTION:    D4D_EditBoxPutString
// SCOPE:       Edit Box object external API function
// DESCRIPTION: The function puts string on cursor position and updates cursor
//              
// PARAMETERS:  D4D_OBJECT_PTR pObj - pointer on edit box object
//              D4D_TCHAR* pText - string that will be puts on edit box
//              
// RETURNS:     result
//-----------------------------------------------------------------------------
D4D_BOOL D4D_EditBoxPutString(D4D_OBJECT_PTR pObj, D4D_TCHAR* pText) 
{  
  if(pText == NULL)
    return D4D_FALSE;
      
  while(*pText) {
    if(!D4D_EditBoxPutChar(pObj, *pText))
      return D4D_FALSE;    
    pText++;  
  }
        
  return D4D_TRUE;
}

//-----------------------------------------------------------------------------
// FUNCTION:    D4D_EditBoxGetCursor
// SCOPE:       Edit Box object external API function
// DESCRIPTION: The function returns the cursor position
//              
// PARAMETERS:  D4D_OBJECT_PTR pObj - pointer on edit box object
//              D4D_INDEX newPos - new position request.
//                           
// RETURNS:     D4D_INDEX real cursor position
//-----------------------------------------------------------------------------
D4D_INDEX D4D_EditBoxSetCursor(D4D_OBJECT_PTR pObj, D4D_INDEX newPos) 
{
  D4D_EDIT_BOX* pEditBox = D4D_GET_EDIT_BOX(pObj);
  D4D_EDIT_BOX_DATA* pData = pEditBox->pData;
  D4D_COOR maxWidth = (D4D_COOR)(pObj->size.cx - 1);
    
      
  // Object exists?
  if(pObj == NULL)
    return 0;
  
  // Check if the cursor is not out of range
  if(newPos >= pData->string.buffSize)
    return pData->curPos;
 
  // Check if the cursor in on requested position
  if(pData->curPos == newPos)
    return newPos;
  
  if(pObj->pMargin)
  {
    maxWidth -= pObj->pMargin->left;
    maxWidth -= pObj->pMargin->right;            
  }
  
  // In case that the cursor overjump non text , fill up overjumped palce by spaces
  if(pData->curPos < newPos)
  {
    D4D_INDEX i = (D4D_INDEX)(newPos - 1);
    D4D_TCHAR* pText = pData->string.pText; 
   // D4D_INDEX cnt = pData->string.printLen;
    while(i && (!pText[i]))
    {
      pText[i--] = ' ';              
   //   cnt++;
    } 
    
    // the visible area is not full
    if(!(pData->flags & D4D_EDIT_BOX_FLAGS_FULL))
    {
    //  D4D_INDEX len = pData->string.printLen;
      if(pData->string.printLen < newPos)
      {
        pData->string.printLen = newPos;
        pData->string.printLen = D4D_GetTextMaxLength(&(pData->string), maxWidth);      
        
        if(pData->string.printLen != newPos)
          pData->flags |= D4D_EDIT_BOX_FLAGS_FULL;    
      }
    }
    
  }
  
  // Check if the cursor is in visible range
  if((newPos >= pData->string.printOff) && (newPos <= (pData->string.printLen + pData->string.printOff - 0)))
  {    
    pData->curPos = newPos;
  }else
  {
    // requested position is not in visible area
    if(pData->curPos > newPos)
    {
      // Go back with visible area
      pData->string.printOff = newPos;
      // Temporary fill up printLen with maximal possible value to be used by function D4D_GetTextMaxLength(...)
      pData->string.printLen = (D4D_INDEX)(pData->string.buffSize - newPos - 1);
      // Compute real printLen
      pData->string.printLen = D4D_GetTextMaxLength(&(pData->string),  maxWidth);      
    }else
    {
      // Go forward with visible area
      D4D_COOR currentWidth = D4D_GetTextBuffWidth(&(pData->string));
      D4D_INDEX currentLast =  (D4D_INDEX)(pData->string.printOff + pData->string.printLen);
      D4D_COOR width;
      D4D_FONT fontIx = pData->string.fontId;
      
      while(currentLast < newPos)
      {
        currentWidth -= D4D_GetCharWidth(fontIx, pData->string.pText[pData->string.printOff]);
        pData->string.printOff++;
        pData->string.printLen--;
        while(1)
        {
          if(!pData->string.pText[currentLast])
            break;
          
          width = D4D_GetCharWidth(fontIx, pData->string.pText[currentLast]);
          
          if((width + currentWidth) > maxWidth)
            break;
          
          currentWidth += width;
          currentLast++;          
          pData->string.printLen++;
        }
      }   
    }
    pData->curPos = newPos;
  }
  
  pData->flags |= (D4D_EDIT_BOX_FLAGS_REDRAWTEXT | D4D_EDIT_BOX_FLAGS_REDRAWCURSOR | D4D_EDIT_BOX_FLAGS_MOVECURSOR);
  D4D_InvalidateObject(pObj, D4D_FALSE);
  return pData->curPos;  
}




//-----------------------------------------------------------------------------
// FUNCTION:    D4D_EditBoxGetCursor
// SCOPE:       Edit Box object external API function
// DESCRIPTION: The function returns the cursor position
//              
// PARAMETERS:  D4D_OBJECT_PTR pObj - pointer on edit box object
//                           
// RETURNS:     D4D_INDEX cursor position
//-----------------------------------------------------------------------------
D4D_INDEX D4D_EditBoxGetCursor(D4D_OBJECT_PTR pObj) 
{
  D4D_EDIT_BOX* pEditBox = D4D_GET_EDIT_BOX(pObj);
  D4D_EDIT_BOX_DATA* pData = pEditBox->pData;
  
  if(pObj == NULL)
    return 0;
  
  return pData->curPos;  
}


//-----------------------------------------------------------------------------
// FUNCTION:    D4D_EditBoxClearAll
// SCOPE:       Edit Box object external API function
// DESCRIPTION: The function clear whole edit box data 
//              
// PARAMETERS:  D4D_OBJECT_PTR pObj - pointer on edit box object
//              
// RETURNS:     none
//-----------------------------------------------------------------------------
void D4D_EditBoxClearAll(D4D_OBJECT_PTR pObj)
{
  D4D_EDIT_BOX* pEditBox = D4D_GET_EDIT_BOX(pObj);
  D4D_EDIT_BOX_DATA* pData = pEditBox->pData;
  
  LWord i,i_max;
  
  // Init the text array   
  i_max = pData->string.buffSize;
      
  for(i=0;i<i_max;i++)
    pData->string.pText[i] = '\0';
  
  pData->curPos = 0;
  pData->string.printOff = 0;
  pData->string.printLen = 0;
  pData->flags &= ~D4D_EDIT_BOX_FLAGS_FULL;  
    
  if(pEditBox->pOnChange)
     pEditBox->pOnChange((D4D_OBJECT*)pObj);
    
  pData->flags |= (D4D_EDIT_BOX_FLAGS_REDRAWTEXT | D4D_EDIT_BOX_FLAGS_REDRAWCURSOR);
  D4D_InvalidateObject(pObj, D4D_FALSE);    
}

//-----------------------------------------------------------------------------
// FUNCTION:    D4D_EditBoxGetText
// SCOPE:       Edit Box object external API function
// DESCRIPTION: The function clear whole edit box data 
//              
// PARAMETERS:  D4D_OBJECT_PTR pObj - pointer on edit box object
//              
// RETURNS:     pointer on edit box text 
//-----------------------------------------------------------------------------
D4D_TCHAR* D4D_EditBoxGetText(D4D_OBJECT_PTR pObj)
{
  D4D_EDIT_BOX* pEditBox = D4D_GET_EDIT_BOX(pObj);
  D4D_EDIT_BOX_DATA* pData = pEditBox->pData;
  
  return pData->string.pText;
}


/**************************************************************//*!
*
*
*
*     THE INTERNAL API FUNCTIONS
*
*
*
******************************************************************/

/*******************************************************
*
* The EDIT_BOX message handler 
*
*******************************************************/

void D4D_EditBoxOnMessage(D4D_MESSAGE* pMsg)
{
  switch(pMsg->nMsgId)
  {
    case D4D_MSG_DRAW:
      D4D_EditBoxOnDraw(pMsg);
      break;
          
    case D4D_MSG_KEYDOWN:
      D4D_EditBoxOnKeyUp(pMsg);
      break;        

#ifdef D4D_LLD_TCH       
    case D4D_MSG_TOUCHED:
    case D4D_MSG_TOUCH_AUTO:
        D4D_EditBoxOnTouch(pMsg);         
      break;
#endif      
    case D4D_MSG_ONINIT:
      pMsg->pObject->pData->flags &= ~D4D_OBJECT_F_NOTINIT;      
      D4D_EditBoxOnInit(pMsg);
      break;
      
    case D4D_MSG_TIMETICK:
      #if (D4D_EDIT_BOX_CURSOR_BLINK_TICK_COUNTER > 0)      
        D4D_EditBoxTimeTick(pMsg);
      #endif
      break;
    
    case D4D_MSG_KILLFOCUS:
      D4D_CaptureKeys(NULL);
    case D4D_MSG_KILLCAPTURE:        
    case D4D_MSG_SETFOCUS:
    case D4D_MSG_SETCAPTURE:
    
      { 
        D4D_EDIT_BOX* pEditBox = D4D_GET_EDIT_BOX(pMsg->pObject);
        D4D_EDIT_BOX_DATA* pData = pEditBox->pData;
        
        D4D_EnableTimeTicks(pMsg->pObject, pMsg->nMsgId == D4D_MSG_SETCAPTURE);
        
        pData->flags |= (D4D_EDIT_BOX_FLAGS_REDRAWBEVEL | D4D_EDIT_BOX_FLAGS_REDRAWTEXT | D4D_EDIT_BOX_FLAGS_REDRAWCURSOR);
        pData->flags &= ~D4D_EDIT_BOX_FLAGS_CURSORSTATE;
        D4D_InvalidateObject(pMsg->pObject, D4D_FALSE);
      }
      break;
   
    default:
      // call the default behavior of all objects
      D4D_ObjOnMessage(pMsg);
  }
}