/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_edit_box.h
*
* @author    b01119
* 
* @version   0.0.1.0
* 
* @date      Jul-11-2012
* 
* @brief     D4D Driver editBox object header file
*
*******************************************************************************/

#ifndef __D4D_EDIT_BOX_H
#define __D4D_EDIT_BOX_H

extern const D4D_OBJECT_SYS_FUNCTION d4d_editBoxSysFunc;
/******************************************************************************
* Constants
******************************************************************************/

/******************************************************************************
* Types
******************************************************************************/

#ifndef D4D_EDIT_BOX_F_DEFAULT
  #define D4D_EDIT_BOX_F_DEFAULT  (D4D_OBJECT_F_VISIBLE | D4D_OBJECT_F_ENABLED | D4D_OBJECT_F_TABSTOP | D4D_OBJECT_F_TOUCHENABLE | D4D_OBJECT_F_FOCUSRECT | D4D_OBJECT_F_BEVEL_DENTED)
#endif

#ifndef D4D_EDIT_BOX_FNT_PRTY_DEFAULT
  #define D4D_EDIT_BOX_FNT_PRTY_DEFAULT  ( 0 )
#endif

#ifndef D4D_EDIT_BOX_TXT_PRTY_DEFAULT
  #define D4D_EDIT_BOX_TXT_PRTY_DEFAULT  ( D4D_ALIGN_V_CENTER_MASK | D4D_ALIGN_H_LEFT_MASK )
#endif

#ifndef D4D_EDIT_BOX_CURSOR_BLINK_TICK_COUNTER
  #define D4D_EDIT_BOX_CURSOR_BLINK_TICK_COUNTER  ( 10 )
#endif

typedef sByte D4D_EDIT_BOX_INDEX;

typedef Byte D4D_EDIT_BOX_FLAGS;
  
#define D4D_EDIT_BOX_FLAGS_REDRAWBEVEL    (0x02)
#define D4D_EDIT_BOX_FLAGS_REDRAWTEXT     (0x04)
#define D4D_EDIT_BOX_FLAGS_MOVECURSOR     (0x08)
#define D4D_EDIT_BOX_FLAGS_REDRAWCURSOR   (0x10)
#define D4D_EDIT_BOX_FLAGS_CURSORSTATE    (0x20)
#define D4D_EDIT_BOX_FLAGS_FULL           (0x40)




typedef struct 
{
   D4D_STRING string;
   D4D_INDEX curPos;
   D4D_INDEX tickCounter;
   D4D_EDIT_BOX_FLAGS flags;
} D4D_EDIT_BOX_DATA;

// EditBox configuration (goes to ROM by default)
typedef struct
{
   D4D_TCHAR* pInitText;
   D4D_EDIT_BOX_DATA* pData;     // EditBox volatile data struct
   void (*pOnChange)(D4D_OBJECT* pThis);
} D4D_EDIT_BOX;

/******************************************************************************
* Macros 
******************************************************************************/

// getting the EDIT_BOX structure from general OBJECT
#define D4D_GET_EDIT_BOX(pObj) ((D4D_EDIT_BOX*)((pObj)->pParam))
#define D4D_GET_EDIT_BOX_DATA(pEditBox) ((pEditBox)->pData)
/*********************************************************
*
* static declaration macros
*
*********************************************************/

    
#define _D4D_DECLARE_EDIT_BOX(type, name, x, y, cx, cy, radius, pMargin, pRelations, flags, pScheme, initText, FontId, maxStr, pUser, pOnChange, pOnUsrMsg) \
    static D4D_STR_PROPERTIES name##_strPrties = {D4D_EDIT_BOX_FNT_PRTY_DEFAULT, D4D_EDIT_BOX_TXT_PRTY_DEFAULT};\
    static D4D_TCHAR name##_txtArray[maxStr + 1];\
    static D4D_EDIT_BOX_DATA name##_data = { {name##_txtArray, maxStr, FontId, &name##_strPrties, 0, 0}, 0};\
    static type D4D_EDIT_BOX name##_params = \
    { \
        initText,\
        &(name##_data), \
        pOnChange \
    }; \
    \
    D4D_DECLARE_OBJECT(type, name, x, y, cx, cy, radius, pMargin, pRelations, pOnUsrMsg, &d4d_editBoxSysFunc, &(name##_params), flags, pUser, pScheme)
    


#define D4D_DECLARE_EDIT_BOX(name, x, y, cx, cy, pRelations, flags, pScheme, pInitText, FontId, maxStr, pUser, pOnChange, pOnUsrMsg) \
 _D4D_DECLARE_EDIT_BOX(D4D_CONST, name, x, y, cx, cy, 0, (D4D_MARGIN*)&editBox_marginDefault, pRelations, flags, pScheme, pInitText,\
    FontId, maxStr, pUser, pOnChange, pOnUsrMsg)

#define D4D_DECLARE_STD_EDIT_BOX(name, x, y, cx, cy, pRelations, pInitText, FontId, maxStr, pOnChange) \
    D4D_DECLARE_EDIT_BOX(name, x, y, cx, cy, pRelations, (D4D_EDIT_BOX_F_DEFAULT), NULL, pInitText, FontId, maxStr, NULL, pOnChange, NULL)

// Rounded editBox definition
#define D4D_DECLARE_REDIT_BOX(name, x, y, cx, cy, radius, pRelations, flags, pScheme, pInitText, FontId, maxStr, pUser, pOnChange, pOnUsrMsg) \
 _D4D_DECLARE_EDIT_BOX(D4D_CONST, name, x, y, cx, cy, radius, (D4D_MARGIN*)&editBox_marginDefault, pRelations, flags, pScheme, pInitText,\
    FontId, maxStr, pUser, pOnChange, pOnUsrMsg)

#define D4D_DECLARE_STD_REDIT_BOX(name, x, y, cx, cy, radius, pRelations, pInitText, FontId, maxStr, pOnChange) \
    D4D_DECLARE_REDIT_BOX(name, x, y, cx, cy, radius, pRelations, (D4D_EDIT_BOX_F_DEFAULT), NULL, pInitText, FontId, maxStr, NULL, pOnChange, NULL)


//IN RAM instantions macros
#define D4D_DECLARE_EDIT_BOX_INRAM(name, x, y, cx, cy, pRelations, flags, pScheme, pInitText, FontId, maxStr, pUser, pOnChange, pOnUsrMsg) \
 _D4D_DECLARE_EDIT_BOX(D4D_NO_CONST, name, x, y, cx, cy, 0, (D4D_MARGIN*)&editBox_marginDefault, pRelations, flags, pScheme, pInitText,\
    FontId, maxStr, pUser, pOnChange, pOnUsrMsg)

#define D4D_DECLARE_STD_EDIT_BOX_INRAM(name, x, y, cx, cy, pRelations, pInitText, FontId, maxStr, pOnChange) \
    D4D_DECLARE_EDIT_BOX_INRAM(name, x, y, cx, cy, pRelations, (D4D_EDIT_BOX_F_DEFAULT), NULL, pInitText, FontId, maxStr, NULL, pOnChange, NULL)

// Rounded editBox definition
#define D4D_DECLARE_REDIT_BOX_INRAM(name, x, y, cx, cy, radius, pRelations, flags, pScheme, pInitText, FontId, maxStr, pUser, pOnChange, pOnUsrMsg) \
 _D4D_DECLARE_EDIT_BOX(D4D_NO_CONST, name, x, y, cx, cy, radius, (D4D_MARGIN*)&editBox_marginDefault, pRelations, flags, pScheme, pInitText,\
    FontId, maxStr, pUser, pOnChange, pOnUsrMsg)

#define D4D_DECLARE_STD_REDIT_BOX_INRAM(name, x, y, cx, cy, radius, pRelations, pInitText, FontId, maxStr, pOnChange) \
    D4D_DECLARE_REDIT_BOX_INRAM(name, x, y, cx, cy, radius, pRelations, (D4D_EDIT_BOX_F_DEFAULT), NULL, pInitText, FontId, maxStr, NULL, pOnChange, NULL)
   
   
/******************************************************************************
* Global variables
******************************************************************************/   
extern const D4D_MARGIN editBox_marginDefault;    
   
/******************************************************************************
* Global functions
******************************************************************************/
D4D_BOOL D4D_EditBoxPutChar(D4D_OBJECT_PTR pObj, D4D_TCHAR ch);
D4D_BOOL D4D_EditBoxPutString(D4D_OBJECT_PTR pObj, D4D_TCHAR* pText);
D4D_INDEX D4D_EditBoxSetCursor(D4D_OBJECT_PTR pObj, D4D_INDEX newPos);
D4D_INDEX D4D_EditBoxGetCursor(D4D_OBJECT_PTR pObj);
void D4D_EditBoxClearAll(D4D_OBJECT_PTR pObj);
D4D_TCHAR* D4D_EditBoxGetText(D4D_OBJECT_PTR pObj);

#endif /* __D4D_EDIT_BOX_H  */
 



