/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_group_box.c
*
* @author    b01119
* 
* @version   0.0.2.0
* 
* @date      Jul-11-2012
* 
* @brief     D4D driver groupBox object c file 
*
******************************************************************************/

#include "d4d.h"
#include "common_files/d4d_private.h"

//Internal API
void D4D_GroupBoxOnMessage(D4D_MESSAGE* pMsg);
D4D_STRING* D4D_GroupBoxGetTextBuffer(D4D_OBJECT* pThis);

D4D_MARGIN groupBox_marginDefault = 
{
  D4D_GROUP_BOX_FRAME_MARGIN, 
  D4D_GROUP_BOX_HEADER_MARGIN,
  D4D_GROUP_BOX_FRAME_MARGIN,
  D4D_GROUP_BOX_FRAME_MARGIN
};

const D4D_OBJECT_SYS_FUNCTION d4d_groupBoxSysFunc = 
{
  D4D_DEFSTR("Group Box"),
  D4D_GroupBoxOnMessage,
  NULL,
  NULL
};

/*******************************************************
*
* GROUP_BOX Drawing routine
*
*******************************************************/

static void D4D_GroupBoxOnDraw(D4D_MESSAGE* pMsg)
{
    D4D_OBJECT* pThis = pMsg->pObject;
    D4D_GROUP_BOX* pGroupBox = D4D_GET_GROUP_BOX(pMsg->pObject);
    
    D4D_POINT pnt = D4D_GetClientToScreenPoint(pThis, &pThis->position);
    D4D_POINT pntLoc = pnt;
    D4D_SIZE size;
    D4D_COLOR bck = D4D_ObjectGetBckgFillColor(pThis);
    D4D_COLOR fore = D4D_ObjectGetForeColor(pThis, pMsg->prm.draw);
                
    // only draw if complete draw is needed
    if(pMsg->prm.draw & D4D_OBJECT_DRAWFLAGS_COMPLETE)
    {
#if D4D_ROUND_CORNER_ENABLE == D4D_TRUE
      if(pThis->radius)
        D4D_FillRRect(&pntLoc, &pThis->size, bck, pThis->radius);
      else
#endif
        D4D_FillRect(&pntLoc, &pThis->size, bck);
     
      if(pThis->initFlags & D4D_OBJECT_F_BEVEL_MASK)
        D4D_Bevel(&pnt, &pThis->size, bck, D4D_OBJECT_FLAGS2BEVEL(pThis), pThis->radius);      
    }

    // Draw Focus rect 
    
    if(pThis->initFlags & D4D_OBJECT_F_FOCUSRECT)
    {
      pntLoc.x += (pThis->pMargin->left / 2);
      pntLoc.y += (pThis->pMargin->top / 2);
      size = pThis->size;
      size.cx -= ((pThis->pMargin->left + pThis->pMargin->right) / 2);
      size.cy -= ((pThis->pMargin->top + pThis->pMargin->bottom) / 2);
  
  #if D4D_ROUND_CORNER_ENABLE == D4D_TRUE
      if(pThis->radius)
      {
        D4D_COOR radius = 0;
        if(pThis->radius - (pThis->pMargin->left / 2) > 0)
          radius = (D4D_COOR)(pThis->radius - (pThis->pMargin->left / 2));
        
        D4D_RRect(&pntLoc, &size, D4D_LINE_THIN, D4D_ChangeColorIntesity(fore, D4D_BEVEL_BCKG_INTENSITY), radius);    
      }
      else
  #endif    
        D4D_Rect(&pntLoc, &size, D4D_LINE_THIN, D4D_ChangeColorIntesity(fore, D4D_BEVEL_BCKG_INTENSITY));
    }

    
    if(pGroupBox->textBuff.pText)
    {
      D4D_COOR activeSize = 2 * D4D_GROUP_BOX_TEXT_OFFSET;
      
      #if D4D_ROUND_CORNER_ENABLE == D4D_TRUE
        activeSize += (D4D_COOR)(pThis->radius * 2);
      #endif
      
      if(pThis->initFlags & D4D_OBJECT_F_BEVEL_MASK)
        activeSize += D4D_BEVEL_WIDTH * 2;
 
      size.cy = pThis->pMargin->top;    
      size.cx = (D4D_COOR)(D4D_GetTextWidth(pGroupBox->textBuff.fontId, pGroupBox->textBuff.pText) + (2 * D4D_GROUP_BOX_TEXT_OFFSET));
  
      if(pThis->initFlags & D4D_OBJECT_F_BEVEL_MASK)
        size.cx += D4D_BEVEL_WIDTH * 2;

      if(size.cx > (pThis->size.cx - activeSize))
        size.cx = (D4D_COOR)(pThis->size.cx - activeSize); 

      pnt.x += D4D_GROUP_BOX_TEXT_OFFSET;
      
      if(pThis->initFlags & D4D_OBJECT_F_BEVEL_MASK)
      { 
        pnt.x += D4D_BEVEL_WIDTH;
        pnt.y += D4D_BEVEL_WIDTH;
            
        if(size.cy - D4D_BEVEL_WIDTH > 0)
          size.cy -= D4D_BEVEL_WIDTH;
        else
          size.cy = 0;
      }
      
  #if D4D_ROUND_CORNER_ENABLE == D4D_TRUE
      if(pThis->radius)
        pnt.x += pThis->radius;
  #endif
      
      D4D_DrawTextRect(&pnt, &size, &pGroupBox->textBuff, fore, bck);
    }
    
    // draw the bitmap
#if D4D_ROUND_CORNER_ENABLE == D4D_TRUE
#else        
#endif        
}

/**************************************************************//*!
*
*
*
*     THE INTERNAL API FUNCTIONS
*
*
*
******************************************************************/

/*******************************************************
*
* The main GROUP_BOX message handler 
*
*******************************************************/

void D4D_GroupBoxOnMessage(D4D_MESSAGE* pMsg)
{
    switch(pMsg->nMsgId)
    {
    case D4D_MSG_DRAW:
        D4D_GroupBoxOnDraw(pMsg);
        break;
        
    default:
        // call the default behavior
        D4D_ObjOnMessage(pMsg);
    }
}