/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_icon.c
*
* @author    b01119
* 
* @version   0.0.46.0
* 
* @date      Jun-29-2012
* 
* @brief     D4D driver icon object c file 
*
******************************************************************************/

#include "d4d.h"
#include "common_files/d4d_private.h"

//Internal API
void D4D_IconOnMessage(D4D_MESSAGE* pMsg);
D4D_STRING* D4D_IconGetTextBuffer(D4D_OBJECT* pThis);
D4D_ICON_INDEX D4D_IconGetBmpCount(D4D_OBJECT_PTR pThis);

const D4D_OBJECT_SYS_FUNCTION d4d_iconSysFunc = 
{
  D4D_DEFSTR("Icon"),
  D4D_IconOnMessage,
  NULL,
  D4D_IconGetTextBuffer
};

// temporary structure for icon coordinate calculation
typedef struct 
{
    D4D_POINT position;
    D4D_POINT txtPos;
} D4D_ICON_TMP_VAL;
  
#define _calc (*((D4D_ICON_TMP_VAL*)d4d_scratchPad))

    
static void D4D_IconValue2Coor(D4D_OBJECT* pThis)
{
  _calc.position = D4D_GetClientToScreenPoint(pThis, &pThis->position);
  _calc.txtPos = _calc.position;
}

/*******************************************************
*
* ICON Drawing routine
*
*******************************************************/

static void D4D_IconOnDraw(D4D_MESSAGE* pMsg)
{
    D4D_OBJECT* pThis  =  pMsg->pObject;
    D4D_ICON* pIcon = D4D_GET_ICON(pThis);    
    D4D_OBJECT_DRAWFLAGS draw = pMsg->prm.draw;
    D4D_COLOR clrT, clrB;
    D4D_BMP_PROPERTIES bmpProp;
    D4D_IconValue2Coor(pThis);
    
    bmpProp = (D4D_ALIGN_V_TOP_MASK | D4D_ALIGN_H_CENTER_MASK);    
    
    clrT = D4D_ObjectGetForeColor(pThis, draw);
    clrB = D4D_ObjectGetBckgColor(pThis, draw);
 
    // when background redraw is needed
        // draw the bitmap        
    if(pIcon->pBmpX[pIcon->pData->index] != NULL)
      D4D_DrawRBmpRect(&_calc.position,&pThis->size, pIcon->pBmpX[pIcon->pData->index], bmpProp, clrB, (D4D_BOOL)(!D4D_IsEnabled(pThis)), pThis->radius);
    
    // draw the text
    if(pIcon->textBuff.pText != NULL)
    {
      D4D_SIZE tmp_size;
      tmp_size.cx = D4D_GetTextWidth(pIcon->textBuff.fontId, pIcon->textBuff.pText);
      tmp_size.cy = D4D_GetFontHeight(pIcon->textBuff.fontId);
        
      D4D_DrawTextRect(&_calc.txtPos, &tmp_size, &pIcon->textBuff, clrT, clrB); 
    }
    if (pThis->initFlags & D4D_OBJECT_F_FOCUSRECT)       
      D4D_RRect(&_calc.position, &pThis->size, D4D_LINE_THIN, clrT, pThis->radius);  // draw focus rectangle

    
}

/*******************************************************
*
* Get ICON count
*
*******************************************************/

D4D_ICON_INDEX D4D_IconGetBmpCount(D4D_OBJECT_PTR pThis)
{
   D4D_ICON* pIcon = D4D_GET_ICON(pThis);
   D4D_ICON_INDEX  ix = 0;
   
   
   while(pIcon->pBmpX[ix] != NULL)
   {
     ix++;  
   }
   
   return ix;
}

/*******************************************************
*
* Set ICON value
*
*******************************************************/

void D4D_IconSetIndex(D4D_OBJECT_PTR pThis, D4D_ICON_INDEX index)
{
    D4D_ICON* pIcon = D4D_GET_ICON(pThis);
    D4D_ICON_DATA* pData = pIcon->pData;

    if(pData->index == index)
        return ;
    
    if(D4D_IconGetBmpCount(pThis) == 0)  // There is no loaded bitmaps - go out
        return;
    
    if(index >= D4D_IconGetBmpCount(pThis))  // If value is bigger then count of BMPs
        return;
    
    // remember new value
    pData->index = index;
    
    D4D_InvalidateObject(pThis, D4D_TRUE);

    // notify user that the value has changed        
    if(pIcon->OnValueChanged)
        pIcon->OnValueChanged(pThis);
}


/*******************************************************
*
* Change ICON value
*
*******************************************************/

void D4D_IconChangeIndex(D4D_OBJECT_PTR pThis, sByte incr)
{
  D4D_ICON_INDEX tmp_ix = (D4D_ICON_INDEX)(D4D_GET_ICON(pThis)->pData->index + incr);
  
  if(tmp_ix >= D4D_IconGetBmpCount(pThis))
    tmp_ix = 0;
  
  D4D_IconSetIndex(pThis, tmp_ix);
}


D4D_ICON_INDEX D4D_IconGetIndex(D4D_OBJECT_PTR pThis)
{
    D4D_ICON* pIcon = D4D_GET_ICON(pThis);
    return pIcon->pData->index;
}

#if D4D_ICON_ENABLE_ANIMATION == D4D_TRUE

/*******************************************************
*
* ICON automatic animation control - Start
*
*******************************************************/
void D4D_IconAnimationStart(D4D_OBJECT_PTR pThis)
{
  D4D_ICON* pIcon = D4D_GET_ICON(pThis);
  D4D_ICON_DATA* pData = pIcon->pData;

  if(D4D_IconGetBmpCount(pThis))
  {
    pData->animationEnabled = D4D_TRUE;
    pData->tickCounter = 0;
    D4D_EnableTimeTicks(pThis, D4D_TRUE);
  }
}

/*******************************************************
*
* ICON automatic animation control - Stop
*
*******************************************************/

void D4D_IconAnimationStop(D4D_OBJECT_PTR pThis)
{
  D4D_ICON* pIcon = D4D_GET_ICON(pThis);
  D4D_ICON_DATA* pData = pIcon->pData;

  pData->animationEnabled = D4D_FALSE;
  D4D_EnableTimeTicks(pThis, D4D_FALSE);
}


/*******************************************************
*
* ICON automatic animation control - Set count ticks
*
*******************************************************/

void D4D_IconAnimationSetCounts(D4D_OBJECT_PTR pThis, Word cnt)
{
	D4D_ICON* pIcon = D4D_GET_ICON(pThis);
  D4D_ICON_DATA* pData = pIcon->pData;


  pData->tickCounterTrshld = cnt;
}

#endif

/**************************************************************//*!
*
*
*
*     THE INTERNAL API FUNCTIONS
*
*
*
******************************************************************/

/*******************************************************
*
* The main ICON message handler 
*
*******************************************************/

void D4D_IconOnMessage(D4D_MESSAGE* pMsg)
{

#if D4D_ICON_ENABLE_ANIMATION == D4D_TRUE
  D4D_ICON* pIcon = D4D_GET_ICON(pMsg->pObject);
  D4D_ICON_DATA* pData = pIcon->pData;
#endif

  switch(pMsg->nMsgId)
  {
    

      
    case D4D_MSG_DRAW:
      D4D_IconOnDraw(pMsg);
      break;
      
#ifdef D4D_LLD_TCH       
    case D4D_MSG_TOUCHED:
      D4D_FocusSet(pMsg->pScreen, pMsg->pObject);
      break;
#endif

#if D4D_ICON_ENABLE_ANIMATION == D4D_TRUE
    
    case D4D_MSG_ONINIT:
      pMsg->pObject->pData->flags &= ~D4D_OBJECT_F_NOTINIT;
      
      pData->tickCounterTrshld = D4D_ICON_ANIMATION_TICK_COUNTER;
      break;
    
    case D4D_MSG_TIMETICK:
      if(pData->animationEnabled)
      {
        if(++(pData->tickCounter) > pData->tickCounterTrshld)
        {
          pData->tickCounter = 0;
          // update animation (change icon)
          D4D_IconChangeIndex(pMsg->pObject, 1);  
        }
      }
      break;
#endif      

    default:
      // call the default behavior of all objects
      D4D_ObjOnMessage(pMsg);
  }
}

/**************************************************************//*!
*
* Get object text buffer
*
******************************************************************/
D4D_STRING* D4D_IconGetTextBuffer(D4D_OBJECT* pThis)
{
  return &(D4D_GET_ICON(pThis)->textBuff);  
}

