/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_label.c
*
* @author    b01119
* 
* @version   0.0.42.0
* 
* @date      Jun-29-2012
* 
* @brief     D4D driver label object c file 
*
******************************************************************************/

#include "d4d.h"
#include "common_files/d4d_private.h"


// Internal API
void D4D_LabelOnMessage(D4D_MESSAGE* pMsg);
D4D_STRING* D4D_LabelGetTextBuffer(D4D_OBJECT* pThis);

const D4D_OBJECT_SYS_FUNCTION d4d_labelSysFunc = 
{
  D4D_DEFSTR("Label"),
  D4D_LabelOnMessage,
  NULL,
  D4D_LabelGetTextBuffer
};

typedef struct 
{
  D4D_POINT position;
  D4D_SIZE size;
} D4D_LBL_TMP_VAL;

#define _calc (*((D4D_LBL_TMP_VAL*)d4d_scratchPad))

    
static void D4D_LblValue2Coor(D4D_OBJECT* pThis)
{
  _calc.position = D4D_GetClientToScreenPoint(pThis, &pThis->position);
  _calc.size = pThis->size;
}
 


/*******************************************************
*
* LABEL Drawing routine
*
*******************************************************/

static void D4D_LabelOnDraw(D4D_MESSAGE* pMsg)
{
    D4D_OBJECT* pThis = pMsg->pObject;
    D4D_LABEL* pLbl = D4D_GET_LABEL(pThis);
    D4D_OBJECT_DRAWFLAGS draw = pMsg->prm.draw;
    D4D_COLOR clrT, clrB;
    
    D4D_LblValue2Coor(pThis);
        
    clrT = D4D_ObjectGetForeColor(pThis, draw);
    clrB = D4D_ObjectGetBckgColor(pThis, draw);  
    
    // draw the rectangle around the text
    if(draw & D4D_OBJECT_DRAWFLAGS_COMPLETE)  
    {
      if(pThis->initFlags & D4D_OBJECT_F_FOCUSRECT)
       D4D_RRect(&_calc.position, &pThis->size, D4D_LINE_THIN, clrT, pThis->radius);
    }
  
    // draw the text
    if(pLbl->textBuff.pText)
    {
      if(pThis->initFlags & D4D_OBJECT_F_FOCUSRECT)
      {        
#if D4D_ROUND_CORNER_ENABLE == D4D_TRUE
        _calc.position.x +=(D4D_COOR)(1 + pThis->radius / 2);
        _calc.position.y +=(D4D_COOR)(1 + pThis->radius / 2);
        _calc.size.cx -= (D4D_COOR)(2 + pThis->radius / 2);
        _calc.size.cy -= (D4D_COOR)(2 + pThis->radius / 2);
#else    
        _calc.position.x++;
        _calc.position.y++;
        _calc.size.cx -= 2;
        _calc.size.cy -= 2;    
#endif    
      }      
      D4D_DrawTextRRect(&_calc.position, &_calc.size, &pLbl->textBuff, clrT, clrB, pThis->radius);
    }   
}

/**************************************************************//*!
*
*
*
*     THE INTERNAL API FUNCTIONS
*
*
*
******************************************************************/

/*******************************************************
*
* The LABEL message handler 
*
*******************************************************/

void D4D_LabelOnMessage(D4D_MESSAGE* pMsg)
{
    switch(pMsg->nMsgId)
    {
    case D4D_MSG_DRAW:
        D4D_LabelOnDraw(pMsg);
        break;
#ifdef D4D_LLD_TCH 
    case D4D_MSG_TOUCHED:                              
        D4D_FocusSet(pMsg->pScreen, pMsg->pObject);       
        break;
#endif
    default:
        // call the default behavior of all objects
        D4D_ObjOnMessage(pMsg);
    }
}

/*******************************************************
*
* The LABEL check coordinators  handler 
*
*******************************************************/

/**************************************************************//*!
*
* Get object text buffer
*
******************************************************************/
D4D_STRING* D4D_LabelGetTextBuffer(D4D_OBJECT* pThis)
{
  return &(D4D_GET_LABEL(pThis)->textBuff);  
}
