/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_list_box.h
*
* @author    b01119
* 
* @version   0.0.4.0
* 
* @date      Jun-29-2012
* 
* @brief     D4D Driver listBox object header file
*
*******************************************************************************/

#ifndef __D4D_LIST_BOX_H
#define __D4D_LIST_BOX_H

extern const D4D_OBJECT_SYS_FUNCTION d4d_listBoxSysFunc;
void D4D_ListBoxScrollBarsFeedBack(D4D_OBJECT* pThis, D4D_INDEX old_position, D4D_INDEX new_position);
/******************************************************************************
* Constants
******************************************************************************/

/******************************************************************************
* Types
******************************************************************************/
#define D4D_LIST_BOX_F_SIDEBAR    (0x01 << D4D_OBJECT_F_OBJECT_SHIFT)

#ifndef D4D_LIST_BOX_F_DEFAULT
  #define D4D_LIST_BOX_F_DEFAULT  (D4D_OBJECT_F_VISIBLE | D4D_OBJECT_F_ENABLED | D4D_OBJECT_F_TABSTOP | D4D_OBJECT_F_TOUCHENABLE | D4D_OBJECT_F_FOCUSRECT | D4D_OBJECT_F_BEVEL_RAISED)
#endif

#ifndef D4D_LIST_BOX_F_SCRLBRS_DEFAULT
  #define D4D_LIST_BOX_F_SCRLBRS_DEFAULT  (D4D_OBJECT_F_ENABLED | D4D_OBJECT_F_TOUCHENABLE | D4D_OBJECT_F_FASTTOUCH | D4D_OBJECT_F_FOCUSRECT)
#endif

#ifndef D4D_LIST_BOX_SCRLBR_WIDTH
  #define D4D_LIST_BOX_SCRLBR_WIDTH  ( 16 )
#endif 

#ifndef D4D_LIST_BOX_SCRLBR_STEP
  #define D4D_LIST_BOX_SCRLBR_STEP  ( 1 )
#endif

#ifndef D4D_LIST_BOX_ITEM_FNT_PRTY_DEFAULT
  #define D4D_LIST_BOX_ITEM_FNT_PRTY_DEFAULT  ( 0 )
#endif

#ifndef D4D_LIST_BOX_ITEM_TXT_PRTY_DEFAULT
  #define D4D_LIST_BOX_ITEM_TXT_PRTY_DEFAULT  ( D4D_ALIGN_V_CENTER_MASK | D4D_ALIGN_H_LEFT_MASK )
#endif


typedef sByte D4D_LIST_BOX_INDEX;

// ListBox configuration (goes to ROM by default)
typedef struct 
{
  D4D_TCHAR* pText;
  void* pUser;    
}D4D_LIST_BOX_ITEM;

typedef struct 
{
   D4D_LIST_BOX_INDEX ix;
   D4D_LIST_BOX_INDEX page_ix;
 //  D4D_LIST_BOX_INDEX item_select; 
   const D4D_LIST_BOX_ITEM*  pItems;
} D4D_LIST_BOX_DATA;

typedef struct
{
    D4D_FONT  itemsFontId;
    D4D_LIST_BOX_INDEX posCnt; // Number of visiable position
    D4D_COOR textOff; // text offset between individual item on screen
    D4D_LIST_BOX_DATA* pData;     // ListBox volatile data struct    
    void (*OnEvent)(D4D_OBJECT* pThis, D4D_EVENTID eventId);
} D4D_LIST_BOX;


#define D4D_LIST_BOX_CHILD_SCROLL_BAR_IX 1

/******************************************************************************
* Macros 
******************************************************************************/

// getting the LIST_BOX structure from general OBJECT
#define D4D_GET_LIST_BOX(pObj) ((D4D_LIST_BOX*)((pObj)->pParam))

#define D4D_GET_LIST_BOX_SCROLL_BAR(pObj) ((D4D_OBJECT*)((pObj)->pRelations[D4D_LIST_BOX_CHILD_SCROLL_BAR_IX]))

/*********************************************************
*
* static declaration macros
*
*********************************************************/

    
#define _D4D_DECLARE_LIST_BOX_BEGIN(type, name, x, y, cx, cy, radius, pMargin, pParent, flags, pScheme,\
    itemsFontId, posCnt, listBoxItemsOff, pUser, pOnChange, pOnUsrMsg) \
    extern type D4D_LIST_BOX_ITEM name##_items[];\
    extern type D4D_OBJECT name##_scrollBarVer;\
    static D4D_LIST_BOX_DATA name##_data = { 0, 0, (const D4D_LIST_BOX_ITEM*)&(name##_items[0])};\
    static const D4D_OBJECT * const name##_relations[] = {pParent, &name##_scrollBarVer, NULL};\
    static type D4D_LIST_BOX name##_params = \
    { \
        itemsFontId,        \
        posCnt, \
        listBoxItemsOff, \
        &(name##_data), \
        pOnChange \
    }; \
    \
    D4D_DECLARE_OBJECT(type, name, x, y, cx, cy, radius, pMargin, name##_relations, pOnUsrMsg, &d4d_listBoxSysFunc, &(name##_params), flags, pUser, pScheme)\
    \
    static const D4D_OBJECT* const name##_scrollBarVerRelations[] = {&name, NULL};\
    _D4D_DECLARE_SCROLL_BAR(type, name##_scrollBarVer,  (D4D_COOR)(cx - D4D_LIST_BOX_SCRLBR_WIDTH - 2*D4D_BEVEL_WIDTH), 0, D4D_LIST_BOX_SCRLBR_WIDTH, (D4D_COOR)(cy - 2*D4D_BEVEL_WIDTH),\
     radius, NULL, name##_scrollBarVerRelations, D4D_LIST_BOX_F_SCRLBRS_DEFAULT, pScheme, (void*)&name, D4D_ListBoxScrollBarsFeedBack, NULL) \
    \
    type D4D_LIST_BOX_ITEM name##_items[] = { \
    
#define D4D_DECLARE_LIST_BOX_ITEM(pText, pUser) { pText, (void*)pUser},


#define D4D_DECLARE_LIST_BOX_END() { NULL, NULL} };



#define D4D_DECLARE_LIST_BOX_BEGIN(name, x, y, cx, cy, flags, pParent, pScheme, itemsFontId, posCnt, listBoxItemsOff, pUser, pOnChange, pOnUsrMsg) \
 _D4D_DECLARE_LIST_BOX_BEGIN(D4D_CONST, name, x, y, cx, cy, 0, (D4D_MARGIN*)&listBox_marginDefault, pParent, flags, pScheme,\
    itemsFontId, posCnt, listBoxItemsOff, pUser, pOnChange, pOnUsrMsg)

#define D4D_DECLARE_STD_LIST_BOX_BEGIN(name, x, y, cx, cy, pParent, itemsFontId, pOnChange) \
    D4D_DECLARE_LIST_BOX_BEGIN(name, x, y, cx, cy, (D4D_LIST_BOX_F_DEFAULT), pParent, NULL, itemsFontId, 0, 0, NULL, pOnChange, NULL)

// Rounded listBox definition
#define D4D_DECLARE_RLIST_BOX_BEGIN(name, x, y, cx, cy, radius, flags, pParent, pScheme, itemsFontId, posCnt, listBoxItemsOff, pUser, pOnChange, pOnUsrMsg) \
 _D4D_DECLARE_LIST_BOX_BEGIN(D4D_CONST, name, x, y, cx, cy, radius, (D4D_MARGIN*)&listBox_marginDefault, pParent, flags, pScheme,\
    itemsFontId, posCnt, listBoxItemsOff, pUser, pOnChange, pOnUsrMsg)

#define D4D_DECLARE_STD_RLIST_BOX_BEGIN(name, x, y, cx, cy, radius, pParent, itemsFontId, pOnChange) \
    D4D_DECLARE_RLIST_BOX_BEGIN(name, x, y, cx, cy, radius, (D4D_LIST_BOX_F_DEFAULT), pParent, NULL, itemsFontId, 0, 0, NULL, pOnChange, NULL)


//IN RAM instantions macros
#define D4D_DECLARE_LIST_BOX_BEGIN_INRAM(name, x, y, cx, cy, flags, pParent, pScheme, itemsFontId, posCnt, listBoxItemsOff, pUser, pOnChange, pOnUsrMsg) \
 _D4D_DECLARE_LIST_BOX_BEGIN(D4D_NO_CONST, name, x, y, cx, cy, 0, (D4D_MARGIN*)&listBox_marginDefault, pParent, flags, pScheme,\
    itemsFontId, posCnt, listBoxItemsOff, pUser, pOnChange, pOnUsrMsg)

#define D4D_DECLARE_STD_LIST_BOX_BEGIN_INRAM(name, x, y, cx, cy, pParent, itemsFontId, pOnChange) \
    D4D_DECLARE_LIST_BOX_BEGIN_INRAM(name, x, y, cx, cy, (D4D_LIST_BOX_F_DEFAULT), pParent, NULL, itemsFontId, 0, 0, NULL, pOnChange, NULL)
    

// Rounded listBox definition
#define D4D_DECLARE_RLIST_BOX_BEGIN_INRAM(name, x, y, cx, cy, radius, flags, pParent, pScheme, itemsFontId, posCnt, listBoxItemsOff, pUser, pOnChange, pOnUsrMsg) \
 _D4D_DECLARE_LIST_BOX_BEGIN(D4D_NO_CONST, name, x, y, cx, cy, radius, (D4D_MARGIN*)&listBox_marginDefault, pParent, flags, pScheme,\
    itemsFontId, posCnt, listBoxItemsOff, pUser, pOnChange, pOnUsrMsg)

#define D4D_DECLARE_STD_RLIST_BOX_BEGIN_INRAM(name, x, y, cx, cy, radius, pParent, itemsFontId, pOnChange) \
    D4D_DECLARE_RLIST_BOX_BEGIN_INRAM(name, x, y, cx, cy, radius, (D4D_LIST_BOX_F_DEFAULT), pParent, NULL, itemsFontId, 0, 0, NULL, pOnChange, NULL)
   
   
/******************************************************************************
* Global variables
******************************************************************************/   
extern const D4D_MARGIN listBox_marginDefault;    
   
/******************************************************************************
* Global functions
******************************************************************************/
D4D_LIST_BOX_INDEX D4D_ListBoxGetIndex(D4D_OBJECT* pThis);

void  D4D_ListBoxSetIndex(D4D_OBJECT* pThis, D4D_LIST_BOX_INDEX ix);
D4D_LIST_BOX_INDEX D4D_ListBoxGetItemCount(D4D_OBJECT_PTR pThis);
D4D_LIST_BOX_INDEX D4D_ListBoxFindUserDataItem(D4D_OBJECT_PTR pThis, void* pUser);
void* D4D_ListBoxGetItemUserData(D4D_OBJECT_PTR pThis);
D4D_TCHAR* D4D_ListBoxGetItemText(D4D_OBJECT_PTR pThis);
void D4D_ListBoxEnsureVisible(D4D_OBJECT* pThis);

void D4D_ListBoxChangeItemList(D4D_OBJECT_PTR pThis, const D4D_LIST_BOX_ITEM* pItems);
D4D_LIST_BOX_ITEM* D4D_ListBoxGetItemList(D4D_OBJECT_PTR pThis);
#endif /* __D4D_LIST_BOX_H  */
 



