/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_menu.c
*
* @author    b01119
* 
* @version   0.0.49.0
* 
* @date      Jun-29-2012
* 
* @brief     D4D driver menu object c file 
*
******************************************************************************/

#include "d4d.h"
#include "common_files/d4d_private.h"

//Internal API
void D4D_MenuOnMessage(D4D_MESSAGE* pMsg);
D4D_STRING* D4D_MenuGetTextBuffer(D4D_OBJECT* pThis);

static void D4D_MenuOnInit(D4D_MESSAGE* pMsg);
static D4D_MENU_INDEX D4D_GetItemsCount(D4D_MENU* pMenu);

const D4D_OBJECT_SYS_FUNCTION d4d_menuSysFunc = 
{
  D4D_DEFSTR("Menu"),
  D4D_MenuOnMessage,
  NULL,
  D4D_MenuGetTextBuffer
};

// temporary structure for menu temporary value calculation
typedef struct 
{
    D4D_POINT position;
    D4D_COOR titleBar_y;
    D4D_SIZE maxIcon;
    D4D_MENU_INDEX posCnt;
    D4D_COOR textOff;
    D4D_TCHAR index_txt[8];
    Byte index_txt_len;
    D4D_MENU_INDEX itemsCnt;
} D4D_MENU_TMP_VAL;

#define _calc (*((D4D_MENU_TMP_VAL*)d4d_scratchPad))




/*******************************************************
*
* MENU Helper computing routines
*
*******************************************************/

static D4D_SIZE D4D_GetIconsMaxSize(D4D_MENU *pMenu)
{
  Byte tmpB;
  D4D_SIZE tmpSize =  {0, 0};
  D4D_MENU_INDEX itemsCnt = D4D_GetItemsCount(pMenu);
  
  
  // compute max width of used Icons
  for(tmpB = 0; tmpB < itemsCnt; tmpB++)
  {
    if(pMenu->pItems[tmpB].pIcon != NULL)
    {      
      if(D4D_GetBmpWidth(pMenu->pItems[tmpB].pIcon) > tmpSize.cx)
        tmpSize.cx = D4D_GetBmpWidth(pMenu->pItems[tmpB].pIcon); 
      
      if(D4D_GetBmpHeight(pMenu->pItems[tmpB].pIcon) > tmpSize.cy)
        tmpSize.cy = D4D_GetBmpHeight(pMenu->pItems[tmpB].pIcon); 
    }
  }   
  return tmpSize;
}

static D4D_COOR D4D_GetTitleBarHeight(D4D_OBJECT* pThis)
{
  D4D_MENU* pMenu = D4D_GET_MENU(pThis);
  
  D4D_COOR bar_height = 0;
  
  if(pMenu->title_text.pText != NULL)
    bar_height = D4D_GetFontHeight(pMenu->title_text.fontId);
  
  if(pThis->initFlags & D4D_MENU_F_INDEX)
    if(D4D_GetFontHeight(pMenu->indexFontId) > bar_height)
      bar_height = D4D_GetFontHeight(pMenu->indexFontId);
  
  if(pMenu->pIcon != NULL)
    if(D4D_GetBmpHeight(pMenu->pIcon) > bar_height)
      bar_height = D4D_GetBmpHeight(pMenu->pIcon);
  
  if(bar_height)
  {
    bar_height += 3;
    
    #if D4D_ROUND_CORNER_ENABLE == D4D_TRUE
      bar_height += pThis->radius / 2;
    #endif
  }
  
  return bar_height;
}


static D4D_MENU_INDEX D4D_GetItemsCount(D4D_MENU* pMenu)
{
  D4D_MENU_INDEX i = 0;
  
  while(pMenu->pItems[i].pIcon || pMenu->pItems[i].text.pText)
  {
    i++;
  }
  
  return i;
  
}

static D4D_MENU_INDEX D4D_GetPositionCount(D4D_OBJECT* pThis)
{
 D4D_MENU* pMenu = D4D_GET_MENU(pThis);
 
   if(pMenu->posCnt)
    return pMenu->posCnt;
  
  if(D4D_GetIconsMaxSize(pMenu).cy > (D4D_GetFontHeight(pMenu->itemsFontId)))
    return (D4D_MENU_INDEX)((pThis->size.cy - D4D_GetTitleBarHeight(pThis)) / (D4D_GetIconsMaxSize(pMenu).cy + 3));
  else       
    return (D4D_MENU_INDEX)((pThis->size.cy - D4D_GetTitleBarHeight(pThis)) / (D4D_GetFontHeight(pMenu->itemsFontId) + 3));     
}


static void D4D_MenuValue2Coor(D4D_OBJECT* pThis)
{
  D4D_MENU* pMenu = D4D_GET_MENU(pThis);
  _calc.position = D4D_GetClientToScreenPoint(pThis, &pThis->position);
  _calc.titleBar_y = D4D_GetTitleBarHeight(pThis);
  _calc.maxIcon = D4D_GetIconsMaxSize(pMenu);
  _calc.posCnt = D4D_GetPositionCount(pThis);
  _calc.textOff = pMenu->textOff;
  _calc.itemsCnt = D4D_GetItemsCount(pMenu);
  
  
  // if count of items position or text offsets are not declared , then use automatic values
  if(!_calc.textOff)
  {
    if(_calc.posCnt > 1)
      _calc.textOff = (D4D_COOR)( (pThis->size.cy - _calc.titleBar_y) / (_calc.posCnt));
    else
      _calc.textOff = (D4D_COOR)((pThis->size.cy - _calc.titleBar_y) / 2);
  }
  
  if(pThis->initFlags & D4D_MENU_F_INDEX)
  {      
    
    // Draw index counter
    _calc.index_txt_len = D4D_SprintDecU8((Byte)(pMenu->pData->ix + 1), _calc.index_txt, (D4D_TCHAR) 0);
    _calc.index_txt[_calc.index_txt_len++] = '/';
    _calc.index_txt_len += D4D_SprintDecU8((Byte)_calc.itemsCnt, &_calc.index_txt[_calc.index_txt_len], (D4D_TCHAR) 0);
    _calc.index_txt[_calc.index_txt_len] = 0;

  }

  
}

/*******************************************************
*
* MENU Drawing routine
*
*******************************************************/

static void D4D_MenuOnDraw(D4D_MESSAGE* pMsg)
{
    D4D_OBJECT* pThis = pMsg->pObject;
    D4D_MENU* pMenu = D4D_GET_MENU(pThis);
    D4D_OBJECT_DRAWFLAGS draw = pMsg->prm.draw;
    D4D_COLOR clrT, clrB;     
    D4D_POINT tmp_point;
    D4D_SIZE tmp_size;
    D4D_STRING tmp_txtbuff;
    D4D_STR_PROPERTIES tmp_str_prty;
    Byte tmpB;
    
    tmp_txtbuff.str_properties = &tmp_str_prty;    
    tmp_txtbuff.printOff = 0;
    // Get background simple color 
    clrT = D4D_ObjectGetForeFillColor(pThis);
    clrB = D4D_ObjectGetBckgFillColor(pThis);
    
    #ifdef D4D_DEBUG
      // sanity check
      D4D_ASSERT(sizeof(_calc) <= D4D_SCRATCHPAD_SIZE);
    #endif
    
    // Compute all tempoarary values
    D4D_MenuValue2Coor(pThis);

#if D4D_ROUND_CORNER_ENABLE == D4D_TRUE
  
    // draw just a color rectangle instead of bitmap
    if(draw & D4D_OBJECT_DRAWFLAGS_COMPLETE)  
    {      
      if(pThis->initFlags & D4D_OBJECT_F_FOCUSRECT)
        D4D_RBox(&_calc.position, &pThis->size, D4D_LINE_THIN, clrT, clrB, pThis->radius);
      else
        D4D_FillRRect(&_calc.position, &pThis->size, clrB, pThis->radius);
      
      if(pMenu->pIcon != NULL)
      {
        tmp_point.x = (D4D_COOR)(_calc.position.x + 3);
        tmp_point.y =  (D4D_COOR)(_calc.position.y + ((_calc.titleBar_y - D4D_GetBmpHeight(pMenu->pIcon)) / 2));
 
        D4D_DrawRBmp(&tmp_point, pMenu->pIcon, (D4D_BOOL)!D4D_IsEnabled(pThis), pThis->radius);
      }
      
      if(pMenu->title_text.pText)
      {                 
        tmp_point.x = (D4D_COOR)(_calc.position.x + 4 + pThis->radius/2);
        if(pMenu->pIcon != NULL)
          tmp_point.x += D4D_GetBmpHeight(pMenu->pIcon);
        
        tmp_point.y = (D4D_COOR)(_calc.position.y + 1 ); 
               
        tmp_size.cx = D4D_GetTextWidth(pMenu->title_text.fontId, pMenu->title_text.pText);
        tmp_size.cy = D4D_GetFontHeight(pMenu->title_text.fontId);
        
        D4D_DrawTextRect(&tmp_point, &tmp_size, &pMenu->title_text, clrT, clrB);        
      }
      
      if(_calc.titleBar_y)
      {
        D4D_MoveToXY((D4D_COOR)(_calc.position.x + 1),\
          (D4D_COOR)(_calc.position.y + _calc.titleBar_y));    
          
        D4D_LineToXY((D4D_COOR)(_calc.position.x + pThis->size.cx - 1),\
          (D4D_COOR)(_calc.position.y + _calc.titleBar_y), D4D_LINE_THIN, clrT);                
      }
    
    }
    
    if(pThis->initFlags & D4D_MENU_F_INDEX)
    {      
      // Draw index counter
      tmp_txtbuff.printLen = (D4D_INDEX)((_calc.itemsCnt < 10)? 3:5);
      tmp_size.cx = (D4D_COOR)(tmp_txtbuff.printLen * D4D_GetFontWidth(pMenu->indexFontId) + 2);
      tmp_size.cy = (D4D_COOR)(_calc.titleBar_y - 2 - pThis->radius / 2);
      tmp_point.x = (D4D_COOR)(_calc.position.x + pThis->size.cx - tmp_size.cx - 2 - pThis->radius / 2);
      tmp_point.y = (D4D_COOR)(_calc.position.y + 1 + pThis->radius / 2);
      
      tmp_txtbuff.pText = _calc.index_txt;
      tmp_txtbuff.fontId = pMenu->indexFontId;
      tmp_str_prty.font_properties = D4D_MENU_IX_FNT_PRTY_DEFAULT;
      tmp_str_prty.text_properties = D4D_MENU_IX_TXT_PRTY_DEFAULT;
      D4D_DrawTextRect(&tmp_point, &tmp_size, &tmp_txtbuff, clrT, clrB);
    }
    
    for(tmpB = 0; (tmpB < _calc.posCnt) && (tmpB < _calc.itemsCnt); tmpB++)
    {
      if(((pMenu->pData->ix - pMenu->pData->focus_pos) != tmpB) || (!D4D_IsEnabled(pThis)))
      {        
        clrT = D4D_ObjectGetForeFillColor(pThis);
        clrB = D4D_ObjectGetBckgFillColor(pThis);        
      }
      else
      {        
        D4D_CLR_SCHEME *pScheme_tmp = D4D_ObjectGetScheme(pThis);
                                  
        if(pMenu->pData->ix == pMenu->pData->item_select)
          clrT = pScheme_tmp->foreCapture;
        else
          clrT = pScheme_tmp->foreFocus;
       
        clrB = pScheme_tmp->bckgFocus;
  
      }
      
      // Draw menu item text
      if(pMenu->pItems[tmpB + pMenu->pData->focus_pos].text.pText != NULL)
      {
        tmp_point.x = (D4D_COOR)(_calc.position.x + _calc.maxIcon.cx + 4 + pThis->radius / 2);
        tmp_point.y = (D4D_COOR)(_calc.position.y + _calc.titleBar_y + 1 + (tmpB * _calc.textOff));
        
        tmp_size.cx = (D4D_COOR)(pThis->size.cx - (_calc.maxIcon.cx + 6 + pThis->radius / 2 + ((pMenu->pScrlBrVer->pData->flags & D4D_OBJECT_F_VISIBLE)? D4D_MENU_SCRLBR_WIDTH:0)));
        tmp_size.cy = D4D_GetFontHeight(pMenu->itemsFontId); // to invoke autosize capability in axis Y
        
        tmp_txtbuff.pText = pMenu->pItems[tmpB + pMenu->pData->focus_pos].text.pText;
        tmp_txtbuff.fontId = pMenu->itemsFontId;
        tmp_txtbuff.buffSize = 0;
        tmp_txtbuff.printLen = D4D_GetTextLength(tmp_txtbuff.pText);
        tmp_str_prty.font_properties = D4D_MENU_ITEM_FNT_PRTY_DEFAULT;
        tmp_str_prty.text_properties = D4D_MENU_ITEM_TXT_PRTY_DEFAULT;
        
        
        D4D_DrawTextRect(&tmp_point, &tmp_size, &tmp_txtbuff, clrT, clrB);        
      }
      
      // Draw menu item icon  
      if(_calc.maxIcon.cx)
      {        
        tmp_point.x = (D4D_COOR)(_calc.position.x + 3);
        
        if(_calc.maxIcon.cy < _calc.textOff)        
          tmp_point.y = (D4D_COOR)(_calc.position.y + _calc.titleBar_y + 3 + (tmpB * _calc.textOff) + ((D4D_GetFontHeight(pMenu->itemsFontId) - _calc.maxIcon.cy) / 2));        
        else
          tmp_point.y = (D4D_COOR)(_calc.position.y + _calc.titleBar_y + 3 + (tmpB * _calc.textOff));
        
        if(pMenu->pItems[tmpB + pMenu->pData->focus_pos].pIcon != NULL)
          D4D_DrawRBmp(&tmp_point, pMenu->pItems[tmpB + pMenu->pData->focus_pos].pIcon, (D4D_BOOL)!D4D_IsEnabled(pThis), pThis->radius);
        else
          D4D_FillRRect(&tmp_point, &_calc.maxIcon, D4D_ObjectGetBckgFillColor(pThis), pThis->radius);
      }
    }

#else
    // draw just a color rectangle instead of bitmap
    if(draw & D4D_OBJECT_DRAWFLAGS_COMPLETE)  
    {      
      if(pThis->initFlags & D4D_OBJECT_F_FOCUSRECT)
        D4D_Box(&_calc.position, &pThis->size, D4D_LINE_THIN, clrT, clrB);
      else
        D4D_FillRect(&_calc.position, &pThis->size, clrB);
      
      if(pMenu->pIcon != NULL)
      {
        tmp_point.x = (D4D_COOR)(_calc.position.x + 3);
        tmp_point.y =  (D4D_COOR)(_calc.position.y + ((_calc.titleBar_y - D4D_GetBmpHeight(pMenu->pIcon)) / 2));
 
        D4D_DrawBmp(&tmp_point, pMenu->pIcon, (D4D_BOOL)!D4D_IsEnabled(pThis));
      }
      
      if(pMenu->title_text.pText)
      {                 
        tmp_point.x = (D4D_COOR)(_calc.position.x + 4);
        if(pMenu->pIcon != NULL)
          tmp_point.x += D4D_GetBmpHeight(pMenu->pIcon);
        
        tmp_point.y = (D4D_COOR)(_calc.position.y + 1 ); 
        
        tmp_size.cx = D4D_GetTextWidth(pMenu->title_text.fontId, pMenu->title_text.pText);
        tmp_size.cy = D4D_GetFontHeight(pMenu->title_text.fontId);
        
        D4D_DrawTextRect(&tmp_point, &tmp_size, &pMenu->title_text, clrT, clrB);         
      }
      
      if(_calc.titleBar_y)
      {
        D4D_MoveToXY((D4D_COOR)(_calc.position.x + 1),\
          (D4D_COOR)(_calc.position.y + _calc.titleBar_y));    
          
        D4D_LineToXY((D4D_COOR)(_calc.position.x + pThis->size.cx - 1),\
          (D4D_COOR)(_calc.position.y + _calc.titleBar_y), D4D_LINE_THIN, clrT);                
      }
    
    }
    
    if(pThis->initFlags & D4D_MENU_F_INDEX)
    {      
      // Draw index counter
      tmp_txtbuff.printLen = (D4D_INDEX)((_calc.itemsCnt < 10)? 3:5);
      tmp_txtbuff.buffSize = 0;
      tmp_size.cx = (D4D_COOR)(tmp_txtbuff.printLen * D4D_GetFontWidth(pMenu->indexFontId) + 2);
      tmp_size.cy = (D4D_COOR)(_calc.titleBar_y - 2);
      tmp_point.x = (D4D_COOR)(_calc.position.x + pThis->size.cx - tmp_size.cx - 2);
      tmp_point.y = (D4D_COOR)(_calc.position.y + 1);
      
      tmp_txtbuff.pText = _calc.index_txt;
      tmp_txtbuff.fontId = pMenu->indexFontId;
      tmp_str_prty.font_properties = D4D_MENU_IX_FNT_PRTY_DEFAULT;
      tmp_str_prty.text_properties = D4D_MENU_IX_TXT_PRTY_DEFAULT;
      D4D_DrawTextRect(&tmp_point, &tmp_size, &tmp_txtbuff, clrT, clrB);
    }
    
    for(tmpB = 0; (tmpB < _calc.posCnt) && (tmpB < _calc.itemsCnt); tmpB++)
    {
      if(((pMenu->pData->ix - pMenu->pData->focus_pos) != tmpB) || (!D4D_IsEnabled(pThis)))
      {        
        clrT = D4D_ObjectGetForeFillColor(pThis);
        clrB = D4D_ObjectGetBckgFillColor(pThis);        
      }
      else
      {        
        D4D_CLR_SCHEME *pScheme_tmp = D4D_ObjectGetScheme(pThis);
                                  
        if(pMenu->pData->ix == pMenu->pData->item_select)
          clrT = pScheme_tmp->foreCapture;
        else
          clrT = pScheme_tmp->foreFocus;
       
        clrB = pScheme_tmp->bckgFocus;
  
      }
      
      // Draw menu item text
      if(pMenu->pItems[tmpB + pMenu->pData->focus_pos].text.pText != NULL)
      {
        tmp_point.x = (D4D_COOR)(_calc.position.x + _calc.maxIcon.cx + 4);
        tmp_point.y = (D4D_COOR)(_calc.position.y + _calc.titleBar_y + 1 + (tmpB * _calc.textOff));
        
        tmp_size.cx = (D4D_COOR)(pThis->size.cx - (_calc.maxIcon.cx + 6 + ((pMenu->pScrlBrVer->pData->flags & D4D_OBJECT_F_VISIBLE)? D4D_MENU_SCRLBR_WIDTH:0)));
        tmp_size.cy = D4D_GetFontHeight(pMenu->itemsFontId); // to invoke autosize capability in axis Y
        
        tmp_txtbuff.pText = pMenu->pItems[tmpB + pMenu->pData->focus_pos].text.pText;
        tmp_txtbuff.fontId = pMenu->itemsFontId;
        tmp_txtbuff.buffSize = 0;
        tmp_txtbuff.printLen = D4D_GetTextLength(tmp_txtbuff.pText);
        tmp_str_prty.font_properties = D4D_MENU_ITEM_FNT_PRTY_DEFAULT;
        tmp_str_prty.text_properties = D4D_MENU_ITEM_TXT_PRTY_DEFAULT;
        
        
        D4D_DrawTextRect(&tmp_point, &tmp_size, &tmp_txtbuff, clrT, clrB);        
      }
      
      // Draw menu item icon  
      if(_calc.maxIcon.cx)
      {        
        tmp_point.x = (D4D_COOR)(_calc.position.x + 3);
        
        if(_calc.maxIcon.cy < _calc.textOff)        
          tmp_point.y = (D4D_COOR)(_calc.position.y + _calc.titleBar_y + 3 + (tmpB * _calc.textOff) + ((D4D_GetFontHeight(pMenu->itemsFontId) - _calc.maxIcon.cy) / 2));        
        else
          tmp_point.y = (D4D_COOR)(_calc.position.y + _calc.titleBar_y + 3 + (tmpB * _calc.textOff));
        
        if(pMenu->pItems[tmpB + pMenu->pData->focus_pos].pIcon != NULL)
          D4D_DrawBmp(&tmp_point, pMenu->pItems[tmpB + pMenu->pData->focus_pos].pIcon, (D4D_BOOL)!D4D_IsEnabled(pThis));
        else
          D4D_FillRect(&tmp_point, &_calc.maxIcon, D4D_ObjectGetBckgFillColor(pThis));
      }
    }  


#endif    
      
}

/*******************************************************
*
* MENU object focus next item routine
*
*******************************************************/
static void D4D_MenuFocusNextItem(D4D_OBJECT* pThis)
{
  D4D_MENU* pMenu = D4D_GET_MENU(pThis);
  D4D_MENU_INDEX tmpPosCnt = D4D_GetPositionCount(pThis);
  D4D_MENU_INDEX tmpItemsCnt = D4D_GetItemsCount(pMenu);
  
  pMenu->pData->item_select = -1;
  pMenu->pData->ix++;
  
  if(tmpItemsCnt >= tmpPosCnt)
  {
    if(pMenu->pData->ix >= tmpItemsCnt)
    {
      pMenu->pData->ix = 0;
      pMenu->pData->focus_pos = 0;
      pMenu->pData->item_select = -2;
    }else if(pMenu->pData->ix - pMenu->pData->focus_pos > (tmpPosCnt - 1))
    {
       pMenu->pData->focus_pos++;
       pMenu->pData->item_select = -2;        
    }
    D4D_ScrlBrSetPosition(pMenu->pScrlBrVer, pMenu->pData->focus_pos);
  } else
  {
    if(pMenu->pData->ix >= tmpItemsCnt)
    {
      pMenu->pData->ix = 0;        
    }
    
  }  
}

/*******************************************************
*
* MENU object focus previous item routine
*
*******************************************************/
static void D4D_MenuFocusPreviousItem(D4D_OBJECT* pThis)
{
  D4D_MENU* pMenu = D4D_GET_MENU(pThis);
  D4D_MENU_INDEX tmpPosCnt = D4D_GetPositionCount(pThis);
  D4D_MENU_INDEX tmpItemsCnt = D4D_GetItemsCount(pMenu);
  
  pMenu->pData->item_select = -1;
  pMenu->pData->ix--;
  
  if(tmpItemsCnt >= tmpPosCnt)
  {
   if(pMenu->pData->ix < 0)
   {
     pMenu->pData->ix = (D4D_MENU_INDEX)(tmpItemsCnt - 1);
     pMenu->pData->focus_pos = (D4D_MENU_INDEX)(tmpItemsCnt - tmpPosCnt);
     pMenu->pData->item_select = -2;
     
   }else if(pMenu->pData->ix - pMenu->pData->focus_pos < 0)
   {
      pMenu->pData->focus_pos--;
      pMenu->pData->item_select = -2;      
   }
   
   D4D_ScrlBrSetPosition(pMenu->pScrlBrVer, pMenu->pData->focus_pos);
  } else
  {
    if(pMenu->pData->ix < 0)
    {          
      pMenu->pData->ix = (D4D_MENU_INDEX)(tmpItemsCnt - 1);
    }
  }  
}


/*******************************************************
*
* MENU key handling routine
*
*******************************************************/

static void D4D_MenuOnKeyDown(D4D_MESSAGE* pMsg)    
{
    D4D_MENU* pMenu = D4D_GET_MENU(pMsg->pObject);   
    
  if(D4D_GetCapturedObject() == pMsg->pObject)
  {
    if(pMsg->prm.key == D4D_KEY_SCANCODE_DOWN)
    {      
      D4D_MenuFocusNextItem(pMsg->pObject);
      D4D_InvalidateObject(pMsg->pObject, D4D_FALSE);

    } else if(pMsg->prm.key == D4D_KEY_SCANCODE_UP)
    {      
      D4D_MenuFocusPreviousItem(pMsg->pObject);
      D4D_InvalidateObject(pMsg->pObject, D4D_FALSE);
        
    } else if(pMsg->prm.key == D4D_KEY_SCANCODE_ENTER)
    {    
      pMenu->pData->item_select = pMenu->pData->ix;
      D4D_InvalidateObject(pMsg->pObject, D4D_FALSE);
    }
  }
}

/*******************************************************
*
* MENU key handling routine
*
*******************************************************/

static void D4D_MenuOnKeyUp(D4D_MESSAGE* pMsg)    
{
    D4D_MENU* pMenu = D4D_GET_MENU(pMsg->pObject);

    if(pMsg->prm.key == D4D_KEY_SCANCODE_ENTER)
    {
      if(D4D_GetCapturedObject() != pMsg->pObject)
      {
        D4D_CaptureKeys(pMsg->pObject);
      }
      else if(pMenu->pData->item_select >= 0)
      {
        if(pMenu->OnClicked)
          pMenu->OnClicked(pMsg->pObject, pMenu->pData->item_select);
        
        pMenu->pData->item_select = -1;
        D4D_InvalidateObject(pMsg->pObject, D4D_FALSE);
      }
    }
    
    if(pMsg->prm.key == D4D_KEY_SCANCODE_ESC)
    {
      D4D_CaptureKeys(NULL);  
    }  
}

/*******************************************************
*
* MENU touch screen handling routine
*
*******************************************************/
#ifdef D4D_LLD_TCH
static void D4D_MenuOnTouch(D4D_MESSAGE* pMsg)
{
  D4D_OBJECT* pThis = pMsg->pObject;
  D4D_MENU* pMenu = D4D_GET_MENU(pThis);
  
  if(D4D_GetFocusedObject(pMsg->pScreen) != pThis)
  {    
    if(pMsg->nMsgId == D4D_MSG_TOUCHED)
      D4D_FocusSet(pMsg->pScreen, pThis);
  }else
  {
    
    D4D_MenuValue2Coor(pThis);
    
    if(D4D_GetTouchScreenCoordinates(pThis).x < (pThis->position.x + pThis->size.cx - ((pMenu->pScrlBrVer->pData->flags & D4D_OBJECT_F_VISIBLE)? D4D_MENU_SCRLBR_WIDTH:0)))      
    {
      // Select Menu Item
      if(pMsg->nMsgId == D4D_MSG_TOUCHED)
      {
        D4D_COOR tmp_y;
        Byte tmpB;
        
        tmp_y = (D4D_COOR)(D4D_GetTouchScreenCoordinates(pThis).y - pThis->position.y);
        
        if(tmp_y > _calc.titleBar_y)
        {
          D4D_MENU_INDEX tmpItemsCnt = D4D_GetItemsCount(pMenu);
          tmp_y -= _calc.titleBar_y;
          // now is in tmp_y offset of y from title bar 
          
          for(tmpB = 0; (tmpB < _calc.posCnt) && (tmpB < tmpItemsCnt); tmpB++)
          {
            if(tmp_y < _calc.textOff)
            {
              // founded touched line   
              if((pMenu->pData->ix - pMenu->pData->focus_pos) != tmpB)
              {
                // Touched line is not focused  
                pMenu->pData->ix = (D4D_MENU_INDEX)(pMenu->pData->focus_pos + tmpB);
              }else
              {
                // Touched line is focused
                pMenu->pData->ix = (D4D_MENU_INDEX)(pMenu->pData->focus_pos + tmpB);
                
                if(pMenu->OnClicked)
                  pMenu->OnClicked(pThis, pMenu->pData->ix);
                
                pMenu->pData->item_select = -1;
                  
              }
              break;
            }
            tmp_y -= _calc.textOff;
          }
          D4D_InvalidateObject(pThis, D4D_FALSE);    
        }
      }          
    } 
  }  
}
#endif

static D4D_BOOL D4D_MenuScrollBarSetup(D4D_OBJECT* pObject)
{
  D4D_MENU* pMenu = D4D_GET_MENU(pObject);
  
  D4D_MENU_INDEX tmpItemsCnt = D4D_GetItemsCount(pMenu);
  
  
  // Try to check if the Menu needs scroll Bar
  D4D_MenuValue2Coor(pObject);
  
  // Switch off possible scroll bar in refresh case
  pMenu->pScrlBrVer->pData->flags &= ~D4D_OBJECT_F_VISIBLE;
   
  if((D4D_GetPositionCount(pObject) < tmpItemsCnt) || (pObject->initFlags & D4D_MENU_F_SIDEBAR))
  {
    D4D_OBJECT* pScrlBr;
    pScrlBr = pMenu->pScrlBrVer;
    
    pScrlBr->pData->flags |= D4D_OBJECT_F_VISIBLE;
    
    // Init child objects coordinations
    
  
    pScrlBr->position.x = (D4D_COOR)(_calc.position.x + pObject->size.cx  - (D4D_MENU_SCRLBR_WIDTH + 2));
    pScrlBr->position.y = (D4D_COOR)(_calc.position.y + _calc.titleBar_y + 1);
    
    pScrlBr->position = D4D_GetScreenToClientPoint( pObject, &pScrlBr->position);
    
    pScrlBr->size.cx = D4D_MENU_SCRLBR_WIDTH + 1;
    pScrlBr->size.cy = (D4D_COOR)(pObject->size.cy - _calc.titleBar_y - 2);

    pScrlBr->radius = D4D_LimitU(pObject->radius, 0, D4D_MENU_SCRLBR_WIDTH / 2);
    
    if(tmpItemsCnt < D4D_GetPositionCount(pObject))  
      D4D_ScrlBrSetRange(pMenu->pScrlBrVer, 0, D4D_GetPositionCount(pObject));
    else  
      D4D_ScrlBrSetRange(pMenu->pScrlBrVer, 0, tmpItemsCnt);
    D4D_ScrlBrSetStep(pMenu->pScrlBrVer, D4D_GetPositionCount(pObject), D4D_MENU_SCRLBR_STEP);
    D4D_ScrlBrSetPosition(pMenu->pScrlBrVer, 0);
    return D4D_TRUE;
  }
  return D4D_FALSE;
}

static void D4D_MenuOnInit(D4D_MESSAGE* pMsg)
{
  D4D_MENU* pMenu = D4D_GET_MENU(pMsg->pObject);

  // init screen pointer of child objects
  pMenu->pScrlBrVer->pData->pScreen = pMsg->pObject->pData->pScreen;

  (void)D4D_MenuScrollBarSetup(pMsg->pObject);

  // set flag to redraw screen
  D4D_InvalidateObject(pMsg->pObject, D4D_TRUE);
}

void D4D_MenuScrollBarsFeedBack(D4D_OBJECT* pThis, D4D_INDEX old_position, D4D_INDEX new_position)
{
  D4D_MENU* pMenu = D4D_GET_MENU((D4D_OBJECT*)(pThis->userData));

  D4D_UNUSED(old_position);

  pMenu->pData->focus_pos = (D4D_MENU_INDEX)new_position;
  pMenu->pData->ix = (D4D_MENU_INDEX)new_position;
  pMenu->pData->item_select = -2;
  
  D4D_InvalidateObject((D4D_OBJECT*)(pThis->userData), D4D_FALSE);
}

static void SendLocalMessage(D4D_MESSAGE* pMsg, D4D_OBJECT* pObj)
{
  D4D_MESSAGE tmpMsg = *pMsg;

  // don't send the OnDraw/done messages if the objects are invisible
  if(!(pObj->pData->flags & D4D_OBJECT_F_VISIBLE) && ((pMsg->nMsgId == D4D_MSG_DRAW) || (pMsg->nMsgId == D4D_MSG_DRAWDONE)))
    return;
  
#ifdef D4D_LLD_TCH 
  
  // don't send the touch scren messages if the objects are untoucheble
  if((pMsg->nMsgId == D4D_MSG_TOUCHED) || (pMsg->nMsgId == D4D_MSG_TOUCH_AUTO) || (pMsg->nMsgId == D4D_MSG_UNTOUCHED))
  {  
    
    if(!(pMsg->pObject->pData->flags & D4D_OBJECT_F_TOUCHENABLE))
      return;
    
    if(!D4D_ObjectCheckCoordinates(pObj, D4D_GetTouchScreenCoordinates(pObj)))
      return;
    
  }
#endif
 
  tmpMsg.pObject = pObj;
 
  if(pObj->pObjFunc->OnSysMessage)
    pObj->pObjFunc->OnSysMessage(&tmpMsg);  
}


/*******************************************************
*
* The MENU Get index function 
*
*******************************************************/

D4D_MENU_INDEX D4D_MenuGetIndex(D4D_OBJECT* pThis)
{
  D4D_MENU* pMenu = D4D_GET_MENU(pThis);
  
  if(pThis == NULL)
    return 0;
  
  return pMenu->pData->ix;
  
}

/*******************************************************
*
* The MENU Set index function 
*
*******************************************************/

void  D4D_MenuSetIndex(D4D_OBJECT* pThis, D4D_MENU_INDEX ix)
{
  D4D_MENU* pMenu = D4D_GET_MENU(pThis);
  
  if(pThis == NULL)
    return;
  
  if(ix > D4D_GetItemsCount(pMenu))
    ix = D4D_GetItemsCount(pMenu);
  
  pMenu->pData->ix = ix;
  pMenu->pData->focus_pos = ix;
  pMenu->pData->item_select = -2;
  
  D4D_ScrlBrSetPosition(pMenu->pScrlBrVer, pMenu->pData->focus_pos);
  
}

/*******************************************************
*
* Get MENU item count
*
*******************************************************/

D4D_MENU_INDEX D4D_MenuGetItemCount(D4D_OBJECT_PTR pThis)
{
   D4D_MENU* pMenu = D4D_GET_MENU(pThis);
   
   return D4D_GetItemsCount(pMenu);
}

/*******************************************************
*
* The MENU find Item by user data function 
*
*******************************************************/
D4D_MENU_INDEX D4D_MenuFindUserDataItem(D4D_OBJECT_PTR pThis, void* pUser)
{
    D4D_MENU* pMenu = D4D_GET_MENU(pThis);
    D4D_MENU_INDEX i_max = D4D_GetItemsCount(pMenu);
    D4D_MENU_INDEX i;
    
    for(i=0;i< i_max;i++)
    {
      if(pMenu->pItems[i].pUser == pUser)
        return i;
    }
    
    return -1;     
}

/*******************************************************
*
* The MENU get Item user data function 
*
*******************************************************/
void* D4D_MenuGetItemUserData(D4D_OBJECT_PTR pThis)
{
  D4D_MENU* pMenu = D4D_GET_MENU(pThis);
  return pMenu->pItems[pMenu->pData->ix].pUser;
}

/*******************************************************
*
* The MENU get Item Text function 
*
*******************************************************/
D4D_STRING* D4D_MenuGetItemText(D4D_OBJECT_PTR pThis)
{
  D4D_MENU* pMenu = D4D_GET_MENU(pThis);
  return (D4D_STRING*)&(pMenu->pItems[pMenu->pData->ix].text);
}
/**************************************************************//*!
*
*
*
*     THE INTERNAL API FUNCTIONS
*
*
*
******************************************************************/

/*******************************************************
*
* The MENU message handler 
*
*******************************************************/

void D4D_MenuOnMessage(D4D_MESSAGE* pMsg)
{      
  switch(pMsg->nMsgId)
  {
    case D4D_MSG_DRAW:
      D4D_MenuOnDraw(pMsg);
      break;
      
    case D4D_MSG_KEYDOWN:
      D4D_MenuOnKeyDown(pMsg);
      break;
    
    case D4D_MSG_KEYUP:
      D4D_MenuOnKeyUp(pMsg);
      break;        

#ifdef D4D_LLD_TCH       
    case D4D_MSG_TOUCHED:
    case D4D_MSG_TOUCH_AUTO:
        D4D_MenuOnTouch(pMsg);         
      break;
#endif      
    case D4D_MSG_ONINIT:
      pMsg->pObject->pData->flags &= ~D4D_OBJECT_F_NOTINIT;
      
      D4D_MenuOnInit(pMsg);
      break;

    default:
      // call the default behavior of all objects
      D4D_ObjOnMessage(pMsg);
  }
    
  // send the message to child object
  // Vertical scroll bar
  SendLocalMessage(pMsg, (D4D_OBJECT*)(D4D_GET_MENU(pMsg->pObject))->pScrlBrVer);
}

/**************************************************************//*!
*
* Get object text buffer
*
******************************************************************/
D4D_STRING* D4D_MenuGetTextBuffer(D4D_OBJECT* pThis)
{
  return &(D4D_GET_MENU(pThis)->title_text);  
}