/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_picture.c
*
* @author    r50233
* 
* @version   0.0.21.0
* 
* @date      Jun-29-2012
* 
* @brief     D4D driver picture object c file 
*
******************************************************************************/

#include "d4d.h"
#include "common_files/d4d_private.h"

//Internal API
void D4D_PictOnMessage(D4D_MESSAGE* pMsg);
D4D_STRING* D4D_PictGetTextBuffer(D4D_OBJECT* pThis);

const D4D_OBJECT_SYS_FUNCTION d4d_pictSysFunc = 
{
  D4D_DEFSTR("Picture"),
  D4D_PictOnMessage,
  NULL,
  NULL
};

/*******************************************************
*
* PICTURE Drawing routine
*
*******************************************************/

static void D4D_PictOnDraw(D4D_MESSAGE* pMsg)
{
    D4D_OBJECT* pThis = pMsg->pObject;
    D4D_PICTURE* pPict = D4D_GET_PICTURE(pThis);
    
    // only draw if complete draw is needed
    if(!(pMsg->prm.draw & D4D_OBJECT_DRAWFLAGS_COMPLETE))
        return;
    
    // draw the bitmap
    if(pPict->pBmp != NULL)
    {
      D4D_POINT tmp_point = D4D_GetClientToScreenPoint(pMsg->pObject, &pThis->position);  
      if(pThis->size.cx && pThis->size.cy)
      {
        D4D_COLOR clrB = D4D_ObjectGetBckgFillColor(pThis); 
        
        D4D_DrawRBmpRect(&tmp_point, &pThis->size, pPict->pBmp, (D4D_PIC_PRTY_DEFAULT), clrB, (D4D_BOOL)!D4D_IsEnabled(pThis), pThis->radius);
      }
      else
        D4D_DrawRBmp(&tmp_point, pPict->pBmp, (D4D_BOOL)!D4D_IsEnabled(pThis), pThis->radius);
    }
}

/**************************************************************//*!
*
*
*
*     THE INTERNAL API FUNCTIONS
*
*
*
******************************************************************/

/*******************************************************
*
* The main PICTURE message handler 
*
*******************************************************/

void D4D_PictOnMessage(D4D_MESSAGE* pMsg)
{
    switch(pMsg->nMsgId)
    {
    case D4D_MSG_DRAW:
        D4D_PictOnDraw(pMsg);
        break;
        
    default:
        // call the default behavior
        D4D_ObjOnMessage(pMsg);
    }
}



