/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_radio_button.c
*
* @author    b01119
* 
* @version   0.0.6.0
* 
* @date      Jun-29-2012
* 
* @brief     D4D driver radio button object c file 
*
******************************************************************************/

#include "d4d.h"
#include "common_files/d4d_private.h"

//Internal API
void D4D_RadioButtonOnMessage(D4D_MESSAGE* pMsg);
D4D_STRING* D4D_RadioButtonGetTextBuffer(D4D_OBJECT* pThis);

static void D4D_RadioButtonChangeValue(D4D_OBJECT_PTR pThis, D4D_BOOL value);

const D4D_OBJECT_SYS_FUNCTION d4d_radioButtonSysFunc = 
{
  D4D_DEFSTR("Radio Button"),
  D4D_RadioButtonOnMessage,
  NULL,
  D4D_RadioButtonGetTextBuffer
};

typedef struct 
{
    D4D_POINT Position;
    D4D_POINT TxtPos;
    D4D_SIZE TxtSize;
    D4D_POINT BmpPos;
    D4D_SIZE  SizeBmpEnabled;
    D4D_POINT iconCentre;
    D4D_COOR iconRadius;    
} D4D_RADIOBUTTON_TMP_VAL;

#define _calc (*((D4D_RADIOBUTTON_TMP_VAL*)d4d_scratchPad))

    
static void D4D_RadioButtonValue2Coor(D4D_OBJECT* pThis)
{
  D4D_RADIOBUTTON* pRadioBtn = D4D_GET_RADIOBUTTON(pThis);
  D4D_COOR tmp_BorderOffset;
  _calc.Position = D4D_GetClientToScreenPoint(pThis, &pThis->position);
  
  if(!pRadioBtn->pBmpChecked)
  {
    _calc.SizeBmpEnabled.cx = D4D_RADIOBUTTON_MIN_SIZE;
    _calc.SizeBmpEnabled.cy = D4D_RADIOBUTTON_MIN_SIZE;  
  }else
    _calc.SizeBmpEnabled = D4D_GetBmpSize(pRadioBtn->pBmpChecked);

    tmp_BorderOffset = (D4D_COOR)((pThis->size.cy - _calc.SizeBmpEnabled.cy) / 2);
    
    _calc.BmpPos.x = (D4D_COOR)(_calc.Position.x + tmp_BorderOffset);
    _calc.BmpPos.y = (D4D_COOR)(_calc.Position.y + tmp_BorderOffset);

    if(pRadioBtn->textBuff.pText && pRadioBtn->textBuff.buffSize)
    {
      _calc.TxtPos.x = (D4D_COOR) (_calc.BmpPos.x + _calc.SizeBmpEnabled.cx + D4D_RADIOBUTTON_TEXT_OFFSET);      
      _calc.TxtPos.y = (D4D_COOR) (_calc.Position.y + 1);
      _calc.TxtSize.cx = (D4D_COOR)(pThis->size.cx - (_calc.TxtPos.x - _calc.Position.x + 1));      
      _calc.TxtSize.cy = (D4D_COOR) (pThis->size.cy - 2);
      #if D4D_ROUND_CORNER_ENABLE != D4D_FALSE
        _calc.TxtSize.cx -= pThis->radius;
      #endif
      
    }
    

  
  _calc.iconRadius = (D4D_COOR)(D4D_GetLongerSide(&_calc.SizeBmpEnabled) / 2);    
  _calc.iconCentre.x = (D4D_COOR)(_calc.BmpPos.x + _calc.SizeBmpEnabled.cx / 2);
  _calc.iconCentre.y = (D4D_COOR)(_calc.Position.y + pThis->size.cy / 2);
           
  

}


/*******************************************************
*
* RADIO BUTTON Drawing routine
*
*******************************************************/

static void D4D_RadioButtonOnDraw(D4D_MESSAGE* pMsg)
{
    D4D_OBJECT* pThis = pMsg->pObject; 
    D4D_RADIOBUTTON* pRadioBtn = D4D_GET_RADIOBUTTON(pThis);
    D4D_RADIOBUTTON_STATUS* pStatus = D4D_GET_RADIOBUTTON_STATUS(pThis);    
    D4D_OBJECT_DRAWFLAGS draw = pMsg->prm.draw;    
    D4D_COLOR clrT, clrB;
    
    D4D_RadioButtonValue2Coor(pThis);
    
    clrT = D4D_ObjectGetForeColor(pThis, draw);
    clrB = D4D_ObjectGetBckgFillColor(pThis);

#if D4D_ROUND_CORNER_ENABLE != D4D_FALSE
    

    if(draw & D4D_OBJECT_DRAWFLAGS_COMPLETE)
      D4D_FillRRect(&_calc.Position, &pThis->size, clrB, pThis->radius);  
    
 
    // normal bitmap (also may be NULL if simple rect button is needed  
    if(*pStatus & D4D_RADIOBUTTON_STATUS_CHECKED_MASK)
    {
      if(pRadioBtn->pBmpChecked != NULL)         
        D4D_DrawRBmp(&_calc.BmpPos, pRadioBtn->pBmpChecked, (D4D_BOOL)!D4D_IsEnabled(pThis), _calc.iconRadius);  // draw the bitmap         
      else if(_calc.iconRadius > D4D_RADIOBUTTON_ICON_RING)
        D4D_FillCircle(&_calc.iconCentre, (D4D_COOR)(_calc.iconRadius - D4D_RADIOBUTTON_ICON_RING), clrT);
          
    }    
    // no bitmap
    else
    {     
      if(pRadioBtn->pBmpUnChecked)
      {
        // draw the bitmap
        D4D_DrawRBmp(&_calc.BmpPos, pRadioBtn->pBmpUnChecked, (D4D_BOOL)!D4D_IsEnabled(pThis), _calc.iconRadius);   
      }else
      {        
        // draw just a color rectangle instead of bitmap
        D4D_CLR_SCHEME* pScheme = D4D_ObjectGetScheme(pThis);
        D4D_FillCircle(&_calc.iconCentre, _calc.iconRadius, clrB);      
      }
    }
    
    if(pThis->initFlags & D4D_RADIOBUTTON_F_ICON_RING)
      D4D_Circle(&_calc.iconCentre, _calc.iconRadius, D4D_LINE_THIN, clrT);       

    
    if(pThis->initFlags & D4D_OBJECT_F_FOCUSRECT)          
      D4D_RRect(&_calc.Position, &pThis->size, D4D_LINE_THIN, clrT, pThis->radius);
    
#else    

    if(draw & D4D_OBJECT_DRAWFLAGS_COMPLETE)
    {
//      if(!(pThis->initFlags & D4D_OBJECT_F_TRANSP_TEXT))
        D4D_FillRect(&_calc.Position, &pThis->size, clrB);  
    }
    
    // normal bitmap (also may be NULL if simple rect button is needed  
    if(*pStatus & D4D_RADIOBUTTON_STATUS_CHECKED_MASK)
    {
      if(pRadioBtn->pBmpChecked != NULL)         
        D4D_DrawBmp(&_calc.BmpPos, pRadioBtn->pBmpChecked, (D4D_BOOL)!D4D_IsEnabled(pThis));  // draw the bitmap         
      else if(_calc.iconRadius > 3)
        D4D_FillCircle(&_calc.iconCentre, (D4D_COOR)(_calc.iconRadius - 3), clrT);  
    }    
    // no bitmap
    else
    {     
      if(pRadioBtn->pBmpUnChecked)
      {
        // draw the bitmap
        D4D_DrawBmp(&_calc.BmpPos, pRadioBtn->pBmpUnChecked, (D4D_BOOL)!D4D_IsEnabled(pThis));   
      }else
      {        
        // draw just a color rectangle instead of bitmap
        D4D_CLR_SCHEME* pScheme = D4D_ObjectGetScheme(pThis);
        D4D_FillCircle(&_calc.iconCentre, _calc.iconRadius, clrB);      
      }
    }
    
    if(pThis->initFlags & D4D_RADIOBUTTON_F_ICON_RING)
      D4D_Circle(&_calc.iconCentre, _calc.iconRadius, D4D_LINE_THIN, clrT);
    
    if(pThis->initFlags & D4D_OBJECT_F_FOCUSRECT)          
      D4D_Rect(&_calc.Position, &pThis->size, D4D_LINE_THIN, clrT);

    
#endif    

    // draw the text
    if(pRadioBtn->textBuff.pText)
      D4D_DrawTextRect(&_calc.TxtPos, &_calc.TxtSize, &pRadioBtn->textBuff, clrT, D4D_ObjectGetBckgColor(pThis, draw));
    
}

/*******************************************************
*
* RADIO BUTTON key handling routine
*
*******************************************************/

static void D4D_RadioButtonOnKeyDown(D4D_MESSAGE* pMsg)    
{
    D4D_RADIOBUTTON_STATUS* pStatus = D4D_GET_RADIOBUTTON_STATUS(pMsg->pObject);
    
    // invoke "Click" event when enter key is pressed
    if(pMsg->prm.key == D4D_KEY_SCANCODE_ENTER)
    {      
      *pStatus |= D4D_RADIOBUTTON_STATUS_PRESSED_MASK;
      D4D_InvalidateObject(pMsg->pObject, D4D_FALSE);
    }
}

/*******************************************************
*
* RADIO BUTTON key handling routine
*
*******************************************************/

static void D4D_RadioButtonOnKeyUp(D4D_MESSAGE* pMsg)    
{
  D4D_OBJECT* pThis = pMsg->pObject;
  D4D_RADIOBUTTON_STATUS* pStatus = D4D_GET_RADIOBUTTON_STATUS(pThis);
  
  // invoke "Click" event when enter key is pressed
  if(pMsg->prm.key == D4D_KEY_SCANCODE_ENTER)
  {
    if(*pStatus & D4D_RADIOBUTTON_STATUS_PRESSED_MASK)
    {
      *pStatus &= ~D4D_RADIOBUTTON_STATUS_PRESSED_MASK;      
      
      if(*pStatus & D4D_RADIOBUTTON_STATUS_CHECKED_MASK)
        return;
  
      if(*pStatus & D4D_RADIOBUTTON_STATUS_CHECKED_MASK)
        *pStatus &= ~D4D_RADIOBUTTON_STATUS_CHECKED_MASK;
      else
        *pStatus |= D4D_RADIOBUTTON_STATUS_CHECKED_MASK;  
       
      D4D_RadioButtonChangeValue(pThis,(D4D_BOOL)(*pStatus & D4D_RADIOBUTTON_STATUS_CHECKED_MASK));            
    }
    D4D_InvalidateObject(pThis, D4D_FALSE); 
  } 
}

/*******************************************************
*
* RADIO BUTTON focus handling routine
*
*******************************************************/

static void D4D_RadioButtonKillFocus(D4D_MESSAGE* pMsg)
{
  D4D_RADIOBUTTON_STATUS* pStatus = D4D_GET_RADIOBUTTON_STATUS(pMsg->pObject);

  *pStatus &= ~D4D_RADIOBUTTON_STATUS_PRESSED_MASK;
}

/*******************************************************
*
* RADIO BUTTON Touched Button handling routine
*
*******************************************************/
#ifdef D4D_LLD_TCH
  static void D4D_RadioButtonTouched(D4D_MESSAGE* pMsg)
  {  
    if(pMsg->pObject->pData->flags & (D4D_OBJECT_F_TABSTOP | D4D_OBJECT_F_FASTTOUCH))
    {
      if((D4D_GetFocusedObject(pMsg->pScreen) == pMsg->pObject) || (pMsg->pObject->pData->flags & D4D_OBJECT_F_FASTTOUCH))
      {               
        D4D_RADIOBUTTON_STATUS* pStatus = D4D_GET_RADIOBUTTON_STATUS(pMsg->pObject);

        if(*pStatus & D4D_RADIOBUTTON_STATUS_CHECKED_MASK)
          return;
    
        if(*pStatus & D4D_RADIOBUTTON_STATUS_CHECKED_MASK)
          *pStatus &= ~D4D_RADIOBUTTON_STATUS_CHECKED_MASK;
        else
          *pStatus |= D4D_RADIOBUTTON_STATUS_CHECKED_MASK;
        
        D4D_RadioButtonChangeValue(pMsg->pObject, (D4D_BOOL)(*pStatus & D4D_RADIOBUTTON_STATUS_CHECKED_MASK));
        
        D4D_InvalidateObject(pMsg->pObject, D4D_FALSE);
      }                    
      D4D_FocusSet(pMsg->pScreen, pMsg->pObject);
    }
  }
#endif
/*******************************************************
*
* Set RADIO BUTTON value
*
*******************************************************/

void D4D_RadioButtonSetValue(D4D_OBJECT_PTR pThis, D4D_BOOL value)
{
  D4D_RADIOBUTTON_STATUS* pStatus = D4D_GET_RADIOBUTTON_STATUS(pThis);
  
//  if(!D4D_IsEnabled((D4D_OBJECT*)pThis))
//    return ;
    
 // if((pStatus->bits.bChecked && value) || (!pStatus->bits.bChecked && !value))
  if(!((*pStatus & D4D_RADIOBUTTON_STATUS_CHECKED_MASK) ^ value))
    return ; 

  
  // remember new value
  if(value)
    *pStatus |= D4D_RADIOBUTTON_STATUS_CHECKED_MASK;
  else
    *pStatus &= ~D4D_RADIOBUTTON_STATUS_CHECKED_MASK;
    
  D4D_InvalidateObject(pThis, D4D_TRUE);

  // notify user that the value has changed        
  D4D_RadioButtonChangeValue(pThis, (D4D_BOOL)(*pStatus & D4D_RADIOBUTTON_STATUS_CHECKED_MASK));
}


D4D_BOOL D4D_RadioButtonGetValue(D4D_OBJECT_PTR pThis)
{
  D4D_RADIOBUTTON_STATUS* pStatus = D4D_GET_RADIOBUTTON_STATUS(pThis);
  
  return (D4D_BOOL)(*pStatus & D4D_RADIOBUTTON_STATUS_CHECKED_MASK);
}

static void D4D_RadioButtonChangeValue(D4D_OBJECT_PTR pThis, D4D_BOOL value)
{
  D4D_RADIOBUTTON* pRadioBtn = D4D_GET_RADIOBUTTON(pThis);
  
  if(value)
  {
    D4D_OBJECT** pObjects = (D4D_OBJECT**)(pThis->pData->pScreen->pObjects);
    D4D_INDEX ix = 0;
    
    // uncheck rest radio buttons on screen!
    if(pThis->pRelations)
    {
      D4D_OBJECT* pParent = D4D_GetParentObject((D4D_OBJECT*)pThis);
      if(pParent)
        pObjects = (D4D_OBJECT**)&pParent->pRelations[D4D_OBJECT_USR_DATA_CHILD_IX];  
    }
    
    while(pObjects[ix])
    {
      if(pObjects[ix]->pObjFunc->strName == d4d_radioButtonSysFunc.strName)
        if(pObjects[ix] != pThis)
          if(D4D_RadioButtonGetValue(pObjects[ix]))
            D4D_RadioButtonSetValue(pObjects[ix], D4D_FALSE);
            
      ix++;        
    }
  }


  // notify user that the value has changed        
  if(pRadioBtn->OnChange)
    pRadioBtn->OnChange((D4D_OBJECT*)pThis);  
}


/**************************************************************//*!
*
*
*
*     THE INTERNAL API FUNCTIONS
*
*
*
******************************************************************/

/*******************************************************
*
* The main RADIO BUTTON message handler 
*
*******************************************************/

void D4D_RadioButtonOnMessage(D4D_MESSAGE* pMsg)
{
    switch(pMsg->nMsgId)
    {
    case D4D_MSG_DRAW:
        D4D_RadioButtonOnDraw(pMsg);
        break;
        
    case D4D_MSG_KEYDOWN:
          D4D_RadioButtonOnKeyDown(pMsg);
        break;

    case D4D_MSG_KEYUP:
          D4D_RadioButtonOnKeyUp(pMsg);
        break;
        
    case D4D_MSG_KILLFOCUS:
        D4D_RadioButtonKillFocus(pMsg);
        break;
        
#ifdef D4D_LLD_TCH     
    case D4D_MSG_TOUCHED:     
      D4D_RadioButtonTouched(pMsg);
      break;
#endif      

    default:
        // call the default behavior
        D4D_ObjOnMessage(pMsg);
    }
}

/**************************************************************//*!
*
* Get object text buffer
*
******************************************************************/
D4D_STRING* D4D_RadioButtonGetTextBuffer(D4D_OBJECT* pThis)
{
  return &(D4D_GET_RADIOBUTTON(pThis)->textBuff);  
}