/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_radio_button.h
*
* @author    b01119
* 
* @version   0.0.5.0
* 
* @date      Jun-29-2012
* 
* @brief     D4D Driver radio button object header file
*
*******************************************************************************/

#ifndef __D4D_RADIOBUTTON_H
#define __D4D_RADIOBUTTON_H

extern const D4D_OBJECT_SYS_FUNCTION d4d_radioButtonSysFunc;

/******************************************************************************
* Constants
******************************************************************************/

#ifndef D4D_RADIOBUTTON_BORDER_OFFSET
  #define D4D_RADIOBUTTON_BORDER_OFFSET 3
#endif

#ifndef D4D_RADIOBUTTON_TEXT_OFFSET
  #define D4D_RADIOBUTTON_TEXT_OFFSET 4
#endif

#ifndef D4D_RADIOBUTTON_ICON_RING
  #define D4D_RADIOBUTTON_ICON_RING 2
#endif

#ifndef D4D_RADIOBUTTON_MIN_SIZE
  #define D4D_RADIOBUTTON_MIN_SIZE 10
#endif

// standard radio button colors
#ifndef D4D_COLOR_RADIOBUTTON_ICON_BCKG
  #define D4D_COLOR_RADIOBUTTON_ICON_BCKG  D4D_COLOR_WHITE
#endif

/******************************************************************************
* Types
******************************************************************************/

// Check box configuration (goes to ROM by default)

#define D4D_RADIOBUTTON_F_ICON_RING   (0x01 << D4D_OBJECT_F_OBJECT_SHIFT)

#ifndef D4D_RADIOBUTTON_F_DEFAULT
  #define D4D_RADIOBUTTON_F_DEFAULT  (D4D_OBJECT_F_VISIBLE | D4D_OBJECT_F_ENABLED | D4D_OBJECT_F_TABSTOP | D4D_OBJECT_F_TOUCHENABLE\
                                  | D4D_OBJECT_F_FOCUSRECT | D4D_RADIOBUTTON_F_ICON_RING )
#endif



#ifndef D4D_RADIOBUTTON_TXT_PRTY_DEFAULT
  #define D4D_RADIOBUTTON_TXT_PRTY_DEFAULT  (D4D_ALIGN_H_LEFT_MASK | D4D_ALIGN_V_CENTER_MASK)
#endif

#ifndef D4D_RADIOBUTTON_FNT_PRTY_DEFAULT
  #define D4D_RADIOBUTTON_FNT_PRTY_DEFAULT  ( 0 )
#endif

typedef Byte D4D_RADIOBUTTON_STATUS;

#define D4D_RADIOBUTTON_STATUS_PRESSED_MASK    (0x01)
#define D4D_RADIOBUTTON_STATUS_CHECKED_MASK    (0x02)

typedef struct 
{
    D4D_STRING textBuff;    // button text
    const D4D_BMP*  pBmpChecked;    // Checked state bitmap
    const D4D_BMP*  pBmpUnChecked;    // Checked state bitmap
    D4D_RADIOBUTTON_STATUS*  pStatus;    
    void (*OnChange)(D4D_OBJECT* pThis);
} D4D_RADIOBUTTON;


/******************************************************************************
* Macros 
******************************************************************************/

// getting the Radio Button structure from general OBJECT
#define D4D_GET_RADIOBUTTON(pObj) ((D4D_RADIOBUTTON*)((pObj)->pParam))
#define D4D_GET_RADIOBUTTON_STATUS(pObj) ((D4D_RADIOBUTTON_STATUS*)(D4D_GET_RADIOBUTTON(pObj)->pStatus))


/*********************************************************
*
* static declaration macros
*
*********************************************************/

#define _D4D_DECLARE_RADIOBUTTON(type, name, text, x, y, cx, cy, radius, pMargin, pRelations, flags, pbmpChecked, pbmpUnChecked, pScheme, fontId, pUser, onchange, pOnUsrMsg) \
    static D4D_RADIOBUTTON_STATUS name##_status = { 0 }; \
    static D4D_STR_PROPERTIES name##_strPrties = { D4D_RADIOBUTTON_FNT_PRTY_DEFAULT, D4D_RADIOBUTTON_TXT_PRTY_DEFAULT}; \
    static type D4D_RADIOBUTTON name##_params = \
    { \
        { text, D4D_TEXT_LEN(text), fontId, &name##_strPrties, D4D_OBJECT_MAX_TEXT_LEN(text), 0}, /* textBuff */ \
        pbmpChecked,      /* pBmpChecked */ \
        pbmpUnChecked,      /* pBmpUnChecked */ \
        &(name##_status), /* Status clear */ \
        onchange     /* Click event */ \
    }; \
    \
    D4D_DECLARE_OBJECT(type, name, x, y, cx, cy, radius, pMargin, pRelations, pOnUsrMsg, &d4d_radioButtonSysFunc, &(name##_params), flags, pUser, pScheme)


#define D4D_DECLARE_RADIOBUTTON(name, text, x, y, cx, cy, flags, pbmpChecked, pbmpUnChecked, pScheme, fontId, pUser, onchange, pOnUsrMsg) \
  _D4D_DECLARE_RADIOBUTTON(D4D_CONST, name, text, x, y, cx, cy, 0, flags, NULL, NULL, pbmpChecked, pbmpUnChecked, pScheme, fontId, pUser, onchange, pOnUsrMsg)

#define D4D_DECLARE_STD_RADIOBUTTON(name, text, x, y, cx, cy, pbmpChecked, pbmpUnChecked, fontId, onchange) \
    D4D_DECLARE_RADIOBUTTON(name, text, x, y, cx, cy, (D4D_RADIOBUTTON_F_DEFAULT), pbmpChecked, pbmpUnChecked, NULL, fontId, NULL, onchange, NULL)
    
// Rounded check box definition

#define D4D_DECLARE_RRADIOBUTTON(name, text, x, y, cx, cy, radius, flags, pbmpChecked, pbmpUnChecked, pScheme, fontId, pUser, onchange, pOnUsrMsg) \
  _D4D_DECLARE_RADIOBUTTON(D4D_CONST, name, text, x, y, cx, cy, radius, NULL, NULL, flags, pbmpChecked, pbmpUnChecked, pScheme, fontId, pUser, onchange, pOnUsrMsg)

#define D4D_DECLARE_STD_RRADIOBUTTON(name, text, x, y, cx, cy, radius, pbmpChecked, pbmpUnChecked, fontId, onchange) \
    D4D_DECLARE_RRADIOBUTTON(name, text, x, y, cx, cy, radius, (D4D_RADIOBUTTON_F_DEFAULT), pbmpChecked, pbmpUnChecked, NULL, fontId, NULL, onchange, NULL)



// IN RAM instantions macros
#define D4D_DECLARE_RADIOBUTTON_INRAM(name, text, x, y, cx, cy, flags, pbmpChecked, pbmpUnChecked, pScheme, fontId, pUser, onchange, pOnUsrMsg) \
  _D4D_DECLARE_RADIOBUTTON(D4D_NO_CONST, name, text, x, y, cx, cy, 0, NULL, NULL, flags, pbmpChecked, pbmpUnChecked, pScheme, fontId, pUser, onchange, pOnUsrMsg)
    
#define D4D_DECLARE_STD_RADIOBUTTON_INRAM(name, text, x, y, cx, cy, pbmpChecked, pbmpUnChecked, fontId, onchange) \
    D4D_DECLARE_RADIOBUTTON_INRAM(name, text, x, y, cx, cy, (D4D_RADIOBUTTON_F_DEFAULT), pbmpChecked, pbmpUnChecked, NULL, fontId, NULL, onchange, NULL)


// Rounded check box definition

#define D4D_DECLARE_RRADIOBUTTON_INRAM(name, text, x, y, cx, cy, radius, flags, pbmpChecked, pbmpUnChecked, pScheme, fontId, pUser, onchange, pOnUsrMsg) \
  _D4D_DECLARE_RADIOBUTTON(D4D_NO_CONST, name, text, x, y, cx, cy, radius, NULL, NULL, flags, pbmpChecked, pbmpUnChecked, pScheme, fontId, pUser, onchange, pOnUsrMsg)

#define D4D_DECLARE_STD_RRADIOBUTTON_INRAM(name, text, x, y, cx, cy, radius, pbmpChecked, pbmpUnChecked, fontId, onchange) \
    D4D_DECLARE_RRADIOBUTTON_INRAM(name, text, x, y, cx, cy, radius, (D4D_RADIOBUTTON_F_DEFAULT), pbmpChecked, pbmpUnChecked, NULL, fontId, NULL, onchange, NULL)
    
    
    

/******************************************************************************
* Global functions
******************************************************************************/

/*********************************************************
*
* internal API
*
*********************************************************/

void D4D_RadioButtonSetValue(D4D_OBJECT_PTR pThis, D4D_BOOL value);
D4D_BOOL D4D_RadioButtonGetValue(D4D_OBJECT_PTR pThis);

// Obsolete functions, replaced by any general
#define D4D_RadioButtonSetText D4D_SetText



#endif /*__D4D_RADIOBUTTON_H*/












