/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4dlcdhw_k70_lcdc.c
*
* @author    b01119
* 
* @version   0.0.15.0
* 
* @date      Aug-22-2012
* 
* @brief     D4D driver - k70_lcdc hardware lcd driver source c file 
*
******************************************************************************/

#include "d4d.h"            // include of all public items (types, function etc) of D4D driver
#include "common_files/d4d_lldapi.h"     // include non public low level driver interface header file (types, function prototypes, enums etc. )
#include "common_files/d4d_private.h"    // include the private header file that contains perprocessor macros as D4D_MK_STR


// identification string of driver - must be same as name D4DTCH_FUNCTIONS structure + "_ID"
// it is used for enable the code for compilation
#define d4dlcdhw_k70_lcdc_ID 1


// copilation enable preprocessor condition
// the string d4dtch_k70_lcdc_ID must be replaced by define created one line up
#if (D4D_MK_STR(D4D_LLD_LCD_HW) == d4dlcdhw_k70_lcdc_ID)
  
  // include of low level driver heaser file
  // it will be included into wole project only in case that this driver is selected in main D4D configuration file
  #include "low_level_drivers/LCD/lcd_hw_interface/k70_lcdc/d4dlcdhw_k70_lcdc.h"
  #include "low_level_drivers/LCD/lcd_hw_interface/k70_lcdc/twr_rgb_rev_a/d4dlcdhw_twrrgba.h"
  /******************************************************************************
  * Macros 
  ******************************************************************************/

  /******************************************************************************
  * Internal function prototypes 
  ******************************************************************************/

  static unsigned char D4DLCDHW_Init_K70LCDC(void);
  static unsigned char D4DLCDHW_DeInit_K70LCDC(void);
  static void D4DLCDHW_WriteData_K70LCDC(unsigned long addr, D4D_COLOR value);
  static D4D_COLOR D4DLCDHW_ReadData_K70LCDC(unsigned long addr);
  static D4DLCD_FRAMEBUFF_DESC* D4DLCDHW_GetFbDescriptor_K70LCDC(void);
  static unsigned char D4DLCDHW_PinCtl_K70LCDC(D4DLCDHW_PINS pinId, D4DHW_PIN_STATE setState);
  static void D4DLCDHW_FlushBuffer_K70LCDC(D4DLCD_FLUSH_MODE mode);
  
#if D4DLCDHWFB_DOUBLE_BUFFER    
  static void D4DLCDHW_CopyBuffers_K70LCDC(void);
#endif  
  
#ifdef D4D_OS_MQX  
  void D4DLCDHW_ISR_K70LCDC(void* pData);
#else
  void D4DLCDHW_ISR_K70LCDC(void);
#endif
 
  /**************************************************************//*!
  *
  * Global variables
  *
  ******************************************************************/
  
  // the main structure that contains low level driver api functions
  // the name fo this structure is used for recognizing of configured low level driver of whole D4D
  // so this name has to be used in main configuration header file of D4D driver to enable this driver
  const D4DLCDHWFB_FUNCTIONS d4dlcdhw_k70_lcdc = 
  {
    D4DLCDHW_Init_K70LCDC,    
    D4DLCDHW_WriteData_K70LCDC,
    D4DLCDHW_ReadData_K70LCDC,
    D4DLCDHW_GetFbDescriptor_K70LCDC,
    D4DLCDHW_PinCtl_K70LCDC,
    D4DLCDHW_FlushBuffer_K70LCDC,
    D4DLCDHW_DeInit_K70LCDC
  };
  /**************************************************************//*!
  *
  * Local variables
  *
  ******************************************************************/

    static D4DLCD_FRAMEBUFF_DESC d4dlcdhw_k70_lcdc_desc = 
    {
            D4DLCDHWFB_START_ADDRESS,
            D4DLCDHWFB_X_MAX,
            D4DLCDHWFB_Y_MAX,
            D4DLCDHWFB_BPP_BYTE		
    };

    static volatile LWord enableWrite = 0;  

#if D4DLCDHWFB_DOUBLE_BUFFER
    static unsigned long fb_start_addr_2nd = D4DLCDHWFB_START_ADDRESS_2ND;
    static signed long fb_start_addr_diff;
    static volatile unsigned long fb_draw_2nd = 0;
    static volatile unsigned long fb_switch = 0;
    static volatile unsigned long fb_shadow_draw = 0;   
#endif
  /**************************************************************//*!
  *
  * Functions bodies
  *
  ******************************************************************/

   
  //-----------------------------------------------------------------------------
  // FUNCTION:    D4DLCDHW_Init_K70LCDC
  // SCOPE:       Low Level Driver API function
  // DESCRIPTION: The function is used for initialization of this low level driver 
  //              
  // PARAMETERS:  none
  //              
  // RETURNS:     result: 1 - Success
  //                      0 - Failed
  //-----------------------------------------------------------------------------  
static unsigned char D4DLCDHW_Init_K70LCDC(void)
{
  
  /* Enable LCDC and DDR Modules */
  SIM_SCGC3 |= SIM_SCGC3_LCDC_MASK;
  
  if(D4D_LLD_LCD_HW_K70LCDC.D4DLCDHWFBK70LCDC_InitPanel == NULL)
    return 0;
  
  
  // Registrer interrupt service routines 
  #ifdef D4D_OS_MQX
    _int_install_isr(INT_LCD, D4DLCDHW_ISR_K70LCDC, NULL);
    _cortex_int_init(INT_LCD, 3, TRUE);
  #else
    NVIC_IP(INT_LCD) = 3;
    NVIC_ISER(INT_LCD / 32) |= (1 << (INT_LCD % 32));
    #warning The LCDC K70 driver is is using interrupts, please registry the "D4DLCDHW_ISR_K70LCDC" on interrupt vector of LCDC (113).
  #endif
  
  
  #if D4DLCDHWFB_START_ADDRESS == 0
    // Allocate frame buffer memory 
    if(d4dlcdhw_k70_lcdc_desc.fb_start_addr)
    {
      D4D_MemFree((void*)d4dlcdhw_k70_lcdc_desc.fb_start_addr);
      d4dlcdhw_k70_lcdc_desc.fb_start_addr = 0;
    }
    d4dlcdhw_k70_lcdc_desc.fb_start_addr = (unsigned long)D4D_MemAlloc(D4DLCDHWFB_X_MAX * D4DLCDHWFB_Y_MAX * D4DLCDHWFB_BPP_BYTE);
    if(!d4dlcdhw_k70_lcdc_desc.fb_start_addr)
      return 0;
  #endif
    
  
  #if D4DLCDHWFB_DOUBLE_BUFFER
    #if D4DLCDHWFB_START_ADDRESS_2ND == 0
      // Allocate frame buffer memory 
      if(fb_start_addr_2nd)
      {
        D4D_MemFree((void*)fb_start_addr_2nd);
        fb_start_addr_2nd = 0;
      }
      fb_start_addr_2nd = (unsigned long)D4D_MemAlloc(D4DLCDHWFB_X_MAX * D4DLCDHWFB_Y_MAX * D4DLCDHWFB_BPP_BYTE);
      if(!fb_start_addr_2nd)
        return 0;
    #endif
      fb_switch = 0;
      fb_draw_2nd = 0;
      fb_shadow_draw = 0;
      fb_start_addr_diff = fb_start_addr_2nd - d4dlcdhw_k70_lcdc_desc.fb_start_addr;
      
#if D4DLCDHWFB_DMA_ENABLE
      // Copy the buffer
      DMA_SADDR(D4DLCDHWFB_DMA_CHANNEL)          = d4dlcdhw_k70_lcdc_desc.fb_start_addr; // first buffer
      DMA_SOFF(D4DLCDHWFB_DMA_CHANNEL)           = 4;                              // 2 byte offset 
      DMA_ATTR(D4DLCDHWFB_DMA_CHANNEL)           = DMA_ATTR_SMOD(0) | DMA_ATTR_SSIZE(2) | DMA_ATTR_DMOD(0) | DMA_ATTR_DSIZE(2);   // no circular addressing S&D, 32 bit S&D 
      DMA_NBYTES_MLNO(D4DLCDHWFB_DMA_CHANNEL)    = (D4DLCDHWFB_X_MAX * D4DLCDHWFB_Y_MAX * 4);                              // one  32bit sample every minor loop 
      DMA_SLAST(D4DLCDHWFB_DMA_CHANNEL)          = 0;         // source address will continue X times to allow defined multiply length of buffer // to do
      DMA_DADDR(D4DLCDHWFB_DMA_CHANNEL)          = fb_start_addr_2nd;     // second buffer
      DMA_DOFF(D4DLCDHWFB_DMA_CHANNEL)           = 4;
      DMA_CITER_ELINKNO(D4DLCDHWFB_DMA_CHANNEL)  = DMA_CITER_ELINKNO_CITER(D4DLCDHWFB_X_MAX * D4DLCDHWFB_Y_MAX);              // total samples
      DMA_DLAST_SGA(D4DLCDHWFB_DMA_CHANNEL)      = 0;                       // no final last adjustment ( does not move )
      DMA_CSR(D4DLCDHWFB_DMA_CHANNEL)            = 0;  // none interrupt when done  
      DMA_BITER_ELINKNO(D4DLCDHWFB_DMA_CHANNEL)  = DMA_BITER_ELINKYES_BITER(D4DLCDHWFB_X_MAX * D4DLCDHWFB_Y_MAX);              // no chan links, total samples ( 128 ) // to do
      
#endif
  #endif  
  
  enableWrite = 1;  
    
  #if D4DLCDHWFB_MIRROWED == 1 
        LCDC_LSSAR = d4dlcdhw_k70_lcdc_desc.fb_start_addr + D4DLCDHWFB_X_MAX * D4DLCDHWFB_Y_MAX * d4dlcdhw_k70_lcdc_desc.bpp_byte ;
  #else
        LCDC_LSSAR = d4dlcdhw_k70_lcdc_desc.fb_start_addr;	  
  #endif	
  LCDC_LSR =   (D4DLCDHWFB_X_MAX / 16) << 20 | (D4DLCDHWFB_Y_MAX);
  
  // set LCD virtual page width
  LCDC_LVPWR = LCDC_LVPWR_VPW( D4DLCDHWFB_X_MAX / (4 / D4DLCDHWFB_BPP_BYTE));

  // set LCD cursor positon & settings (turn off)
  LCDC_LCPR = 0;
  LCDC_LCWHB = 0;
  
  // set LCD panning offset
  LCDC_LPOR = 0;

  // set LCD interrupt configuration
  LCDC_LICR = LCDC_LICR_INTCON_MASK;

  // set LCD interrupt enable
  LCDC_LIER = LCDC_LIER_BOF_EN_MASK | LCDC_LIER_EOF_EN_MASK;
  
  //Set background plane DMA to burst mode
  LCDC_LDCR &= ~(LCDC_LDCR_BURST_MASK);

  D4D_LLD_LCD_HW_K70LCDC.D4DLCDHWFBK70LCDC_InitPanel();
  
  #if D4DLCDHWFB_MIRROWED == 1 
    LCDC_LPCR |= LCDC_LPCR_REV_VS_MASK;
  #endif
    
    
#if 0
  // set LCD graphic window start address
  LCDC_LGWSAR = 0x80000000;

  // set LCD graphic window size
  LCDC_LGWSR =
    LCDC_LGWSR_GWW(0) |
    LCDC_LGWSR_GWH(0);

  //  set LCD graphic window virtual page width
  LCDC_LGWVPWR = 0;

  //  set LCD graphic window panning offset
  LCDC_LGWPOR = 0;

  //  set LCD graphic window position
  LCDC_LGWPR =
    LCDC_LGWPR_GWXP(0) |
    LCDC_LGWPR_GWYP(0);

  //  set LCD graphic window control
  LCDC_LGWCR =
    LCDC_LGWCR_GWAV(0xF)          | // alpha-transparent
  //  (1 << LCDC_LGWCR_GWE_SHIFT)   | // enable
    (1 << LCDC_LGWCR_GWCKE_SHIFT) | // color key enable
    LCDC_LGWCR_GWCKR(0xFF)        | // color key
    LCDC_LGWCR_GWCKG(0)           |
    LCDC_LGWCR_GWCKB(80);
  
 // LCDC_LAUSCR = LCDC_LAUSCR_AGWCKB_MASK;
#endif
      /* Enable LCD */
  SIM_MCR|=SIM_MCR_LCDSTART_MASK;    

      
      return 1;
}
  
  
  //-----------------------------------------------------------------------------
  // FUNCTION:    D4DLCDHW_DeInit_K70LCDC
  // SCOPE:       Low Level Driver API function
  // DESCRIPTION: The function is used for deinitialization of this low level driver 
  //              
  // PARAMETERS:  none
  //              
  // RETURNS:     result: 1 - Success
  //                      0 - Failed
  //-----------------------------------------------------------------------------  
  static unsigned char D4DLCDHW_DeInit_K70LCDC(void)
  { 
    #if D4DLCDHWFB_START_ADDRESS == 0
    if(d4dlcdhw_k70_lcdc_desc.fb_start_addr)
    {
      D4D_MemFree((void*)d4dlcdhw_k70_lcdc_desc.fb_start_addr);
      d4dlcdhw_k70_lcdc_desc.fb_start_addr = 0;
    }
    #endif
    return 1;
  }

  //-----------------------------------------------------------------------------
  // FUNCTION:    D4DLCDHW_SendDataWord_K70LCDC
  // SCOPE:       Low Level Driver API function
  // DESCRIPTION: The function send the one 16 bit variable into LCD  
  //
  // PARAMETERS:  unsigned long addr  		address to write data
  //							unsigned short value    variable to send
  //
  // RETURNS:     none
  //-----------------------------------------------------------------------------
  static void D4DLCDHW_WriteData_K70LCDC(unsigned long addr, D4D_COLOR value)
  { 
    unsigned long data;
    
    #if D4D_COLOR_SYSTEM  == D4D_COLOR_SYSTEM_RGB565
    data = ((value & 0xf800)<<8)|((value & 0x07e0)<<5)|((value & 0x001F)<<3);        
    #elif D4D_COLOR_SYSTEM == D4D_COLOR_SYSTEM_RGB888
    data = value;
    #else
    data = D4D_COLOR_RGB888(D4D_COLOR_GET_R(value), D4D_COLOR_GET_G(value), D4D_COLOR_GET_B(value));
    #endif
    
    #if D4DLCDHWFB_DOUBLE_BUFFER    
    if(fb_draw_2nd)
      addr += fb_start_addr_diff; 
    
    if(!fb_shadow_draw)
    #endif
      while(!enableWrite);
    
    
    *((unsigned long*)addr) = data;    
  }
  
  
  //-----------------------------------------------------------------------------
  // FUNCTION:    D4DLCDHW_ReadDataWord_K70LCDC
  // SCOPE:       Low Level Driver API function
  // DESCRIPTION: The function reads the one 16 bit variable from LCD (if this function is supported)  
  //
  // PARAMETERS:  unsigned long addr  		address to read data
  //
  // RETURNS:     unsigned short - the readed value
  //              
  //-----------------------------------------------------------------------------
  static D4D_COLOR D4DLCDHW_ReadData_K70LCDC(unsigned long addr)
  {       
    #if D4DLCDHWFB_DOUBLE_BUFFER
    if(fb_draw_2nd)
      addr += fb_start_addr_diff;
    #endif
    
    #if D4D_COLOR_SYSTEM == D4D_COLOR_SYSTEM_RGB888
        return *((unsigned long*)addr);
    #else
        unsigned long value = *((unsigned long*)addr);
        return D4D_COLOR_RGB(D4D_COLOR888_GET_R(value), D4D_COLOR888_GET_G(value), D4D_COLOR888_GET_B(value));
    #endif
  }
  
  
  //-----------------------------------------------------------------------------
  // FUNCTION:    D4DLCDHW_GetFbDescriptor_K70LCDC
  // SCOPE:       Low Level Driver API function
  // DESCRIPTION: The function return the pointer on filled frame buffer descriptor  
  //
  // PARAMETERS:  none
  //
  // RETURNS:     D4DLCD_FRAMEBUFF_DESC* - pointer on frame buffer descriptor
  //              
  //-----------------------------------------------------------------------------
  static D4DLCD_FRAMEBUFF_DESC* D4DLCDHW_GetFbDescriptor_K70LCDC(void)
  {
  	return (D4DLCD_FRAMEBUFF_DESC*) &d4dlcdhw_k70_lcdc_desc;	
  }
  
  //-----------------------------------------------------------------------------
  // FUNCTION:    D4DLCDHW_PinCtl_K70LCDC
  // SCOPE:       Low Level Driver API function
  // DESCRIPTION: allows control GPIO pins for LCD conrol purposes
  //
  // PARAMETERS:  D4DLCDHW_PINS pinId - Pin identification
  //              D4DHW_PIN_STATE setState - Pin action
  // RETURNS:     for Get action retuns the pin value
  //-----------------------------------------------------------------------------
  static unsigned char D4DLCDHW_PinCtl_K70LCDC(D4DLCDHW_PINS pinId, D4DHW_PIN_STATE setState)
  {       
		(void)pinId;
		(void)setState;
		return 0;
  }
  
  //-----------------------------------------------------------------------------
  // FUNCTION:    D4DLCD_FlushBuffer_K70LCDC
  // SCOPE:       Low Level Driver API function
  // DESCRIPTION: For buffered low level interfaces is used to inform
  //              driver the complete object is drawed and pending pixels should be flushed
  //
  // PARAMETERS:  none
  //              
  // RETURNS:     none
  //-----------------------------------------------------------------------------
  static void D4DLCDHW_FlushBuffer_K70LCDC(D4DLCD_FLUSH_MODE mode)
  {  

#if (D4DLCDHWFB_DOUBLE_BUFFER == 0) && (D4DLCDHWFB_CACHE_FLUSH_ENABLE == 0)
    D4D_UNUSED(mode);
#endif    
    
#if D4DLCDHWFB_DOUBLE_BUFFER
    switch(mode)
    {
      case D4DLCD_FLSH_SCR_START:
        // Switch the drawing buffer
        fb_shadow_draw = 1;  
        fb_draw_2nd = !fb_draw_2nd;        
        break;

      case D4DLCD_FLSH_SCR_END:        
        fb_switch = 1;    
        break;
    }  
#endif    
    
    
#if D4DLCDHWFB_CACHE_FLUSH_ENABLE != 0
    if(mode == D4DLCD_FLSH_SCR_END || mode == D4DLCD_FLSH_FORCE)
    {
      LMEM_PSCCR |= LMEM_PSCCR_GO_MASK | LMEM_PSCCR_PUSHW0_MASK | LMEM_PSCCR_PUSHW1_MASK;
      /* wait until the command completes */
      while (LMEM_PSCCR & LMEM_PSCCR_GO_MASK)
      {};
    }
#endif
  }

#ifdef D4D_OS_MQX
  void D4DLCDHW_ISR_K70LCDC(void* pData)
#else
  void D4DLCDHW_ISR_K70LCDC(void)  
#endif
  {
    LWord mirror_LISR = LCDC_LISR;
    
    if(mirror_LISR & LCDC_LISR_EOF_MASK)
    {
      LCDC_LICR |= LCDC_LICR_INTCON_MASK;
     
#if D4DLCDHWFB_DOUBLE_BUFFER      
      if(fb_switch)
      {
        fb_switch = 0;
        fb_shadow_draw = 0;
        // Switch the frame buffer
        D4DLCDHW_CopyBuffers_K70LCDC();
        
        if(fb_draw_2nd)
        #if D4DLCDHWFB_MIRROWED == 1 
          LCDC_LSSAR = fb_start_addr_2nd + D4DLCDHWFB_X_MAX * D4DLCDHWFB_Y_MAX * d4dlcdhw_k70_lcdc_desc.bpp_byte ;
        #else
          LCDC_LSSAR = fb_start_addr_2nd;	  
        #endif  
        else
        #if D4DLCDHWFB_MIRROWED == 1 
          LCDC_LSSAR = d4dlcdhw_k70_lcdc_desc.fb_start_addr + D4DLCDHWFB_X_MAX * D4DLCDHWFB_Y_MAX * d4dlcdhw_k70_lcdc_desc.bpp_byte ;
        #else
          LCDC_LSSAR = d4dlcdhw_k70_lcdc_desc.fb_start_addr;	  
        #endif  
      }else        // Give the DMA time to recopy the buffer to each other for one screen refresh
#endif
      enableWrite = 1;
      
    }
    else if(mirror_LISR & LCDC_LISR_BOF_MASK)
    {
      enableWrite = 0;
      LCDC_LICR &= ~LCDC_LICR_INTCON_MASK;
    }
  }

#if D4DLCDHWFB_DOUBLE_BUFFER  
static void D4DLCDHW_CopyBuffers_K70LCDC(void)
{
  
#if D4DLCDHWFB_DMA_ENABLE  
  if(!fb_draw_2nd)
  {
    DMA_SADDR(D4DLCDHWFB_DMA_CHANNEL)      = d4dlcdhw_k70_lcdc_desc.fb_start_addr;
    DMA_DADDR(D4DLCDHWFB_DMA_CHANNEL)      = fb_start_addr_2nd;
  }else                 
  {                     
    DMA_SADDR(D4DLCDHWFB_DMA_CHANNEL)      = fb_start_addr_2nd;
    DMA_DADDR(D4DLCDHWFB_DMA_CHANNEL)      = d4dlcdhw_k70_lcdc_desc.fb_start_addr;
  }  
  DMA_CSR( D4DLCDHWFB_DMA_CHANNEL)         = DMA_CSR_START_MASK;
   
#else
  
  LWord* pSource;
  LWord* pDest;
  LWord cnt = D4DLCDHWFB_X_MAX * D4DLCDHWFB_Y_MAX;
  
  if(!fb_draw_2nd)
  {
    pSource     = (LWord*)d4dlcdhw_k70_lcdc_desc.fb_start_addr;
    pDest       = (LWord*)fb_start_addr_2nd;
  }else
  {
    pSource     = (LWord*)fb_start_addr_2nd;
    pDest       = (LWord*)d4dlcdhw_k70_lcdc_desc.fb_start_addr;
  }
  
  while(cnt--)
    *pDest++ = *pSource++;
#endif
}
#endif

#endif //(D4D_MK_STR(D4D_LLD_LCD_HW) == d4dlcdhw_k70_lcdc_ID)
