/******************************************************************************
* 
* Copyright (c) 2011 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_extsrc.h
*
* @author    b01119
* 
* @version   0.0.15.0
* 
* @date      Jun-29-2012
* 
* @brief     D4D Driver extscr helper functions header file
*
*******************************************************************************/

#ifndef __D4D_EXTSRC_H
#define __D4D_EXTSRC_H

#if !defined(__MQX__) && !defined(D4D_OS_MQX)
 // #include <stdlib.h>
#endif

/******************************************************************************
* Constants
******************************************************************************/

#ifndef D4D_IMG_WORK_PATH_DEFAULT
  #define D4D_IMG_WORK_PATH_DEFAULT "a:\\"
#endif  

#ifndef D4D_FNT_WORK_PATH_DEFAULT
  #define D4D_FNT_WORK_PATH_DEFAULT "a:\\"
#endif

/******************************************************************************
* Types
******************************************************************************/



typedef struct D4DFILE_FUNCTIONS_S           
{
  D4D_FILEPTR (*D4DFILE_fopen)(D4D_CHAR* filename, D4D_CHAR* mode); // THE MODE WILL BE ALWAYS READ ONLY 'r'
  int (*D4DFILE_fclose)(D4D_FILEPTR p_file);  
  int (*D4DFILE_read)(D4D_FILEPTR p_file, void* buffer, int size);
  int (*D4DFILE_write)(D4D_FILEPTR p_file, const void* buffer, int size);
  int (*D4DFILE_fseek)(D4D_FILEPTR p_file, long offset, int origin);
  long (*D4DFILE_ftell)(D4D_FILEPTR p_file);  
}D4DFILE_FUNCTIONS;

typedef struct D4DLOCALE_FUNCTIONS_S           
{
  char strTranslatePrefix;
  int (*D4DLOCALE_TranslateStr)(const D4D_TCHAR* originTxt, D4D_TCHAR* newTxt, int offset, int size);
  int (*D4DLOCALE_GetTranslateStrLength)(const D4D_TCHAR* originTxt);
}D4DLOCALE_FUNCTIONS;


/******************************************************************************
* Macros 
******************************************************************************/

#define D4D_DECLARE_LOCALE_FUNCTIONS(strTranslatePrefix, D4DLOCALE_TranslateStr, D4DLOCALE_GetTranslateStrLength) \
  const D4DLOCALE_FUNCTIONS d4d_extsrcLocale = \
  { \
    strTranslatePrefix, /* Translate Char prefix */ \
    D4DLOCALE_TranslateStr, /* Translate locale function pointer */ \
    D4DLOCALE_GetTranslateStrLength  /* Translate locale get str Length function pointer */ \
  };
    
#define D4D_DECLARE_FILE_FUNCTIONS(D4DFILE_fopen, D4DFILE_fclose, D4DFILE_read, D4DFILE_write, D4DFILE_fseek, D4DFILE_ftell) \
  const D4DFILE_FUNCTIONS d4d_extsrcFile = \
  { \
    D4DFILE_fopen, /* D4D File - fopen */ \
    D4DFILE_fclose, /* D4D File - fclose */ \
    D4DFILE_read, /* D4D File - fread */ \
    D4DFILE_write, /* D4D File - fwrite */ \
    D4DFILE_fseek, /* D4D File - fseek */ \
    D4DFILE_ftell /* D4D File - ftell */ \
  };

#define D4D_SEEK_SET    1
#define D4D_SEEK_CUR    2
#define D4D_SEEK_END    3


#define D4D_FileOpen    d4d_extsrcFile.D4DFILE_fopen
#define D4D_FileClose   d4d_extsrcFile.D4DFILE_fclose
#define D4D_FileRead    d4d_extsrcFile.D4DFILE_read
#define D4D_FileWrite   d4d_extsrcFile.D4DFILE_write
#define D4D_FileSeek    d4d_extsrcFile.D4DFILE_fseek
#define D4D_FileTell    d4d_extsrcFile.D4DFILE_ftell 

#ifndef D4D_MemAlloc
  #if defined(__MQX__) || defined(D4D_OS_MQX)
    #define D4D_MemAlloc(size)  _mem_alloc(size)
  #else
    #define D4D_MemAlloc(size)  malloc(size)   
  #endif
#endif

#ifndef D4D_MemFree  
  #if defined(__MQX__) || defined(D4D_OS_MQX)
    #define D4D_MemFree(pMem)   _mem_free(pMem)
  #else
    #define D4D_MemFree(pMem)   free(pMem)
  #endif
#endif


                      

/******************************************************************************
* Global variables
******************************************************************************/

extern const D4DLOCALE_FUNCTIONS d4d_extsrcLocale;
extern const D4DFILE_FUNCTIONS d4d_extsrcFile;

/******************************************************************************
* Global functions
******************************************************************************/
D4D_CHAR* D4D_GetFileExtension(D4D_CHAR* pFileName);
D4D_BOOL D4D_FileIsAbsolutePath(D4D_CHAR* pFileName);
D4D_BOOL D4D_PrintScreenToBMP(D4D_CHAR* pFileName);
int D4D_StrLen(register const D4D_CHAR *s);
D4D_CHAR* D4D_StrCopy(register D4D_CHAR *sd, register const D4D_CHAR *ss);
D4D_WCHAR* D4D_StrCopyAscii2Unicode(register D4D_WCHAR *sd, register const D4D_CHAR *ss);
D4D_CHAR* D4D_StrCopyUnicode2Ascii(register D4D_CHAR *sd, register const D4D_WCHAR *ss);
D4D_CHAR* D4D_StrCat(register D4D_CHAR *sd, register const D4D_CHAR *ss);
D4D_WCHAR* D4D_StrCatUnicode(register D4D_WCHAR *sd, register const D4D_WCHAR *ss);
int D4D_CompareStrings(const D4D_CHAR* s1, const D4D_CHAR* s2);
void D4D_ToUpper(D4D_CHAR* s);
LWord D4D_SwapLong(LWord n);
Word D4D_SwapWord(Word n);


#if D4D_ENDIAN == D4D_ENDIAN_LITTLE
  #define D4D_READ8L(x) (x)
  #define D4D_READ16L(x) (x)
  #define D4D_READ32L(x) (x)
  
  #define D4D_READ8B(x) (x)
  #define D4D_READ16B(x) D4D_SwapWord((x))
  #define D4D_READ32B(x) D4D_SwapLong((x))

  #define D4D_WRITE8L(x) (x)
  #define D4D_WRITE16L(x) (x)
  #define D4D_WRITE32L(x) (x)
  
  #define D4D_WRITE8B(x) (x)
  #define D4D_WRITE16B(x) D4D_SwapWord((x))
  #define D4D_WRITE32B(x) D4D_SwapLong((x))


#else
  #define D4D_READ8L(x) (x)
  #define D4D_READ16L(x) D4D_SwapWord((x))
  #define D4D_READ32L(x) D4D_SwapLong((x))
  
  #define D4D_READ8B(x) (x)
  #define D4D_READ16B(x) (x)
  #define D4D_READ32B(x) (x)

  #define D4D_WRITE8L(x) (x)
  #define D4D_WRITE16L(x) D4D_SwapWord((x))
  #define D4D_WRITE32L(x) D4D_SwapLong((x))
  
  #define D4D_WRITE8B(x) (x)
  #define D4D_WRITE16B(x) (x)
  #define D4D_WRITE32B(x) (x)

#endif

#endif /* __D4D_EXTSRC_H */