/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_imgdec_d4dint.h
*
* @author    b01119
* 
* @version   0.0.6.0
* 
* @date      Jun-29-2012
* 
* @brief     D4D driver external pictures d4d decoder functions header file 
*
******************************************************************************/

#ifndef __D4D_IMGDEC_D4DINT_H
#define __D4D_IMGDEC_D4DINT_H

/******************************************************************************
* Types
******************************************************************************/

typedef struct D4D_IMG_HEADER_D4DINT_S
{
  unsigned short  width;        // width of bitmap
  unsigned short  height;       // height of bitmap
  unsigned short  colorMode;    // color mode of stored bitmap
}D4D_IMG_HEADER_D4DINT;

typedef struct D4D_D4DBMP_DECODE_S
{
  LWord       pxlClr;
  LWord       pxlClrPrv;
  Byte        pxlCode;
  D4D_COLOR   pxlClrRet;  
  LWord       ix;
  LWord       rptCnt;
  Byte        curSubPxl;
  D4D_BOOL    compress;
  D4D_BOOL    greyScale;
  const void* pBmp;
  const void* pPalette;
  void*       GetDataFunc;
  D4D_COLOR   (*GetPxlDecoder)(struct D4D_D4DBMP_DECODE_S* pDecode);
  
#if (D4D_BMP_EXTSRC_SUPPORT_D4D == D4D_TRUE) && (D4D_EXTSRC_FILE_ENABLE == D4D_TRUE) 
  LWord       dataCnt;
  LWord       dataIx;
  D4D_FILEPTR pFile;  
#endif
}D4D_D4DBMP_DECODE;


typedef Byte (*GET_DATA_BYTE_FUNC)(D4D_D4DBMP_DECODE* pDecode, D4D_BOOL increment);  
typedef Word (*GET_DATA_WORD_FUNC)(D4D_D4DBMP_DECODE* pDecode, D4D_BOOL increment);  
typedef LWord (*GET_DATA_LONG_FUNC)(D4D_D4DBMP_DECODE* pDecode, D4D_BOOL increment);  

#define GET_DATA_BYTE(pBmp_dec, inc) ((GET_DATA_BYTE_FUNC)pBmp_dec->GetDataFunc)(pBmp_dec, inc)
#define GET_DATA_WORD(pBmp_dec, inc) ((GET_DATA_WORD_FUNC)pBmp_dec->GetDataFunc)(pBmp_dec, inc)
#define GET_DATA_LONG(pBmp_dec, inc) ((GET_DATA_LONG_FUNC)pBmp_dec->GetDataFunc)(pBmp_dec, inc)


/******************************************************************************
* Constants
******************************************************************************/

#define D4D_BMP_COMPRESS_MASK   0x0001

#define D4D_BMP_COMPRESSED      0x0001
#define D4D_BMP_UNCOMPRESSED    0x0000

#define D4D_BMP_TYPE_MASK       0x001E

#define D4D_BMP_16MNOPAL        0x0014
#define D4D_BMP_256KNOPAL       0x0012
#define D4D_BMP_65536NOPAL      0x0000
#define D4D_BMP_4096NOPAL       0x0002
#define D4D_BMP_256NOPAL        0x0004


#define D4D_BMP_PAL256_32       0x0006
#define D4D_BMP_PAL16_8         0x0008
#define D4D_BMP_PAL4            0x000A
#define D4D_BMP_PAL2            0x000C
#define D4D_BMP_MON_VER         0x0010

#define D4D_BMP_CLR_DEPTH_MASK  0x0700

#define D4D_BMP_CLR_DEPTH256    0x0000
#define D4D_BMP_CLR_DEPTH128    0x0100
#define D4D_BMP_CLR_DEPTH64     0x0200
#define D4D_BMP_CLR_DEPTH32     0x0300
#define D4D_BMP_CLR_DEPTH16     0x0400
#define D4D_BMP_CLR_DEPTH8      0x0500
#define D4D_BMP_CLR_DEPTH4      0x0600
#define D4D_BMP_CLR_DEPTH2      0x0700

#define D4D_BMP_ENDIAN_FLAG     0x8000

#define D4D_BMP_PAL_256         (D4D_BMP_PAL256_32 | D4D_BMP_CLR_DEPTH256)
#define D4D_BMP_PAL_16          (D4D_BMP_PAL16_8 | D4D_BMP_CLR_DEPTH16)
#define D4D_BMP_PAL_2           (D4D_BMP_PAL2 | D4D_BMP_CLR_DEPTH2)


#ifndef D4D_BMP_16MNOPAL_ENABLE
  #define D4D_BMP_16MNOPAL_ENABLE D4D_FALSE
#endif  

#ifndef D4D_BMP_256KNOPAL_ENABLE
  #define D4D_BMP_256KNOPAL_ENABLE D4D_FALSE
#endif  

#ifndef D4D_BMP_65536NOPAL_ENABLE
  #define D4D_BMP_65536NOPAL_ENABLE D4D_FALSE
#endif  

#ifndef D4D_BMP_256NOPAL_ENABLE
  #define D4D_BMP_256NOPAL_ENABLE D4D_FALSE
#endif  

#ifndef D4D_BMP_PAL_256_ENABLE
  #define D4D_BMP_PAL_256_ENABLE D4D_FALSE
#endif  

#ifndef D4D_BMP_PAL_16_ENABLE
  #define D4D_BMP_PAL_16_ENABLE D4D_FALSE
#endif  

#ifndef D4D_BMP_PAL_2_ENABLE
  #define D4D_BMP_PAL_2_ENABLE D4D_FALSE
#endif  

/******************************************************************************
* External source constants
******************************************************************************/


#define D4D_IMG_HEADER_D4DINT_SIZE sizeof(D4D_IMG_HEADER_D4DINT)

   

/******************************************************************************
* Global D4D bitmap drawing level functions
******************************************************************************/

#if 1
  void D4D_ImgDecIntD4DDraw(D4D_COOR x, D4D_COOR y, D4D_CHAR* pFileName, D4D_BOOL greyScale, D4D_COOR radius);
  D4D_COOR D4D_ImgDecIntD4DGetWidth(D4D_CHAR* pFileName);
  D4D_COOR D4D_ImgDecIntD4DGetHeight(D4D_CHAR* pFileName);
  D4D_SIZE D4D_ImgDecIntD4DGetSize(D4D_CHAR* pFileName); 
  int D4D_ImgDecIntD4DGetHeader(D4D_CHAR* pFileName, void* pBuff, int max_size);
#else
  #define D4D_ImgDecIntD4DDraw       NULL
  #define D4D_ImgDecIntD4DGetWidth   NULL
  #define D4D_ImgDecIntD4DGetHeight  NULL
  #define D4D_ImgDecIntD4DGetSize    NULL
  #define D4D_ImgDecIntD4DGetHeader  NULL
#endif                     
                     
#endif /* __D4D_IMGDEC_D4DINT_H */