/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d.h
*
* @author    r50233
* 
* @version   0.0.88.0
* 
* @date      Aug-9-2012
* 
* @brief     D4D Driver main header file
*
*******************************************************************************/

#ifndef __D4D_H
#define __D4D_H

/******************************************************************************
* Includes
******************************************************************************/

/**************************************************************//*!
*
* Types used by driver (standard HCxx: Byte, Word, LWord, ...)
*
******************************************************************/


/**************************************************************//*!
*
* Implementation header files
*
******************************************************************/
#include "d4d_user_cfg.h"
#include "common_files/d4d_types.h"
#include "common_files/d4d_base.h"
#include "common_files/d4d_object.h"
#include "common_files/d4d_low.h"
#include "common_files/d4d_bmp.h"
#include "common_files/d4d_extsrc.h" 
#include "common_files/d4d_font.h"
#include "common_files/d4d_math.h"
#include "common_files/d4d_scheme.h"
#include "common_files/d4d_screen.h"
#include "graphic_objects/d4d_picture.h"
#include "graphic_objects/d4d_button.h"
#include "graphic_objects/d4d_check_box.h"
#include "graphic_objects/d4d_gauge.h"
#include "graphic_objects/d4d_slider.h"
#include "graphic_objects/d4d_label.h"
#include "graphic_objects/d4d_icon.h"
#include "graphic_objects/d4d_menu.h"
#include "graphic_objects/d4d_graph.h"
#include "graphic_objects/d4d_scroll_bar.h"
#include "graphic_objects/d4d_console.h"
#include "graphic_objects/d4d_text_box.h"
#include "graphic_objects/d4d_progress_bar.h"
#include "graphic_objects/d4d_radio_button.h"
#include "graphic_objects/d4d_group_box.h"
#include "graphic_objects/d4d_list_box.h"
#include "graphic_objects/d4d_edit_box.h"

//#include <string.h>
 // #include <stdlib.h>
/******************************************************************************
* Constants
******************************************************************************/

/**************************************************************//*!
*
* Basic driver configuration
*
******************************************************************/ 


/******************************************************************************
* Types
******************************************************************************/

/******************************************************************************
* Global functions
******************************************************************************/

/**************************************************************//*!
*
* Main driver API Functions
*
******************************************************************/

/* General driver API */
D4D_BOOL D4D_Init(D4D_SCREEN* pInitScreen);
void D4D_Poll(void);
void D4D_SetOrientation(D4D_ORIENTATION orient);
void D4D_KeysChanged(D4D_KEYS keys);
void D4D_NewKeyEvent(D4D_KEY_SCANCODE scanCode);
void D4D_TimeTickPut(void);
void D4D_FlushOutput(void);
D4D_POINT D4D_GetTouchScreenCoordinates(D4D_OBJECT* pObject);
void D4D_CheckTouchScreen(void);
void D4D_PutTouchScreen(D4D_BOOL touched, D4D_COOR x, D4D_COOR y);
void D4D_PutRawTouchScreen(D4D_BOOL touched, D4D_COOR x, D4D_COOR y);



/* screen API */


/* object API */
void D4D_EnableTabStop(D4D_OBJECT_PTR pObj, D4D_BOOL bEnable);
void D4D_EnableObject(D4D_OBJECT_PTR pObj, D4D_BOOL bEnable);
void D4D_EnableTouchScreen(D4D_OBJECT_PTR pObj, D4D_BOOL bEnable, D4D_BOOL bFastTouch);
void D4D_ShowObject(D4D_OBJECT_PTR pObject, D4D_BOOL bShow);
void D4D_EnableTimeTicks(D4D_OBJECT_PTR pObj, D4D_BOOL bEnable);
D4D_OBJECT_PTR D4D_GetFocusedObject(D4D_SCREEN* pScreen);
void D4D_FocusSet(D4D_SCREEN* pScreen, D4D_OBJECT_PTR pObject);
void D4D_FocusNextObject(D4D_SCREEN* pScreen, D4D_BOOL bInitialSearch);
void D4D_FocusPrevObject(D4D_SCREEN* pScreen);
void D4D_InvalidateObject(D4D_OBJECT_PTR pObject, D4D_BOOL bComplete);
void* D4D_GetUserPointer(D4D_OBJECT *pThis);
void D4D_CaptureKeys(D4D_OBJECT_PTR pObj);
D4D_OBJECT* D4D_GetCapturedObject(void);

/* Color Schemes */
D4D_CLR_SCHEME* D4D_ObjectGetScheme(D4D_OBJECT * pObj);
D4D_CLR_SCHEME* D4D_ScreenGetScheme(D4D_SCREEN * pScreen);
D4D_CLR_SCHEME* D4D_GetDefaultScheme(void);
void D4D_SetDefaultScheme(D4D_CLR_SCHEME* pScheme);
D4D_COLOR D4D_ObjectGetForeColor(D4D_OBJECT * pObj, D4D_OBJECT_DRAWFLAGS draw);
D4D_COLOR D4D_ObjectGetBckgColor(D4D_OBJECT * pObj, D4D_OBJECT_DRAWFLAGS draw);
D4D_COLOR D4D_ObjectGetForeFillColor(D4D_OBJECT * pObj);
D4D_COLOR D4D_ObjectGetBckgFillColor(D4D_OBJECT * pObj);
                             
/* general helpers */
D4D_KEYS D4D_GetKeys(void);
void D4D_ClearKeysBuffer(void);
void D4D_ClearTouchScreenEvents(void);
void D4D_ClearScreen(D4D_COLOR color);

void D4D_PutPixel(D4D_POINT* ppt, D4D_LINETYPE type, D4D_COLOR color);
void D4D_PutPixelTo(D4D_LINETYPE type, D4D_COLOR color);
void D4D_PutPixelXY(D4D_COOR x, D4D_COOR y, D4D_LINETYPE type, D4D_COLOR color);
void D4D_MoveTo(D4D_POINT* ppt);
void D4D_MoveToXY(D4D_COOR x, D4D_COOR y);
D4D_POINT D4D_SwapCoor(D4D_POINT point);
void D4D_LineTo(D4D_POINT* ppt, D4D_LINETYPE ltype, D4D_COLOR color);
void D4D_LineToXY(D4D_COOR x, D4D_COOR y, D4D_LINETYPE ltype, D4D_COLOR color);
void D4D_FillRect(D4D_POINT* ppt, D4D_SIZE* psz, D4D_COLOR color);
void D4D_FillRectXY(D4D_COOR x1, D4D_COOR y1, D4D_COOR x2, D4D_COOR y2, D4D_COLOR color);
void D4D_FillRectTo(D4D_POINT* ppt, D4D_COLOR color);
void D4D_FillRectToXY(D4D_COOR x, D4D_COOR y, D4D_COLOR color);
void D4D_FillRectColorScale(D4D_POINT* ppt, D4D_SIZE* psz, D4D_COLOR startColor, D4D_COLOR endColor, D4D_DIRECTION direction);
void D4D_FillRectColorScaleXY(D4D_COOR x1, D4D_COOR y1, D4D_COOR x2, D4D_COOR y2, D4D_COLOR startColor, D4D_COLOR endColor, D4D_DIRECTION direction);
void D4D_FillRectColorScaleTo(D4D_POINT* ppt, D4D_COLOR startColor, D4D_COLOR endColor, D4D_DIRECTION direction);
void D4D_FillRectColorScaleToXY(D4D_COOR x, D4D_COOR y, D4D_COLOR startColor, D4D_COLOR endColor, D4D_DIRECTION direction);
void D4D_Rect(D4D_POINT* ppt, D4D_SIZE* psz, D4D_LINETYPE ltype, D4D_COLOR color);
void D4D_RectXY(D4D_COOR x1, D4D_COOR y1, D4D_COOR x2, D4D_COOR y2, D4D_LINETYPE ltype, D4D_COLOR color);
void D4D_RectTo(D4D_POINT* ppt, D4D_LINETYPE ltype, D4D_COLOR color);
void D4D_RectToXY(D4D_COOR x, D4D_COOR y, D4D_LINETYPE ltype, D4D_COLOR color);
void D4D_Box(D4D_POINT* ppt, D4D_SIZE* psz, D4D_LINETYPE ltype, D4D_COLOR colorLine, D4D_COLOR colorFill);
void D4D_BoxXY(D4D_COOR x1, D4D_COOR y1, D4D_COOR x2, D4D_COOR y2, D4D_LINETYPE ltype, D4D_COLOR colorLine, D4D_COLOR colorFill);
void D4D_BoxTo(D4D_POINT* ppt, D4D_LINETYPE ltype, D4D_COLOR colorLine, D4D_COLOR colorFill);
void D4D_BoxToXY(D4D_COOR x, D4D_COOR y, D4D_LINETYPE ltype, D4D_COLOR colorLine, D4D_COLOR colorFill);
void D4D_FillCircle(D4D_POINT* pCenter, D4D_COOR r, D4D_COLOR color);
void D4D_FillCircleXY(D4D_COOR x, D4D_COOR y, D4D_COOR r, D4D_COLOR color);
void D4D_Circle(D4D_POINT* pCenter, D4D_COOR r, D4D_LINETYPE ltype, D4D_COLOR color);
void D4D_CircleXY(D4D_COOR x, D4D_COOR y, D4D_COOR r, D4D_LINETYPE ltype, D4D_COLOR color);
void D4D_Bevel(D4D_POINT* ppt, D4D_SIZE* psz, D4D_COLOR color, D4D_BEVEL type, D4D_COOR radius);

#define D4D_DrawBmp( ppt, pBmp, greyScale) D4D_DrawRBmpXY((ppt)->x, (ppt)->y, pBmp, greyScale, 0)
#define D4D_DrawBmpXY(x, y, pBmp, greyScale) D4D_DrawRBmpXY(x, y, pBmp, greyScale, 0)

#if D4D_ROUND_CORNER_ENABLE != D4D_FALSE

  void D4D_RRect(D4D_POINT* ppt, D4D_SIZE* psz, D4D_LINETYPE ltype, D4D_COLOR color, D4D_COOR radius);
  void D4D_RRectXY(D4D_COOR x1, D4D_COOR y1, D4D_COOR x2, D4D_COOR y2, D4D_LINETYPE ltype, D4D_COLOR color, D4D_COOR radius);
  void D4D_RRectTo(D4D_POINT* ppt, D4D_LINETYPE ltype, D4D_COLOR color, D4D_COOR radius);
  void D4D_RRectToXY(D4D_COOR x, D4D_COOR y, D4D_LINETYPE ltype, D4D_COLOR color, D4D_COOR radius);
  void D4D_FillRRect(D4D_POINT* ppt, D4D_SIZE* psz, D4D_COLOR color, D4D_COOR radius);
  void D4D_FillRRectXY(D4D_COOR x1, D4D_COOR y1, D4D_COOR x2, D4D_COOR y2, D4D_COLOR color, D4D_COOR radius);
  void D4D_FillRRectTo(D4D_POINT* ppt, D4D_COLOR color, D4D_COOR radius);
  void D4D_FillRRectToXY(D4D_COOR x, D4D_COOR y, D4D_COLOR color, D4D_COOR radius);
  void D4D_RBox(D4D_POINT* ppt, D4D_SIZE* psz, D4D_LINETYPE ltype, D4D_COLOR colorLine, D4D_COLOR colorFill, D4D_COOR radius);
  void D4D_RBoxXY(D4D_COOR x1, D4D_COOR y1, D4D_COOR x2, D4D_COOR y2, D4D_LINETYPE ltype, D4D_COLOR colorLine, D4D_COLOR colorFill, D4D_COOR radius);
  void D4D_RBoxTo(D4D_POINT* ppt, D4D_LINETYPE ltype, D4D_COLOR colorLine, D4D_COLOR colorFill, D4D_COOR radius);
  void D4D_RBoxToXY(D4D_COOR x, D4D_COOR y, D4D_LINETYPE ltype, D4D_COLOR colorLine, D4D_COLOR colorFill, D4D_COOR radius);
  void D4D_FillQuadrant(D4D_POINT* pCenter, D4D_COOR radius, D4D_COLOR color, D4D_QUADRANT quadrant);
  void D4D_FillQuadrantXY(D4D_COOR x, D4D_COOR y, D4D_COOR radius, D4D_COLOR color, D4D_QUADRANT quadrant);
  void D4D_Quadrant(D4D_POINT* pCenter, D4D_COOR radius, D4D_LINETYPE ltype, D4D_COLOR color, D4D_QUADRANT quadrant);
  void D4D_QuadrantXY(D4D_COOR x, D4D_COOR y, D4D_COOR radius, D4D_LINETYPE ltype, D4D_COLOR color, D4D_QUADRANT quadrant);
  void D4D_FillRRectColorScale(D4D_POINT* ppt, D4D_SIZE* psz, D4D_COLOR startColor, D4D_COLOR endColor, D4D_DIRECTION direction, D4D_COOR radius); // experimental function
  void D4D_FillRRectColorScaleXY(D4D_COOR x1, D4D_COOR y1, D4D_COOR x2, D4D_COOR y2, D4D_COLOR startColor, D4D_COLOR endColor, D4D_DIRECTION direction, D4D_COOR radius); // same
  void D4D_FillRRectColorScaleTo(D4D_POINT* ppt, D4D_COLOR startColor, D4D_COLOR endColor, D4D_DIRECTION direction, D4D_COOR radius); // same
  void D4D_FillRRectColorScaleToXY(D4D_COOR x, D4D_COOR y, D4D_COLOR startColor, D4D_COLOR endColor, D4D_DIRECTION direction, D4D_COOR radius); // same
  void D4D_DrawRBmpRect(D4D_POINT* ppt, D4D_SIZE* psz, const D4D_BMP* pBmp, D4D_BMP_PROPERTIES bmpProp, D4D_COLOR colorBkgd, D4D_BOOL greyScale, D4D_COOR radius);
 // void D4D_DrawRBmp(D4D_POINT* ppt, const D4D_BMP* pBmp, D4D_BOOL greyScale, D4D_COOR radius);
  void D4D_DrawTextRRectTab(D4D_POINT* ppt, D4D_SIZE* psz, D4D_STRING* buffText, D4D_TAB* pTab, D4D_COLOR colorText, D4D_COLOR colorBkgd, D4D_COOR radius);
  void D4D_DrawTextRRectTabTo(D4D_POINT* ppt, D4D_STRING* buffText, D4D_TAB* pTab, D4D_COLOR colorText, D4D_COLOR colorBkgd, D4D_COOR radius);
  void D4D_DrawTextRRectTabXY(D4D_COOR x1, D4D_COOR y1, D4D_COOR x2, D4D_COOR y2, D4D_STRING* buffText, D4D_TAB* pTab, D4D_COLOR colorText, D4D_COLOR colorBkgd, D4D_COOR radius);
  void D4D_DrawTextRRectTabToXY(D4D_COOR x, D4D_COOR y, D4D_STRING* buffText, D4D_TAB* pTab, D4D_COLOR colorText, D4D_COLOR colorBkgd, D4D_COOR radius);
  
#else
  #define D4D_RRect(ppt, psz, ltype, color, radius)                                             D4D_Rect(ppt, psz, ltype, color)
  #define D4D_RRectXY(x1, y1, x2, y2, ltype, color, radius)                                     D4D_RectXY(x1, y1, x2, y2, ltype, color)
  #define D4D_RRectTo(ppt, ltype, color, radius)                                                D4D_RectTo(ppt, ltype, color)
  #define D4D_RRectToXY(x, y, ltype, color, radius)                                             D4D_RectToXY(x, y, ltype, color)
  #define D4D_FillRRect(ppt, psz, color, radius)                                                D4D_FillRect(ppt, psz, color)
  #define D4D_FillRRectXY(x1, y1, x2, y2, color, radius)                                        D4D_FillRectXY(x1, y1, x2, y2, color)               
  #define D4D_FillRRectTo(ppt, color, radius)                                                   D4D_FillRectTo(ppt, color)
  #define D4D_FillRRectToXY(x, y, color, radius)                                                D4D_FillRectToXY(x, y, color)
  #define D4D_RBox(ppt, psz, ltype, colorLine, colorFill, radius)                               D4D_Box(ppt, psz, ltype, colorLine, colorFill)
  #define D4D_RBoxXY(x1, y1, x2, y2, ltype, colorLine, colorFill, radius)                       D4D_BoxXY(x1, y1, x2, y2, ltype, colorLine, colorFill)
  #define D4D_RBoxTo(ppt, ltype, colorLine, colorFill, radius)                                  D4D_BoxTo(ppt, ltype, colorLine, colorFill)
  #define D4D_RBoxToXY(x, y, ltype, colorLine, colorFill, radius)                               D4D_BoxToXY(x, y, ltype, colorLine, colorFill)
  #define D4D_FillQuadrant(pCenter, radius, color, quadrant)                                    ;
  #define D4D_FillQuadrantXY(x, y, radius, color, quadrant)                                     ;
  #define D4D_Quadrant(pCenter, radius, ltype, color, quadrant)                                 ;
  #define D4D_QuadrantXY(x, y, radius, ltype, color, quadrant)                                  ;
  #define D4D_FillRRectColorScale(ppt, psz, startColor, endColor, direction, radius)            D4D_FillRectColorScale(ppt, psz, startColor, endColor, direction)
  #define D4D_FillRRectColorScaleXY(x1, y1, x2, y2, startColor, endColor, direction, radius)    D4D_FillRectColorScaleXY(x1, y1, x2, y2, startColor, endColor, direction)
  #define D4D_FillRRectColorScaleTo(ppt, startColor, endColor, direction, radius)               D4D_FillRectColorScaleTo(ppt, startColor, endColor, direction)
  #define D4D_FillRRectColorScaleToXY(x, y, startColor, endColor, direction, radius)            D4D_FillRectColorScaleToXY(x, y, startColor, endColor, direction)
  #define D4D_DrawRBmpRect(ppt,  psz, pBmp, bmpProp, colorBkgd, greyScale, radius)              D4D_DrawBmpRect(ppt,  psz, pBmp, bmpProp, colorBkgd, greyScale)
  #define D4D_DrawTextRRectTab(ppt, psz, buffText, pTab, colorText, colorBkgd, radius)          D4D_DrawTextRectTab(ppt, psz, buffText, pTab, colorText, colorBkgd)
  #define D4D_DrawTextRRectTabTo(ppt, buffText, pTab, colorText, colorBkgd, radius)             D4D_DrawTextRectTabTo(ppt, buffText, pTab, colorText, colorBkgd)                   
  #define D4D_DrawTextRRectTabXY(x1, y1, x2, y2, buffText, pTab, colorText, colorBkgd, radius)  D4D_DrawTextRectTabXY(x1, y1, x2, y2, buffText, pTab, colorText, colorBkgd)
  #define D4D_DrawTextRRectTabToXY(x, y, buffText, pTab, colorText, colorBkgd, radius)          D4D_DrawTextRectTabToXY(x, y, buffText, pTab, colorText, colorBkgd)
  
#endif

#define D4D_DrawRBmp( ppt, pBmp, greyScale, radius) D4D_DrawRBmpXY((ppt)->x, (ppt)->y, pBmp, greyScale, radius)
void D4D_DrawRBmpXY(D4D_COOR x, D4D_COOR y, const D4D_BMP* pBmp, D4D_BOOL greyScale, D4D_COOR radius);

void D4D_DrawBmpRect(D4D_POINT* ppt, D4D_SIZE* psz, const D4D_BMP* pBmp, D4D_BMP_PROPERTIES bmpProp, D4D_COLOR colorBkgd, D4D_BOOL greyScale);
//void D4D_DrawText(D4D_POINT* ppt, D4D_STRING* buffText, D4D_COLOR colorText, D4D_COLOR colorBkgd);
//void D4D_DrawTextXY(D4D_COOR x, D4D_COOR y, D4D_STRING* buffText, D4D_COLOR colorText, D4D_COLOR colorBkgd);
//void D4D_DrawTextTab(D4D_POINT* ppt, D4D_STRING* buffText, D4D_COOR* pTabTable, D4D_COLOR colorText, D4D_COLOR colorBkgd);
void D4D_DrawTextTabXY(D4D_COOR x, D4D_COOR y, D4D_STRING* buffText, D4D_TAB* pTab, D4D_COLOR colorText, D4D_COLOR colorBkgd);


void D4D_DrawTextRectTab(D4D_POINT* ppt, D4D_SIZE* psz, D4D_STRING* buffText, D4D_TAB* pTab, D4D_COLOR colorText, D4D_COLOR colorBkgd);
void D4D_DrawTextRectTabTo(D4D_POINT* ppt, D4D_STRING* buffText, D4D_TAB* pTab, D4D_COLOR colorText, D4D_COLOR colorBkgd);
void D4D_DrawTextRectTabXY(D4D_COOR x1, D4D_COOR y1, D4D_COOR x2, D4D_COOR y2, D4D_STRING* buffText, D4D_TAB* pTab, D4D_COLOR colorText, D4D_COLOR colorBkgd);
void D4D_DrawTextRectTabToXY(D4D_COOR x, D4D_COOR y, D4D_STRING* buffText, D4D_TAB* pTab, D4D_COLOR colorText, D4D_COLOR colorBkgd);

//void D4D_DrawTextRectTab(D4D_POINT* ppt, D4D_SIZE* psz, D4D_STRING* buffText, D4D_TAB* pTab, D4D_COLOR colorText, D4D_COLOR colorBkgd);
//void D4D_DrawTextRectTabTo(D4D_POINT* ppt, D4D_STRING* buffText, D4D_TAB* pTab, D4D_COLOR colorText, D4D_COLOR colorBkgd);
//void D4D_DrawTextRectTabXY(D4D_COOR x1, D4D_COOR y1, D4D_COOR x2, D4D_COOR y2, D4D_STRING* buffText, D4D_TAB* pTab, D4D_COLOR colorText, D4D_COLOR colorBkgd);
//void D4D_DrawTextRectTabToXY(D4D_COOR x, D4D_COOR y, D4D_STRING* buffText, D4D_TAB* pTab, D4D_COLOR colorText, D4D_COLOR colorBkgd);


//void D4D_DrawTextRect(D4D_POINT* ppt, D4D_SIZE* psz, D4D_STRING* buffText, D4D_COLOR colorText, D4D_COLOR colorBkgd);
//void D4D_DrawTextRectTo(D4D_POINT* ppt, D4D_STRING* buffText, D4D_COLOR colorText, D4D_COLOR colorBkgd);
//void D4D_DrawTextRectXY(D4D_COOR x1, D4D_COOR y1, D4D_COOR x2, D4D_COOR y2, D4D_STRING* buffText, D4D_COLOR colorText, D4D_COLOR colorBkgd);
//void D4D_DrawTextRectToXY(D4D_COOR x, D4D_COOR y, D4D_STRING* buffText, D4D_COLOR colorText, D4D_COLOR colorBkgd);





D4D_SIZE D4D_GetBmpSize(const D4D_BMP* pBmp);
D4D_COOR D4D_GetBmpWidth(const D4D_BMP* pBmp);
D4D_COOR D4D_GetBmpHeight(const D4D_BMP* pBmp);
int D4D_GetBmpHeader(D4D_BMP* pBmp, void* pBuff, int max_size);

void D4D_SetText(D4D_OBJECT_PTR pObject, D4D_TCHAR* pText);
void D4D_SetTextProperties(D4D_OBJECT_PTR pObject, D4D_TEXT_PROPERTIES property);
void D4D_SetFontProperties(D4D_OBJECT_PTR pObject, D4D_FONT_PROPERTIES property);
D4D_INDEX D4D_GetTextLength(D4D_TCHAR* pText);
D4D_COOR D4D_GetTextWidth(D4D_FONT ix, D4D_TCHAR* pText);
//D4D_COOR D4D_GetTextBuffWidth(D4D_STRING* text_buffer);
D4D_COOR D4D_GetTextBuffWidthTab(D4D_STRING* text_buffer, D4D_TAB* pTab);
D4D_INDEX D4D_GetTextMaxLength(D4D_STRING* pString, D4D_COOR maxWidth);

Byte D4D_SprintDecU8(Byte val, D4D_TCHAR *pText, D4D_TCHAR fill);
Byte D4D_SprintDecS8(sByte val, D4D_TCHAR *pText, D4D_TCHAR fill);
Byte D4D_SprintDecU16(Word val, D4D_TCHAR *pText, D4D_TCHAR fill);
Byte D4D_SprintDecS16(sWord val, D4D_TCHAR *pText, D4D_TCHAR fill);
Byte D4D_SprintDecU32(LWord val, D4D_TCHAR *pText, D4D_TCHAR fill);
Byte D4D_SprintDecS32(sLWord val, D4D_TCHAR *pText, D4D_TCHAR fill);

void D4D_CalibrateTouchScreen(void);
D4D_TOUCHSCREEN_CALIB D4D_GetTouchScreenCalibration(void);
void D4D_SetTouchScreenCalibration(D4D_TOUCHSCREEN_CALIB newCalib);

D4D_COLOR D4D_GetCrossColor(D4D_COLOR startColor, D4D_COLOR endColor, Byte value);
D4D_COLOR D4D_ChangeColorIntesity(D4D_COLOR color, sByte intensity);

D4D_COLOR D4D_GetGreyScale(D4D_COLOR color);
D4D_COOR D4D_GetLongerSide(D4D_SIZE* pSz);


#endif /* __D4D_H */









