/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_button.h
*
* @author    r50233
* 
* @version   0.0.44.0
* 
* @date      Jun-29-2012
* 
* @brief     D4D Driver button object header file
*
*******************************************************************************/

#ifndef __D4D_BUTTON_H
#define __D4D_BUTTON_H

extern const D4D_OBJECT_SYS_FUNCTION d4d_btnSysFunc;

/******************************************************************************
* Constants
******************************************************************************/
#ifndef D4D_BTN_3D_ENABLE
  #define D4D_BTN_3D_ENABLE D4D_FALSE
#endif

#ifndef D4D_BTN_3D_FRAME_WIDTH
  #define D4D_BTN_3D_FRAME_WIDTH 2
#endif

#ifndef D4D_BTN_3D_SHIFT
  #define D4D_BTN_3D_SHIFT 1
#endif


/******************************************************************************
* Types
******************************************************************************/

#define D4D_BTN_F_3D            (0x01 << D4D_OBJECT_F_OBJECT_SHIFT)
#define D4D_BTN_F_CONT_RECT     (0x02 << D4D_OBJECT_F_OBJECT_SHIFT)

// button configuration (goes to ROM by default)

#ifndef D4D_BTN_F_DEFAULT
  #define D4D_BTN_F_DEFAULT  (D4D_OBJECT_F_VISIBLE | D4D_OBJECT_F_ENABLED | D4D_OBJECT_F_TABSTOP | D4D_OBJECT_F_TOUCHENABLE | D4D_OBJECT_F_FOCUSRECT)
#endif

#ifndef D4D_BTN_TXT_PRTY_DEFAULT
  #define D4D_BTN_TXT_PRTY_DEFAULT  (D4D_ALIGN_H_CENTER_MASK | D4D_ALIGN_V_CENTER_MASK)
#endif

#ifndef D4D_BTN_FNT_PRTY_DEFAULT
  #define D4D_BTN_FNT_PRTY_DEFAULT  ( 0 )
#endif

#ifndef D4D_BUTTON_3D_FORE_INTENSITY
  #define D4D_BUTTON_3D_FORE_INTENSITY    D4D_BEVEL_FORE_INTENSITY
#endif

#ifndef D4D_BUTTON_3D_BCKG_INTENSITY
  #define D4D_BUTTON_3D_BCKG_INTENSITY    D4D_BEVEL_BCKG_INTENSITY
#endif

typedef Byte D4D_BUTTON_STATUS;

#define D4D_BUTTON_STATUS_PRESSED_MASK    (0x01)
#define D4D_BUTTON_STATUS_1ST_TOUCH_MASK  (0x02)


typedef struct 
{
    D4D_STRING textBuff;    // button text
    const D4D_BMP*  pBmpNormal;    // normal state bitmap
    const D4D_BMP*  pBmpFocus;     // focused state bitmap
    D4D_BUTTON_STATUS*  pStatus;    
    void (*OnClicked)(D4D_OBJECT* pThis);
} D4D_BUTTON;


/******************************************************************************
* Macros 
******************************************************************************/

// getting the BUTTON structure from general OBJECT
#define D4D_GET_BUTTON(pObj) ((D4D_BUTTON*)((pObj)->pParam))
#define D4D_GET_BUTTON_STATUS(pObj) ((D4D_BUTTON_STATUS*)(D4D_GET_BUTTON(pObj)->pStatus))


/*********************************************************
*
* static declaration macros
*
*********************************************************/

#define _D4D_DECLARE_BUTTON(type, name, text, x, y, cx, cy, radius, pMargin, pRelations, flags, pbmpN, pbmpF, pScheme, fontId, pUser, onclick, pOnUsrMsg) \
    static D4D_BUTTON_STATUS  name##_status = { 0 }; \
    static D4D_STR_PROPERTIES name##_strPrties = { D4D_BTN_FNT_PRTY_DEFAULT, D4D_BTN_TXT_PRTY_DEFAULT}; \
    static type D4D_BUTTON name##_params = \
    { \
        { text, D4D_TEXT_LEN(text), fontId, &name##_strPrties, D4D_OBJECT_MAX_TEXT_LEN(text), 0}, /* textBuff */ \
        pbmpN,      /* pBmpNormal */ \
        pbmpF,      /* pBmpFocus */ \
        &(name##_status), /* Status clear */ \
        onclick     /* Click event */ \
    }; \
    \
    D4D_DECLARE_OBJECT(type, name, x, y, cx, cy, radius, pMargin, pRelations, pOnUsrMsg,  &d4d_btnSysFunc, &(name##_params), flags, pUser, pScheme)

#define D4D_DECLARE_BUTTON(name, text, x, y, cx, cy, flags, pbmpN, pbmpF, pScheme, fontId, pUser, onclick, pOnUsrMsg) \
  _D4D_DECLARE_BUTTON(D4D_CONST, name, text, x, y, cx, cy, 0, NULL, NULL, flags, pbmpN, pbmpF, pScheme, fontId, pUser, onclick, pOnUsrMsg)
  
#define D4D_DECLARE_STD_BUTTON(name, text, x, y, cx, cy, bmpN, bmpF, fontId, onclick) \
    D4D_DECLARE_BUTTON(name, text, x, y, cx, cy, (D4D_BTN_F_DEFAULT), bmpN, bmpF, NULL, fontId, NULL, onclick, NULL)

#define D4D_DECLARE_TXT_BUTTON(name, text, x, y, cx, cy, fontId, onclick) \
    D4D_DECLARE_STD_BUTTON(name, text, x, y, cx, cy,  NULL, NULL, \
        fontId, onclick)


// Rounded button definition
#define D4D_DECLARE_RBUTTON(name, text, x, y, cx, cy, radius, flags, pbmpN, pbmpF, pScheme, fontId, pUser, onclick, pOnUsrMsg) \
  _D4D_DECLARE_BUTTON(D4D_CONST, name, text, x, y, cx, cy, radius, NULL, NULL, flags, pbmpN, pbmpF, pScheme, fontId, pUser, onclick, pOnUsrMsg)

#define D4D_DECLARE_STD_RBUTTON(name, text, x, y, cx, cy, radius, bmpN, bmpF, fontId, onclick) \
    D4D_DECLARE_RBUTTON(name, text, x, y, cx, cy, radius, (D4D_BTN_F_DEFAULT), bmpN, bmpF, NULL, fontId, NULL, onclick, NULL)

#define D4D_DECLARE_TXT_RBUTTON(name, text, x, y, cx, cy, radius, fontId, onclick) \
    D4D_DECLARE_STD_RBUTTON(name, text, x, y, cx, cy,  radius, NULL, NULL, \
        fontId, onclick)

// IN RAM instantion macros
#define D4D_DECLARE_BUTTON_INRAM(name, text, x, y, cx, cy, flags, pbmpN, pbmpF, pScheme, fontId, pUser, onclick, pOnUsrMsg) \
  _D4D_DECLARE_BUTTON(D4D_NO_CONST, name, text, x, y, cx, cy, 0, NULL, NULL, flags, pbmpN, pbmpF, pScheme, fontId, pUser, onclick, pOnUsrMsg)

#define D4D_DECLARE_STD_BUTTON_INRAM(name, text, x, y, cx, cy, bmpN, bmpF, fontId, onclick) \
    D4D_DECLARE_BUTTON_INRAM(name, text, x, y, cx, cy, (D4D_BTN_F_DEFAULT), bmpN, bmpF, NULL, fontId, NULL, onclick, NULL)

#define D4D_DECLARE_TXT_BUTTON_INRAM(name, text, x, y, cx, cy, fontId, onclick) \
    D4D_DECLARE_STD_BUTTON_INRAM(name, text, x, y, cx, cy,  NULL, NULL, \
        fontId, onclick)
    
// Rounded button definition
#define D4D_DECLARE_RBUTTON_INRAM(name, text, x, y, cx, cy, radius, flags, pbmpN, pbmpF, pScheme, fontId, pUser, onclick, pOnUsrMsg) \
  _D4D_DECLARE_BUTTON(D4D_NO_CONST, name, text, x, y, cx, cy, radius, NULL, NULL, flags, pbmpN, pbmpF, pScheme, fontId, pUser, onclick, pOnUsrMsg)

#define D4D_DECLARE_STD_RBUTTON_INRAM(name, text, x, y, cx, cy, radius, bmpN, bmpF, fontId, onclick) \
    D4D_DECLARE_RBUTTON_INRAM(name, text, x, y, cx, cy, radius, (D4D_BTN_F_DEFAULT), bmpN, bmpF, NULL, fontId, NULL, onclick, NULL)

#define D4D_DECLARE_TXT_RBUTTON_INRAM(name, text, x, y, cx, cy, radius, fontId, onclick) \
    D4D_DECLARE_STD_RBUTTON_INRAM(name, text, x, y, cx, cy,  radius, NULL, NULL, \
        fontId, onclick)

/******************************************************************************
* Global functions
******************************************************************************/

/*********************************************************
*
* Object API
*
*********************************************************/

// Obsolete functions, replaced by any general
#define  D4D_BtnSetText D4D_SetText

#endif /*__D4D_BUTTON_H*/












