/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_group_box.h
*
* @author    r50233
* 
* @version   0.0.4.0
* 
* @date      Jun-29-2012
* 
* @brief     D4D Driver groupBox object header file
*
*******************************************************************************/

#ifndef __D4D_GROUP_BOX_H
#define __D4D_GROUP_BOX_H

extern const D4D_OBJECT_SYS_FUNCTION d4d_groupBoxSysFunc;

/******************************************************************************
* Constants
******************************************************************************/

#ifndef D4D_GROUP_BOX_F_DEFAULT
  #define D4D_GROUP_BOX_F_DEFAULT  (D4D_OBJECT_F_VISIBLE | D4D_OBJECT_F_ENABLED | D4D_OBJECT_F_TABSTOP | D4D_OBJECT_F_TOUCHENABLE | D4D_OBJECT_F_FOCUSRECT | D4D_OBJECT_F_BEVEL_RAISED)
#endif

#ifndef D4D_GROUP_BOX_TEXT_OFFSET
  #define D4D_GROUP_BOX_TEXT_OFFSET 6
#endif

#ifndef D4D_GROUP_BOX_HEADER_MARGIN
  #define D4D_GROUP_BOX_HEADER_MARGIN 16
#endif

#ifndef D4D_GROUP_BOX_FRAME_MARGIN
  #define D4D_GROUP_BOX_FRAME_MARGIN 4
#endif


/******************************************************************************
* Types
******************************************************************************/

// groupBox configuration (goes to ROM by default)

typedef struct
{
    D4D_STRING textBuff;    // group box text    
} D4D_GROUP_BOX;


/******************************************************************************
* Macros 
******************************************************************************/

// getting the GROUP_BOX structure from general OBJECT
#define D4D_GET_GROUP_BOX(pObj) ((D4D_GROUP_BOX*)((pObj)->pParam))

/*********************************************************
*
* static declaration macros
*
*********************************************************/

#define _D4D_DECLARE_GROUP_BOX(type, name, x, y, cx, cy, radius, pMargin, pRelations, text, fontId, flags, pScheme, pUser, pOnUsrMsg) \
    static D4D_STR_PROPERTIES name##_strPrties = { D4D_FNT_PRTY_TRANSPARENT_NO_MASK, (D4D_TXT_PRTY_ALIGN_H_CENTER_MASK | D4D_TXT_PRTY_ALIGN_V_CENTER_MASK)}; \
    static type D4D_GROUP_BOX name##_params = \
    { \
        { text, sizeof(text), fontId,  &name##_strPrties, D4D_OBJECT_MAX_TEXT_LEN(text), 0}, /* textBuff */ \
    }; \
    \
    D4D_DECLARE_OBJECT(type, name, x, y, cx, cy, radius, pMargin, pRelations, pOnUsrMsg, &d4d_groupBoxSysFunc, &(name##_params), flags, pUser, pScheme)
   
   
   
#define D4D_DECLARE_GROUP_BOX_BEGIN(name, x, y, cx, cy, radius, pParent, pMargin, text, fontId, flags, pScheme, pUser, pOnUsrMsg)\
    extern const D4D_OBJECT* const name##_pRelations[];\
    _D4D_DECLARE_GROUP_BOX(D4D_CONST, name, x, y, cx, cy, radius, pMargin, name##_pRelations, text, fontId, flags, pScheme, pUser, pOnUsrMsg)\
    D4D_DECLARE_OBJECT_RELATIONS_BEGIN(name##_pRelations, pParent)
       
#define D4D_DECLARE_GROUP_BOX_OBJECT(name)  D4D_DECLARE_OBJECT_RELATIONS_CHILD(name)
   
#define D4D_DECLARE_GROUP_BOX_OBJECT_END    D4D_DECLARE_OBJECT_RELATIONS_END

#define D4D_DECLARE_STD_GROUP_BOX_BEGIN(name, x, y, cx, cy, radius, pParent, text, fontId) \
  D4D_DECLARE_GROUP_BOX_BEGIN(name, x, y, cx, cy, radius, pParent, &groupBox_marginDefault, text, fontId, (D4D_GROUP_BOX_F_DEFAULT), NULL, NULL, NULL)


//IN RAM instantions macros
#define D4D_DECLARE_GROUP_BOX_BEGIN_INRAM(name, x, y, cx, cy, radius, pParent, pMargin, text, fontId, flags, pScheme, pUser, pOnUsrMsg)\
    extern const D4D_OBJECT* const name##_pRelations[];\
    _D4D_DECLARE_GROUP_BOX(D4D_NO_CONST, name, x, y, cx, cy, radius, pMargin, name##_pRelations, text, fontId, flags, pScheme, pUser, pOnUsrMsg)\
    D4D_DECLARE_OBJECT_RELATIONS_BEGIN(name##_pRelations, pParent)      

#define D4D_DECLARE_STD_GROUP_BOX_BEGIN_INRAM(name, x, y, cx, cy, radius, pParent, text, fontId) \
  D4D_DECLARE_GROUP_BOX_BEGIN_INRAM(name, x, y, cx, cy, radius, pParent, &groupBox_marginDefault, text, fontId, (D4D_GROUP_BOX_F_DEFAULT), NULL,  NULL, NULL) 

/******************************************************************************
* Global variables
******************************************************************************/
   
extern D4D_MARGIN groupBox_marginDefault;

/******************************************************************************
* Global functions
******************************************************************************/



#endif /* __D4D_GROUP_BOX_H */