/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4dlcdhw_k70_lcdc.h
*
* @author    b01119
* 
* @version   0.0.6.0
* 
* @date      Aug-22-2012
* 
* @brief     D4D driver - k70_lcdc hardware lcd driver function header file 
*
*******************************************************************************/

#ifndef __D4DLCDHW_K70_LCDC_H
#define __D4DLCDHW_K70_LCDC_H

  #if (D4D_MK_STR(D4D_LLD_LCD_HW) == d4dlcdhw_k70_lcdc_ID)

    /******************************************************************************
    * Includes
    ******************************************************************************/
    #include "d4dlcdhw_k70_lcdc_cfg.h"
    
    /******************************************************************************
    * Constants
    ******************************************************************************/

    // TWR-LCDRBG board revA - with TWR-K70 on SPI2
    #define TWR_LCDRGB_A 1    

    // TWR-LCDRBG board revB
    #define TWR_LCDRGB_B 2

    #ifndef D4DLCDHWFB_DOUBLE_BUFFER 
      #define D4DLCDHWFB_DOUBLE_BUFFER 0
    #endif

    #ifndef D4DLCDHWFB_DMA_ENABLE 
      #define D4DLCDHWFB_DMA_ENABLE 0
    #endif      

    #ifndef D4DLCDHWFB_DMA_CHANNEL 
      #define D4DLCDHWFB_DMA_CHANNEL 0
    #endif


    #ifndef D4DLCDHWFB_START_ADDRESS 
      #define D4DLCDHWFB_START_ADDRESS 0
    #endif
     
    #ifndef D4DLCDHWFB_START_ADDRESS_2ND 
      #define D4DLCDHWFB_START_ADDRESS_2ND 0
    #endif
    
    #ifndef D4DLCDHWFB_X_MAX 
      #define D4DLCDHWFB_X_MAX 480
    #endif	
    
    #ifndef D4DLCDHWFB_Y_MAX 
      #define D4DLCDHWFB_Y_MAX 800
    #endif	
              
    #ifndef D4DLCDHWFB_BPP_BYTE 
      #define D4DLCDHWFB_BPP_BYTE 4
    #endif	
    
    #ifndef D4DLCDHWFB_BPP 
      #define D4DLCDHWFB_BPP BPP24
    #endif
              
    #ifndef D4DLCDHWFB_MIRROWED
      #define D4DLCDHWFB_MIRROWED 0
    #endif
    
    #ifndef D4DLCDHWFB_CACHE_FLUSH_ENABLE
      #define D4DLCDHWFB_CACHE_FLUSH_ENABLE 0
    #endif
      
// List of supported LCD Panels
//  d4dlcdhw_k70_lcdc_twr_rgbA - TWR-LCD-RGB rev A; on TWR-K70 and SPI2 - it must be on TWR-PROTO board rerouted SPI2 signals
//  d4dlcdhw_k70_lcdc_twr_rgbB - TWR-LCD_RGB rev B; general (it doesn't need SPI initialization)

    #ifndef D4D_LLD_LCD_HW_K70LCDC	
      #define D4D_LLD_LCD_HW_K70LCDC d4dlcdhw_k70_lcdc_twr_rgbB
    #endif	
    
	


    /******************************************************************************
    * Types
    ******************************************************************************/
    typedef struct D4DLCDHWFBK70LCDC_FUNCTIONS_S
    {
      unsigned char (*D4DLCDHWFBK70LCDC_InitPanel)(void);      
    }D4DLCDHWFBK70LCDC_FUNCTIONS;


    /******************************************************************************
    * Macros 
    ******************************************************************************/
    
      
    /******************************************************************************
    * Global variables
    ******************************************************************************/
    
    extern const D4DLCDHWFBK70LCDC_FUNCTIONS D4D_LLD_LCD_HW_K70LCDC;
    
    /******************************************************************************
    * Global functions
    ******************************************************************************/



/************************* Structure/Union Data types ************************/

typedef enum
{ ACD_DIV_0=0,
  ACD_DIV_1,
  ACD_DIV_2,
  ACD_DIV_3,
  ACD_DIV_4,
  ACD_DIV_5,
  ACD_DIV_6,
  ACD_DIV_7,
  ACD_DIV_8,
  ACD_DIV_9,
  ACD_DIV_10,
  ACD_DIV_11,
  ACD_DIV_12,
  ACD_DIV_13,
  ACD_DIV_14,
  ACD_DIV_15  	
} ACD_DIV;


typedef enum 
{ MONO_PANEL=0,
  CSTN_PANEL,
  PANEL_UNUSED,
  TFT_PANEL=3
} LCDC_PANEL_TYPE;

typedef enum
{
 BPP1=0,
 BPP2=1,
 BPP4=2,
 BPP8=3,
 BPP12=4,
 BPP16=5,
 BPP18=6,
 BPP24=7 
} BPP_TYPE;

  #endif
#endif /* __D4DLCDHW_K70_LCDC_H */










