/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4dtch_cr_touch.h
*
* @author    b01119
* 
* @version   0.0.3.0
* 
* @date      Dec-7-2011
* 
* @brief     D4D driver - cr_touch touch screen driver function header file 
*
*******************************************************************************/

#ifndef __D4DTCH_CR_TOUCH_H
#define __D4DTCH_CR_TOUCH_H

  #if (D4D_MK_STR(D4D_LLD_TCH) == d4dtch_cr_touch_ID)

    /******************************************************************************
    * Includes
    ******************************************************************************/
    #include "d4dtch_cr_touch_cfg.h"

    /******************************************************************************
    * Constants
    ******************************************************************************/
    #define CRTCH_STATUS      0x00 
    #define CRTCH_STATUS1     0x01
    #define CRTCH_STATUS2     0x02
    #define CRTCH_X           0x03
    #define CRTCH_Y           0x05
    #define CRTCH_PRESS       0x07
    #define CRTCH_FIFO_ST     0x09 
    #define CRTCH_FIFO_X      0x0A
    #define CRTCH_FIFO_Y      0x0C
    #define CRTCH_FIFO_PRESS  0x0E
    #define CRTCH_UART_BAUD   0x10

    #define CRTCH_RES_CFG     0x40


    #define CRTCH_STATUS1_RTST_MASK         0x80
    #define CRTCH_STATUS1_RTSRDY_MASK       0x01

    #define CRTCH_RES_CFG_RTPRESEN_MASK     0x80
    
    #define CRTCH_STATUS2_RTSD_MASK     0x03



    /****** Algorithm constants/macros *****/
    
    #ifndef D4DTCH_FULL_SCALE
      #define D4DTCH_FULL_SCALE           0x0FFF
    #endif 
                         
    #ifndef D4DTCH_X_TOUCH_MIN
      #define D4DTCH_X_TOUCH_MIN              (D4DTCH_FULL_SCALE / 10)
    #endif
   
    #ifndef D4DTCH_Y_TOUCH_MIN
      #define D4DTCH_Y_TOUCH_MIN              (D4DTCH_FULL_SCALE / 10)
    #endif
   
    #ifndef D4DTCH_X_TOUCH_OFFMAX
      #define D4DTCH_X_TOUCH_OFFMAX           (D4DTCH_X_TOUCH_MIN * 4 / 2)
    #endif
   
    #ifndef D4DTCH_Y_TOUCH_OFFMAX
      #define D4DTCH_Y_TOUCH_OFFMAX           (D4DTCH_Y_TOUCH_MIN * 4 / 2)
    #endif
   
    #ifndef D4DTCH_SAMPLE_MARGIN
      #define D4DTCH_SAMPLE_MARGIN            (D4DTCH_FULL_SCALE / 256)
    #endif


    #ifndef D4DTCH_IIC_BAUDRATE
      #define D4DTCH_IIC_BAUDRATE           400000
    #endif 
    
    #ifndef D4DTCH_IIC_ADDRESS
      #define D4DTCH_IIC_ADDRESS           0x49
    #endif 
    
    #ifndef D4DTCH_INVERSE_X
      #define D4DTCH_INVERSE_X               0
    #endif

    #ifndef D4DTCH_INVERSE_Y
      #define D4DTCH_INVERSE_Y               0
    #endif

    #ifndef D4DTCH_SWAP_AXES   
      #define D4DTCH_SWAP_AXES               0
    #endif

    #ifndef D4D_LLD_TCH_HW_CRTOUCH
      #define D4D_LLD_TCH_HW_CRTOUCH d4dtchhw_crtouch_iicBareMetal
    #endif

    /******************************************************************************
    * Macros 
    ******************************************************************************/


    #if D4DTCH_INVERSE_X != 0
      #define D4DTCH_INVERSE_X_INT D4DTCH_FULL_SCALE
    #endif
  
    #if D4DTCH_INVERSE_Y != 0
      #define D4DTCH_INVERSE_Y_INT D4DTCH_FULL_SCALE
    #endif

    /******************************************************************************
    * Types
    ******************************************************************************/
#pragma pack (push)
#pragma pack (1)    

    typedef struct
    {
      Byte status1;
      Byte status2;
      Word X;
      Word Y;
    }CRTOUCH_RES_SIMPLE;
#pragma pack (pop)

    typedef struct D4DTCHHWCRTOUCH_FUNCTIONS_S
    {
      unsigned char (*D4DTCHHWCRTOUCH_Init)(void);
      unsigned char (*D4DTCHHWCRTOUCH_DeInit)(void);
      void (*D4DTCHHWCRTOUCH_SetAddress)(unsigned char);
      signed long (*D4DTCHHWCRTOUCH_SetBaudRate)(unsigned long baudRate);
      unsigned char (*D4DTCHHWCRTOUCH_MultiReadBytes)(unsigned char regAddr, unsigned char* pBuffer, unsigned char cnt);
      unsigned char (*D4DTCHHWCRTOUCH_MultiWriteBytes)(unsigned char regAddr, unsigned char* pBuffer, unsigned char cnt);            
      unsigned char COMM_OK;
    }D4DTCHHWCRTOUCH_FUNCTIONS;
      
    /******************************************************************************
    * Global variables
    ******************************************************************************/
    
    extern const D4DTCHHWCRTOUCH_FUNCTIONS D4D_LLD_TCH_HW_CRTOUCH;
    
    /******************************************************************************
    * Global functions
    ******************************************************************************/

  #endif
#endif /* __D4DTCH_CR_TOUCH_H */










