/**************************************************************************
* 
* Copyright (c) 2008 Freescale Semiconductor;
* All Rights Reserved                       
*
*************************************************************************** 
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
****************************************************************************
***************************************************************************//*!
*
* @file      hw_iic_cfg.h
* @author    B15307
* @version   1.0.5.0
* @date      Jun-29-2012
* @brief     HardI2C configuration
*
******************************************************************************/

#ifndef _HW_I2C_CFG_H
#define _HW_I2C_CFG_H


/******************************************************************************
* Includes
******************************************************************************/
#include "d4d.h"            // include of all public items (types, function etc) of D4D driver
#include "common_files/d4d_lldapi.h"     // include non public low level driver interface header file (types, function prototypes, enums etc. )
#include "common_files/d4d_private.h"    // include the private header file that contains perprocessor macros as D4D_MK_STR


// identification string of driver - must be same as name D4DTCH_FUNCTIONS structure + "_ID"
// it is used for enable the code for compilation
#define d4dtch_cr_touch_ID 1

// copilation enable preprocessor condition
// the string d4dtch_cr_touch_ID must be replaced by define created one line up
#if (D4D_MK_STR(D4D_LLD_TCH) == d4dtch_cr_touch_ID)


#include "low_level_drivers\touch_screen\touch_screen_drivers\cr_touch\d4dtch_cr_touch.h"

// identification string of driver - must be same as name D4DTCH_FUNCTIONS structure + "_ID"
// it is used for enable the code for compilation
#define d4dtchhw_crtouch_iicBareMetal_ID 1


 

// copilation enable preprocessor condition
// the string d4dtch_k70_lcdc_ID must be replaced by define created one line up
#if (D4D_MK_STR(D4D_LLD_TCH_HW_CRTOUCH) == d4dtchhw_crtouch_iicBareMetal_ID)
  
  /******************************************************************************
  * Constants
  ******************************************************************************/
    

  #define I2C_ENABLE_COMPILATION 1
  #define I2C_MCU_TYPE I2C_MCU_TYPE_MK
/*
    #define I2C_BASE      I2C0_BASE_PTR
  
    #define I2C_FREQ_REG   0x17        // 400Khz for 100MHz       
  
    #define I2C_FREQ_BUS   50000000u
  
       
    #define I2C_SDA_PCR   PORTE_PCR18
    #define I2C_SCL_PCR   PORTE_PCR19
  
    #define I2C_SCL   19
    #define I2C_SCL_PORT  GPIOE_PDOR   // Data output register
    #define I2C_SCL_DDR   GPIOE_PDDR   // Output enable register
    
    #define I2C_SDA   18
    #define I2C_SDA_PORT  GPIOE_PDOR   // Data output register
    #define I2C_SDA_DDR   GPIOE_PDDR   // Output enable register
  */
  /******************************************************************************
  * Macros 
  ******************************************************************************/
  
  /******************************************************************************
  * Types
  ******************************************************************************/
  
  /******************************************************************************
  * Global functions
  ******************************************************************************/
  // Init Denit functions
  extern Byte HwIIC_Init(void);
  extern Byte HwIIC_DeInit(void);
  // Address related functions
  extern void HwIIC_SetAddress(Byte address);
  // Multi Byte read Write functions - 8 bit device address mode
  extern Byte HwIIC_MultiReadBytes(Byte regAdr, Byte *pData, Byte cnt);
  extern Byte HwIIC_MultiWriteBytes(Byte regAdr, Byte *pData, Byte cnt);
  
  //Help functions
  extern sLWord HwIIC_SetBaudRate(LWord baudRate);  


  /******************************************************************************
  * Global variables
  ******************************************************************************/
  
  const D4DTCHHWCRTOUCH_FUNCTIONS d4dtchhw_crtouch_iicBareMetal = 
  {
    HwIIC_Init,
    HwIIC_DeInit,
    HwIIC_SetAddress,
    HwIIC_SetBaudRate,
    HwIIC_MultiReadBytes,
    HwIIC_MultiWriteBytes,
    0
  };
  #else
    #define I2C_ENABLE_COMPILATION 0
  #endif
  
#endif

#endif