/**************************************************************************
* 
* Copyright (c) 2008 Freescale Semiconductor;
* All Rights Reserved                       
*
*************************************************************************** 
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
****************************************************************************
***************************************************************************//*!
*
* @file      hw_iic.h
* @author    B15307
* @version   1.0.1.0
* @date      Nov-24-2011
* @brief     HardI2C definitions
*
******************************************************************************/

#ifndef _HW_I2C_H
#define _HW_I2C_H
     

/******************************************************************************
* Constants
******************************************************************************/

#define I2C_OK 0
#define I2C_FALSE  1                                         
#define I2C_ACK  I2C_OK
#define I2C_NACK 2                       
  

#define I2C_MCU_TYPE_S08 1
#define I2C_MCU_TYPE_CV 2
#define I2C_MCU_TYPE_MK 3


/******************************************************************************
* Macros 
******************************************************************************/



/******************************************************************************
* Types
******************************************************************************/
typedef union
{
  Word w;
  struct
  {
    Byte h;
    Byte l;
  }b;
}I2C_WORD_BYTE;

/******************************************************************************
* Global functions
******************************************************************************/

// Init Denit functions
Byte HwIIC_Init(void);
Byte HwIIC_DeInit(void);
// Address related functions
void HwIIC_SetAddress(Byte address);
Byte HwIIC_GetAddress(void);
// Single Byte read Write functions - 8 bit device address mode
Byte HwIIC_ReadByte(Byte regAdr, Byte* pData);
Byte HwIIC_WriteByte(Byte regAdr, Byte data);
// Multi Byte read Write functions - 8 bit device address mode
Byte HwIIC_MultiReadBytes(Byte regAdr, Byte *pData, Byte cnt);
Byte HwIIC_MultiWriteBytes(Byte regAdr, Byte *pData, Byte cnt);
Byte HwIIC_ReadArrayDelayed(Byte regAdr, Byte* pData, Byte cnt, Word attemps);
// Single Byte read Write functions - 16 bit device address mode
Byte HwIIC_ReadByte16(I2C_WORD_BYTE regAdr, Byte* pData);
Byte HwIIC_WriteByte16(I2C_WORD_BYTE regAdr, Byte data);
// Multi Byte read Write functions - 16 bit device address mode
Byte HwIIC_MultiReadBytes16(I2C_WORD_BYTE regAdr, Byte *pData, Byte cnt);
Byte HwIIC_MultiWriteBytes16(I2C_WORD_BYTE regAdr, Byte *pData, Byte cnt);

//Help functions
Byte HwIIC_LookForDevice(Byte adr);
sLWord HwIIC_SetBaudRate(LWord baudRate);



#endif