/**************************************************************************
* 
* Copyright (c) 2008 Freescale Semiconductor;
* All Rights Reserved                       
*
*************************************************************************** 
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
****************************************************************************
***************************************************************************//*!
*
* @file      hw_iic_prv.h
* @author    B15307
* @version   1.0.3.0
* @date      Feb-21-2012
* @brief     HardI2C private definitions
*
******************************************************************************/

#ifndef _HW_I2C_PRV_H
#define _HW_I2C_PRV_H



/******************************************************************************
* Constants
******************************************************************************/

/******************************************************************************
* Macros 
******************************************************************************/

#define SET(x)       (x##_PORT) |= (1 << (x))
#define RESET(x)     (x##_PORT) &= ~(1 << (x))

#define GET(x)      ((x##_PORT) & ~(1 << (x)))


#define OUTPUT(x)    (x##_DDR) |= (1 << (x)) ;
#define INPUT(x)     (x##_DDR) &= ~(1 << (x));

#ifndef I2C_MCU_TYPE
  #define I2C_MCU_TYPE I2C_MCU_TYPE_MK
#endif

#ifndef I2C_FREQ_REG
  #define I2C_FREQ_REG   0x17
#endif

#ifndef I2C_FREQ_BUS
  #define I2C_FREQ_BUS   50000000u
#endif

#ifndef I2C_PCR_PIN_GPIO
  #define I2C_PCR_PIN_GPIO  1
#endif

#ifndef I2C_PCR_PIN_I2C
  #define I2C_PCR_PIN_I2C   4
#endif

#ifndef I2C_SCL_PIN_ENABLE
  #ifndef I2C_SCL_PCR
    #error "The Macro I2C_SCL_PCR must be set in Touch Screen low level driver configuration file." 
  #else
    #define I2C_SCL_PIN_ENABLE(alt)  (I2C_SCL_PCR = PORT_PCR_MUX(alt) | PORT_PCR_ODE_MASK);
  #endif
#endif

#ifndef I2C_INIT_SCL  
    #define I2C_INIT_SCL  I2C_SCL_PIN_ENABLE(I2C_PCR_PIN_I2C); 
#endif

#ifndef RESET_I2C_SCL
    #ifndef I2C_SCL
    #error "The Macro I2C_SCL must be set in Touch Screen low level driver configuration file." 
  #else
    #define RESET_I2C_SCL RESET(I2C_SCL);
  #endif	
#endif  

#ifndef SET_I2C_SCL
  #ifndef I2C_SCL
    #error "The Macro I2C_SCL must be set in Touch Screen low level driver configuration file." 
  #else
    #define SET_I2C_SCL SET(I2C_SCL);
  #endif	
#endif

#ifndef SET_I2C_SCL
  #ifndef I2C_SCL
    #error "The Macro I2C_SCL must be set in Touch Screen low level driver configuration file." 
  #else
    #define SET_I2C_SCL SET(I2C_SCL);
  #endif	
#endif

#ifndef SET_OUTPUT_I2C_SCL
  #ifndef I2C_SCL
    #error "The Macro I2C_SCL must be set in Touch Screen low level driver configuration file." 
  #else
    #define SET_OUTPUT_I2C_SCL OUTPUT(I2C_SCL);
  #endif	
#endif

#ifndef SET_INPUT_I2C_SCL
  #ifndef I2C_SCL
    #error "The Macro I2C_SCL must be set in Touch Screen low level driver configuration file." 
  #else
    #define SET_INPUT_I2C_SCL INPUT(I2C_SCL);
  #endif	
#endif

#ifndef GET_I2C_SCL
  #ifndef I2C_SCL
    #error "The Macro I2C_SCL must be set in Touch Screen low level driver configuration file." 
  #else
    #define GET_I2C_SCL GET(I2C_SCL);
  #endif	
#endif

#ifndef I2C_SDA_PIN_ENABLE
  #ifndef I2C_SDA_PCR
    #error "The Macro I2C_SDA_PCR must be set in Touch Screen low level driver configuration file." 
  #else
    #define I2C_SDA_PIN_ENABLE(alt) (I2C_SDA_PCR = PORT_PCR_MUX(alt) | PORT_PCR_ODE_MASK); 
  #endif
#endif


#ifndef I2C_INIT_SDA  
    #define I2C_INIT_SDA  I2C_SDA_PIN_ENABLE(I2C_PCR_PIN_I2C)
#endif

#ifndef RESET_I2C_SDA
    #ifndef I2C_SDA
    #error "The Macro I2C_SDA must be set in Touch Screen low level driver configuration file." 
  #else
    #define RESET_I2C_SDA RESET(I2C_SDA);
  #endif	
#endif  

#ifndef SET_I2C_SDA
  #ifndef I2C_SDA
    #error "The Macro I2C_SDA must be set in Touch Screen low level driver configuration file." 
  #else
    #define SET_I2C_SDA SET(I2C_SDA);
  #endif	
#endif

#ifndef SET_I2C_SDA
  #ifndef I2C_SDA
    #error "The Macro I2C_SDA must be set in Touch Screen low level driver configuration file." 
  #else
    #define SET_I2C_SDA SET(I2C_SDA);
  #endif	
#endif

#ifndef SET_OUTPUT_I2C_SDA
  #ifndef I2C_SDA
    #error "The Macro I2C_SDA must be set in Touch Screen low level driver configuration file." 
  #else
    #define SET_OUTPUT_I2C_SDA OUTPUT(I2C_SDA);
  #endif	
#endif

#ifndef SET_INPUT_I2C_SDA
  #ifndef I2C_SDA
    #error "The Macro I2C_SDA must be set in Touch Screen low level driver configuration file." 
  #else
    #define SET_INPUT_I2C_SDA INPUT(I2C_SDA);
  #endif	
#endif

#ifndef GET_I2C_SDA
  #ifndef I2C_SDA
    #error "The Macro I2C_SDA must be set in Touch Screen low level driver configuration file." 
  #else
    #define GET_I2C_SDA GET(I2C_SDA);
  #endif	
#endif


#ifndef I2C_DISABLE_COMPILATION
  #define I2C_DISABLE_COMPILATION 0
#endif

#ifndef I2C_C1_I2CEN_MASK
  #define I2C_C1_I2CEN_MASK                         0x80u
#endif

#ifndef I2C_C1_MST_MASK
  #define I2C_C1_MST_MASK                           0x20u
#endif

#ifndef I2C_C1_TX_MASK
  #define I2C_C1_TX_MASK                            0x10u
#endif

#ifndef I2C_C1_TXAK_MASK
  #define I2C_C1_TXAK_MASK                          0x08u
#endif

#ifndef I2C_C1_RSTA_MASK
  #define I2C_C1_RSTA_MASK                          0x04u
#endif


#ifndef I2C_S_BUSY_MASK
  #define I2C_S_BUSY_MASK                           0x20u
#endif

#ifndef I2C_S_I2CIF_MASK
  #define I2C_S_I2CIF_MASK                          0x02u
#endif

#ifndef I2C_S_RXAK_MASK
  #define I2C_S_RXAK_MASK                           0x01u
#endif

#if (I2C_MCU_TYPE == I2C_MCU_TYPE_S08)
  // S08 
  #error "S08 is currently unssuported MCU type for delay loop in loe level common driver!"  
#elif (I2C_MCU_TYPE == I2C_MCU_TYPE_CV)
  //ColdFire
  #error "ColdFire is currently unssuported MCU type for delay loop in loe level common driver!" 
#elif (I2C_MCU_TYPE == I2C_MCU_TYPE_MK)
  // Kinetis
  #define I2C_A1                                  I2C_A1_REG(I2C_BASE)
  #define I2C_F                                   I2C_F_REG(I2C_BASE)
  #define I2C_C1                                  I2C_C1_REG(I2C_BASE)
  #define I2C_S                                   I2C_S_REG(I2C_BASE)
  #define I2C_D                                   I2C_D_REG(I2C_BASE)
  #define I2C_C2                                  I2C_C2_REG(I2C_BASE)
  #define I2C_FLT                                 I2C_FLT_REG(I2C_BASE)
  #define I2C_RA                                  I2C_RA_REG(I2C_BASE)
  #define I2C_SMB                                 I2C_SMB_REG(I2C_BASE)
  #define I2C_A2                                  I2C_A2_REG(I2C_BASE)
  #define I2C_SLTH                                I2C_SLTH_REG(I2C_BASE)
  #define I2C_SLTL                                I2C_SLTL_REG(I2C_BASE)
#else
  #error "Unsupported MCU type for delay loop in low level common driver!"
#endif

/******************************************************************************
* Types
******************************************************************************/

/******************************************************************************
* Global functions
******************************************************************************/

#endif //_HW_I2C_PRV_H