/**************************************************************************
* 
* Copyright (c) 2008 Freescale Semiconductor;
* All Rights Reserved                       
*
*************************************************************************** 
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
****************************************************************************
***************************************************************************//*!
*
* @file      mqx_iic.h
* @author    B15307
* @version   1.0.3.0
* @date      Mar-19-2012
* @brief     MqxI2C definitions
*
******************************************************************************/

#ifndef _MQX_I2C_H
#define _MQX_I2C_H
     
// copilation enable preprocessor condition
// the string d4dtch_k70_lcdc_ID must be replaced by define created one line up
#if (D4D_MK_STR(D4D_LLD_TCH_HW_CRTOUCH) == d4dtchhw_crtouch_iicMqx_ID)
  
  /******************************************************************************
  * Constants
  ******************************************************************************/
#ifndef I2C_OK
  #define I2C_OK 0
#endif
  #define I2C_FALSE  1                                         
  #define I2C_ACK  I2C_OK
  #define I2C_NACK 2                       
    
  
  /******************************************************************************
  * Macros 
  ******************************************************************************/
  
  #ifndef MQX_I2C_DEV
    #define MQX_I2C_DEV "i2c0:"
  #endif
  
  /******************************************************************************
  * Types
  ******************************************************************************/
  typedef union
  {
    Word w;
    struct
    {
      Byte h;
      Byte l;
    }b;
  }I2C_WORD_BYTE;
  
  /******************************************************************************
  * Global functions
  ******************************************************************************/
  
  // Init Denit functions
  Byte MqxIIC_Init(void);
  Byte MqxIIC_DeInit(void);
  // Address related functions
  void MqxIIC_SetAddress(Byte address);
  Byte MqxIIC_GetAddress(void);
  // Single Byte read Write functions - 8 bit device address mode
  Byte MqxIIC_ReadByte(Byte regAdr, Byte* pData);
  Byte MqxIIC_WriteByte(Byte regAdr, Byte data);
  // Multi Byte read Write functions - 8 bit device address mode
  Byte MqxIIC_MultiReadBytes(Byte regAdr, Byte *pData, Byte cnt);
  Byte MqxIIC_MultiWriteBytes(Byte regAdr, Byte *pData, Byte cnt);
  Byte MqxIIC_ReadArrayDelayed(Byte regAdr, Byte* pData, Byte cnt, Word attemps);
  // Single Byte read Write functions - 16 bit device address mode
  Byte MqxIIC_ReadByte16(I2C_WORD_BYTE regAdr, Byte* pData);
  Byte MqxIIC_WriteByte16(I2C_WORD_BYTE regAdr, Byte data);
  // Multi Byte read Write functions - 16 bit device address mode
  Byte MqxIIC_MultiReadBytes16(I2C_WORD_BYTE regAdr, Byte *pData, Byte cnt);
  Byte MqxIIC_MultiWriteBytes16(I2C_WORD_BYTE regAdr, Byte *pData, Byte cnt);
  
  //Help functions
  Byte MqxIIC_LookForDevice(Byte adr);
  sLWord MqxIIC_SetBaudRate(LWord baudRate);


  #endif
#endif