/**HEADER********************************************************************
*
* $FileName: nand_flash.c$
* $Version : 
* $Date    : 
*
* Comments:
*
*END************************************************************************/

#include <stdlib.h>
#include <mqx.h>
#include <bsp.h>
#include <fio.h>
#include <string.h>
#include <mfs.h>

#include "nand_flash_fcn.h"
#include "..\File_management\file_management.h"
#include "..\main.h"
#include "../format/format.h"

#if BSP_TWRMCF54418 | BSP_M5329EVB | BSP_TWRMPC5125 | BSP_TWR_K70F120M | BSP_TWR_K60F120M | BSP_SQM4_K70_EB
    #if ! BSPCFG_ENABLE_NANDFLASH
    #error This application requires BSPCFG_ENABLE_NANDFLASH defined non-zero in user_config.h. Please recompile BSP with this option.
    #endif
    // external flash name
    #define         FLASH_NAME "nandflash:"
#endif

// Set to 1 to call FLASH_IOCTL_ERASE_CHIP if chip erase function is present
#define FLASH_TEST_ERASE_CHIP   (1)     // whether to erase NAND flash before writing data from SD card
#define WRITE_FILES             (0)     // set to 1 if you want to write files into NAND Flash
 
#define HEADER_LOCATION         1       //Block #1
#define FILE_LOCATION           2       //First file location block #2

volatile char bWriteFiles = WRITE_FILES;

static char *www_files[][2] = {
  { "A:\\HTML\\BARGRAPH.BMP",   "BARGRAPH.BMP"  },
  { "A:\\HTML\\FAVICON.ICO",    "FAVICON.ICO"   },
  { "A:\\HTML\\HEADER.GIF",     "HEADER.GIF"    },
  { "A:\\HTML\\ICMPSTAT.HTM",   "ICMPSTAT.HTM"  },
  { "A:\\HTML\\INDEX.HTM",      "INDEX.HTM"     },
  { "A:\\HTML\\IPSTAT.HTM",     "IPSTAT.HTM"    },
  { "A:\\HTML\\LED.HTM",        "LED.HTM"       },
  { "A:\\HTML\\LINE.BMP",       "LINE.BMP"      },
  { "A:\\HTML\\MQX.CSS",        "MQX.CSS"       },
  { "A:\\HTML\\REQUEST.JS",     "REQUEST.JS"    },
  { "A:\\HTML\\RTC.HTM",        "RTC.HTM"       },
  { "A:\\HTML\\TCPSTAT.HTM",    "TCPSTAT.HTM"   },
  { "A:\\HTML\\UDPSTAT.HTM",    "UDPSTAT.HTM"   },
  { "A:\\DATASHT.PDF",          "DATASHT.PDF"   },
  { "A:\\LEAFLET.PDF",          "LEAFLET.PDF"   },
  { "A:\\GETTIN~1.PDF",         "GETTIN~1.PDF"  },
  { NULL, NULL }
};

/*FUNCTION*-----------------------------------------------------
*
* Task Name    : nandflash_open
* @param[in]    name    NAND Flash name.
* @return               NAND Flash handle 
* Comments     :
*    Open the flash device
*
*END*-----------------------------------------------------*/
MQX_FILE_PTR nandflash_open(char_ptr name) {
    MQX_FILE_PTR nandflash_hdl = NULL;

    /* Open the flash device */

   nandflash_hdl = fopen(name, NULL);
    if (nandflash_hdl == NULL) {
        printf("\nUnable to open NAND flash device %s", name);
        return nandflash_hdl;
    } /* Endif */
    printf("\nNAND Flash device %s opened", name);
    return nandflash_hdl;
}

/*FUNCTION*-----------------------------------------------------
*
* Task Name    : nandflash_close
* @param[in]    nandflash       NAND Flash handle
* Comments     :
*    Close the flash device
*
*END*-----------------------------------------------------*/
void nandflash_close(MQX_FILE_PTR nandflash) {
    fclose(nandflash);
    printf("\n\nNAND Flash closed.");
}

/*FUNCTION*-----------------------------------------------------
*
* Task Name    : allocate_buffer
* @param[in]    buffer_size     Buffer size
* @param[in]    string          Buffer name
* @return                       Pointer to memory buffer
* Comments     :
*    Allocate the buffer
*
*END*-----------------------------------------------------*/
uchar_ptr allocate_buffer(_mem_size buffer_size, char_ptr string) {
    uchar_ptr buffer_ptr;

    //buffer_ptr = _mem_alloc_zero(buffer_size);
    buffer_ptr = _mem_alloc_from(d4dMemoryPoolId, buffer_size);
    if (buffer_ptr == NULL) {
        printf("\nFailed to get %s buffer", string);
        return buffer_ptr;
    } /* Endif */
    return buffer_ptr;
}


/*FUNCTION*-----------------------------------------------------
*
* Task Name    : compare_test
* @param[in]    write_buffer            Write buffer
* @param[in]    read_buffer             Read buffer
* @param[in]    read_write_size         Number of bytes to compare
* Comments     :
*    function compare write_buffer and read_buffer
*
*END*-----------------------------------------------------*/
void compare_test( uchar_ptr write_buffer, uchar_ptr read_buffer, _mem_size read_write_size) {
    if (memcmp(write_buffer, read_buffer, read_write_size)) {
         printf("Compare failed\n");
    }
    else {
        printf("Compare OK\n");
    }
}

/*FUNCTION*-----------------------------------------------------
*
* Task Name    : size_compare
* @param[in]    flash_hdl               NAND Flash handle
* @param[in]    i                       First variable to compare
* @param[in]    read_write_size         Second variable to compare
* Comments     :
*    function compare i and read_write_size
*
*END*-----------------------------------------------------------*/
void size_compare( MQX_FILE_PTR flash_hdl, _mqx_int i, _mqx_int read_write_size )
{
    if (i != read_write_size ) {
        printf("\nFailed to write flash, size returned:%d expected %d", i,
        read_write_size);
        printf("\nTEC:0x%X FERROR:0x%X", _task_get_error(), ferror(flash_hdl));
    }
    else {
        printf("Done");
    }/* Endif */
}

/*FUNCTION*-----------------------------------------------------
* Inicialize NAND Flash and fill RAM disk with files from NAND Flash
* Function Name    : nand_flash_init
* Comments     :
*
*END*-----------------------------------------------------*/
void nand_flash_init(void) {
   MQX_FILE_PTR   nandflash_hdl;
   _mqx_uint i, j, file_size;
   _mqx_uint ID, num_blocks, virt_page_size, block_size, tmp = 0, header_size;
   _mem_size num_virt_pages_per_block;
   uint_8_ptr bbt;
   MQX_FILE_PTR fr;
   uchar * header;
   _mqx_uint location;
   char file_name[FILENAME_SHORT_SIZE], abs_path[PATHNAME_SIZE];

   _int_install_unexpected_isr();

   // Open the flash device
   nandflash_hdl = nandflash_open(FLASH_NAME);

   // Get NAND Flash organization data
   puts("\nObtaining NAND Flash organization data ...");
   if(MQX_OK == ioctl(nandflash_hdl, NANDFLASH_IOCTL_GET_ID, &ID))
       printf("\nID:                      0x%x", ID);
   else
       printf("\nID not obtained");
   if(MQX_OK == ioctl(nandflash_hdl, NANDFLASH_IOCTL_GET_PHY_PAGE_SIZE, &tmp))
       printf("\nPhysical page size:      %d bytes", tmp);
   else
       printf("\nPhysical page size not obtained");
   if(MQX_OK == ioctl(nandflash_hdl, NANDFLASH_IOCTL_GET_SPARE_AREA_SIZE, &tmp))
       printf("\nSpare area size:         %d bytes", tmp);
   else
       printf("\nSpare area size not obtained");
   if(MQX_OK == ioctl(nandflash_hdl, NANDFLASH_IOCTL_GET_BLOCK_SIZE, &block_size))
       printf("\nBlock size:              %d bytes", block_size);
   else
       printf("\nBlock size not obtained");
   if(MQX_OK == ioctl(nandflash_hdl, NANDFLASH_IOCTL_GET_NUM_BLOCKS, &num_blocks))
       printf("\nNumber of blocks:        %d", num_blocks);
   else
       printf("\nNumber of blocks not obtained");
   if(MQX_OK == ioctl(nandflash_hdl, NANDFLASH_IOCTL_GET_WIDTH, &tmp))
       printf("\nWidth:                   %d", tmp);
   else
       printf("\nWidth not obtained");
   if(MQX_OK == ioctl(nandflash_hdl, NANDFLASH_IOCTL_GET_NUM_VIRT_PAGES, &tmp))
       printf("\nNumber of virtual pages: %d", tmp);
   else
       printf("\nNumber of virtual pages not obtained");
   if(MQX_OK == ioctl(nandflash_hdl, NANDFLASH_IOCTL_GET_VIRT_PAGE_SIZE, &virt_page_size))
       printf("\nVirtual page size:       %d bytes\n", virt_page_size);
   else
       printf("\nVirtual page size not obtained\n");

   // Get Bad Block Table
   bbt = _mem_alloc_zero(num_blocks);
   if(MQX_OK == ioctl(nandflash_hdl, NANDFLASH_IOCTL_GET_BAD_BLOCK_TABLE, bbt)) {
     for (i=0; i<num_blocks; i++) {
       if(bbt[i]==0) {
         printf("\nBlock #%d is bad", i);
         if(NANDFLASHERR_BLOCK_BAD != ioctl(nandflash_hdl, NANDFLASH_IOCTL_CHECK_BLOCK, (void*)i))
           printf("\nCHECK BLOCK IOCTL command testing failed");
       }
     }
   } else {
     printf("\nReading Bad block table not successful ");
   }

   // Calculate the number of virtual pages per one block
   num_virt_pages_per_block = block_size / virt_page_size;

  if(bWriteFiles) {                     // Write files from SD card to NAND flash
#if FLASH_TEST_ERASE_CHIP  
    // Erase memory before writing
    if(MQX_OK != ioctl(nandflash_hdl, NANDFLASH_IOCTL_ERASE_CHIP, NULL)) {
      printf("Chip erase error!");
    }
#endif

    header=NULL;
    header_size=0;

    _time_delay(3000); // wait for SD Card initialization

    location=FILE_LOCATION*num_virt_pages_per_block;

    for (i=FILE_LOCATION; i<num_blocks; i++) {
      if(NANDFLASHERR_NO_ERROR != ioctl(nandflash_hdl, NANDFLASH_IOCTL_ERASE_BLOCK_FORCE, (void*)i)) {
        printf("\nChip erase error!\n");
      }
    }

    i=0;
    while(www_files[i][0] && www_files[i][1]) {
      fr=fopen(www_files[i][0], "r");
      NANDFlash_addfile2header(fr, www_files[i][1], &header, &header_size);
      NANDFlash_writefile(fr, &location);
      fclose(fr);
      ++i;
    }

    NANDFlash_writeheader(header, header_size);
    _mem_free(header);
  }
  
  header=NULL;
  header_size = 0;
  NANDFlash_readheader(&header, &header_size);
  location=FILE_LOCATION*num_virt_pages_per_block;
  
  MFS_make_dir(FTP_DRIVE, "\\", "html");
  i=4;          // header[0-3] = header size
  while(i<header_size) {
    j=0;
    while(header[i]!='\0') {
      file_name[j]=header[i];
      i++;
      j++;
    }
    
    file_name[j]=header[i];
    i++;
    
    file_size=header[i]&0xFF; 
    i++;
    file_size+=((_mqx_uint)((header[i])<<8))&0x0000FF00; 
    i++;
    file_size+=((_mqx_uint)((header[i])<<16))&0x00FF0000; 
    i++;
    file_size+=((_mqx_uint)((header[i])<<24))&0xFF000000; 
    i++;
    
    _io_rel2abs(abs_path, "\\html", file_name, PATHNAME_SIZE, FTP_DRIVE);
    fr=NANDFlash_readfile(abs_path, file_size, &location);
    fclose(fr);
  }
  _mem_free(header);  

   // Close the device
   nandflash_close(nandflash_hdl);
}

/*FUNCTION*-----------------------------------------------------
*
* Function Name    : NANDFlash_writeheader
* @param[in]    header     Data to be written to NAND Flash
* @param[in]    size       Number of bytes
* @return                  1 - Writing failed
*                          0 - Operation OK
* Comments     :
*    Write header describing a list of files stored in the NAND Flash
*
*END*-----------------------------------------------------------*/
int_32 NANDFlash_writeheader(uchar *header, uint_32 size) {
  MQX_FILE_PTR   nandflash_hdl;
  _mqx_uint i;
  _mqx_uint num_blocks, virt_page_size, block_size;
  _mem_size num_virt_pages_per_block, seek_location;
  uchar_ptr write_buffer;
  
  nandflash_hdl = nandflash_open(FLASH_NAME);
   
  if(MQX_OK != ioctl(nandflash_hdl, NANDFLASH_IOCTL_GET_BLOCK_SIZE, &block_size)) {
    nandflash_close(nandflash_hdl);
    return 1;
  }  
  if(MQX_OK != ioctl(nandflash_hdl, NANDFLASH_IOCTL_GET_NUM_BLOCKS, &num_blocks)) {
    nandflash_close(nandflash_hdl);
    return 1;
  }  
  if(MQX_OK != ioctl(nandflash_hdl, NANDFLASH_IOCTL_GET_VIRT_PAGE_SIZE, &virt_page_size)) {
    nandflash_close(nandflash_hdl);
    return 1;
  }

   // Calculate the number of virtual pages per one block
  num_virt_pages_per_block = block_size / virt_page_size;
  
  if(size>block_size) {
    nandflash_close(nandflash_hdl);
    return 1;
  }
  
  
  if(NANDFLASHERR_NO_ERROR != ioctl(nandflash_hdl, NANDFLASH_IOCTL_ERASE_BLOCK_FORCE, (void*)HEADER_LOCATION)) {
    nandflash_close(nandflash_hdl);
    return 1;
  }
   
  write_buffer = allocate_buffer(block_size, "header");
  
  for(i=0;i<size;i++) {
    write_buffer[i]=header[i];
  }
  
   // Write header to header block

   seek_location = HEADER_LOCATION*num_virt_pages_per_block;
   fseek(nandflash_hdl, seek_location, IO_SEEK_SET);
   i = write(nandflash_hdl, &write_buffer[0], num_virt_pages_per_block);
   if(i!=num_virt_pages_per_block) {
     nandflash_close(nandflash_hdl);
     _mem_free(write_buffer);
     return 1;
   }

   nandflash_close(nandflash_hdl);
   _mem_free(write_buffer);
   return 0;
}

/*FUNCTION*-----------------------------------------------------
*
* Function Name    : NANDFlash_readheader
* @param[out]   header     Pointer to the buffer to write header
* @param[out]   size       Number of bytes
* @return                  1 - Reading failed
*                          0 - Operation OK
* Comments     :
*    Read header describing a list of files stored in the NAND Flash from NAND Flash
*
*END*-----------------------------------------------------------*/
int_32 NANDFlash_readheader(uchar **header, _mqx_uint *size) {
  MQX_FILE_PTR nandflash_hdl;
  _mqx_uint i;
  _mqx_uint num_blocks, virt_page_size, block_size;
  uchar_ptr read_buffer;
  _mem_size num_virt_pages_per_block, seek_location;
  
  nandflash_hdl = nandflash_open(FLASH_NAME);
    if(MQX_OK != ioctl(nandflash_hdl, NANDFLASH_IOCTL_GET_BLOCK_SIZE, &block_size)) {
    nandflash_close(nandflash_hdl);
    return 1;
  }  
  if(MQX_OK != ioctl(nandflash_hdl, NANDFLASH_IOCTL_GET_NUM_BLOCKS, &num_blocks)) {
    nandflash_close(nandflash_hdl);
    return 1;
  }  
  if(MQX_OK != ioctl(nandflash_hdl, NANDFLASH_IOCTL_GET_VIRT_PAGE_SIZE, &virt_page_size)) {
    nandflash_close(nandflash_hdl);
    return 1;
  }
  
  num_virt_pages_per_block = block_size / virt_page_size;
  read_buffer = allocate_buffer(block_size, "header");
  
   seek_location = HEADER_LOCATION*num_virt_pages_per_block;
   fseek(nandflash_hdl, seek_location, IO_SEEK_SET);
   i = read(nandflash_hdl, read_buffer, num_virt_pages_per_block);
   if(i!=num_virt_pages_per_block) {
     nandflash_close(nandflash_hdl);
     _mem_free(read_buffer);
     return 1;
   }

   *size=read_buffer[0]&0xFF;
   *size+=((read_buffer[1]<<8)&0xFF00);
   *size+=((read_buffer[2]<<16)&0xFF0000);
   *size+=((read_buffer[3]<<24)&0xFF000000); 

   (*header) = allocate_buffer(*size, "header");
   i=0;
   for(i=0;i<*size;i++) {
     (*header)[i]=read_buffer[i];
   }
   _mem_free(read_buffer);
  nandflash_close(nandflash_hdl);
  return 0;
}

/*FUNCTION*-----------------------------------------------------
*
* Function Name    : NANDFlash_writefile
* @param[in]    file            File handle
* @param[out]   location       Memory address of file
* @return                  1 - Writing failed
*                          0 - Operation OK
* Comments     :
*    Write file into NAND Flash at a specific address
*
*END*-----------------------------------------------------------*/
int_32 NANDFlash_writefile(MQX_FILE_PTR file, _mqx_uint *location) {
    MQX_FILE_PTR nandflash_hdl;
  _mqx_uint size, file_size_in_pages;
  _mqx_uint num_blocks, virt_page_size, block_size, num_virt_pages_per_block;
  _mem_size seek_location;
  uchar_ptr write_buffer;
  
  nandflash_hdl = nandflash_open(FLASH_NAME);
   
  if(MQX_OK != ioctl(nandflash_hdl, NANDFLASH_IOCTL_GET_BLOCK_SIZE, &block_size)) {
    nandflash_close(nandflash_hdl);
    return 1;
  }  
  if(MQX_OK != ioctl(nandflash_hdl, NANDFLASH_IOCTL_GET_NUM_BLOCKS, &num_blocks)) {
    nandflash_close(nandflash_hdl);
    return 1;
  }  
  if(MQX_OK != ioctl(nandflash_hdl, NANDFLASH_IOCTL_GET_VIRT_PAGE_SIZE, &virt_page_size)) {
    nandflash_close(nandflash_hdl);
    return 1;
  }
  
  file_size_in_pages=(file->SIZE)/virt_page_size+1;
  num_virt_pages_per_block = block_size / virt_page_size;
  
  if((*location)<(FILE_LOCATION*num_virt_pages_per_block)) {      //Attempt write to header block
    nandflash_close(nandflash_hdl);
    return 1;
  }
   
  write_buffer = allocate_buffer(file_size_in_pages*virt_page_size+1, "file");
  size = read(file, write_buffer, file->SIZE);
  if(size!=file->SIZE) {
    nandflash_close(nandflash_hdl);
    _mem_free(write_buffer);
    return 1;
  }
    
  // Write file to file block
  seek_location = *location;
  fseek(nandflash_hdl, seek_location, IO_SEEK_SET);
  size = write(nandflash_hdl, &write_buffer[0], file_size_in_pages);
   if(size!=file_size_in_pages) {
     nandflash_close(nandflash_hdl);
     _mem_free(write_buffer);
     return 1;
   }
   seek_location += file_size_in_pages;
  *location=seek_location;

   nandflash_close(nandflash_hdl);
   _mem_free(write_buffer);
   return 0;
}

/*FUNCTION*-----------------------------------------------------
*
* Function Name    : NANDFlash_readfile
* @param[in]   file_name       File name including the path, example: x:\html\index.html
* @param[in]   size            Number of bytes
* @param[out]   location       Memory address of file
* @return                  File handle
* Comments     :
*    Read file from NAND Flash
*
*END*-----------------------------------------------------------*/
MQX_FILE_PTR NANDFlash_readfile(char *file_name, _mqx_uint size, _mqx_uint *location) {
  MQX_FILE_PTR nandflash_hdl, f;
  _mqx_uint file_size_in_pages;
  _mqx_uint num_blocks, virt_page_size, block_size, rsize;
  uchar_ptr read_buffer;
  _mem_size seek_location;
  
  f=fopen(file_name, "w");
  if(f==NULL) {
    return (MQX_FILE_PTR)NULL;
  }
  
  nandflash_hdl = nandflash_open(FLASH_NAME);
  if(MQX_OK != ioctl(nandflash_hdl, NANDFLASH_IOCTL_GET_BLOCK_SIZE, &block_size)) {
    nandflash_close(nandflash_hdl);
    return (MQX_FILE_PTR)NULL;
  }  
  if(MQX_OK != ioctl(nandflash_hdl, NANDFLASH_IOCTL_GET_NUM_BLOCKS, &num_blocks)) {
    nandflash_close(nandflash_hdl);
    return (MQX_FILE_PTR)NULL;
  }  
  if(MQX_OK != ioctl(nandflash_hdl, NANDFLASH_IOCTL_GET_VIRT_PAGE_SIZE, &virt_page_size)) {
    nandflash_close(nandflash_hdl);
    return (MQX_FILE_PTR)NULL;
  }

  file_size_in_pages=(size)/virt_page_size+1;

  read_buffer = allocate_buffer(file_size_in_pages*virt_page_size+1, "file");
  
  seek_location = *location;
  fseek(nandflash_hdl, seek_location, IO_SEEK_SET);
  rsize = read(nandflash_hdl, read_buffer, file_size_in_pages);
  if(rsize!=file_size_in_pages) {
    nandflash_close(nandflash_hdl);
    _mem_free(read_buffer);
    return (MQX_FILE_PTR)NULL;
   }
  rsize = write(f, &read_buffer[0], size);
  if(rsize!=size) {
    nandflash_close(nandflash_hdl);
    _mem_free(read_buffer);
    return (MQX_FILE_PTR)NULL;
  }

  _mem_free(read_buffer);
  nandflash_close(nandflash_hdl);
  seek_location += file_size_in_pages;
  *location=seek_location;

  return f;
}

/*FUNCTION*-----------------------------------------------------
*
* Function Name    : NANDFlash_addfile2header
* @param[in]    file       File to add into header
* @param[in]    name       File name
* @param[out]   header     Pointer to NAND Flash header
* @param[out]   headersize      Number of header bytes
* @return                       TRUE - OK
*                               FALSE - Operation failed
* Comments     :
*    Add file to NAND Flash header
*
*END*-----------------------------------------------------------*/
boolean NANDFlash_addfile2header(MQX_FILE_PTR file, char *name, uchar** header,
                                 _mqx_uint *headersize) {
  _mqx_uint i, shift=0, name_size, old_header_size;
  uchar * buffer;
  
  old_header_size = *headersize;
    
  if(!(*headersize)) {
    *headersize+=sizeof(uint_32);         //Header size uint_32
  }
  
  *headersize+=sizeof(uint_32);         //File size uint_32
  
  name_size=0;
  while(name[name_size]!='\0') {name_size++;}
  name_size++;
  *headersize+=name_size;
  
  buffer = _mem_alloc_from(d4dMemoryPoolId, *headersize);
  if(buffer==NULL) {
    return FALSE;
  }
  
  if((*header)!=NULL) {
    for(shift=0;shift<old_header_size;shift++) {
      buffer[shift] = (*header)[shift];
    }
    _mem_free(*header);
  }
  *header = buffer;
  
  i=0;
  if(!old_header_size) {
    shift = 4;          //header[0-3] = header size
  } else {
    shift = old_header_size;
  }
  while(name[i]!='\0') {
    (*header)[i+shift]=name[i];
    i++;
  }
  (*header)[i+shift]=name[i];
  i++; 
  shift+=i;
  
  (*header)[shift] = (file->SIZE)&0xFF; 
  shift++;
  (*header)[shift] = ((file->SIZE)>>8)&0xFF; 
  shift++;
  (*header)[shift] = ((file->SIZE)>>16)&0xFF; 
  shift++;
  (*header)[shift] = ((file->SIZE)>>24)&0xFF; 
  shift++;
  
  (*header)[0]=(*headersize)&0xFF; 
  (*header)[1]=((*headersize)>>8)&0xFF; 
  (*header)[2]=((*headersize)>>16)&0xFF; 
  (*header)[3]=((*headersize)>>24)&0xFF;
  
  return TRUE;
}
/* EOF */
