/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      wave_task.h
*
* @author    b01119
* 
* @version   0.0.2.0
* 
* @date      Feb-9-2012
* 
* @brief     D4D driver bmp decoder header file 
*
******************************************************************************/

#ifndef __WAVE_TASK
#define __WAVE_TASK

/******************************************************************************
* Include
******************************************************************************/


/******************************************************************************
* Constants
******************************************************************************/
#define READBUF_SIZE		(512 * 8)	/* accurate for SD card and USB bit rate */
#define WAVE_OUTBUFF_CNT        2

#define NUM_MESSAGES            16

#define WAVE_TASK_BUFFER_REC_MODE  0
#define WAVE_TASK_BUFFER_REC_SIZE  (44100 * 2 * 2 * 60)
/******************************************************************************
* Types
******************************************************************************/

typedef struct _WAV_ID {
    char        ChunkID[4];
    uint_32     ChunkSize;
    char        Format[4];
    char        Subchunk1ID[4];
    uint_32     Subchunk1Size;
    uint_16     AudioFormat;
    uint_16     NumChannels;
    uint_32     SampleRate;
    uint_32     ByteRate;
    uint_16     BlockAlign;
    uint_16     BitsPerSample;
    char        Subchunk2ID[4];
    uint_32     Subchunk2Size;
} WAV_ID;

typedef enum
{
  WAVE_PLAY,
  WAVE_REC,
  WAVE_PAUSE,
  WAVE_STOP,
  WAVE_INIT,
  WAVE_DEINIT,
  WAVE_NEEDDATA,
  WAVE_HAVEDATA
}WAVE_MSG_TYPE;

typedef enum
{
  WAVEST_NOTINIT,
  WAVEST_READY,
  WAVEST_RECORDING,
  WAVEST_PLAYING,
  WAVEST_PLAYING_HOLD,
  WAVEST_PAUSE 
}WAVE_MSG_STATE;

typedef struct
{
  WAVE_MSG_TYPE type;
  void* pointer;
}WAVE_MSG;


typedef enum
{
  WAVE_OUTST_READY,
  WAVE_OUTST_REQUEST,
  WAVE_OUTST_FULL,
  WAVE_OUTST_PLAY
}WAVE_OUTPUT_STATE;


/******************************************************************************
* Static declaration macros
******************************************************************************/
#define MSG_SIZE sizeof(WAVE_MSG)

/******************************************************************************
* External source constants
******************************************************************************/
extern uint_32 wave_queue[];



/******************************************************************************
* Global functions
******************************************************************************/
#ifdef __cplusplus
extern "C" {
#endif

uint_32 WAVE_GetSongDuration(char* pFileName);
WAVE_MSG_STATE WAVE_GetState(void);
#ifdef __cplusplus
}
#endif

#endif //__WAVE_TASK
