/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_iconbutton.c
*
* @author    b01119
* @author    Petr Kubiznak
* 
* @version   0.0.46.0
* 
* @date      Jan-19-2013
* 
* @brief     D4D iconbutton object extended by ELNICO Ltd. from Freescale's
*            icon object.
*
******************************************************************************/

#include "d4d.h"
#include "common_files/d4d_private.h"
#include "d4d_iconbutton.h"

//Internal API
void D4D_IconbuttonOnMessage(D4D_MESSAGE* pMsg);
D4D_STRING* D4D_IconbuttonGetTextBuffer(D4D_OBJECT* pThis);
unsigned char D4D_IconbuttonGetBmpCount(D4D_OBJECT_PTR pThis);

const D4D_OBJECT_SYS_FUNCTION d4d_iconbuttonSysFunc = {
  D4D_DEFSTR("IconButton"),
  D4D_IconbuttonOnMessage,
  NULL,
  D4D_IconbuttonGetTextBuffer
};

// temporary structure for icon coordinate calculation
typedef struct {
    D4D_POINT position;
    D4D_POINT txtPos;
} D4D_ICONBUTTON_TMP_VAL;
  
#define _calc (*((D4D_ICONBUTTON_TMP_VAL*)d4d_scratchPad))

    
static void D4D_IconbuttonValue2Coor(D4D_OBJECT* pThis) {
  _calc.position = D4D_GetClientToScreenPoint(pThis, &pThis->position);
  _calc.txtPos = _calc.position;
}

/******************************************************************************/
static void D4D_IconbuttonOnDraw(D4D_MESSAGE* pMsg) {
    D4D_OBJECT*           pThis       = pMsg->pObject;
    D4D_ICONBUTTON_PTR    pIconbutton = D4D_GET_ICONBUTTON(pThis);    
    D4D_OBJECT_DRAWFLAGS  draw        = pMsg->prm.draw;
    D4D_COLOR clrT, clrB;
    D4D_BMP_PROPERTIES bmpProp;
    D4D_IconbuttonValue2Coor(pThis);
    
    bmpProp = (D4D_ALIGN_V_TOP_MASK | D4D_ALIGN_H_CENTER_MASK);    
    
    clrT = D4D_ObjectGetForeColor(pThis, draw);
    clrB = D4D_ObjectGetBckgColor(pThis, draw);
 
    // when background redraw is needed
        // draw the bitmap        
    if(pIconbutton->pBmpX[pIconbutton->pData->index] != NULL)
      D4D_DrawRBmpRect(&_calc.position,&pThis->size, pIconbutton->pBmpX[pIconbutton->pData->index], bmpProp, clrB, (D4D_BOOL)(!D4D_IsEnabled(pThis)), pThis->radius);
    
    // draw the text
    if(pIconbutton->textBuff.pText != NULL) {
      D4D_SIZE tmp_size;
      tmp_size.cx = D4D_GetTextWidth(pIconbutton->textBuff.fontId, pIconbutton->textBuff.pText);
      tmp_size.cy = D4D_GetFontHeight(pIconbutton->textBuff.fontId);
        
      D4D_DrawTextRect(&_calc.txtPos, &tmp_size, &pIconbutton->textBuff, clrT, clrB); 
    }
    if (pThis->initFlags & D4D_OBJECT_F_FOCUSRECT)       
      D4D_RRect(&_calc.position, &pThis->size, D4D_LINE_THIN, clrT, pThis->radius);  // draw focus rectangle
}

/******************************************************************************/
unsigned char D4D_IconbuttonGetBmpCount(D4D_OBJECT_PTR pThis) {
   D4D_ICONBUTTON_PTR pIconbutton = D4D_GET_ICONBUTTON(pThis);
   unsigned char  ix = 0;   
   
   while(pIconbutton->pBmpX[ix] != NULL) {
     ix++;  
   }
   
   return ix;
}

/******************************************************************************/
void D4D_IconbuttonSetIndex(D4D_OBJECT_PTR pThis, D4D_ICON_INDEX index)
{
    D4D_ICONBUTTON_PTR pIconbutton = D4D_GET_ICONBUTTON(pThis);
    D4D_ICONBUTTON_DATA_PTR pData = pIconbutton->pData;

    if(pData->index == index)
        return ;
    
    if(D4D_IconbuttonGetBmpCount(pThis) == 0)  // There is no loaded bitmaps - go out
        return;
    
    if(index >= D4D_IconbuttonGetBmpCount(pThis))  // If value is bigger then count of BMPs
        return;
    
    // remember new value
    pData->index = index;
    
    D4D_InvalidateObject(pThis, D4D_TRUE);

    // notify user that the value has changed        
    if(pIconbutton->OnValueChanged)
        pIconbutton->OnValueChanged(pThis);
}

/******************************************************************************/
void D4D_IconbuttonChangeIndex(D4D_OBJECT_PTR pThis, sByte incr) {
  unsigned char tmp_ix = (unsigned char)(D4D_GET_ICONBUTTON(pThis)->pData->index + incr);
  
  if(tmp_ix >= D4D_IconbuttonGetBmpCount(pThis))
    tmp_ix = 0;
  
  D4D_IconbuttonSetIndex(pThis, tmp_ix);
}

/******************************************************************************/
unsigned char D4D_IconbuttonGetIndex(D4D_OBJECT_PTR pThis) {
    D4D_ICONBUTTON_PTR pIconbutton = D4D_GET_ICONBUTTON(pThis);
    return pIconbutton->pData->index;
}

/******************************************************************************/
int D4D_IconbuttonGetState(D4D_OBJECT_PTR pThis) {
  if(!pThis) return -1;
  return D4D_GET_ICONBUTTON(pThis)->pData->iButtonState;
}

/******************************************************************************/
unsigned char D4D_IconbuttonSetState(D4D_OBJECT_PTR pThis, unsigned char state) {
  D4D_ICONBUTTON_DATA_PTR pData;

  if(!pThis) return 2;
  if(D4D_IconbuttonGetBmpCount(pThis)-1 < 2*state+1) return 1;

  pData = D4D_GET_ICONBUTTON(pThis)->pData;
  pData->iButtonState = state;
  D4D_IconbuttonSetIndex(pThis, (pData->iButtonState*2) + (D4D_IconbuttonGetIndex(pThis)%2));
  
  return 0;
}

/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
void D4D_IconbuttonOnMessage(D4D_MESSAGE* pMsg) {
  D4D_ICONBUTTON_PTR    pIconbutton;
  D4D_ICONBUTTON_DATA_PTR pIconbuttonData;
  D4D_OBJECT_PTR        pObject;
  char                  bDoClick = 0;
  
  if(!pMsg || !pMsg->pObject) return;
  
  pIconbutton = D4D_GET_ICONBUTTON(pMsg->pObject);
  pIconbuttonData = pIconbutton->pData;

  switch(pMsg->nMsgId) {

  case D4D_MSG_DRAW:                    //1
    D4D_IconbuttonOnDraw(pMsg);
    break;

  case D4D_MSG_SETFOCUS:                //3
    D4D_IconbuttonSetIndex(pMsg->pObject, (pIconbuttonData->iButtonState*2)+1);
    break;
    
  case D4D_MSG_KILLFOCUS:               //4
    D4D_IconbuttonSetIndex(pMsg->pObject, (pIconbuttonData->iButtonState*2));
    break;
    
#ifdef D4D_LLD_TCH
  case D4D_MSG_TOUCHED:                 //9
    pIconbuttonData->bButtonDown=1;
    D4D_FocusSet(pMsg->pScreen, pMsg->pObject);
    break;
    
  case D4D_MSG_TOUCH_AUTO:              //10
    if(!pIconbuttonData->bButtonDown) D4D_FocusSet(pMsg->pScreen, pMsg->pObject);
    pIconbuttonData->bButtonDown=1;
    break;
    
  case D4D_MSG_TOUCH_LOST:              //11
    D4D_FocusSet(pMsg->pScreen, pIconbutton->pFocusKeeper);
    pIconbuttonData->bButtonDown=0;
    break;
    
  case D4D_MSG_UNTOUCHED:               //12
    if(pIconbuttonData->bButtonDown) {                          //OnClick
      if(pIconbuttonData->iStateIncrement &&
         ((pIconbuttonData->iButtonState += pIconbuttonData->iStateIncrement) >= pIconbuttonData->iButtonStatesCnt)) {
        pIconbuttonData->iButtonState = 0;
      }
      if(pIconbutton->OnClick) bDoClick=1;
    }
    pObject = pMsg->pObject;
    D4D_FocusSet(pMsg->pScreen, pIconbutton->pFocusKeeper);                     //(by sending a system message, it changes content of pMsg! -> pObject needs to be saved)
    pIconbuttonData->bButtonDown=0;
    if(bDoClick) pIconbutton->OnClick(pObject);
    break;
#endif /* D4D_LLD_TCH */
    
  default:
    // call the default behavior of all objects
    D4D_ObjOnMessage(pMsg);
    break;
  }
}


/******************************************************************************/
D4D_STRING* D4D_IconbuttonGetTextBuffer(D4D_OBJECT* pThis)
{
  return &(D4D_GET_ICONBUTTON(pThis)->textBuff);  
}

