/******************************************************************************
* 
* Copyright (c) 2010 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_iconbutton.h
*
* @author    b01119
* @author    Petr Kubiznak (ELNICO Ltd.)
* 
* @version   0.0.44.0
* 
* @date      Jan-19-2013
* 
* @brief     D4D iconbutton object extended from the Freescale's icon object.
*
*******************************************************************************/

#ifndef __D4D_ICONBUTTON_H
#define __D4D_ICONBUTTON_H

extern const D4D_OBJECT_SYS_FUNCTION d4d_iconbuttonSysFunc;

/******************************************************************************
* Constants
******************************************************************************/

/******************************************************************************
* Types
******************************************************************************/

#ifndef D4D_ICONBUTTON_F_DEFAULT
  #define D4D_ICONBUTTON_F_DEFAULT  (D4D_OBJECT_F_VISIBLE | D4D_OBJECT_F_ENABLED | D4D_OBJECT_F_TABSTOP | D4D_OBJECT_F_TOUCHENABLE)
#endif

#ifndef D4D_ICONBUTTON_TXT_PRTY_DEFAULT
  #define D4D_ICONBUTTON_TXT_PRTY_DEFAULT  (D4D_ALIGN_H_CENTER_MASK | D4D_ALIGN_V_CENTER_MASK)
#endif

#ifndef D4D_ICONBUTTON_FNT_PRTY_DEFAULT
  #define D4D_ICONBUTTON_FNT_PRTY_DEFAULT  ( 0 )
#endif


typedef D4D_BMP D4D_ICONBUTTON_BMPS;

typedef struct d4d_iconbuttondata {
  unsigned  bButtonDown       : 1;
  unsigned  iButtonStatesCnt  : 8;
  unsigned  iButtonState      : 8;
  signed    iStateIncrement   : 7;
  unsigned  index             : 8;      // index of the icon (from icon object)
  void (*OnClick)(D4D_OBJECT* pThis);
} D4D_ICONBUTTON_DATA, *D4D_ICONBUTTON_DATA_PTR;


typedef struct d4d_iconbutton {
  void (*OnClick)(D4D_OBJECT_PTR pThis);
  void (*OnValueChanged)(D4D_OBJECT_PTR pThis);
  D4D_STRING                  textBuff;       // ICONBUTTON text
  D4D_POINT                   txtOff;         // text coordinates as offset from scrPos
  const D4D_ICON_BMPS* const* pBmpX;          // ICONBUTTON state X idication bitmaps
  D4D_ICONBUTTON_DATA_PTR     pData;          // current object state
  D4D_OBJECT_PTR              pFocusKeeper;   // pointer to focus keeper
} D4D_ICONBUTTON, *D4D_ICONBUTTON_PTR;

/******************************************************************************
* Macros 
******************************************************************************/

// getting the ICONBUTTON structure from general OBJECT
#define D4D_GET_ICONBUTTON(pObj) ((D4D_ICONBUTTON_PTR)((pObj)->pParam))

/*********************************************************
*
* static declaration macros
*
*********************************************************/
          
#define _D4D_DECLARE_ICONBUTTON_BEGIN(type, name, text, x, y, cx, cy, tx, ty, radius, pMargin, pRelations, flags, pScheme, fontId, pFocusKeeper, pUser, pOnClick, pOnValch, pOnUsrMsg) \
    extern const D4D_ICONBUTTON_BMPS*  const name##_bmps[];\
    static D4D_ICONBUTTON_DATA name##_data; \
    static D4D_STR_PROPERTIES name##_strPrties = { D4D_ICONBUTTON_FNT_PRTY_DEFAULT, D4D_ICONBUTTON_TXT_PRTY_DEFAULT}; \
    static type D4D_ICONBUTTON name##_params = \
    { \
      pOnClick, /* OnClick */ \
      pOnValch, /* OnValueCanged */ \
      { text, D4D_TEXT_LEN(text), fontId, &name##_strPrties, D4D_OBJECT_MAX_TEXT_LEN(text), 0}, /* textBuff */ \
      { tx, ty },   /* txtOff     */ \
      &(name##_bmps[0]),  /* pBmpX     */ \
      &(name##_data), /* RAM-based data */ \
      pFocusKeeper /* pFocusKeeper */ \
    }; \
    \
    D4D_DECLARE_OBJECT(type, name, x, y, cx, cy, radius, pMargin, pRelations, pOnUsrMsg, &d4d_iconbuttonSysFunc, &(name##_params), (flags), pUser, pScheme)\
    \
    const D4D_ICON_BMPS*   const name##_bmps[] = \
    {

#define D4D_DECLARE_ICONBUTTON_BMP(pBmp) (pBmp),

#define D4D_DECLARE_ICONBUTTON_END() NULL };


#define D4D_DECLARE_ICONBUTTON_BEGIN(name, text, x, y, cx, cy, tx, ty, flags, pScheme, fontId, pFocusKeeper, pUser, pOnClick, pOnValch, pOnUsrMsg) \
    _D4D_DECLARE_ICONBUTTON_BEGIN(D4D_CONST, name, text, x, y, cx, cy, tx, ty, 0, NULL, NULL, flags, pScheme, fontId, pFocusKeeper, pUser, pOnClick, pOnValch, pOnUsrMsg)

#define D4D_DECLARE_STD_ICONBUTTON_BEGIN(name, text, x, y, cx, cy, tx, ty, fontId, focuskeeper, onclick) \
    D4D_DECLARE_ICONBUTTON_BEGIN(name, text, x, y, cx, cy, tx, ty, (D4D_ICONBUTTON_F_DEFAULT), NULL, fontId, focuskeeper, NULL, onclick, NULL, NULL)


// IN RAM instantions macros
#define D4D_DECLARE_ICONBUTTON_BEGIN_INRAM(name, text, x, y, cx, cy, tx, ty, flags, pScheme, fontId, pFocusKeeper, pUser, pOnClick, pOnValch, pOnUsrMsg) \
    _D4D_DECLARE_ICONBUTTON_BEGIN(D4D_NO_CONST, name, text, x, y, cx, cy, tx, ty, 0, NULL, NULL, flags, pScheme, fontId, pFocusKeeper, pUser, pOnClick, pOnValch, pOnUsrMsg)

#define D4D_DECLARE_STD_ICONBUTTON_BEGIN_INRAM(name, text, x, y, cx, cy, tx, ty, fontId, focuskeeper, onclick) \
    D4D_DECLARE_ICONBUTTON_BEGIN_INRAM(name, text, x, y, cx, cy, tx, ty, (D4D_ICONBUTTON_F_DEFAULT), NULL, fontId, focuskeeper, NULL, onclick, NULL, NULL)


/******************************************************************************
* Global variables
******************************************************************************/

/******************************************************************************
* Global functions
******************************************************************************/

/*********************************************************
*
* global functions
*
*********************************************************/

/** Retrieves state of the ICONBUTTON object. See D4D_IconbuttonSetState() for
*   more information.
*   @param[in]  pThis   Object to read its state.
*   @return             State of the button (non-negative integer)
*                       or -1 in case of failure, e.g. pThis==NULL. */
int D4D_IconbuttonGetState(D4D_OBJECT_PTR pThis);

/** Sets state of the ICONBUTTON object. Each state requires two bitmaps for
*   proper functionality. E.g. if setting state 2, the object needs to have
*   at least 6 bitmaps, using bitmaps at indices 4 and 5 for the new state.
*   Default state is 0, requiring two bitmaps (0 and 1).
* @param[in,out]  pThis   Object to be modified.
* @param[in]      state   Index of new state.
* @return                 Zero on success, 1 for insufficient number of bitmaps,
*                         2 in case of other failure, e.g. pThis==NULL. */
unsigned char D4D_IconbuttonSetState(D4D_OBJECT_PTR pThis, unsigned char state);

D4D_ICON_INDEX D4D_IconbuttonGetBmpCount(D4D_OBJECT_PTR pThis);
// Obsolete functions, replaced by any general
#define D4D_IconSetText D4D_SetText




#endif /* __D4D_ICON_H */        


