
#include <mqx.h>
#include <bsp.h>
#include <rtcs.h>
#include <ftpd.h>
#include "ethernet_ftp.h"
#include "../tasks.h"
#include "..\format\format.h"

#include <mfs.h>
#include <shell.h>
#include <ipcfg.h>

#if ! BSPCFG_ENABLE_IO_SUBSYSTEM
#error This application requires BSPCFG_ENABLE_IO_SUBSYSTEM defined non-zero in user_config.h. Please recompile BSP with this option.
#endif


#ifndef BSP_DEFAULT_IO_CHANNEL_DEFINED
#error This application requires BSP_DEFAULT_IO_CHANNEL to be not NULL. Please set corresponding BSPCFG_ENABLE_TTYx to non-zero in user_config.h and recompile BSP with this option.
#endif

#if ! FTPDCFG_USES_MFS
#error This application requires FTPDCFG_USES_MFS defined non-zero in user_config.h. Please recompile BSP with this option.
#endif

#if ! FTPDCFG_ENABLE_USERNAME_AND_PASSWORD
#warning This application requires FTPDCFG_ENABLE_USERNAME_AND_PASSWORD defined non-zero in user_config.h. Please recompile BSP with this option.
#endif

/****************************************************************************
*       Global variables declarations
***************************************************************************/
// IP address
IPCFG_IP_ADDRESS_DATA gIPAddress;
// MAC address
_enet_address         gMacAddress;
MQX_FILE_PTR          ftp_filesystem_handle;

//Set root directory (MFS disk) for FTP server
char FTPd_rootdir[] = {"A:\\"}; 

//List of FTP commands
const FTPd_COMMAND_STRUCT FTPd_commands[] = {

   { "abor", FTPd_unimplemented },
   { "acct", FTPd_unimplemented },
   { "cdup", FTPd_cdup },        
   { "cwd",  FTPd_cd   },        
   { "feat", FTPd_feat },       
   { "help", FTPd_help },       
   { "dele", FTPd_dele },       
   { "list", FTPd_list },       
   { "mkd",  FTPd_mkdir},       
   { "noop", FTPd_noop },
   { "nlst", FTPd_nlst },       
   { "opts", FTPd_opts },
   { "pass", FTPd_pass },
   { "pasv", FTPd_pasv },
   { "port", FTPd_port },
   { "pwd",  FTPd_pwd  },       
   { "quit", FTPd_quit },
   { "rnfr", FTPd_rnfr },
   { "rnto", FTPd_rnto },
   { "retr", FTPd_retr },
   { "stor", FTPd_stor },
   { "rmd",  FTPd_rmdir},       
   { "site", FTPd_site },
   { "size", FTPd_size },
   { "syst", FTPd_syst },
   { "type", FTPd_type },
   { "user", FTPd_user },
   { "xcwd", FTPd_cd    },        
   { "xmkd", FTPd_mkdir },       
   { "xpwd", FTPd_pwd   },       
   { "xrmd", FTPd_rmdir },       
   { NULL,   NULL } 
};
/******************************************************************************/

/*******************************************************************************
* Initialize RTCS stack (ethernet).
* @return                 TRUE - Initialization OK
*                         FALSE - Initialization failed
*******************************************************************************/
boolean Enet_init(void) {
_enet_address address;
uint_32 error;

#if RTCS_MINIMUM_FOOTPRINT
    /* We are running on a memory constrained device, lower RTCS's defaults */

#if PSP_MQX_CPU_IS_MCF5223X
    /* runtime RTCS configuration for small memory <32kB*/
    _RTCSPCB_init = 3;
    _RTCSPCB_grow = 0;
    _RTCSPCB_max = 3;
    _RTCS_msgpool_init = 3;
    _RTCS_msgpool_grow = 0;
    _RTCS_msgpool_max  = 3;
    _RTCS_socket_part_init = 2;
    _RTCS_socket_part_grow = 1;
    _RTCS_socket_part_max  = 6;
#else
    /* runtime RTCS configuration */
    _RTCSPCB_init = 4;
    _RTCSPCB_grow = 2;
    _RTCSPCB_max = 6;
    _RTCS_msgpool_init = 4;
    _RTCS_msgpool_grow = 2;
    _RTCS_msgpool_max  = 8;
    _RTCS_socket_part_init = 4;
    _RTCS_socket_part_grow = 2;
    _RTCS_socket_part_max  = 6;
#endif

    _RTCSTASK_stacksize = 2200;
#endif /* RTCS_MINIMUM_FOOTPRINT */

  // Set RTCS priority to 6  
  _RTCSTASK_priority = 6;

  //Create RTCS stack
error = RTCS_create();    
if((error!=RTCS_OK) && (error!=RTCSERR_INITIALIZED)) {
printf("\nError initializing RTCS: 0x%0X", error);
return FALSE;
}

// Get MAC address
ENET_get_mac_address(BSP_DEFAULT_ENET_DEVICE, ENET_IPADDR, address);

error = ipcfg_init_device(BSP_DEFAULT_ENET_DEVICE, address);   
if(error) {
printf("\nError initializing IPCFG: 0x%0X", error);
return FALSE;
}
return TRUE;
  
}

/*******************************************************************************
* Initialize FTP server.
* @return                 TRUE - Initialization OK
*                         FALSE - Initialization failed
* @comment FTP server priority set to 8
*******************************************************************************/
boolean FTP_init(void) {
  uint_32 error;
  
   RAM_disk_format(&ftp_filesystem_handle, (uchar_ptr)FTP_MSD_DISK_BASE, (_file_size)FTP_MSD_DISK_SIZE, FTP_DRIVE, "mfs_ramdisk_f:", "SQM4_FTP");
  
   //FTPd_userfile = "users.txt";         //users file name
   FTPd_userfile = NULL;         //users file name
   
   error = FTPd_init("FTP_server", 8, 5000 );   // Init FTP server better than FTPSRV_init(), task priority is lower than RTCS task priority
   if(error!=RTCS_OK) {
     printf("\nError initializing FTP server: 0x%0X", error);
     return FALSE;
   }
   return TRUE;
}

/*******************************************************************************
* Get IP address from DHCP server and save it to global variable gIPAddress.
* @return                 void
*******************************************************************************/
void Get_IP_address(void) {
    IPCFG_STATE state;
    static boolean done = FALSE;
    uint_32 result = IPCFG_OK;

    while(1) {
      do {
        _time_delay(200);
        result = ipcfg_bind_dhcp(BSP_DEFAULT_ENET_DEVICE, FALSE);
      } while (result == RTCSERR_IPCFG_BUSY);
      if (result != IPCFG_OK) {
        continue;
      }
      
      do {
        _time_delay (200);
        result = ipcfg_poll_dhcp(BSP_DEFAULT_ENET_DEVICE, FALSE, NULL);
      } while (result == RTCSERR_IPCFG_BUSY);
      
      state = ipcfg_get_state (BSP_DEFAULT_ENET_DEVICE);
      if ((state != IPCFG_STATE_INIT) && (result == IPCFG_OK)) {
        ipcfg_get_mac(BSP_DEFAULT_ENET_DEVICE, gMacAddress);
        ipcfg_get_ip (BSP_DEFAULT_ENET_DEVICE, &gIPAddress);

        printf( "MAC : %02X:%02X:%02X:%02X:%02X:%02X\n", gMacAddress[0],gMacAddress[1],
          gMacAddress[2],gMacAddress[3],gMacAddress[4],gMacAddress[5]);
        printf ("IP  : %d.%d.%d.%d\n", IPBYTES(gIPAddress.ip));
        printf ("MASK  : %d.%d.%d.%d\n", IPBYTES(gIPAddress.mask));
        printf ("GATEWAY  : %d.%d.%d.%d\n", IPBYTES(gIPAddress.gateway));
        done=TRUE;
      }
      
      if(done) {
        done=FALSE;
        return;
      }
    }
}
