#ifndef __file_management_h__
#define __file_management_h__

#include <mfs.h>
#include <shell.h>
#include <mqx.h>
#include "..\main.h"

#define FILENAME_SHORT_SIZE             14

/** Checks whether a file is a directory.
* @param[in]  absPath   Absolute path to the file, e.g. "A:\abs\path\file.ext".
* @return               Non-zero value for any directory, zero for other files. */
boolean MFS_isDir(char *absPath);

/** Gets file attributes as bit array. See MFS_ATTR_??? macros.
* @param[in]  absPath   Absolute path to the file, e.g. "A:\abs\path\file.ext".
* @param[out] attrs     File attributes will be stored here.
* @return               0 on success, 1 for invalid arguments, 2 if ioctl fails. */
unsigned char MFS_getFileAttrs(char *absPath, unsigned char *attrs);

/** Copies a file. If it is a directory, only empty directory is copied,
*   the content has to be handled manually.
*   TODO: Copy directories recursively. */
boolean MFS_copy_files(char _PTR_ in_MFS, char _PTR_ in_dir, char _PTR_ in_file, 
                       char _PTR_ out_MFS, char _PTR_ out_dir,
                       char _PTR_ out_file);

void printf_dir(char * fs_name);

boolean MFS_delete_file(char * file_system, char *dir, char * file_name);

boolean MFS_make_dir(char * file_system, char *dir, char *dir_name);

#endif
