#ifndef __format_h__
#define __format_h__

#include <mfs.h>
#include <shell.h>
#include <mqx.h>
#include "..\main.h"

#define RAM_DISK_APP        (1)

#define USB_MSD_DISK_BASE     ((D4D_MEMPOOL_BASE)+(D4D_MEMPOOL_SIZE)+1)

#if RAM_DISK_APP
    /* Length of Each Logical Address Block */
    #define LENGTH_OF_EACH_LAB              (512)    
    /* total number of logical blocks present */
    #define USB_TOTAL_LOGICAL_ADDRESS_BLOCKS    ((0x77FFFFF0-(USB_MSD_DISK_BASE)-1)/LENGTH_OF_EACH_LAB / 2)                                       
    /* Net Disk Size */
    #define USB_MSD_DISK_SIZE                       (LENGTH_OF_EACH_LAB * USB_TOTAL_LOGICAL_ADDRESS_BLOCKS)
#endif



#define FTP_MSD_DISK_BASE     ((USB_MSD_DISK_BASE)+(USB_MSD_DISK_SIZE)+1)

#if RAM_DISK_APP
    /* total number of logical blocks present */
    #define FTP_TOTAL_LOGICAL_ADDRESS_BLOCKS    ((0x77FFFFF0-(FTP_MSD_DISK_BASE)-1)/LENGTH_OF_EACH_LAB)                                       
    /* Net Disk Size */
    #define FTP_MSD_DISK_SIZE                       (LENGTH_OF_EACH_LAB * FTP_TOTAL_LOGICAL_ADDRESS_BLOCKS)
#endif


#define USB_DRIVE       "B:"
#define FTP_DRIVE       "F:"


/** Creates an MFS virtual drive in RAM memory, occupying area from [base] to
*   [base+size].
* @param[out] fd_ptr  Address to store filesystem descriptor to.
* @param[in]  base    Base address of the drive in RAM.
* @param[in]  size    Number of bytes to allocate for the drive.
* @param[in]  drive   Drive identifier, e.g. "c:". Must contain the colon!
* @param[in]  device  Device identifier, e.g. "mfs_ramdisk_c:". Must contain
*                     the semicolon!
* @param[in]  label   Up to 8 characters labeling the disk, e.g. "MY_RAM_C".
* @return             TRUE on success, FALSE on failure. */
boolean RAM_disk_format(MQX_FILE_PTR *fd_ptr, pointer base, _file_size size, char *drive, char *device, char *label);
#endif