/******************************************************************************
* 
* Copyright (c) 2012 Elnico s.r.o.
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY ELNICO "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL ELNICO OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
****************************************************************************//*!
*
* @file      d4d_screen_audioplr.c
*
* @author    Petr Kubiznak
* 
* @version   1.0
* 
* @date      Sep-14-2012
* 
* @brief     D4D demo - audioplayer screen
*
******************************************************************************/


/****************************************************************************
*       Includes header files
****************************************************************************/
#include <d4d.h>
#include <lwmsgq.h>

#include "d4d_iconbutton.h"
#include "fonts.h"
#include "strings.h"
#include "audio/wave_task.h"
#include "file_list.h"
#include "utils.h"
#include "schemes.h"

/****************************************************************************
*       Local functions declarations
***************************************************************************/

static void changeState(WAVE_MSG_STATE newState);

static void btnPlay_OnClick(D4D_OBJECT_PTR pThis);
static void btnStop_OnClick(D4D_OBJECT_PTR pThis);
static void btnClose_OnClick(D4D_OBJECT_PTR pThis);


/****************************************************************************
*       Local variables
***************************************************************************/

static char             gsSongFilename[PATHNAME_SIZE] = "A:\\TEST.WAV";
static D4D_TCHAR        gsState[PATHNAME_SIZE+50]     = D4D_DEFSTR("???");
static WAVE_MSG_STATE   gLastState                    = WAVEST_NOTINIT;

/****************************************************************************
*       D4D widgets (graphic objects) declarations
****************************************************************************/

#if ((D4D_SCREEN_SIZE_LONGER_SIDE >= 800) && (D4D_SCREEN_SIZE_SHORTER_SIDE >= 480))
# define  LCD_800X480                   1
#else
# define  LCD_800X480                   0
#endif

#define SCREEN_SIZE_X                   (D4D_SCREEN_SIZE_LONGER_SIDE)
#define SCREEN_SIZE_Y                   (D4D_SCREEN_SIZE_SHORTER_SIDE)

#if LCD_800X480
# define BUTTON_PLAY_SIZE_X             (236)
# define BUTTON_PLAY_SIZE_Y             (210)
#else /* LCD_800X480 */
# define BUTTON_PLAY_SIZE_X             (144)
# define BUTTON_PLAY_SIZE_Y             (120)
#endif /* LCD_800X480 */
#define BUTTON_PLAY_POS_X               (((SCREEN_SIZE_X)-(BUTTON_PLAY_SIZE_X))/2)
#define BUTTON_PLAY_POS_Y               (((SCREEN_SIZE_Y)-(BUTTON_PLAY_SIZE_Y))/2)

#define BUTTON_PAUSE_SIZE_X             (BUTTON_PLAY_SIZE_X)
#define BUTTON_PAUSE_SIZE_Y             (BUTTON_PLAY_SIZE_Y)
#define BUTTON_PAUSE_POS_X              (BUTTON_PLAY_POS_X)
#define BUTTON_PAUSE_POS_Y              (BUTTON_PLAY_POS_Y)

#define BUTTON_CLOSE_SIZE_X             ((BUTTON_PLAY_SIZE_X)/2)
#define BUTTON_CLOSE_SIZE_Y             ((BUTTON_PLAY_SIZE_Y)/2)
#define BUTTON_CLOSE_POS_X              (((BUTTON_PLAY_POS_X)-(BUTTON_CLOSE_SIZE_X))/2)
#define BUTTON_CLOSE_POS_Y              (((SCREEN_SIZE_Y)-(BUTTON_CLOSE_SIZE_Y))/2)

#define BUTTON_STOP_SIZE_X              (BUTTON_CLOSE_SIZE_X)
#define BUTTON_STOP_SIZE_Y              (BUTTON_CLOSE_SIZE_Y)
#define BUTTON_STOP_POS_X               ((3*(BUTTON_PLAY_POS_X) + 2*(BUTTON_PLAY_SIZE_X) - (BUTTON_STOP_SIZE_X))/2)
#define BUTTON_STOP_POS_Y               (BUTTON_CLOSE_POS_Y)

#define LABEL_STATE_SIZE_X              ((SCREEN_SIZE_X)-2)
#define LABEL_STATE_SIZE_Y              (20)
#define LABEL_STATE_POS_X               (1)
#define LABEL_STATE_POS_Y               (((BUTTON_PLAY_POS_Y)-(LABEL_STATE_SIZE_Y))/2)


/*******************************************
* Main content objects - button icons
*******************************************/

// Bitmap filenames

#if LCD_800X480
# define FILEPATH_BMP_BUTTONS           "A:\\SYSTEM\\IMAGES\\AUDIOPLR\\S800X480\\"
#else /* LCD_800X480 */
# define FILEPATH_BMP_BUTTONS           "A:\\SYSTEM\\IMAGES\\AUDIOPLR\\S480X272\\"
#endif /* LCD_800X480 */

#define FILENAME_BMP_PLAY               FILEPATH_BMP_BUTTONS "PLAYN.D4L"
#define FILENAME_BMP_PLAY_A             FILEPATH_BMP_BUTTONS "PLAYA.D4L"
#define FILENAME_BMP_PAUSE              FILEPATH_BMP_BUTTONS "PAUSEN.D4L"
#define FILENAME_BMP_PAUSE_A            FILEPATH_BMP_BUTTONS "PAUSEA.D4L"
#define FILENAME_BMP_STOP               FILEPATH_BMP_BUTTONS "STOPN.D4L"
#define FILENAME_BMP_STOP_A             FILEPATH_BMP_BUTTONS "STOPA.D4L"
#define FILENAME_BMP_CLOSE              FILEPATH_BMP_BUTTONS "CLOSEN.D4L"
#define FILENAME_BMP_CLOSE_A            FILEPATH_BMP_BUTTONS "CLOSEA.D4L"

// Bitmap structures - will be initialized in ScreenInit()
D4D_BMP sAudioplr_bmpPlay       = { NULL, NULL };
D4D_BMP sAudioplr_bmpPlayA      = { NULL, NULL };
D4D_BMP sAudioplr_bmpPause      = { NULL, NULL };
D4D_BMP sAudioplr_bmpPauseA     = { NULL, NULL };
D4D_BMP sAudioplr_bmpStop       = { NULL, NULL };
D4D_BMP sAudioplr_bmpStopA      = { NULL, NULL };
D4D_BMP sAudioplr_bmpClose      = { NULL, NULL };
D4D_BMP sAudioplr_bmpCloseA     = { NULL, NULL };


/*******************************************
* Main content objects - buttons
*******************************************/
D4D_DECLARE_BUTTON(sAudioplr_btnFocusKeeper, NULL, 0, 0, 1, 1, (D4D_OBJECT_F_ENABLED | D4D_OBJECT_F_TABSTOP), NULL, NULL, NULL, FONT_SYSTEM, NULL, NULL, NULL)

// PLAY
D4D_DECLARE_STD_ICONBUTTON_BEGIN(sAudioplr_btnPlay, NULL, BUTTON_PLAY_POS_X, BUTTON_PLAY_POS_Y, BUTTON_PLAY_SIZE_X, BUTTON_PLAY_SIZE_Y, 0, 0, FONT_SYSTEM, &sAudioplr_btnFocusKeeper, btnPlay_OnClick)
  D4D_DECLARE_ICON_BMP(&sAudioplr_bmpPlay)
  D4D_DECLARE_ICON_BMP(&sAudioplr_bmpPlayA)
  D4D_DECLARE_ICON_BMP(&sAudioplr_bmpPause)
  D4D_DECLARE_ICON_BMP(&sAudioplr_bmpPauseA)
D4D_DECLARE_ICON_END()

// STOP
D4D_DECLARE_STD_ICONBUTTON_BEGIN(sAudioplr_btnStop, NULL, BUTTON_STOP_POS_X, BUTTON_STOP_POS_Y, BUTTON_STOP_SIZE_X, BUTTON_STOP_SIZE_Y, 0, 0, FONT_SYSTEM, &sAudioplr_btnFocusKeeper, btnStop_OnClick)
  D4D_DECLARE_ICON_BMP(&sAudioplr_bmpStop)
  D4D_DECLARE_ICON_BMP(&sAudioplr_bmpStopA)
D4D_DECLARE_ICON_END()

// CLOSE
D4D_DECLARE_STD_ICONBUTTON_BEGIN(sAudioplr_btnClose, NULL, BUTTON_CLOSE_POS_X, BUTTON_CLOSE_POS_Y, BUTTON_CLOSE_SIZE_X, BUTTON_CLOSE_SIZE_Y, 0, 0, FONT_SYSTEM, &sAudioplr_btnFocusKeeper, btnClose_OnClick)
  D4D_DECLARE_ICON_BMP(&sAudioplr_bmpClose)
  D4D_DECLARE_ICON_BMP(&sAudioplr_bmpCloseA)
D4D_DECLARE_ICON_END()

// STATE LABEL
D4D_DECLARE_LABEL_INRAM(sAudioplr_lblState, gsState, LABEL_STATE_POS_X, LABEL_STATE_POS_Y, LABEL_STATE_SIZE_X, LABEL_STATE_SIZE_Y, (D4D_OBJECT_F_VISIBLE | D4D_OBJECT_F_ENABLED), &schemeWhiteOnBlack, FONT_SYSTEM_SMALL, NULL, NULL);

/****************************************************************************
* Screen declaration
****************************************************************************/
D4D_DECLARE_STD_SCREEN_BEGIN(screen_audioplr, ScreenAudioplr_)  
  D4D_DECLARE_SCREEN_OBJECT(sAudioplr_btnFocusKeeper)
  D4D_DECLARE_SCREEN_OBJECT(sAudioplr_btnPlay)
  D4D_DECLARE_SCREEN_OBJECT(sAudioplr_btnStop)
  D4D_DECLARE_SCREEN_OBJECT(sAudioplr_btnClose)
  D4D_DECLARE_SCREEN_OBJECT(sAudioplr_lblState)
D4D_DECLARE_SCREEN_END()


/*******************************************
* External screens
*******************************************/


/****************************************************************************
*       D4D standard screen functions definitions
****************************************************************************/

// One time called screen function in screen initialization proces
static void ScreenAudioplr_OnInit() {
  loadBitmap(&sAudioplr_bmpPlay,        FILENAME_BMP_PLAY);
  loadBitmap(&sAudioplr_bmpPlayA,       FILENAME_BMP_PLAY_A);
  loadBitmap(&sAudioplr_bmpPause,       FILENAME_BMP_PAUSE);
  loadBitmap(&sAudioplr_bmpPauseA,      FILENAME_BMP_PAUSE_A);
  loadBitmap(&sAudioplr_bmpStop,        FILENAME_BMP_STOP);
  loadBitmap(&sAudioplr_bmpStopA,       FILENAME_BMP_STOP_A);
  loadBitmap(&sAudioplr_bmpClose,       FILENAME_BMP_CLOSE);
  loadBitmap(&sAudioplr_bmpCloseA,      FILENAME_BMP_CLOSE_A);
}

/******************************************************************************/
// Screen on Activate function called with each screen activation
static void ScreenAudioplr_OnActivate() {
  changeState(gLastState);
}

/******************************************************************************/
// Screen "Main" function called periodically in each D4D_poll runs
static void ScreenAudioplr_OnMain() {
  WAVE_MSG_STATE currentState = WAVE_GetState();
  if(currentState!=gLastState) changeState(currentState);
}

/******************************************************************************/
// Screen on DeActivate function called with each screen deactivation
static void ScreenAudioplr_OnDeactivate() {
}

/******************************************************************************/
// Screen on message function called with each internal massage for this screen
static Byte ScreenAudioplr_OnObjectMsg(D4D_MESSAGE* pMsg) {
  if(pMsg->pObject == NULL) {
    switch(pMsg->nMsgId) {
      case D4D_MSG_DRAW:
        // redraw background under buttons 
        D4D_ClearScreen(D4D_COLOR_BLACK);
        break;
    }
  }
  return 0;
}


/****************************************************************************
*       D4D widgets static events functions definitions
****************************************************************************/

// On Click Event of "Play" button
static void btnPlay_OnClick(D4D_OBJECT_PTR pThis) {
  WAVE_MSG message = { D4D_IconbuttonGetState(&sAudioplr_btnPlay)==0 ? WAVE_PLAY : WAVE_PAUSE, gsSongFilename };  
  _lwmsgq_send((pointer)wave_queue, (uint_32*)&message, 0);
}

// On Click Event of "Stop" button
static void btnStop_OnClick(D4D_OBJECT_PTR pThis) {
  WAVE_MSG message = { WAVE_STOP, NULL };
  _lwmsgq_send((pointer)wave_queue, (uint_32*)&message, 0);
}

// On Click Event of "Close" button
static void btnClose_OnClick(D4D_OBJECT_PTR pThis) {
  D4D_EscapeScreen();
}

/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
/** Changes state of the player to the given one.
* @param[in]  newState  New state to set to gLastState and the GUI. */
static void changeState(WAVE_MSG_STATE newState) {
  D4D_CHAR  pAscii[sizeof(gsState)];
  D4D_TCHAR pUnicode[sizeof(gsState)];
  gLastState = newState;
  
  switch(gLastState) {
  case WAVEST_NOTINIT:
    D4D_StrCopyAscii2Unicode(pUnicode, "Player not initialized.");
    D4D_SetText(&sAudioplr_lblState, pUnicode);
    break;
    
  case WAVEST_READY:
    D4D_IconbuttonSetState(&sAudioplr_btnPlay, 0);

    D4D_StrCopy(pAscii, "Ready: ");
    D4D_StrCat(pAscii, gsSongFilename);
    D4D_SetText(&sAudioplr_lblState, D4D_StrCopyAscii2Unicode(pUnicode, pAscii));
    break;
    
  case WAVEST_RECORDING:
    D4D_StrCopyAscii2Unicode(pUnicode, "Recording...");
    D4D_SetText(&sAudioplr_lblState, pUnicode);
    break;
    
  case WAVEST_PLAYING:
    D4D_IconbuttonSetState(&sAudioplr_btnPlay, 1);
    
    D4D_StrCopy(pAscii, "Playing: ");
    D4D_StrCat(pAscii, gsSongFilename);
    D4D_SetText(&sAudioplr_lblState, D4D_StrCopyAscii2Unicode(pUnicode, pAscii));
    break;
    
  case WAVEST_PLAYING_HOLD:
    D4D_StrCopyAscii2Unicode(pUnicode, "Playing hold.");
    D4D_SetText(&sAudioplr_lblState, pUnicode);
    break;
    
  case WAVEST_PAUSE:
    D4D_IconbuttonSetState(&sAudioplr_btnPlay, 0);
    
    D4D_StrCopy(pAscii, "Paused: ");
    D4D_StrCat(pAscii, gsSongFilename);
    D4D_SetText(&sAudioplr_lblState, D4D_StrCopyAscii2Unicode(pUnicode, pAscii));
    break;
  }
}

/** Opens the screen and starts playing a song.
* @param[in]  filename  Name of song to play. */
void ScrAudioplr_Show(const D4D_CHAR *filename) {
  D4D_StrCopy(gsSongFilename, filename);
  btnStop_OnClick(NULL);
  btnPlay_OnClick(NULL);
  D4D_ActivateScreen((D4D_SCREEN*)&screen_audioplr, D4D_FALSE);
}

