/******************************************************************************
* 
* Copyright (c) 2012 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_screen_fileviewer.c
*
* @author    b01119
* 
* @version   0.0.1.0
* 
* @date      Aug-14-2012
* 
* @brief     D4D demo - fileviewer
*
******************************************************************************/


/****************************************************************************
*       Includes header files
****************************************************************************/
#include "d4d.h"
//#include "d4d_screen_openfile.h"
#include "../file_list.h"
#include "fonts.h"
#include "strings.h"

/****************************************************************************
*       Local type definitions
***************************************************************************/
typedef struct
{
  D4D_CHAR* strFileExt;
  void (*pFileHandler)(const D4D_CHAR *pFileName);
}FILE_ASSOCIATION;

/****************************************************************************
*       Local functions declarations
***************************************************************************/

//static void SFL_BtnOpenOnClick(D4D_OBJECT* pThis);
static void SFL_BtnQuitOnClick(D4D_OBJECT* pThis);
static void RadioButnTxtType_OnChange(D4D_OBJECT* pThis);
static void ScrFileviewer_FileHandleUnsupported (const D4D_CHAR* pFileName);
static void ScrFileviewer_FileHandlePicture     (const D4D_CHAR* pFileName);
static void ScrFileviewer_FileHandleText        (const D4D_CHAR* pFileName);


/****************************************************************************
*       Local variables declarations
***************************************************************************/

const static FILE_ASSOCIATION fileAssociationTable[] = {
  {"BMP",       ScrFileviewer_FileHandlePicture},
  {"D4L",       ScrFileviewer_FileHandlePicture},
  {"TXT",       ScrFileviewer_FileHandleText},
  {"C",         ScrFileviewer_FileHandleText},
  {"CPP",       ScrFileviewer_FileHandleText},
  {"H",         ScrFileviewer_FileHandleText},
  {"HPP",       ScrFileviewer_FileHandleText},
  {"ICF",       ScrFileviewer_FileHandleText},
  {"LCF",       ScrFileviewer_FileHandleText},
  {"HTM",       ScrFileviewer_FileHandleText},
  {NULL,        NULL}
};

const D4D_CHAR *fileName = NULL;

/****************************************************************************
*       D4D widgets (graphic objects) declarations
****************************************************************************/

#define CONTENT_POS_X           D4D_BEVEL_WIDTH
#define CONTENT_POS_Y           D4D_BEVEL_WIDTH

#define CONTENT_SIZE_X          320
#define CONTENT_SIZE_Y          230

#define PANEL_SIZE_X            (D4D_SCREEN_SIZE_LONGER_SIDE - CONTENT_SIZE_X - 2*D4D_BEVEL_WIDTH)
#define PANEL_SIZE_Y            (CONTENT_SIZE_Y + 2*D4D_BEVEL_WIDTH)

#define PANEL_POS_X             (CONTENT_SIZE_X + 2*D4D_BEVEL_WIDTH)
#define PANEL_POS_Y             0

#define BTN_SIZE_X              100
#define BTN_SIZE_Y              34

#define BTN_POS_X(x)            (10 + (x) * (BTN_SIZE_X + 10))
#define BTN_POS_Y               (CONTENT_SIZE_Y + 2 + 2*D4D_BEVEL_WIDTH)

#define PNL_OBJ_SIZE_X          (PANEL_SIZE_X - 8)
#define PNL_OBJ_SIZE_Y          22

#define PNL_OBJ_POS_X           0
#define PNL_OBJ_POS_Y(y)        ((y) * (PNL_OBJ_SIZE_Y + 4))

// My help macro to simplify label declaration for my application purposes
#define D4D_DECLARE_MY_LABEL(name, text, x, y, cx, cy, font, pRelations) \
        _D4D_DECLARE_LABEL(D4D_CONST, name, text, x, y, cx, cy, 0, NULL, pRelations, (D4D_OBJECT_F_VISIBLE | D4D_OBJECT_F_ENABLED), NULL, font, NULL, NULL)

/*******************************************
* Main content objects - not supported file
*******************************************/          
D4D_TCHAR sFileviewer_CntntMsgStr[5] = DEF_EXTSTR(STR_DUMMY_LABEL);
D4D_DECLARE_STD_LABEL(sFileviewer_CntntMsg, sFileviewer_CntntMsgStr, CONTENT_POS_X, CONTENT_POS_Y, CONTENT_SIZE_X, CONTENT_SIZE_Y, FONT_SYSTEM)

/*******************************************
* Main content objects - picture file
*******************************************/
// Declare the array of file path in RAM
D4D_CHAR pictureFileName[PATHNAME_SIZE] = "a:\\";
// Declare the the bitmap structure
D4D_DECLARE_BMPFILE(sFileviewer_CntntPict_bmp, pictureFileName)
// Declare the picture object
_D4D_DECLARE_PICTURE(D4D_CONST, sFileviewer_CntntPict, CONTENT_POS_X, CONTENT_POS_Y, CONTENT_SIZE_X, CONTENT_SIZE_Y, 0, NULL, NULL, &sFileviewer_CntntPict_bmp, D4D_OBJECT_F_ENABLED, NULL, NULL)

/*******************************************
* Main content objects - text file
*******************************************/
// Declare the tab table
D4D_DECLARE_TAB_TABLE_BEGIN(sFileviewer_CntntTextTabs)
  D4D_DECLARE_TAB(10) // tab space in pixels
  D4D_DECLARE_TAB(20) // tab space in pixels
  D4D_DECLARE_TAB(30) // tab space in pixels
  D4D_DECLARE_TAB(40) // tab space in pixels
  D4D_DECLARE_TAB(50) // tab space in pixels
  D4D_DECLARE_TAB(60) // tab space in pixels
  D4D_DECLARE_TAB(70) // tab space in pixels
  D4D_DECLARE_TAB(80) // tab space in pixels
  D4D_DECLARE_TAB(90) // tab space in pixels
  D4D_DECLARE_TAB(100) // tab space in pixels
D4D_DECLARE_TAB_TABLE_END

//Declare text box object to showing text files
D4D_DECLARE_TEXTBOX(sFileviewer_CntntText, NULL, CONTENT_POS_X, CONTENT_POS_Y, CONTENT_SIZE_X, CONTENT_SIZE_Y, NULL, &sFileviewer_CntntTextTabs, NULL, (D4D_OBJECT_F_ENABLED | D4D_OBJECT_F_TOUCHENABLE), NULL, FONT_SYSTEM_SMALL, 0, NULL, NULL)

/*******************************************
* Information panel objects - not supported file
*******************************************/

// Dummy panel information group box
D4D_DECLARE_STD_GROUP_BOX_BEGIN(sFileviewer_GrpBxMsg, PANEL_POS_X, PANEL_POS_Y, PANEL_SIZE_X, PANEL_SIZE_Y, 0, NULL, DEF_EXTSTR(STR_INFO), FONT_SYSTEM_SMALL)

D4D_DECLARE_GROUP_BOX_OBJECT_END


/*******************************************
* Information panel objects - picture file
*******************************************/
// Objects relations structure declaration for picture files information panel
D4D_EXTERN_OBJECT(sFileviewer_GrpBxPict)
D4D_DECLARE_OBJECT_RELATIONS_BEGIN(grpBxPic_Rel, &sFileviewer_GrpBxPict)
D4D_DECLARE_OBJECT_RELATIONS_END

// Declaration of label object of string "Type"
D4D_DECLARE_MY_LABEL(grpBx_LblTypeName, DEF_EXTSTR(STR_TYPE), PNL_OBJ_POS_X, PNL_OBJ_POS_Y(0), PNL_OBJ_SIZE_X, PNL_OBJ_SIZE_Y, FONT_SYSTEM, grpBxPic_Rel) // Type text Name

// Declaration of label object of string type value
D4D_TCHAR grpBx_LblTypeStr[8] = D4D_DEFSTR("None");
D4D_DECLARE_MY_LABEL(grpBx_LblType, grpBx_LblTypeStr, PNL_OBJ_POS_X, PNL_OBJ_POS_Y(1), PNL_OBJ_SIZE_X, PNL_OBJ_SIZE_Y, FONT_SYSTEM, grpBxPic_Rel) // Type text

// Declaration of label object of string "Width"
D4D_DECLARE_MY_LABEL(grpBx_LblWidthName, DEF_EXTSTR(STR_WIDTH), PNL_OBJ_POS_X, PNL_OBJ_POS_Y(2), PNL_OBJ_SIZE_X, PNL_OBJ_SIZE_Y, FONT_SYSTEM, grpBxPic_Rel) // Width text Name

// Declaration of label object of string width value
D4D_TCHAR grpBx_LblWidthStr[10] = D4D_DEFSTR("0px");
D4D_DECLARE_MY_LABEL(grpBx_LblWidth, grpBx_LblWidthStr, PNL_OBJ_POS_X, PNL_OBJ_POS_Y(3), PNL_OBJ_SIZE_X, PNL_OBJ_SIZE_Y, FONT_SYSTEM, grpBxPic_Rel) // Width text

// Declaration of label object of string "Height"
D4D_DECLARE_MY_LABEL(grpBx_LblHeightName, DEF_EXTSTR(STR_HEIGHT), PNL_OBJ_POS_X, PNL_OBJ_POS_Y(4), PNL_OBJ_SIZE_X, PNL_OBJ_SIZE_Y, FONT_SYSTEM, grpBxPic_Rel) // Height text Name

// Declaration of label object of string height value
D4D_TCHAR grpBx_LblHeightStr[10] = D4D_DEFSTR("0px");
D4D_DECLARE_MY_LABEL(grpBx_LblHeight, grpBx_LblHeightStr, PNL_OBJ_POS_X, PNL_OBJ_POS_Y(5), PNL_OBJ_SIZE_X, PNL_OBJ_SIZE_Y, FONT_SYSTEM, grpBxPic_Rel) // Height text

// Declaration of label object of string "Size"
D4D_DECLARE_MY_LABEL(grpBx_LblSizeName, DEF_EXTSTR(STR_SIZE), PNL_OBJ_POS_X, PNL_OBJ_POS_Y(6), PNL_OBJ_SIZE_X, PNL_OBJ_SIZE_Y, FONT_SYSTEM, grpBxPic_Rel) // Size text Name

// Declaration of label object of string size value
D4D_TCHAR grpBx_LblSizeStr[10] = D4D_DEFSTR("0B");
D4D_DECLARE_MY_LABEL(grpBx_LblSize, grpBx_LblSizeStr, PNL_OBJ_POS_X, PNL_OBJ_POS_Y(7), PNL_OBJ_SIZE_X, PNL_OBJ_SIZE_Y, FONT_SYSTEM, grpBxPic_Rel) // Size text

// Group Box Bitmap panel information
D4D_DECLARE_GROUP_BOX_BEGIN(sFileviewer_GrpBxPict, PANEL_POS_X, PANEL_POS_Y, PANEL_SIZE_X, PANEL_SIZE_Y, 0, NULL, &groupBox_marginDefault, DEF_EXTSTR(STR_IMAGE_INFO), FONT_SYSTEM_SMALL, (D4D_OBJECT_F_ENABLED | D4D_OBJECT_F_TABSTOP | D4D_OBJECT_F_TOUCHENABLE | D4D_OBJECT_F_FOCUSRECT | D4D_OBJECT_F_BEVEL_RAISED), NULL, NULL, NULL)
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBx_LblTypeName)
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBx_LblType)
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBx_LblWidthName)
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBx_LblWidth)
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBx_LblHeightName)
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBx_LblHeight)
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBx_LblSizeName)
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBx_LblSize)    
D4D_DECLARE_GROUP_BOX_OBJECT_END

/*******************************************
* Information panel objects - text file
*******************************************/
// Objects relations structure declaration for text files information panel
D4D_EXTERN_OBJECT(sFileviewer_GrpBxText)
D4D_DECLARE_OBJECT_RELATIONS_BEGIN(grpBxTxtBx_Rel, &sFileviewer_GrpBxText)
D4D_DECLARE_OBJECT_RELATIONS_END

// Declaration of label object of string "Type"
D4D_DECLARE_MY_LABEL(grpBxT_LblTypeName, DEF_EXTSTR(STR_TYPE), PNL_OBJ_POS_X, PNL_OBJ_POS_Y(0), PNL_OBJ_SIZE_X, PNL_OBJ_SIZE_Y, FONT_SYSTEM, grpBxTxtBx_Rel) // Type text Name

// Declaration of label object of string type value
D4D_TCHAR grpBxT_LblTypeStr[8] = D4D_DEFSTR("None");
D4D_DECLARE_MY_LABEL(grpBxT_LblType, grpBxT_LblTypeStr, PNL_OBJ_POS_X, PNL_OBJ_POS_Y(1), PNL_OBJ_SIZE_X, PNL_OBJ_SIZE_Y, FONT_SYSTEM, grpBxTxtBx_Rel) // Type text

// Declaration of label object of string "Size"
D4D_DECLARE_MY_LABEL(grpBxT_LblSizeName, DEF_EXTSTR(STR_SIZE), PNL_OBJ_POS_X, PNL_OBJ_POS_Y(2), PNL_OBJ_SIZE_X, PNL_OBJ_SIZE_Y, FONT_SYSTEM, grpBxTxtBx_Rel) // Size text Name

// Declaration of label object of string size value
D4D_TCHAR grpBxT_LblSizeStr[10] = D4D_DEFSTR("0B");
D4D_DECLARE_MY_LABEL(grpBxT_LblSize, grpBxT_LblSizeStr, PNL_OBJ_POS_X, PNL_OBJ_POS_Y(3), PNL_OBJ_SIZE_X, PNL_OBJ_SIZE_Y, FONT_SYSTEM, grpBxTxtBx_Rel) // Size text

// Declaration of radio buttons ASCII/UNICODE
_D4D_DECLARE_RADIOBUTTON(D4D_CONST, grpBxT_RbAscii, DEF_EXTSTR(STR_ASCII), PNL_OBJ_POS_X, PNL_OBJ_POS_Y(4), PNL_OBJ_SIZE_X, PNL_OBJ_SIZE_Y, 0, NULL, grpBxTxtBx_Rel, D4D_RADIOBUTTON_F_DEFAULT, NULL, NULL, NULL, FONT_SYSTEM, NULL, RadioButnTxtType_OnChange, NULL)
_D4D_DECLARE_RADIOBUTTON(D4D_CONST, grpBxT_RbUnicode, DEF_EXTSTR(STR_UNICODE), PNL_OBJ_POS_X, PNL_OBJ_POS_Y(5), PNL_OBJ_SIZE_X, PNL_OBJ_SIZE_Y, 0, NULL, grpBxTxtBx_Rel, D4D_RADIOBUTTON_F_DEFAULT, NULL, NULL, NULL, FONT_SYSTEM, NULL, NULL, NULL)

// Declaration of Text panel information group box
D4D_DECLARE_GROUP_BOX_BEGIN(sFileviewer_GrpBxText, PANEL_POS_X, PANEL_POS_Y, PANEL_SIZE_X, PANEL_SIZE_Y, 0, NULL, &groupBox_marginDefault, DEF_EXTSTR(STR_TEXT_INFO), FONT_SYSTEM_SMALL, (D4D_OBJECT_F_ENABLED | D4D_OBJECT_F_TABSTOP | D4D_OBJECT_F_TOUCHENABLE | D4D_OBJECT_F_FOCUSRECT | D4D_OBJECT_F_BEVEL_RAISED), NULL, NULL, NULL)
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBxT_LblTypeName)
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBxT_LblType)
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBxT_LblSizeName)
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBxT_LblSize)  
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBxT_RbAscii)
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBxT_RbUnicode)
D4D_DECLARE_GROUP_BOX_OBJECT_END

/*******************************************
* Control buttons objects
*******************************************/
//D4D_DECLARE_TXT_BUTTON(sFileviewer_btnOpenFie,  DEF_EXTSTR(STR_OPEN), BTN_POS_X(0), BTN_POS_Y, BTN_SIZE_X, BTN_SIZE_Y, FONT_SYSTEM, SFL_BtnOpenOnClick)
D4D_DECLARE_TXT_BUTTON(sFileviewer_btnQuit,     DEF_EXTSTR(STR_QUIT), BTN_POS_X(1), BTN_POS_Y, BTN_SIZE_X, BTN_SIZE_Y, FONT_SYSTEM, SFL_BtnQuitOnClick)

// Test chinese
D4D_WCHAR helloChn[] = {0x54C7, 0xFF0C, 0x0020, 0x0045, 0x0047, 0x0055, 0x0049, 0x5F00, 0x59CB, 0x652F, 0x6301, 0x4E2D, 0x6587, 0x5B57, 0x5E93, 0x4E86, 0xFF01, 0x0000};

D4D_DECLARE_STD_LABEL(lbl_test, helloChn, BTN_POS_X(2), BTN_POS_Y, 240, BTN_SIZE_Y, FONT_SYSTEM_UNICODE)

D4D_EXTERN_SCREEN(screen_settings);

/****************************************************************************
*       D4D screen declarations
****************************************************************************/
D4D_DECLARE_STD_SCREEN_BEGIN(screen_fileviewer, ScreenFileviewer_)  
  D4D_DECLARE_SCREEN_OBJECT(sFileviewer_CntntMsg)
  D4D_DECLARE_SCREEN_OBJECT(sFileviewer_GrpBxMsg)
  D4D_DECLARE_SCREEN_OBJECT(sFileviewer_CntntPict)
  D4D_DECLARE_SCREEN_OBJECT(sFileviewer_GrpBxPict)
  D4D_DECLARE_SCREEN_OBJECT(sFileviewer_CntntText)
  D4D_DECLARE_SCREEN_OBJECT(sFileviewer_GrpBxText)
//  D4D_DECLARE_SCREEN_OBJECT(sFileviewer_btnOpenFie)    
  D4D_DECLARE_SCREEN_OBJECT(sFileviewer_btnQuit)
  D4D_DECLARE_SCREEN_OBJECT(lbl_test)
D4D_DECLARE_SCREEN_END()    

/****************************************************************************
*       D4D widgets static events functions definitions
****************************************************************************/
// On Click Event of "OPEN" button
//static void SFL_BtnOpenOnClick(D4D_OBJECT* pThis)
//{
//  D4D_FileDialogShow(pFileDialog);
//}

// On Click Event of "QUIT" button
static void SFL_BtnQuitOnClick(D4D_OBJECT* pThis) {
  D4D_EscapeScreen();
}

// On Change Event of "ASCII / UNICODE"  radio buttons
static void RadioButnTxtType_OnChange(D4D_OBJECT* pThis) {
  // It's called when the ASCII radio button is changed
  if(D4D_IsVisible((D4D_OBJECT*)&sFileviewer_CntntText))
    ScrFileviewer_FileHandleText(fileName);
}

/*-----------------------------------------------------------------------------
*-----------------------------------------------------------------------------*/
void ScrFileviewer_Show(const D4D_CHAR *filename) {
  D4D_CHAR* pExt;
  LWord i;
  
  fileName = filename;
  
  // Get the file extension
  pExt = D4D_GetFileExtension((D4D_CHAR*)fileName);
  
  // Just switch off redraw of last component
//  sFileviewer_CntntMsg.pData->flags &= ~D4D_OBJECT_F_VISIBLE;
//  sFileviewer_CntntPict.pData->flags &= ~D4D_OBJECT_F_VISIBLE;
//  sFileviewer_CntntText.pData->flags &= ~D4D_OBJECT_F_VISIBLE;
//  
//  sFileviewer_GrpBxMsg.pData->flags &= ~D4D_OBJECT_F_VISIBLE;
//  sFileviewer_GrpBxPict.pData->flags &= ~D4D_OBJECT_F_VISIBLE;
//  sFileviewer_GrpBxText.pData->flags &= ~D4D_OBJECT_F_VISIBLE;
    
  // Look to association table for right file handler
  i = 0;  
  while(fileAssociationTable[i].strFileExt && fileAssociationTable[i].pFileHandler) {
    if(!D4D_CompareStrings(pExt, fileAssociationTable[i].strFileExt)) {
      // Run the file content open Handler
      fileAssociationTable[i].pFileHandler(fileName);
      break;
    }
    i++;
  }
  
  // if there is no any usable file handler just show unsupported type
  if(fileAssociationTable[i].strFileExt == NULL) {
    // Not Supported File type
    // Show the screen that this file is not supported.
    ScrFileviewer_FileHandleUnsupported(fileName);
  }
  
  // Show the screen
  D4D_ActivateScreen((D4D_SCREEN*)&screen_fileviewer, D4D_FALSE);
}


/****************************************************************************
*       D4D standard screen functions definitions
****************************************************************************/

// One time called screen function in screen initialization proces
static void ScreenFileviewer_OnInit() {
  // Create the File dialog
//  pFileDialog = D4D_FileDialogCreate("A:\\", FileDialogEnds);
  
  // Initialized some one time properties of some objects
  D4D_SetFontProperties(&grpBx_LblTypeName, D4D_FNT_PRTY_UNDERLINE_LINE_MASK);
  D4D_SetFontProperties(&grpBx_LblWidthName, D4D_FNT_PRTY_UNDERLINE_LINE_MASK);
  D4D_SetFontProperties(&grpBx_LblHeightName, D4D_FNT_PRTY_UNDERLINE_LINE_MASK);
  D4D_SetFontProperties(&grpBx_LblSizeName, D4D_FNT_PRTY_UNDERLINE_LINE_MASK);
  D4D_SetFontProperties(&grpBxT_LblTypeName, D4D_FNT_PRTY_UNDERLINE_LINE_MASK);
  D4D_SetFontProperties(&grpBxT_LblSizeName, D4D_FNT_PRTY_UNDERLINE_LINE_MASK);
  D4D_RadioButtonSetValue(&grpBxT_RbAscii, D4D_TRUE);
}

// Screen on Activate function called with each screen activation
static void ScreenFileviewer_OnActivate() {
  /* nothing */
}

// Screen "Fileviewer" function called periodically in each D4D_poll runs
static void ScreenFileviewer_OnMain() {
  /* nothing */
}


// Screen on DeActivate function called with each screen deactivation
static void ScreenFileviewer_OnDeactivate() {
  /* nothing */
}

// Screen on message function called with each internal massage for this screen
static Byte ScreenFileviewer_OnObjectMsg(D4D_MESSAGE* pMsg) {  
  if(pMsg->pObject == NULL) {
    switch(pMsg->nMsgId) {
      case D4D_MSG_DRAW:
        // redraw background under buttons 
        D4D_MoveToXY(0, PANEL_SIZE_Y);
        D4D_FillRectToXY(D4D_SCREEN_SIZE_LONGER_SIDE - 1, D4D_SCREEN_SIZE_SHORTER_SIDE - 1, D4D_ScreenGetScheme((D4D_SCREEN*)&screen_fileviewer)->screen.desktop);        
        break;
        
      case D4D_MSG_DRAWDONE: {
        // When the screen content is drawed - just add one extra bevel around the content area    
          D4D_POINT pnt = {0,0};
          D4D_SIZE sz = {CONTENT_SIZE_X + 2*D4D_BEVEL_WIDTH ,CONTENT_SIZE_Y + 2*D4D_BEVEL_WIDTH};
          D4D_Bevel(&pnt, &sz, D4D_ScreenGetScheme((D4D_SCREEN*)&screen_fileviewer)->bckg, D4D_BEVEL_RAISED, 0);
        }
        break;
    }
  }
  return 0;
}


/*-----------------------------------------------------------------------------
* FUNCTION:    ScrFileviewer_FileHandleUnsupported
* SCOPE:       Fileviewer screen related function
* DESCRIPTION: Function shows that the selected file is unssuported 
*
* PARAMETERS:  pFileName - pointer to file name
*              
* RETURNS:     none
*-----------------------------------------------------------------------------*/
static void ScrFileviewer_FileHandleUnsupported(const D4D_CHAR* pFileName) {
  // Show dummy panels
  D4D_SetText(&sFileviewer_CntntMsg, DEF_EXTSTR(STR_COULD_NOT_SHOW));
  D4D_ShowObject(&sFileviewer_CntntMsg, D4D_TRUE);
  D4D_ShowObject((D4D_OBJECT*)&sFileviewer_GrpBxMsg, D4D_TRUE);
  D4D_InvalidateObject((D4D_OBJECT*)&sFileviewer_GrpBxMsg, D4D_TRUE);
}

/*-----------------------------------------------------------------------------
* FUNCTION:    ScrFileviewer_FileHandlePicture
* SCOPE:       Fileviewer screen related function
* DESCRIPTION: Function load the picture file to D4D_PICTURE and set up the side information panel 
*
* PARAMETERS:  pFileName - pointer to file name
*              
* RETURNS:     none
*-----------------------------------------------------------------------------*/
static void ScrFileviewer_FileHandlePicture(const D4D_CHAR* pFileName) {
  D4D_BMP tmp_bmp;
  D4D_SIZE tmp_size;
  D4D_CHAR tmp_str[20];  
  D4D_WCHAR tmp_wstr[20];
  D4D_FILEPTR filePtr;
  
  // Update the d4d bitmap structure
  tmp_bmp.pData = NULL;
  tmp_bmp.pParam = pFileName;
  
  // Fill up the information panel
  tmp_size = D4D_GetBmpSize(&tmp_bmp);
  
  D4D_ShowObject(&sFileviewer_GrpBxPict, D4D_TRUE);
  
  // Update file type label
  D4D_StrCopyAscii2Unicode(tmp_wstr, D4D_GetFileExtension((D4D_CHAR*)pFileName));
  D4D_SetText(&grpBx_LblType, tmp_wstr);
  
  // Update file width label
  snprintf(tmp_str, 10, "%dpx", tmp_size.cx);
  D4D_StrCopyAscii2Unicode(tmp_wstr, tmp_str);
  D4D_SetText(&grpBx_LblWidth, tmp_wstr);
  
  // Update file height label
  snprintf(tmp_str, 10, "%dpx", tmp_size.cy);
  D4D_StrCopyAscii2Unicode(tmp_wstr, tmp_str);
  D4D_SetText(&grpBx_LblHeight, tmp_wstr);
  
  // Get the size of file
  filePtr = D4D_FileOpen((D4D_CHAR*)pFileName, "r");
  
  if(filePtr)
  {
    LWord size;
    
    D4D_FileSeek(filePtr, 0, D4D_SEEK_END);    
    size = D4D_FileTell(filePtr);
    D4D_FileClose(filePtr);
    
    if(size < 1024)
      snprintf(tmp_str, 20, "%dB", size);
    else if(size < 1024*1024)
      snprintf(tmp_str, 20, "%dKB", size / 1024);
    else
      snprintf(tmp_str, 20, "%dMB", size / (1024 * 1024));
    // Update file size label  
    D4D_StrCopyAscii2Unicode(tmp_wstr, tmp_str);
    D4D_SetText(&grpBx_LblSize, tmp_wstr);
  }else
    D4D_SetText(&grpBx_LblSize, DEF_EXTSTR(STR_UNKNOWN));
  
  // Show the picture
  if(tmp_size.cx > CONTENT_SIZE_X || tmp_size.cy > CONTENT_SIZE_Y)
  {
    // So big picture - we not able to show it in our content area
    D4D_SetText(&sFileviewer_CntntMsg, DEF_EXTSTR(STR_COULD_NOT_SHOW));
    D4D_ShowObject(&sFileviewer_CntntMsg, D4D_TRUE);
  }else
  {
    D4D_StrCopy(pictureFileName, pFileName);
    D4D_ShowObject(&sFileviewer_CntntPict, D4D_TRUE);
    D4D_InvalidateObject(&sFileviewer_CntntPict, D4D_TRUE);    
  }
}

/*-----------------------------------------------------------------------------
* FUNCTION:    ScrFileviewer_FileHandleText
* SCOPE:       Fileviewer screen related function
* DESCRIPTION: Function load the text file to D4D_TEXT_BOX and set up the side information panel 
*
* PARAMETERS:  pFileName - pointer to file name
*              
* RETURNS:     none
*-----------------------------------------------------------------------------*/
static void ScrFileviewer_FileHandleText(const D4D_CHAR* pFileName) {
  static D4D_TCHAR* pText = NULL;
  D4D_FILEPTR filePtr;
  D4D_CHAR tmp_str[20];  
  D4D_WCHAR tmp_wstr[20];
  LWord size = 0;
  // Get the size of file
  filePtr = D4D_FileOpen((D4D_CHAR*)pFileName, "r");
  
  // Get the file size
  if(filePtr)
  {
    D4D_FileSeek(filePtr, 0, D4D_SEEK_END);    
    size = D4D_FileTell(filePtr);
    D4D_FileClose(filePtr);  
  }
  
  // Change the file type label
  D4D_StrCopyAscii2Unicode(tmp_wstr, D4D_GetFileExtension((D4D_CHAR*)pFileName));
  D4D_SetText(&grpBxT_LblType, tmp_wstr);
  
  
  if(size)
  {
    // Change the text
    if(pText)
      D4D_MemFree(pText);
    
    // Show the size of file
    
    if(size < 1024)
      snprintf(tmp_str, 20, "%dB", size);
    else if(size < 1024*1024)
      snprintf(tmp_str, 20, "%dKB", size / 1024);
    else
      snprintf(tmp_str, 20, "%dMB", size / (1024 * 1024));
      
    D4D_StrCopyAscii2Unicode(tmp_wstr, tmp_str);
    D4D_SetText(&grpBxT_LblSize, tmp_wstr);
    
    
    if(size < 100000)
    {    
      Byte array[512];
      LWord i = 0;
      D4D_FILEPTR pFile;
      D4D_TCHAR* pTextTmp;
      
      // Encode the text file to RAM
      if(D4D_RadioButtonGetValue(&grpBxT_RbUnicode))
      {
        pText = D4D_MemAlloc(size + 2); // + termination char
        if(pText == NULL)
          return;
        
        pFile = D4D_FileOpen((D4D_CHAR*)pFileName, "r");
        if(pFile == NULL)
          return;
        
        i = D4D_FileRead(pFile, pText, size);
        
        pText[i/2] = 0;

      }else
      {
        pText = D4D_MemAlloc((size * 2) + 2); // + termination char
        if(pText == NULL)
          return;
        
        pFile = D4D_FileOpen((D4D_CHAR*)pFileName, "r");
        if(pFile == NULL)
          return;
        
        pTextTmp = pText;
        do
        {
          LWord j;
          i = D4D_FileRead(pFile, array, 512);
          for(j = 0; j<i; j++)
            *(pTextTmp++) = array[j]; 
           
        }while(512 == i);
        *((D4D_TCHAR*)pTextTmp) = 0; // End char
      }
 
      // Show the text box + text information panel
      D4D_TextBoxChangeText((D4D_OBJECT*)&sFileviewer_CntntText, pText);
      D4D_ShowObject((D4D_OBJECT*)&sFileviewer_CntntText, D4D_TRUE);
      D4D_InvalidateObject((D4D_OBJECT*)&sFileviewer_CntntText, D4D_TRUE);
      
      D4D_ShowObject((D4D_OBJECT*)&sFileviewer_GrpBxText, D4D_TRUE);
      D4D_InvalidateObject((D4D_OBJECT*)&sFileviewer_GrpBxText, D4D_TRUE);      
    }
  }
}


