/******************************************************************************
* 
* Copyright (c) 2012 Elnico s.r.o.
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY ELNICO "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL ELNICO OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
****************************************************************************//*!
*
* @file      d4d_screen_main.c
*
* @author    Petr Kubiznak
* 
* @version   1.0
* 
* @date      Sep-03-2012
* 
* @brief     D4D demo - main screen
*
******************************************************************************/


/****************************************************************************
*       Includes header files
****************************************************************************/
#include <d4d.h>
#include <lwmsgq.h>

#include "../main.h"
#include "fonts.h"
#include "strings.h"
#include "file_list.h"
#include "utils.h"
#include "lwcfg.h"
#include "audio/wave_task.h"
#include "d4d_iconbutton.h"
//#include "../usb/MFS_USB.h"
//#include "../usb/USB_MSD_Disk.h"
#include "../usb/USB_init.h"

/****************************************************************************
*       Local functions declarations
***************************************************************************/
static void btnFilebrowser_OnClick(D4D_OBJECT_PTR pThis);
static void btnAudioplayer_OnClick(D4D_OBJECT_PTR pThis);
static void btnRtcsInfo_OnClick(D4D_OBJECT_PTR pThis);
static void btnToggleLED_OnClick(D4D_OBJECT_PTR pThis);
static void btnCalibrateTS_OnClick(D4D_OBJECT_PTR pThis);
static void btnSettings_OnClick(D4D_OBJECT_PTR pThis);


/****************************************************************************
*       Local variables
***************************************************************************/
#define ICON_BUTTONS_COUNT              6

extern LWGPIO_STRUCT led1;
static LWGPIO_VALUE gLed1LastValue;

/****************************************************************************
*       D4D widgets (graphic objects) declarations
****************************************************************************/

#if ((D4DLCDHWFB_X_MAX >= 800) && (D4DLCDHWFB_Y_MAX >= 480))
# define  LCD_800X480                   1
#else
# define  LCD_800X480                   0
#endif

#if LCD_800X480

# define MARGIN_X                       15
# define MARGIN_Y                       15

# define PANEL_POS_X                    1
# define PANEL_POS_Y                    1

# define BUTTON_SIZE_X                  236
# define BUTTON_SIZE_Y                  210

#else /* LCD_800X480 */

# define MARGIN_X                       8
# define MARGIN_Y                       8

# define PANEL_POS_X                    0
# define PANEL_POS_Y                    0

# define BUTTON_SIZE_X                  144
# define BUTTON_SIZE_Y                  120

#endif /* LCD_800X480 */

#define COLUMN_SIZE_X                   (BUTTON_SIZE_X + 2*MARGIN_X)
#define COLUMN_SIZE_Y                   (BUTTON_SIZE_Y + 2*MARGIN_Y)

#define BUTTON_POS_X(col)               (PANEL_POS_X + (col)*COLUMN_SIZE_X + MARGIN_X)
#define BUTTON_POS_Y(row)               (PANEL_POS_Y + (row)*COLUMN_SIZE_Y + MARGIN_Y)

/*******************************************
* Main content objects - button icons
*******************************************/
// Bitmap filenames
#if LCD_800X480
# define FILEPATH_BMP_BUTTONS           "A:\\SYSTEM\\IMAGES\\BUTTONS\\S800X480\\"
#else /* LCD_800X480 */
# define FILEPATH_BMP_BUTTONS           "A:\\SYSTEM\\IMAGES\\BUTTONS\\S480X272\\"
#endif /* LCD_800X480 */
#define FILENAME_BMP_FILEBROWSER        FILEPATH_BMP_BUTTONS "FILEBRON.D4L"
#define FILENAME_BMP_FILEBROWSER_A      FILEPATH_BMP_BUTTONS "FILEBROA.D4L"
#define FILENAME_BMP_AUDIOPLAYER        FILEPATH_BMP_BUTTONS "AUDIOPLN.D4L"
#define FILENAME_BMP_AUDIOPLAYER_A      FILEPATH_BMP_BUTTONS "AUDIOPLA.D4L"
#define FILENAME_BMP_RTCSINFO           FILEPATH_BMP_BUTTONS "RTCSINFN.D4L"
#define FILENAME_BMP_RTCSINFO_A         FILEPATH_BMP_BUTTONS "RTCSINFA.D4L"
#define FILENAME_BMP_LEDON              FILEPATH_BMP_BUTTONS "LEDONN.D4L"
#define FILENAME_BMP_LEDON_A            FILEPATH_BMP_BUTTONS "LEDONA.D4L"
#define FILENAME_BMP_LEDOFF             FILEPATH_BMP_BUTTONS "LEDOFFN.D4L"
#define FILENAME_BMP_LEDOFF_A           FILEPATH_BMP_BUTTONS "LEDOFFA.D4L"
#define FILENAME_BMP_TSCALIBRATION      FILEPATH_BMP_BUTTONS "TSCALIBN.D4L"
#define FILENAME_BMP_TSCALIBRATION_A    FILEPATH_BMP_BUTTONS "TSCALIBA.D4L"
#define FILENAME_BMP_SETTINGS           FILEPATH_BMP_BUTTONS "SETTINGN.D4L"
#define FILENAME_BMP_SETTINGS_A         FILEPATH_BMP_BUTTONS "SETTINGA.D4L"

// Bitmap structures - will be initialized in ScreenInit()
D4D_BMP sMain_bmpFilebrowser  = { NULL, NULL };
D4D_BMP sMain_bmpFilebrowserA = { NULL, NULL };
D4D_BMP sMain_bmpAudioplayer  = { NULL, NULL };
D4D_BMP sMain_bmpAudioplayerA = { NULL, NULL };
D4D_BMP sMain_bmpRtcsInfo     = { NULL, NULL };
D4D_BMP sMain_bmpRtcsInfoA    = { NULL, NULL };
D4D_BMP sMain_bmpLedOn        = { NULL, NULL };
D4D_BMP sMain_bmpLedOnA       = { NULL, NULL };
D4D_BMP sMain_bmpLedOff       = { NULL, NULL };
D4D_BMP sMain_bmpLedOffA      = { NULL, NULL };
D4D_BMP sMain_bmpCalibrate    = { NULL, NULL };
D4D_BMP sMain_bmpCalibrateA   = { NULL, NULL };
D4D_BMP sMain_bmpSettings     = { NULL, NULL };
D4D_BMP sMain_bmpSettingsA    = { NULL, NULL };


/*******************************************
* Main content objects - buttons
*******************************************/
D4D_DECLARE_BUTTON(sMain_btnFocusKeeper,        DEF_EXTSTR(STR_LANGRES2),     0, 0, 1, 1, D4D_OBJECT_F_ENABLED | D4D_OBJECT_F_TABSTOP, NULL, NULL, NULL, FONT_SYSTEM, NULL, NULL, NULL)

// FILEMANAGER
D4D_DECLARE_STD_ICONBUTTON_BEGIN(sMain_btnFilebrowser, NULL, BUTTON_POS_X(0), BUTTON_POS_Y(0), BUTTON_SIZE_X, BUTTON_SIZE_Y, 0, 0, FONT_SYSTEM, &sMain_btnFocusKeeper, btnFilebrowser_OnClick)
  D4D_DECLARE_ICON_BMP(&sMain_bmpFilebrowser)
  D4D_DECLARE_ICON_BMP(&sMain_bmpFilebrowserA)
D4D_DECLARE_ICON_END()

// AUDIOPLAYER
D4D_DECLARE_STD_ICONBUTTON_BEGIN(sMain_btnAudioplayer, NULL, BUTTON_POS_X(1), BUTTON_POS_Y(0), BUTTON_SIZE_X, BUTTON_SIZE_Y, 0, 0, FONT_SYSTEM, &sMain_btnFocusKeeper, btnAudioplayer_OnClick)
  D4D_DECLARE_ICON_BMP(&sMain_bmpAudioplayer)
  D4D_DECLARE_ICON_BMP(&sMain_bmpAudioplayerA)
D4D_DECLARE_ICON_END()

// NOTHING
D4D_DECLARE_STD_ICONBUTTON_BEGIN(sMain_btnRtcsInfo, NULL, BUTTON_POS_X(2), BUTTON_POS_Y(0), BUTTON_SIZE_X, BUTTON_SIZE_Y, 0, 0, FONT_SYSTEM, &sMain_btnFocusKeeper, btnRtcsInfo_OnClick)
  D4D_DECLARE_ICON_BMP(&sMain_bmpRtcsInfo)
  D4D_DECLARE_ICON_BMP(&sMain_bmpRtcsInfoA)
D4D_DECLARE_ICON_END()

// NOTHING
D4D_DECLARE_STD_ICONBUTTON_BEGIN(sMain_btnToggleLED, NULL, BUTTON_POS_X(0), BUTTON_POS_Y(1), BUTTON_SIZE_X, BUTTON_SIZE_Y, 0, 0, FONT_SYSTEM, &sMain_btnFocusKeeper, btnToggleLED_OnClick)
  D4D_DECLARE_ICON_BMP(&sMain_bmpLedOn)
  D4D_DECLARE_ICON_BMP(&sMain_bmpLedOnA)
  D4D_DECLARE_ICON_BMP(&sMain_bmpLedOff)
  D4D_DECLARE_ICON_BMP(&sMain_bmpLedOffA)
D4D_DECLARE_ICON_END()

// NOTHING
D4D_DECLARE_STD_ICONBUTTON_BEGIN(sMain_btnCalibrateTS, NULL, BUTTON_POS_X(1), BUTTON_POS_Y(1), BUTTON_SIZE_X, BUTTON_SIZE_Y, 0, 0, FONT_SYSTEM, &sMain_btnFocusKeeper, btnCalibrateTS_OnClick)
  D4D_DECLARE_ICON_BMP(&sMain_bmpCalibrate)
  D4D_DECLARE_ICON_BMP(&sMain_bmpCalibrateA)
D4D_DECLARE_ICON_END()

// SETTINGS
D4D_DECLARE_STD_ICONBUTTON_BEGIN(sMain_btnSettings, NULL, BUTTON_POS_X(2), BUTTON_POS_Y(1), BUTTON_SIZE_X, BUTTON_SIZE_Y, 0, 0, FONT_SYSTEM, &sMain_btnFocusKeeper, btnSettings_OnClick)
  D4D_DECLARE_ICON_BMP(&sMain_bmpSettings)
  D4D_DECLARE_ICON_BMP(&sMain_bmpSettingsA)
D4D_DECLARE_ICON_END()

/****************************************************************************
* Screen declaration
****************************************************************************/
D4D_DECLARE_STD_SCREEN_BEGIN(screen_main, ScreenMain_)  
  D4D_DECLARE_SCREEN_OBJECT(sMain_btnFocusKeeper)
  D4D_DECLARE_SCREEN_OBJECT(sMain_btnFilebrowser)
  D4D_DECLARE_SCREEN_OBJECT(sMain_btnAudioplayer)
  D4D_DECLARE_SCREEN_OBJECT(sMain_btnRtcsInfo)
  D4D_DECLARE_SCREEN_OBJECT(sMain_btnToggleLED)
  D4D_DECLARE_SCREEN_OBJECT(sMain_btnCalibrateTS)
  D4D_DECLARE_SCREEN_OBJECT(sMain_btnSettings)
D4D_DECLARE_SCREEN_END()


/*******************************************
* External screens
*******************************************/
D4D_EXTERN_SCREEN(screen_filemngr);
D4D_EXTERN_SCREEN(screen_audioplr);
D4D_EXTERN_SCREEN(screen_rtcsinfo);
D4D_EXTERN_SCREEN(screen_settings);


/****************************************************************************
*       D4D standard screen functions definitions
****************************************************************************/

// One time called screen function in screen initialization proces
static void ScreenMain_OnInit() {
  loadBitmap(&sMain_bmpFilebrowser,   FILENAME_BMP_FILEBROWSER);
  loadBitmap(&sMain_bmpFilebrowserA,  FILENAME_BMP_FILEBROWSER_A);
  loadBitmap(&sMain_bmpAudioplayer,   FILENAME_BMP_AUDIOPLAYER);
  loadBitmap(&sMain_bmpAudioplayerA,  FILENAME_BMP_AUDIOPLAYER_A);
  loadBitmap(&sMain_bmpRtcsInfo,      FILENAME_BMP_RTCSINFO);
  loadBitmap(&sMain_bmpRtcsInfoA,     FILENAME_BMP_RTCSINFO_A);
  loadBitmap(&sMain_bmpLedOn,         FILENAME_BMP_LEDON);
  loadBitmap(&sMain_bmpLedOnA,        FILENAME_BMP_LEDON_A);
  loadBitmap(&sMain_bmpLedOff,        FILENAME_BMP_LEDOFF);
  loadBitmap(&sMain_bmpLedOffA,       FILENAME_BMP_LEDOFF_A);
  loadBitmap(&sMain_bmpCalibrate,     FILENAME_BMP_TSCALIBRATION);
  loadBitmap(&sMain_bmpCalibrateA,    FILENAME_BMP_TSCALIBRATION_A);
  loadBitmap(&sMain_bmpSettings,      FILENAME_BMP_SETTINGS);
  loadBitmap(&sMain_bmpSettingsA,     FILENAME_BMP_SETTINGS_A);
  
  gLed1LastValue = LWGPIO_VALUE_NOCHANGE;
}

/******************************************************************************/
// Screen on Activate function called with each screen activation
static void ScreenMain_OnActivate() {
}

/******************************************************************************/
// Screen "Main" function called periodically in each D4D_poll runs
static void ScreenMain_OnMain() {
  LWGPIO_VALUE led1CurrentValue = lwgpio_get_value(&led1);
  if(led1CurrentValue != gLed1LastValue) {
    D4D_IconbuttonSetState(&sMain_btnToggleLED, (led1CurrentValue==LWGPIO_VALUE_LOW ? 0 : 1));
    gLed1LastValue = led1CurrentValue;
  }
}

/******************************************************************************/
// Screen on DeActivate function called with each screen deactivation
static void ScreenMain_OnDeactivate() {
}

/******************************************************************************/
// Screen on message function called with each internal massage for this screen
static Byte ScreenMain_OnObjectMsg(D4D_MESSAGE* pMsg) {
  if(pMsg->pObject == NULL) {
    switch(pMsg->nMsgId) {
      case D4D_MSG_DRAW:
        // redraw background under buttons 
        D4D_ClearScreen(D4D_COLOR_BLACK);
        break;
    }
  }
  return 0;
}


/****************************************************************************
*       D4D widgets static events functions definitions
****************************************************************************/

// On Click Event of "Filebrowser" button
static void btnFilebrowser_OnClick(D4D_OBJECT_PTR pThis) {
  D4D_ActivateScreen(&screen_filemngr, D4D_FALSE);
  printf("btnFilebrowser clicked!\n");
}

// On Click Event of "Audioplayer" button
static void btnAudioplayer_OnClick(D4D_OBJECT_PTR pThis) {
  D4D_ActivateScreen(&screen_audioplr, D4D_FALSE);
  printf("btnAudioplayer clicked!\n");
}

// On Click Event of "RTCS info" button
static void btnRtcsInfo_OnClick(D4D_OBJECT_PTR pThis) {
  D4D_ActivateScreen(&screen_rtcsinfo, D4D_FALSE);
  printf("btnRtcsinfo clicked!\n");
}

// On Click Event of "Toggle LED" button
static void btnToggleLED_OnClick(D4D_OBJECT_PTR pThis) {
  lwgpio_toggle_value(&led1);
}

// On Click Event of "Calibrate Touchscreen" button
static void btnCalibrateTS_OnClick(D4D_OBJECT_PTR pThis) {
#if 1
  LWCFG_FILE_PTR pCfgFile = lwcfg_open("A:\\SYSTEM\\CONFIG.TXT");
  main_calibrateTouchscreen(pCfgFile);
  lwcfg_close(pCfgFile);
  main_calibrateTouchscreen(NULL);
#else
  static char bHost = 1;
  
  if(bHost) {   // switch to host
    USB_MSD_Disk_deinit();
    MFS_USB_Task_init();
  } else {      // switch to device
    MFS_USB_Task_deinit();
    USB_MSD_Disk_init();
  }
  bHost = !bHost;
#endif
}

// On Click Event of "Settings" button
static void btnSettings_OnClick(D4D_OBJECT_PTR pThis) {
  D4D_ActivateScreen(&screen_settings, D4D_FALSE);
  printf("btnSettings clicked!\n");
}


