/******************************************************************************
* 
* Copyright (c) 2012 Elnico s.r.o.
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY ELNICO "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL ELNICO OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
****************************************************************************//*!
*
* @file      d4d_screen_audioplr.c
*
* @author    Petr Kubiznak
* 
* @version   1.0
* 
* @date      Sep-14-2012
* 
* @brief     D4D demo - RTCS information screen
*
******************************************************************************/

/****************************************************************************
*       Includes header files
****************************************************************************/
#include <d4d.h>
#include <rtcs.h>
#include <ipcfg.h>

#include "fonts.h"
#include "strings.h"

/****************************************************************************
*       Local functions declarations
***************************************************************************/

static void btnClose_OnClick(D4D_OBJECT* pThis);

/****************************************************************************
*       Local variables declarations
***************************************************************************/
#define LABEL_BUFFER_SIZE               30
static D4D_TCHAR gsMacValue     [LABEL_BUFFER_SIZE] = D4D_DEFSTR("???");
static D4D_TCHAR gsIpValue      [LABEL_BUFFER_SIZE] = D4D_DEFSTR("???");
static D4D_TCHAR gsMaskValue    [LABEL_BUFFER_SIZE] = D4D_DEFSTR("???");
static D4D_TCHAR gsGatewayValue [LABEL_BUFFER_SIZE] = D4D_DEFSTR("???");

extern IPCFG_IP_ADDRESS_DATA gIPAddress;
extern _enet_address         gMacAddress;

/****************************************************************************
*       D4D widgets (graphic objects) declarations
****************************************************************************/

#define MARGIN_X                        (4)
#define MARGIN_Y                        (6)
#define RADIUS                          (5)

#define SCREEN_SIZE_X                   (380)
#define SCREEN_SIZE_Y                   (162)
#define SCREEN_POS_X                    (((D4D_SCREEN_SIZE_LONGER_SIDE)-(SCREEN_SIZE_X))/2)
#define SCREEN_POS_Y                    (((D4D_SCREEN_SIZE_SHORTER_SIDE)-(SCREEN_SIZE_Y))/2)

#define LABEL_SIZE_X                    (((SCREEN_SIZE_X)-(MARGIN_X))/2 - (MARGIN_X))
#define LABEL_SIZE_Y                    (22)
#define LABEL_POS_X(col)                ((col)*(LABEL_SIZE_X) + MARGIN_X)
#define LABEL_POS_Y(row)                ((MARGIN_Y) + ((LABEL_SIZE_Y)+(MARGIN_Y))*(row))

#define BUTTON_SIZE_X                   (120)
#define BUTTON_SIZE_Y                   (36)
#define BUTTON_POS_X                    ((SCREEN_SIZE_X)-(BUTTON_SIZE_X)-(MARGIN_X))
#define BUTTON_POS_Y                    ((SCREEN_SIZE_Y)-(BUTTON_SIZE_Y)-(MARGIN_Y))

#define LABEL_FLAGS                     (D4D_OBJECT_F_VISIBLE | D4D_OBJECT_F_ENABLED)
#define LABEL_TEXT_PROPERTIES           (D4D_ALIGN_H_LEFT_MASK)
#define LABEL_FONT_PROPERTIES           (D4D_FNT_PRTY_TRANSPARENT_MASK)
#define SCREEN_FLAGS                    (D4D_SCR_F_OUTLINE | D4D_SCR_F_TITLEBAR | D4D_SCR_F_BCKG)

/*******************************************
* Info labels
*******************************************/
D4D_DECLARE_LABEL       (sRtcsinfo_lblMacText,      DEF_EXTSTR(STR_MAC),      LABEL_POS_X(0), LABEL_POS_Y(0), LABEL_SIZE_X, LABEL_SIZE_Y, (LABEL_FLAGS), NULL, FONT_SYSTEM, NULL, NULL)
D4D_DECLARE_LABEL_INRAM (sRtcsinfo_lblMacValue,     gsMacValue,               LABEL_POS_X(1), LABEL_POS_Y(0), LABEL_SIZE_X, LABEL_SIZE_Y, (LABEL_FLAGS), NULL, FONT_SYSTEM, NULL, NULL)
D4D_DECLARE_LABEL       (sRtcsinfo_lblIpText,       DEF_EXTSTR(STR_IP),       LABEL_POS_X(0), LABEL_POS_Y(1), LABEL_SIZE_X, LABEL_SIZE_Y, (LABEL_FLAGS), NULL, FONT_SYSTEM, NULL, NULL)
D4D_DECLARE_LABEL_INRAM (sRtcsinfo_lblIpValue,      gsIpValue,                LABEL_POS_X(1), LABEL_POS_Y(1), LABEL_SIZE_X, LABEL_SIZE_Y, (LABEL_FLAGS), NULL, FONT_SYSTEM, NULL, NULL)
D4D_DECLARE_LABEL       (sRtcsinfo_lblMaskText,     DEF_EXTSTR(STR_MASK),     LABEL_POS_X(0), LABEL_POS_Y(2), LABEL_SIZE_X, LABEL_SIZE_Y, (LABEL_FLAGS), NULL, FONT_SYSTEM, NULL, NULL)
D4D_DECLARE_LABEL_INRAM (sRtcsinfo_lblMaskValue,    gsMaskValue,              LABEL_POS_X(1), LABEL_POS_Y(2), LABEL_SIZE_X, LABEL_SIZE_Y, (LABEL_FLAGS), NULL, FONT_SYSTEM, NULL, NULL)
D4D_DECLARE_LABEL       (sRtcsinfo_lblGatewayText,  DEF_EXTSTR(STR_GATEWAY),  LABEL_POS_X(0), LABEL_POS_Y(3), LABEL_SIZE_X, LABEL_SIZE_Y, (LABEL_FLAGS), NULL, FONT_SYSTEM, NULL, NULL)
D4D_DECLARE_LABEL_INRAM (sRtcsinfo_lblGatewayValue, gsGatewayValue,           LABEL_POS_X(1), LABEL_POS_Y(3), LABEL_SIZE_X, LABEL_SIZE_Y, (LABEL_FLAGS), NULL, FONT_SYSTEM, NULL, NULL)


/*******************************************
* Control buttons
*******************************************/
D4D_DECLARE_TXT_RBUTTON(sRtcsinfo_btnClose, DEF_EXTSTR(STR_CLOSE), BUTTON_POS_X, BUTTON_POS_Y, BUTTON_SIZE_X, BUTTON_SIZE_Y, RADIUS, FONT_SYSTEM, btnClose_OnClick)


/****************************************************************************
* Screen declaration
****************************************************************************/
D4D_DECLARE_SCREEN_BEGIN(screen_rtcsinfo, ScreenRtcsinfo_, SCREEN_POS_X-1, SCREEN_POS_Y-1, SCREEN_SIZE_X+2, SCREEN_SIZE_Y+20, DEF_EXTSTR(STR_RTCS_INFO), FONT_SYSTEM, NULL, SCREEN_FLAGS, NULL)
  D4D_DECLARE_SCREEN_OBJECT(sRtcsinfo_lblMacText)
  D4D_DECLARE_SCREEN_OBJECT(sRtcsinfo_lblMacValue)
  D4D_DECLARE_SCREEN_OBJECT(sRtcsinfo_lblIpText)
  D4D_DECLARE_SCREEN_OBJECT(sRtcsinfo_lblIpValue)
  D4D_DECLARE_SCREEN_OBJECT(sRtcsinfo_lblMaskText)
  D4D_DECLARE_SCREEN_OBJECT(sRtcsinfo_lblMaskValue)
  D4D_DECLARE_SCREEN_OBJECT(sRtcsinfo_lblGatewayText)
  D4D_DECLARE_SCREEN_OBJECT(sRtcsinfo_lblGatewayValue)
  D4D_DECLARE_SCREEN_OBJECT(sRtcsinfo_btnClose)
D4D_DECLARE_SCREEN_END()


/****************************************************************************
*       D4D widgets static events functions definitions
****************************************************************************/

// The Close button action
static void btnClose_OnClick(D4D_OBJECT* pThis) {
  D4D_EscapeScreen();
}

/****************************************************************************
*       D4D standard screen functions definitions
****************************************************************************/

// One time called screen function in screen initialization proces
static void ScreenRtcsinfo_OnInit() {
  D4D_SetTextProperties(&sRtcsinfo_lblMacText,      LABEL_TEXT_PROPERTIES);
  D4D_SetTextProperties(&sRtcsinfo_lblMacValue,     LABEL_TEXT_PROPERTIES);
  D4D_SetTextProperties(&sRtcsinfo_lblIpText,       LABEL_TEXT_PROPERTIES);
  D4D_SetTextProperties(&sRtcsinfo_lblIpValue,      LABEL_TEXT_PROPERTIES);
  D4D_SetTextProperties(&sRtcsinfo_lblMaskText,     LABEL_TEXT_PROPERTIES);
  D4D_SetTextProperties(&sRtcsinfo_lblMaskValue,    LABEL_TEXT_PROPERTIES);
  D4D_SetTextProperties(&sRtcsinfo_lblGatewayText,  LABEL_TEXT_PROPERTIES);
  D4D_SetTextProperties(&sRtcsinfo_lblGatewayValue, LABEL_TEXT_PROPERTIES);
  
  D4D_SetFontProperties(&sRtcsinfo_lblMacText,      LABEL_FONT_PROPERTIES);
  D4D_SetFontProperties(&sRtcsinfo_lblMacValue,     LABEL_FONT_PROPERTIES);
  D4D_SetFontProperties(&sRtcsinfo_lblIpText,       LABEL_FONT_PROPERTIES);
  D4D_SetFontProperties(&sRtcsinfo_lblIpValue,      LABEL_FONT_PROPERTIES);
  D4D_SetFontProperties(&sRtcsinfo_lblMaskText,     LABEL_FONT_PROPERTIES);
  D4D_SetFontProperties(&sRtcsinfo_lblMaskValue,    LABEL_FONT_PROPERTIES);
  D4D_SetFontProperties(&sRtcsinfo_lblGatewayText,  LABEL_FONT_PROPERTIES);
  D4D_SetFontProperties(&sRtcsinfo_lblGatewayValue, LABEL_FONT_PROPERTIES);
}



// Screen on Activate function called with each screen activation
static void ScreenRtcsinfo_OnActivate() {
  D4D_CHAR  pAscii[LABEL_BUFFER_SIZE];
  D4D_TCHAR pUnicode[LABEL_BUFFER_SIZE];

  sprintf(pAscii, "%02X:%02X:%02X:%02X:%02X:%02X", gMacAddress[0],gMacAddress[1],
          gMacAddress[2],gMacAddress[3],gMacAddress[4],gMacAddress[5]);
  D4D_SetText(&sRtcsinfo_lblMacValue, D4D_StrCopyAscii2Unicode(pUnicode, pAscii));
  sprintf(pAscii, "%d.%d.%d.%d", IPBYTES(gIPAddress.ip));
  D4D_SetText(&sRtcsinfo_lblIpValue, D4D_StrCopyAscii2Unicode(pUnicode, pAscii));
  sprintf(pAscii, "%d.%d.%d.%d", IPBYTES(gIPAddress.mask));
  D4D_SetText(&sRtcsinfo_lblMaskValue, D4D_StrCopyAscii2Unicode(pUnicode, pAscii));
  sprintf(pAscii, "%d.%d.%d.%d", IPBYTES(gIPAddress.gateway));
  D4D_SetText(&sRtcsinfo_lblGatewayValue, D4D_StrCopyAscii2Unicode(pUnicode, pAscii));
}

// Screen "Main" function called periodically in each D4D_poll runs
static void ScreenRtcsinfo_OnMain() {
  // nothing
}


// Screen on DeActivate function called with each screen deactivation
static void ScreenRtcsinfo_OnDeactivate() {
  // nothing
}

// Screen on message function called with each internal massage for this screen
static Byte ScreenRtcsinfo_OnObjectMsg(D4D_MESSAGE* pMsg) {
  D4D_UNUSED(pMsg);
  
  return D4D_MSG_NOSKIP;
}



