/******************************************************************************
* 
* Copyright (c) 2012 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_screen_scrsvr.c
*
* @author    b01119
* 
* @version   0.0.1.0
* 
* @date      Aug-14-2012
* 
* @brief     D4D driver - scrsvr of d4d typical screen source file
*
******************************************************************************/

/****************************************************************************
*       Includes header files
****************************************************************************/
#include <mfs.h>
#include <stdlib.h>
#include <lwmsgq.h>

#include "d4d.h"
#include "main.h"
#include "fonts.h"
#include "pictures.h"
#include "utils.h"
#include "audio/wave_task.h"
#include "schemes.h"


/****************************************************************************
*       Local functions declarations
***************************************************************************/
//static Byte SCRSVR_PicOnUsrMessage(D4D_MESSAGE* pMsg);

/****************************************************************************
*       Local variables declarations
***************************************************************************/

/****************************************************************************
*       D4D widgets (graphic objects) declarations
****************************************************************************/

#if defined(FRD43040CA)
# define FILENAME_BMP_BACKGROUND_FILE           "S480X272\\F43040CA.D4L"
#elif defined(FRD500A4001)
# define FILENAME_BMP_BACKGROUND_FILE           "S480X272\\F500A400.D4L"
#elif defined(FRD5040TPTB)
# define FILENAME_BMP_BACKGROUND_FILE           "S480X272\\F5040B.D4L"
#elif defined(FRD5040TPT)
# define FILENAME_BMP_BACKGROUND_FILE           "S800X480\\F5040T.D4L"
#elif defined(FRD7040TPT)
# define FILENAME_BMP_BACKGROUND_FILE           "S800X480\\F7040TPT.D4L"
#else
# define FILENAME_BMP_BACKGROUND_FILE           "BG.D4L"
#endif

// Bitmap filenames
#define FILENAME_BMP_BACKGROUND         "A:\\SYSTEM\\IMAGES\\SCRNSVR\\" FILENAME_BMP_BACKGROUND_FILE
#define FILENAME_BMP_LOGO_SQM4          "A:\\SYSTEM\\IMAGES\\SCRNSVR\\LOGO\\SQM4LOGO.D4L"
#define FILENAME_BMP_LOGO_ELNICO        "A:\\SYSTEM\\IMAGES\\SCRNSVR\\LOGO\\ELNCLOGO.D4L"
#define FILENAME_BMP_LOGO_FREESCALE     "A:\\SYSTEM\\IMAGES\\SCRNSVR\\LOGO\\FRSCLOGO.D4L"

// Bitmap structures - will be initialized in ScreenInit()
D4D_BMP sScrsvr_bmpBackground    = { NULL, NULL };
D4D_BMP sScrsvr_bmpLogoSQM4      = { NULL, NULL };
D4D_BMP sScrsvr_bmpLogoElnico    = { NULL, NULL };
D4D_BMP sScrsvr_bmpLogoFreescale = { NULL, NULL };

// Declare the picture object
D4D_DECLARE_STD_PICTURE       (scrsvr_picBg,   0, 0, &sScrsvr_bmpBackground);
D4D_DECLARE_ICON_BEGIN_INRAM(scrsvr_icnLogo, NULL, 0, 0, 0, 0, 0, 0, (D4D_ICON_F_DEFAULT), &schemeTransparent, FONT_SYSTEM, NULL, NULL, NULL)
  D4D_DECLARE_ICON_BMP(&sScrsvr_bmpLogoSQM4)
  D4D_DECLARE_ICON_BMP(&sScrsvr_bmpLogoElnico)
  D4D_DECLARE_ICON_BMP(&sScrsvr_bmpLogoFreescale)
D4D_DECLARE_ICON_END()

/****************************************************************************
*       D4D screen declarations
****************************************************************************/
D4D_DECLARE_SCREEN_BEGIN(screen_scrsvr, ScreenScrSvr_, 0 , 0, (D4D_COOR)(D4D_SCREEN_SIZE_LONGER_SIDE), (D4D_COOR)(D4D_SCREEN_SIZE_SHORTER_SIDE), NULL, 0, NULL, 0, NULL)
  D4D_DECLARE_SCREEN_OBJECT(scrsvr_picBg)    
  D4D_DECLARE_SCREEN_OBJECT(scrsvr_icnLogo)    
D4D_DECLARE_SCREEN_END()


/*-----------------------------------------------------------------------------
* FUNCTION:    ScreenScrSvr_MovePicture
* SCOPE:       Screen saver screen related function - local
* DESCRIPTION: Local function to move the screen saver picture
*
* PARAMETERS:  none
*              
* RETURNS:     none
*-----------------------------------------------------------------------------*/
static void ScreenScrSvr_MovePicture(void) { 
  D4D_SIZE pictSize;
  
  D4D_IconSetIndex(&scrsvr_icnLogo, rand()%3);
  pictSize = D4D_GetBmpSize((D4D_GET_ICON(&scrsvr_icnLogo))->pBmpX[D4D_IconGetIndex(&scrsvr_icnLogo)]);
  
  // Move the picture on new position
  scrsvr_icnLogo.position.x = rand()%(D4D_SCREEN_SIZE_LONGER_SIDE -pictSize.cx-1);
  scrsvr_icnLogo.position.y = rand()%(D4D_SCREEN_SIZE_SHORTER_SIDE-pictSize.cy-1);

  D4D_InvalidateScreen(scrsvr_icnLogo.pData->pScreen, D4D_TRUE);
}

/****************************************************************************
*       D4D standard screen functions definitions
****************************************************************************/
// One time called screen function in screen initialization proces
static void ScreenScrSvr_OnInit() {
  loadBitmap(&sScrsvr_bmpBackground,    FILENAME_BMP_BACKGROUND);
  loadBitmap(&sScrsvr_bmpLogoSQM4,      FILENAME_BMP_LOGO_SQM4);
  loadBitmap(&sScrsvr_bmpLogoElnico,    FILENAME_BMP_LOGO_ELNICO);
  loadBitmap(&sScrsvr_bmpLogoFreescale, FILENAME_BMP_LOGO_FREESCALE);
}

// Screen on Activate function called with each screen activation
static void ScreenScrSvr_OnActivate() {
}

// Screen "Main" function called periodically in each D4D_poll runs
static void ScreenScrSvr_OnMain() {
  if(time.bits.b3200ms) {
    time.bits.b3200ms = 0;    
    ScreenScrSvr_MovePicture();
  }
}


// Screen on DeActivate function called with each screen deactivation
static void ScreenScrSvr_OnDeactivate() {
/*  WAVE_MSG message;
  message.type = WAVE_STOP;
  _lwmsgq_send((pointer)wave_queue, (uint_32*)&message, 0); */
}

// Screen on message function called with each internal massage for this screen
static Byte ScreenScrSvr_OnObjectMsg(D4D_MESSAGE* pMsg) {
  return D4D_MSG_NOSKIP;
}



