/******************************************************************************
* 
* Copyright (c) 2012 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      d4d_screen_settings.c
*
* @author    b01119
* 
* @version   0.0.1.0
* 
* @date      Aug-14-2012
* 
* @brief     D4D driver - settings of d4d typical screen source file
*
******************************************************************************/

/****************************************************************************
*       Includes header files
****************************************************************************/
#include "d4d.h"
#include "main.h"
#include "fonts.h"
#include "strings.h"
#include "schemes.h"

/****************************************************************************
*       Local functions declarations
***************************************************************************/

static void RadioButnLang_OnChange(D4D_OBJECT* pThis);
static void RadioButnScheme_OnChange(D4D_OBJECT* pThis);
static void RadioButnFonts_OnChange(D4D_OBJECT* pThis);
static void ChbxScrSvrEnable_OnChange(D4D_OBJECT *pThis);
static void SliderScrSvrTime_OnChange(D4D_OBJECT_PTR pThis);

static void BtnOkOnClick(D4D_OBJECT* pThis);

/****************************************************************************
*       Local variables declarations
***************************************************************************/

/****************************************************************************
*       D4D widgets (graphic objects) declarations
****************************************************************************/

#define SET_SCREEN_SIZE_X       432
#define SET_SCREEN_SIZE_Y       220

#define SET_OFFSET_X            4
#define SET_OFFSET_Y            4

#define SET_SIZE_GB_X           (SET_SCREEN_SIZE_X - 3*SET_OFFSET_X) / 2
#define SET_SIZE_GB_Y           120
#define SET_SIZE_GB_SMALL_Y     (SET_SCREEN_SIZE_Y - SET_POS_GB_Y(1) - SET_OFFSET_Y)
#define SET_SIZE_GB_SMALL_X     110

#define SET_POS_GB_X(x)         (SET_OFFSET_X + ((SET_SIZE_GB_X + SET_OFFSET_X) * (x)))
#define SET_POS_GB_Y(y)         (SET_OFFSET_Y + ((SET_SIZE_GB_Y + SET_OFFSET_Y) * (y))) 


#define SET_SIZE_RB_X           140
#define SET_SIZE_RB_SMALL_X     92
#define SET_SIZE_RB_Y           24

#define SET_RB_POS_Y(y)         (SET_OFFSET_Y + (y) * SET_SIZE_RB_Y)

#define SET_BTN_SIZE_X          (SET_SCREEN_SIZE_X - SET_POS_GB_X(1) - SET_SIZE_GB_SMALL_X - 2*SET_OFFSET_X)
#define SET_BTN_SIZE_Y          SET_SIZE_GB_SMALL_Y

#define SET_BTN_POS_X           (SET_POS_GB_X(1) + SET_SIZE_GB_SMALL_X + SET_OFFSET_X)
#define SET_BTN_POS_Y           SET_POS_GB_Y(1)                   

#define SET_RADIUS              0

#define SET_BUTTON_FLAGS (D4D_OBJECT_F_VISIBLE | D4D_OBJECT_F_ENABLED | D4D_OBJECT_F_TABSTOP | D4D_OBJECT_F_TOUCHENABLE  | D4D_OBJECT_F_FASTTOUCH | D4D_OBJECT_F_BEVEL_RAISED | D4D_BTN_F_3D)
#define D4D_SCRSET_F    (D4D_SCR_F_OUTLINE | D4D_SCR_F_TITLEBAR | D4D_SCR_F_EXIT | D4D_SCR_F_BCKG)
#define SET_CHECKBOX_F  (D4D_OBJECT_F_VISIBLE | D4D_OBJECT_F_ENABLED | D4D_OBJECT_F_TABSTOP | D4D_OBJECT_F_TOUCHENABLE | D4D_OBJECT_F_FASTTOUCH | D4D_OBJECT_F_FOCUSRECT | D4D_CHECKBOX_F_ICON_RECTANGLE )
#define SET_SLIDER_F    (D4D_OBJECT_F_VISIBLE | D4D_OBJECT_F_ENABLED | D4D_OBJECT_F_TABSTOP | D4D_OBJECT_F_TOUCHENABLE | D4D_OBJECT_F_FASTTOUCH | D4D_OBJECT_F_FOCUSRECT | D4D_SLDR_F_AUTOTOUCH_OFF)

// Extern the parent group box objects
D4D_EXTERN_OBJECT(scrSett_grpBxLang)
D4D_EXTERN_OBJECT(scrSett_grpBxScheme)
D4D_EXTERN_OBJECT(scrSett_grpBxFonts)
D4D_EXTERN_OBJECT(scrSett_grpBxScrSvr)

// Declare the relations structures of group boxes
D4D_DECLARE_OBJECT_RELATIONS_BEGIN(grpBxLang_Rel, &scrSett_grpBxLang)
D4D_DECLARE_OBJECT_RELATIONS_END

D4D_DECLARE_OBJECT_RELATIONS_BEGIN(grpBxScheme_Rel, &scrSett_grpBxScheme)
D4D_DECLARE_OBJECT_RELATIONS_END

D4D_DECLARE_OBJECT_RELATIONS_BEGIN(grpBxFonts_Rel, &scrSett_grpBxFonts)
D4D_DECLARE_OBJECT_RELATIONS_END

D4D_DECLARE_OBJECT_RELATIONS_BEGIN(grpBxScrSvr_Rel, &scrSett_grpBxScrSvr)
D4D_DECLARE_OBJECT_RELATIONS_END

/*******************************************
* Language settings Group box 
*******************************************/
// Radio buttons declaration
_D4D_DECLARE_RADIOBUTTON(D4D_CONST, grpBxLang_Eng, DEF_EXTSTR(STR_LANGEN), SET_OFFSET_X, SET_RB_POS_Y(0), SET_SIZE_RB_X, SET_SIZE_RB_Y, SET_RADIUS, NULL, grpBxLang_Rel, D4D_RADIOBUTTON_F_DEFAULT, NULL, NULL, NULL, FONT_SYSTEM, STRING_LANG_ENG, RadioButnLang_OnChange, NULL)
_D4D_DECLARE_RADIOBUTTON(D4D_CONST, grpBxLang_Cze, DEF_EXTSTR(STR_LANGCZ), SET_OFFSET_X, SET_RB_POS_Y(1), SET_SIZE_RB_X, SET_SIZE_RB_Y, SET_RADIUS, NULL, grpBxLang_Rel, D4D_RADIOBUTTON_F_DEFAULT, NULL, NULL, NULL, FONT_SYSTEM, STRING_LANG_CZE, RadioButnLang_OnChange, NULL)
_D4D_DECLARE_RADIOBUTTON(D4D_CONST, grpBxLang_Chn, DEF_EXTSTR(STR_LANGCHI), SET_OFFSET_X, SET_RB_POS_Y(2), SET_SIZE_RB_X, SET_SIZE_RB_Y, SET_RADIUS, NULL, grpBxLang_Rel, D4D_RADIOBUTTON_F_DEFAULT, NULL, NULL, NULL, FONT_SYSTEM, STRING_LANG_CHN, RadioButnLang_OnChange, NULL)


// Declare language Settings Group Box
D4D_DECLARE_STD_GROUP_BOX_BEGIN(scrSett_grpBxLang, SET_POS_GB_X(0), SET_POS_GB_Y(0), SET_SIZE_GB_X, SET_SIZE_GB_Y, SET_RADIUS, NULL, DEF_EXTSTR(STR_LANG), FONT_SYSTEM_SMALL)
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBxLang_Eng)
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBxLang_Cze)
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBxLang_Chn)  
D4D_DECLARE_GROUP_BOX_OBJECT_END

/*******************************************
* Color scheme settings Group box 
*******************************************/
_D4D_DECLARE_RADIOBUTTON(D4D_CONST, grpBxScheme_Grey, DEF_EXTSTR(STR_SCHEME_GREY), SET_OFFSET_X, SET_RB_POS_Y(0), SET_SIZE_RB_X, SET_SIZE_RB_Y, SET_RADIUS, NULL, grpBxScheme_Rel, D4D_RADIOBUTTON_F_DEFAULT, NULL, NULL, NULL, FONT_SYSTEM, NULL, RadioButnScheme_OnChange, NULL)
_D4D_DECLARE_RADIOBUTTON(D4D_CONST, grpBxScheme_Green, DEF_EXTSTR(STR_SCHEME_GREEN), SET_OFFSET_X, SET_RB_POS_Y(1), SET_SIZE_RB_X, SET_SIZE_RB_Y, SET_RADIUS, NULL, grpBxScheme_Rel, D4D_RADIOBUTTON_F_DEFAULT, NULL, NULL, NULL, FONT_SYSTEM, &d4d_clr_scheme_green, RadioButnScheme_OnChange, NULL)
_D4D_DECLARE_RADIOBUTTON(D4D_CONST, grpBxScheme_Blue, DEF_EXTSTR(STR_SCHEME_BLUE), SET_OFFSET_X, SET_RB_POS_Y(2), SET_SIZE_RB_X, SET_SIZE_RB_Y, SET_RADIUS, NULL, grpBxScheme_Rel, D4D_RADIOBUTTON_F_DEFAULT, NULL, NULL, NULL, FONT_SYSTEM, &d4d_clr_scheme_navy, RadioButnScheme_OnChange, NULL)
_D4D_DECLARE_RADIOBUTTON(D4D_CONST, grpBxScheme_Red, DEF_EXTSTR(STR_SCHEME_RED), SET_OFFSET_X, SET_RB_POS_Y(3), SET_SIZE_RB_X, SET_SIZE_RB_Y, SET_RADIUS, NULL, grpBxScheme_Rel, D4D_RADIOBUTTON_F_DEFAULT, NULL, NULL, NULL, FONT_SYSTEM, &d4d_clr_scheme_red, RadioButnScheme_OnChange, NULL)


// Declare scheme Settings Group Box
D4D_DECLARE_STD_GROUP_BOX_BEGIN(scrSett_grpBxScheme, SET_POS_GB_X(1), SET_POS_GB_Y(0), SET_SIZE_GB_X, SET_SIZE_GB_Y, SET_RADIUS, NULL, DEF_EXTSTR(STR_SCHEME), FONT_SYSTEM_SMALL)
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBxScheme_Grey)
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBxScheme_Green)
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBxScheme_Blue)
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBxScheme_Red)  
D4D_DECLARE_GROUP_BOX_OBJECT_END

/*******************************************
* Fonts settings Group box 
*******************************************/ 
_D4D_DECLARE_RADIOBUTTON(D4D_CONST, grpBxFonts_Courier, DEF_EXTSTR(STR_FONTS_COURIER), SET_OFFSET_X, SET_RB_POS_Y(0), SET_SIZE_RB_SMALL_X, SET_SIZE_RB_Y, SET_RADIUS, NULL, grpBxFonts_Rel, D4D_RADIOBUTTON_F_DEFAULT, NULL, NULL, NULL, FONT_SYSTEM, NULL, RadioButnFonts_OnChange, NULL)
_D4D_DECLARE_RADIOBUTTON(D4D_CONST, grpBxFonts_Arial, DEF_EXTSTR(STR_FONTS_ARIAL), SET_OFFSET_X, SET_RB_POS_Y(1), SET_SIZE_RB_SMALL_X, SET_SIZE_RB_Y, SET_RADIUS, NULL, grpBxFonts_Rel, D4D_RADIOBUTTON_F_DEFAULT, NULL, NULL, NULL, FONT_SYSTEM, &fontTable_Arial, RadioButnFonts_OnChange, NULL)

// Declare font Settings Group Box
D4D_DECLARE_STD_GROUP_BOX_BEGIN(scrSett_grpBxFonts, SET_POS_GB_X(1), SET_POS_GB_Y(1), SET_SIZE_GB_SMALL_X, SET_SIZE_GB_SMALL_Y, SET_RADIUS, NULL, DEF_EXTSTR(STR_FONTS), FONT_SYSTEM_SMALL)
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBxFonts_Courier)
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBxFonts_Arial)  
D4D_DECLARE_GROUP_BOX_OBJECT_END

/*******************************************
* Screen saver settings Group box 
*******************************************/                      
_D4D_DECLARE_CHECKBOX(D4D_CONST, grpBxScrsvr_enable, DEF_EXTSTR(STR_ENABLE), SET_OFFSET_X, SET_RB_POS_Y(0), SET_SIZE_RB_X, SET_SIZE_RB_Y, SET_RADIUS, NULL, grpBxScrSvr_Rel, SET_CHECKBOX_F, NULL, NULL, NULL, FONT_SYSTEM, NULL, ChbxScrSvrEnable_OnChange, NULL)

D4D_TCHAR grpBxScrsvr_timeStr[16] = D4D_DEFSTR("30s");
_D4D_DECLARE_SLIDER(D4D_CONST, grpBxScrsvr_time, grpBxScrsvr_timeStr, SET_OFFSET_X, SET_RB_POS_Y(1) + SET_OFFSET_Y, (SET_SIZE_GB_X - 20), SET_SIZE_RB_Y, SET_RADIUS, NULL, grpBxScrSvr_Rel, 0, 0, 0, 0, 0, 0, SET_SLIDER_F, NULL, NULL, FONT_SYSTEM, NULL, SliderScrSvrTime_OnChange, NULL)
// Declare screen saver Settings Group Box
D4D_DECLARE_STD_GROUP_BOX_BEGIN(scrSett_grpBxScrSvr, SET_POS_GB_X(0), SET_POS_GB_Y(1), SET_SIZE_GB_X, SET_SIZE_GB_SMALL_Y, SET_RADIUS, NULL, DEF_EXTSTR(STR_SCRSVR), FONT_SYSTEM_SMALL)
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBxScrsvr_enable)
  D4D_DECLARE_GROUP_BOX_OBJECT(grpBxScrsvr_time)  
D4D_DECLARE_GROUP_BOX_OBJECT_END

// Declare OK button
D4D_DECLARE_RBUTTON(sSett_btnOk, DEF_EXTSTR(STR_OK), SET_BTN_POS_X, SET_BTN_POS_Y, SET_BTN_SIZE_X, SET_BTN_SIZE_Y, SET_RADIUS, SET_BUTTON_FLAGS, NULL, NULL, NULL, FONT_SYSTEM_BIG, NULL, BtnOkOnClick, NULL)


/****************************************************************************
*       D4D screen declarations
****************************************************************************/
D4D_DECLARE_SCREEN_BEGIN(screen_settings, ScreenSettings_, 20 ,20, (SET_SCREEN_SIZE_X + 2), (SET_SCREEN_SIZE_Y + 20), DEF_EXTSTR(STR_SETTINGS), FONT_SYSTEM, NULL, D4D_SCRSET_F, NULL)
  D4D_DECLARE_SCREEN_OBJECT(scrSett_grpBxLang)    
  D4D_DECLARE_SCREEN_OBJECT(scrSett_grpBxScheme)    
  D4D_DECLARE_SCREEN_OBJECT(scrSett_grpBxScrSvr)
  D4D_DECLARE_SCREEN_OBJECT(scrSett_grpBxFonts)
  D4D_DECLARE_SCREEN_OBJECT(sSett_btnOk)    
D4D_DECLARE_SCREEN_END()    


/****************************************************************************
*       D4D widgets static events functions definitions
****************************************************************************/
static void RadioButnLang_OnChange(D4D_OBJECT* pThis)
{
  //Change used language
  Strings_ChangeLanguage((LWord)D4D_GetUserPointer(pThis));
  
  //Invalidate screen to show change on screen
  D4D_InvalidateScreen(pThis->pData->pScreen, D4D_TRUE);
}

static void RadioButnScheme_OnChange(D4D_OBJECT* pThis)
{
  // Change used color scheme
  D4D_SetDefaultScheme((D4D_CLR_SCHEME*)D4D_GetUserPointer(pThis));
  
  //Invalidate screen to show change on screen
  D4D_InvalidateScreen(pThis->pData->pScreen, D4D_TRUE);
}

static void RadioButnFonts_OnChange(D4D_OBJECT* pThis)
{
  // Change used font
  D4D_SetFontTable((D4D_FONT_TYPE*)D4D_GetUserPointer(pThis));
  
  //Invalidate screen to show change on screen
  D4D_InvalidateScreen(pThis->pData->pScreen, D4D_TRUE);
}

static void ChbxScrSvrEnable_OnChange(D4D_OBJECT *pThis) {
  screenSaverEnabled = D4D_CheckBoxGetValue(pThis);
}

static void SliderScrSvrTime_OnChange(D4D_OBJECT_PTR pThis)
{
  //Chenge the timeout for screen saver
  // there is 4 step 
  switch(D4D_SldrGetValue(pThis))
  {
    case 1:
      D4D_SetText(pThis, D4D_DEFSTR("10s"));
      screenSaverTimeoutPreset = 100;
      break;
    
    case 2:
      D4D_SetText(pThis, D4D_DEFSTR("30s"));
      screenSaverTimeoutPreset = 300;
      break;
    
    case 3:
      D4D_SetText(pThis, D4D_DEFSTR("1m"));
      screenSaverTimeoutPreset = 600;
      break;
    
    case 4:
      D4D_SetText(pThis, D4D_DEFSTR("10m"));
      screenSaverTimeoutPreset = 6000;
      break;
      
  }
  
}

// The OK button action
static void BtnOkOnClick(D4D_OBJECT* pThis)
{
  D4D_EscapeScreen();
}

/****************************************************************************
*       D4D standard screen functions definitions
****************************************************************************/

// One time called screen function in screen initialization proces
static void ScreenSettings_OnInit()
{
  const D4D_SLIDER_LIMITS sliderLimits = {1, 4, 1, 1};
  
  D4D_RadioButtonSetValue(&grpBxLang_Eng, D4D_TRUE);
  D4D_RadioButtonSetValue(&grpBxScheme_Blue, D4D_TRUE);
  D4D_RadioButtonSetValue(&grpBxFonts_Courier, D4D_TRUE);
  D4D_CheckBoxSetValue(&grpBxScrsvr_enable, D4D_TRUE);
  
  D4D_SldrSetLimits(&grpBxScrsvr_time, &sliderLimits);
  D4D_SldrSetValue(&grpBxScrsvr_time, 2);
}



// Screen on Activate function called with each screen activation
static void ScreenSettings_OnActivate()
{
  
}

// Screen "Main" function called periodically in each D4D_poll runs
static void ScreenSettings_OnMain()
{
    
}


// Screen on DeActivate function called with each screen deactivation
static void ScreenSettings_OnDeactivate()
{
    
}

// Screen on message function called with each internal massage for this screen
static Byte ScreenSettings_OnObjectMsg(D4D_MESSAGE* pMsg)
{
  D4D_UNUSED(pMsg);
  
  return 0;
}



