/************************************************************************
*
*  Freescale INC.
*  ALL RIGHTS RESERVED
*  (c) Copyright 2008 Freescale, Inc.
*
*************************************************************************
*
*  FILE NAME: strings.c
*
*  PURPOSE: Fonts used in project.
*
*  AUTHOR: B01119
*
***********************************************************************/

/******************************************************************************
* Includes
******************************************************************************/
#include "d4d.h"   
#include "strings.h"

/****************************************************************************
*       Local functions declarations
***************************************************************************/
static int Strings_TranslateStr(const D4D_TCHAR* originTxt, D4D_TCHAR* newTxt, int offset, int size);
static int Strings_GetTranslateStrLength(const D4D_TCHAR* originTxt);

/*****************************************************************************
*       Local variables declarations
****************************************************************************/
D4D_DECLARE_LOCALE_FUNCTIONS(STRINGS_PREFIX, Strings_TranslateStr, Strings_GetTranslateStrLength)

/*******************************************
* String table - English
*******************************************/
static D4D_TCHAR* strings_eng[] = 
{
  D4D_DEFSTR("Open"),           // STR_OPEN       
  D4D_DEFSTR("Open File"),      // STR_OPENFILE   
  D4D_DEFSTR("Close"),          // STR_CLOSE      
  D4D_DEFSTR("Cancel"),         // STR_CANCEL     
  D4D_DEFSTR("OK"),             // STR_OK         
  D4D_DEFSTR("Quit"),           // STR_QUIT     
  D4D_DEFSTR("Directories"),    // STR_DIRECTORIES
  D4D_DEFSTR("Files"),          // STR_FILES      
  D4D_DEFSTR("Disk"),           // STR_DISK         
  D4D_DEFSTR("Language"),       // STR_LANG   
  D4D_DEFSTR("English"),        // STR_LANGEN 
  D4D_DEFSTR("Czech"),          // STR_LANGCZ 
  D4D_DEFSTR("Chinese"),        // STR_LANGCHI
  D4D_DEFSTR("Reserved1"),      // STR_LANGRES1
  D4D_DEFSTR("Reserved2"),      // STR_LANGRES2
  D4D_DEFSTR("Reserved3"),      // STR_LANGRES3
  D4D_DEFSTR("Reserved4"),      // STR_LANGRES4
  D4D_DEFSTR("Settings"),       // STR_SETTINGS  
  D4D_DEFSTR("Scheme"),         // STR_SCHEME  
  D4D_DEFSTR("Grey"),           // STR_SCHEME_GREY   
  D4D_DEFSTR("Green"),          // STR_SCHEME_GREEN  
  D4D_DEFSTR("Blue"),           // STR_SCHEME_BLUE   
  D4D_DEFSTR("Red"),            // STR_SCHEME_RED     
  D4D_DEFSTR("Fonts"),          // STR_FONTS          
  D4D_DEFSTR("Courier"),        // STR_FONTS_COURIER  
  D4D_DEFSTR("Arial"),          // STR_FONTS_ARIAL     
  D4D_DEFSTR("Screen Saver"),   // STR_SCRSVR  
  D4D_DEFSTR("Enabled"),        // STR_ENABLE   
  D4D_DEFSTR("File Content. Use OPEN button!"),        // STR_DUMMY_LABEL   
  D4D_DEFSTR("Could not show!"),// STR_COULD_NOT_SHOW
  D4D_DEFSTR("Image Info"),     // STR_IMAGE_INFO
  D4D_DEFSTR("Type:"),          // STR_TYPE
  D4D_DEFSTR("Width:"),         // STR_WIDTH
  D4D_DEFSTR("Height:"),        // STR_HEIGHT
  D4D_DEFSTR("Size:"),          // STR_SIZE
  D4D_DEFSTR("Text Info"),      // STR_TEXT_INFO
  D4D_DEFSTR("ASCII"),          // STR_ASCII    
  D4D_DEFSTR("UniCode"),        // STR_UNICODE  
  D4D_DEFSTR("Info"),           // STR_INFO
  D4D_DEFSTR("Filebrowser"),    // STR_FILEBROWSER
  D4D_DEFSTR("View"),           // STR_VIEW
  D4D_DEFSTR("Copy"),           // STR_COPY
  D4D_DEFSTR("Remove"),         // STR_REMOVE
  D4D_DEFSTR("Exit"),           // STR_EXIT
  D4D_DEFSTR("Left"),           // STR_LEFT
  D4D_DEFSTR("Right"),          // STR_RIGHT
  D4D_DEFSTR("RTCS stack info"),  // STR_RTCS_INFO
  D4D_DEFSTR("MAC Address:"),   // STR_MAC
  D4D_DEFSTR("IP Address:"),    // STR_IP
  D4D_DEFSTR("Net Mask:"),      // STR_MASK
  D4D_DEFSTR("Default Gateway:"),   // STR_GATEWAY

  
  D4D_DEFSTR("Unknown") 
};


/*******************************************
* String table - Czech
*******************************************/
static D4D_TCHAR* strings_cze[] = 
{
  D4D_DEFSTR("Otevrit"),        // STR_OPEN       
  D4D_DEFSTR("Otevrit soubor"), // STR_OPENFILE   
  D4D_DEFSTR("Zavrit"),         // STR_CLOSE      
  D4D_DEFSTR("Zrusit"),         // STR_CANCEL     
  D4D_DEFSTR("OK"),             // STR_OK         
  D4D_DEFSTR("Konec"),          // STR_QUIT  
  D4D_DEFSTR("Adresare"),       // STR_DIRECTORIES
  D4D_DEFSTR("Soubory"),        // STR_FILES      
  D4D_DEFSTR("Disk"),           // STR_DISK 
  D4D_DEFSTR("Jazyk"),          // STR_LANG   
  D4D_DEFSTR("Anglictina"),     // STR_LANGEN 
  D4D_DEFSTR("Cestina"),        // STR_LANGCZ 
  D4D_DEFSTR("Cinstina"),       // STR_LANGCHI
  D4D_DEFSTR("Rezervovano1"),   // STR_LANGRES1
  D4D_DEFSTR("Rezervovano2"),   // STR_LANGRES2
  D4D_DEFSTR("Rezervovano3"),   // STR_LANGRES3
  D4D_DEFSTR("Rezervovano4"),   // STR_LANGRES4
  D4D_DEFSTR("Nastaveni"),      // STR_SETTINGS  
  D4D_DEFSTR("Schema"),         // STR_SCHEME  
  D4D_DEFSTR("Sede"),           // STR_SCHEME_GREY   
  D4D_DEFSTR("Zelene"),         // STR_SCHEME_GREEN  
  D4D_DEFSTR("Modre"),          // STR_SCHEME_BLUE   
  D4D_DEFSTR("Cervene"),        // STR_SCHEME_RED     
  D4D_DEFSTR("Pismo"),          // STR_FONTS          
  D4D_DEFSTR("Courier"),        // STR_FONTS_COURIER  
  D4D_DEFSTR("Arial"),          // STR_FONTS_ARIAL     
  D4D_DEFSTR("Sporic obrazovky"),// STR_SCRSVR  
  D4D_DEFSTR("Povolit"),        // STR_ENABLE   
  D4D_DEFSTR("Obsah souboru. Pouzij tlacitko OTEVRIT!"),        // STR_DUMMY_LABEL   
  D4D_DEFSTR("Nejde zobrazit!"),// STR_COULD_NOT_SHOW
  D4D_DEFSTR("Informace"),      // STR_IMAGE_INFO
  D4D_DEFSTR("Typ:"),           // STR_TYPE
  D4D_DEFSTR("Sirka:"),         // STR_WIDTH
  D4D_DEFSTR("Vyska:"),         // STR_HEIGHT
  D4D_DEFSTR("Velikost:"),      // STR_SIZE
  D4D_DEFSTR("Informace"),      // STR_TEXT_INFO
  D4D_DEFSTR("ASCII"),          // STR_ASCII    
  D4D_DEFSTR("UniCode"),        // STR_UNICODE  
  D4D_DEFSTR("Informace"),      // STR_INFO
  D4D_DEFSTR("Prohlizec souboru"),  // STR_FILEBROWSER
  D4D_DEFSTR("Nahled"),         // STR_VIEW
  D4D_DEFSTR("Kopirovat"),      // STR_COPY
  D4D_DEFSTR("Odstranit"),      // STR_REMOVE
  D4D_DEFSTR("Konec"),          // STR_EXIT
  D4D_DEFSTR("Levy"),           // STR_LEFT
  D4D_DEFSTR("Pravy"),          // STR_RIGHT
  D4D_DEFSTR("RTCS stack info"),  // STR_RTCS_INFO
  D4D_DEFSTR("MAC adresa:"),    // STR_MAC
  D4D_DEFSTR("IP adresa:"),     // STR_IP
  D4D_DEFSTR("Maska site:"),    // STR_MASK
  D4D_DEFSTR("Vychozi brana:"), // STR_GATEWAY

  
  
  D4D_DEFSTR("Neznamy") 
};

/*******************************************
* String table - Chinese
*******************************************/
static D4D_TCHAR* strings_chn[] = 
{
  D4D_DEFSTR("Hobolo"),          // STR_OPEN       
  D4D_DEFSTR("Hoodo fado"),     // STR_OPENFILE   
  D4D_DEFSTR("Boko"),           // STR_CLOSE      
  D4D_DEFSTR("Belo"),           // STR_CANCEL     
  D4D_DEFSTR("Ja"),             // STR_OK         
  D4D_DEFSTR("Sustila"),        // STR_SYSTEM     
  D4D_DEFSTR("Homolo"),         // STR_DIRECTORIES
  D4D_DEFSTR("Komoho"),         // STR_FILES      
  D4D_DEFSTR("Besku"),          // STR_DISK         
  D4D_DEFSTR("Begudena"),       // STR_LANG   
  D4D_DEFSTR("Enovona"),        // STR_LANGEN 
  D4D_DEFSTR("Genof"),          // STR_LANGCZ 
  D4D_DEFSTR("Behodss"),        // STR_LANGCHI
  D4D_DEFSTR("Hegeho1"),        // STR_LANGRES1
  D4D_DEFSTR("Hegeho2"),        // STR_LANGRES2
  D4D_DEFSTR("Hegeho3"),        // STR_LANGRES3
  D4D_DEFSTR("Hegeho4"),        // STR_LANGRES4
  D4D_DEFSTR("Kumulo"),         // STR_SETTINGS  
  D4D_DEFSTR("Jojojo"),         // STR_SCHEME  
  D4D_DEFSTR("Tako"),           // STR_SCHEME_GREY   
  D4D_DEFSTR("Nejap"),          // STR_SCHEME_GREEN  
  D4D_DEFSTR("Coma"),           // STR_SCHEME_BLUE   
  D4D_DEFSTR("Wah"),            // STR_SCHEME_RED     
  D4D_DEFSTR("Hotmoloklo"),     // STR_FONTS          
  D4D_DEFSTR("Courier"),        // STR_FONTS_COURIER  
  D4D_DEFSTR("Arial"),          // STR_FONTS_ARIAL     
  D4D_DEFSTR("Hendsf demok"),   // STR_SCRSVR  
  D4D_DEFSTR("Hedenad"),        // STR_ENABLE   
  D4D_DEFSTR("Hemod Undafvo. Sdfasf fot sdfsdfg!"),        // STR_DUMMY_LABEL   
  D4D_DEFSTR("Hunodo kopolo!"), // STR_COULD_NOT_SHOW
  D4D_DEFSTR("Hojofo"),         // STR_IMAGE_INFO
  D4D_DEFSTR("Heko:"),          // STR_TYPE
  D4D_DEFSTR("Bleko:"),         // STR_WIDTH
  D4D_DEFSTR("Mako:"),          // STR_HEIGHT
  D4D_DEFSTR("Umola:"),         // STR_SIZE
  D4D_DEFSTR("Hego moro"),      // STR_TEXT_INFO
  D4D_DEFSTR("ASCII"),          // STR_ASCII    
  D4D_DEFSTR("UniCode"),        // STR_UNICODE  
  D4D_DEFSTR("Hunfolr"),        // STR_INFO
  
  D4D_DEFSTR("Heno") 
};

/*******************************************
* Table of string tables
*******************************************/
static STRINGS_LANG_TABLE stringsLangTable[] =
{
  {STRING_LANG_ENG, strings_eng},
  {STRING_LANG_CZE, strings_cze},
  {STRING_LANG_CHN, strings_chn},
  {0, NULL}
};

// Current used language
static D4D_TCHAR** pStrTable = strings_eng;


/*-----------------------------------------------------------------------------
* FUNCTION:    Strings_ChangeLanguage
* SCOPE:       language strings related function
* DESCRIPTION: Function changes the current language
*
* PARAMETERS:  ix - index of language
*              
* RETURNS:     none
*-----------------------------------------------------------------------------*/
void Strings_ChangeLanguage(LWord ix)
{
  LWord i = 0;
  while(stringsLangTable[i].langTable != NULL)
  {
    if(stringsLangTable[i].index == ix)
    {
      pStrTable = stringsLangTable[i].langTable;
      return;
    }
    i++;
  }
}


static int Strings_stoi(const D4D_TCHAR* pText)
{
  // simple implementation of string to integer (just only for 2 chars)
  // just for example  of external strings
  
  return (pText[0] - '0') * 10 + (pText[1] - '0');
}


static int Strings_TranslateStr(const D4D_TCHAR* originTxt, D4D_TCHAR* newTxt, int offset, int size)
{
  LWord tableIx;
  LWord strLen;
  LWord i;
    
  D4D_TCHAR* pText;
  
  if(originTxt[0] != STRINGS_PREFIX)
    tableIx = STRINGS_GET_COUNT(strings_eng) - 1;
  else
  {
    tableIx = Strings_stoi(&originTxt[1]);
    if(tableIx >= STRINGS_GET_COUNT(strings_eng))
      tableIx = STRINGS_GET_COUNT(strings_eng) - 1;
  }
  
  pText = pStrTable[tableIx];
  
  strLen = D4D_GetTextLength(pText) + 1; // + termination char \0
    
  if(offset >= strLen)
    return 0;
  
  strLen -= offset;
  
  pText += offset;
  
  if(strLen > (size / sizeof(D4D_TCHAR)))
    strLen = (size / sizeof(D4D_TCHAR));
  
  for(i=0; i< strLen; i++)
    newTxt[i] = pText[i];
  
  return strLen;
}

static int Strings_GetTranslateStrLength(const D4D_TCHAR* originTxt)
{
  LWord tableIx;
    
  D4D_TCHAR* pText;
  
  if(originTxt[0] != STRINGS_PREFIX)
    tableIx = STRINGS_GET_COUNT(strings_eng) - 1;
  else
  {
    tableIx = Strings_stoi(&originTxt[1]);
    if(tableIx >= STRINGS_GET_COUNT(strings_eng))
      tableIx = STRINGS_GET_COUNT(strings_eng) - 1;
  }
  
  pText = pStrTable[tableIx];
  
  return D4D_GetTextLength(pText);
}