/**HEADER********************************************************************
* 
* Copyright (c) 2008 Freescale Semiconductor;
* All Rights Reserved                       
*
* Copyright (c) 1989-2008 ARC International;
* All Rights Reserved
*
*************************************************************************** 
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
**************************************************************************
*
* $FileName: main.c$
* $Version : 3.6.1.0$
* $Date    : Aug-14-2012$
*
* Comments:
*
*   This file contains the source for the eGUI example program.
*
*END************************************************************************/

/****************************************************************************
*       Includes header files
****************************************************************************/
#include <mqx.h>
#include <bsp.h> 
#include <lwtimer.h>
#include "main.h"
#include "tasks.h"
#include "lwcfg.h"
#include "gui/schemes.h"
#include "ethernet/ethernet_ftp.h"
#include "usb/MFS_USB.h"
#include "usb/USB_MSD_Disk.h"
#include "audio/wave_task.h"
#include "NANDFlash/nand_flash_fcn.h"

/****************************************************************************
*       MQX checks
****************************************************************************/
#if ! BSPCFG_ENABLE_IO_SUBSYSTEM
#error This application requires BSPCFG_ENABLE_IO_SUBSYSTEM defined non-zero in user_config.h. Please recompile BSP with this option.
#endif

#ifndef BSP_DEFAULT_IO_CHANNEL_DEFINED
#error This application requires BSP_DEFAULT_IO_CHANNEL to be not NULL. Please set corresponding BSPCFG_ENABLE_TTYx to non-zero in user_config.h and recompile BSP with this option.
#endif

#if ! MQX_USE_LWTIMER
#error This application requires MQX_USE_LWTIMER defined non-zero in user_config.h. Please recompile BSP with this option.
#endif


/****************************************************************************
*       Macro definitions
****************************************************************************/
// Configuration file keys definitions
#define CONFIG_KEY_TS_CALIBRATED        "TS_Calibrated"
#define CONFIG_KEY_TS_OFFSET_X          "TS_Xoffset"
#define CONFIG_KEY_TS_OFFSET_Y          "TS_Yoffset"
#define CONFIG_KEY_TS_BITSPERPIXEL_X    "TS_XBitsPerPixelx16"
#define CONFIG_KEY_TS_BITSPERPIXEL_Y    "TS_YBitsPerPixelx16"

#if PSP_MQX_CPU_IS_MCF51JM
   /* small ram taget */
   #define MFS_USB_STACK_SIZE   1000
#else
   #define MFS_USB_STACK_SIZE   2500   
#endif 


/****************************************************************************
*       Global variables declarations
***************************************************************************/
_lwmem_pool_id d4dMemoryPoolId;

/****************************************************************************
*       Local functions declarations
***************************************************************************/
static void lcd_task(uint_32); // task function
extern void WAVE_task(uint_32);
extern void httpdsrv_task(uint_32);

static D4D_FILEPTR My_FileOpen(char* filename, char* mode); // THE MODE WILL BE ALWAYS READ ONLY 'r'
static int My_FileClose(D4D_FILEPTR p_file);
static int My_FileRead(D4D_FILEPTR p_file, void* buffer, int size);
static int My_FileWrite(D4D_FILEPTR p_file, const void* buffer, int size);
static int My_FileFseek(D4D_FILEPTR p_file, long offset, int origin);
static long My_FileFtell(D4D_FILEPTR p_file);
static void getScreenshot(void);
extern uint_32 InitSdCard(void);

D4D_EXTERN_SCREEN(screen_main)
D4D_EXTERN_SCREEN(screen_scrsvr)

/****************************************************************************
*       Local variables declarations
***************************************************************************/
TIME_FLAGS time;
LWord time100sm_cnt = 0;
LWord screenSaverTimeout = 0;
LWord screenSaverTimeoutPreset = SCREENSAVER_TIMEOUT_100MS;
D4D_BOOL screenSaverEnabled = D4D_TRUE;
LWord screenSaverTouchProtection = 0;
volatile unsigned char gbGetScreenshot;
unsigned char gucScreenshotsCnt;


/* Initial structure for TCHRES taken from init_tchres.c */

extern MQX_FILE_PTR filesystem_handle;

    
/****************************************************************************
*       MQX tasks definition
***************************************************************************/
const TASK_TEMPLATE_STRUCT  MQX_template_list[] = { 
  /* Task Index,          Function,            Stack,Pri., Name,                  Attributes,       Param,  Time Slice */
  { LCD_TASK,             lcd_task,             4000, 11, "LCD",                  MQX_AUTO_START_TASK,  0,  0 },
  { WAVE_PLAYER_TASK,     WAVE_task,            2000, 10, "Wave_player",          MQX_AUTO_START_TASK,  0,  0 },
  { HTTPDSRV_TASK,        httpdsrv_task,        2500, 12, "HTTPd_server",         MQX_AUTO_START_TASK,  0,  0 },
  { USB_MSD_DISK_TASK,    USB_MSD_disk,         6000, 17, "USB_MSD_disk",         MQX_AUTO_START_TASK,  0,  0 },   
  { MFS_USB_TASK,         MFS_USB_task, MFS_USB_STACK_SIZE, 14, "MFS_USB",        MQX_AUTO_START_TASK,  0,  0 },
  
  { 0,                    0,                    0,    0,  0,                      0,                    0,  0 }
};

/**************************************************************************************************
*   D4D file operation API
**************************************************************************************************/

D4D_DECLARE_FILE_FUNCTIONS(My_FileOpen, My_FileClose, My_FileRead, My_FileWrite, My_FileFseek, My_FileFtell)  

/**********************
* D4D open file
**********************/
static D4D_FILEPTR My_FileOpen(char* filename, char* mode) // THE MODE WILL BE ALWAYS READ ONLY 'r'
{    
  return (D4D_FILEPTR)fopen(filename, mode);
}

/**********************
* D4D close file
**********************/
static int My_FileClose(D4D_FILEPTR p_file)
{
  return fclose((MQX_FILE_PTR)p_file);
}

/**********************
* D4D read file
**********************/
static int My_FileRead(D4D_FILEPTR p_file, void* buffer, int size)
{   
  return fread(buffer, 1, size, (MQX_FILE_PTR)p_file);
}

/**********************
* D4D write file
**********************/
static int My_FileWrite(D4D_FILEPTR p_file, const void* buffer, int size)
{
  return fwrite((void*)buffer, 1, size, (MQX_FILE_PTR)p_file);
}

/**********************
* D4D seek file
**********************/
static int My_FileFseek(D4D_FILEPTR p_file, long offset, int origin)
{ 
  return fseek((MQX_FILE_PTR)p_file, offset, origin);
}

/**********************
* D4D tell file
**********************/
static long My_FileFtell(D4D_FILEPTR p_file)
{ 
  return ftell((MQX_FILE_PTR)p_file);
} 

/*LW Timer CallBack*---------------------------------------
* 
* Function Name    : Time_event25ms
* Comments     :
*    This function do: 	periodical events in 25ms
*
*END*-----------------------------------------------------*/
static void Time_event25ms(void* pUser)
{
  static Byte actual_time = 0;  
  TIME_FLAGS t_flags;	
  
  D4D_UNUSED(pUser);
  
  
  // Notify eGUI about new time event
  D4D_TimeTickPut();
  
  // Check the touches, the protection is here to protect short moment after leaving the screen saver
  if(!screenSaverTouchProtection)
    D4D_CheckTouchScreen();
  
  if(screenSaverTouchProtection)
    screenSaverTouchProtection--;
  
  // set the internal time flags
  actual_time++;
  t_flags.all = (uint_32)(actual_time ^ (actual_time - 1));

  time.all |= t_flags.all;
  
  if(t_flags.bits.b100ms)
  {
    time100sm_cnt++;    
    // manage start of screen saver
    if(screenSaverTimeout && screenSaverEnabled)
    {
      screenSaverTimeout--;
      if(screenSaverTimeout == 0)
      {
        // Timeout occur - Start Screen Saver
        printf("Activate Screen Saver\n");
        D4D_ActivateScreen(&screen_scrsvr, D4D_FALSE);
      }
    }
  }
}

/**************************************************************//*!
* D4D Input event call back function
******************************************************************/
void D4D_InputEventCB(void)
{
  if(screenSaverTimeout == 0)
  {
    // Escape from Screen Saver screen
    if(D4D_GetActiveScreen() == &screen_scrsvr)
    {
      printf("Deactivate Screen Saver\n");
      D4D_EscapeScreen();
      D4D_ClearKeysBuffer();
      D4D_ClearTouchScreenEvents();    
      
      screenSaverTouchProtection = 8;
    }
  }
 
  screenSaverTimeout = screenSaverTimeoutPreset;
}

/*******************************************************************************
* Tries to load a touchscreen calibration from a config file.
* @param[in]  pCfgFile  Initialized config file structure.
* @param[out] pDst      Initialized calibration structure to store the data to.
* @return               Zero on success, 1 on invalid argument, other value
*                       for a failure (please refer to lwcfg_getInteger()).
*                       pDst keeps unchanged unless zero returned.
*******************************************************************************/
static int ts_loadCalibration(LWCFG_FILE_PTR pCfgFile,
                              D4D_TOUCHSCREEN_CALIB *pDst) {
  unsigned char   ScreenCalibrated;
  unsigned short  TouchScreenXoffset;
  unsigned short  TouchScreenYoffset;
  unsigned short  TouchScreenXBitsPerPixelx16;
  unsigned short  TouchScreenYBitsPerPixelx16;
  long int value;
  int ret;

  ret = lwcfg_getInteger(pCfgFile, CONFIG_KEY_TS_CALIBRATED, &value);
  if(ret) return ret;
  ScreenCalibrated              = (unsigned char)value;

  ret = lwcfg_getInteger(pCfgFile, CONFIG_KEY_TS_OFFSET_X, &value);
  if(ret) return ret;
  TouchScreenXoffset            = (unsigned short)value;

  ret = lwcfg_getInteger(pCfgFile, CONFIG_KEY_TS_OFFSET_Y, &value);
  if(ret) return ret;
  TouchScreenYoffset            = (unsigned short)value;

  ret = lwcfg_getInteger(pCfgFile, CONFIG_KEY_TS_BITSPERPIXEL_X, &value);
  if(ret) return ret;
  TouchScreenXBitsPerPixelx16   = (unsigned short)value;

  ret = lwcfg_getInteger(pCfgFile, CONFIG_KEY_TS_BITSPERPIXEL_Y, &value);
  if(ret) return ret;
  TouchScreenYBitsPerPixelx16   = (unsigned short)value;
  
  pDst->ScreenCalibrated            = ScreenCalibrated;
  pDst->TouchScreenXoffset          = TouchScreenXoffset;
  pDst->TouchScreenYoffset          = TouchScreenYoffset;
  pDst->TouchScreenXBitsPerPixelx16 = TouchScreenXBitsPerPixelx16;
  pDst->TouchScreenYBitsPerPixelx16 = TouchScreenYBitsPerPixelx16;
  return 0;
}

/*TASK*-----------------------------------------------------
* 
* Task Name    : lcd_task
* Comments     :
*    This task handle the D4D display
*
*END*-----------------------------------------------------*/

static void lcd_task(uint_32 initial_data) {
  LWTIMER_PERIOD_STRUCT lwt_ps;
  LWTIMER_STRUCT lwt_st25;
#ifdef BSP_BUTTON2
  LWGPIO_STRUCT button;
#endif
  
  // Create memory pool for D4D library (DDR memory)
  d4dMemoryPoolId = _mem_create_pool((void*)D4D_MEMPOOL_BASE, D4D_MEMPOOL_SIZE);
  if(d4dMemoryPoolId==NULL) {
    printf("Fatal error: Cannot allocate memory pool for D4D graphics library.\n");
    printf("Terminating MQX...\n");
    _mqx_exit(0);
  }
  
#if 0 //defined(BSPCFG_ENABLE_IODEBUG) && BSPCFG_ENABLE_IODEBUG 
   MQX_FILE_PTR      fh_ptr;
   if(NULL == (fh_ptr = fopen("iodebug:", NULL))) { 
      printf("Cannot open the debug output\n"); 
   } else { 
      _io_set_handle(IO_STDOUT, fh_ptr); 
      printf("The debug print output is enabled.\n"); 
   }
#endif
  
  printf("Hello from LCD demo\n\r"); 
 
  // Init SD card - for this demo application 
  if(InitSdCard() == 0)
  {
    printf("SD Card is not ready.\r\n"); 
 //   _mqx_exit(0);
  }
  
  // Create the lwtimer component
  _lwtimer_create_periodic_queue(&lwt_ps, 25 / 5, 40);
  _lwtimer_add_timer_to_queue(&lwt_ps, &lwt_st25, 0, Time_event25ms, 0);

  if(!D4D_Init(&screen_main))
  {
    // D4D initialization failed
    printf("eGUI/D4D initialization failed\n\r");   
    _mqx_exit(0);
  }

  D4D_SetOrientation(D4D_ORIENT_LANDSCAPE);

  // Touchscreen calibration
{
    D4D_TOUCHSCREEN_CALIB tchCalibration;
    LWCFG_FILE_PTR pCfgFile = lwcfg_open("A:\\SYSTEM\\CONFIG.TXT");
    if(ts_loadCalibration(pCfgFile, &tchCalibration) != 0) {
      printf("Touchscreen needs to be calibrated...\n");
      main_calibrateTouchscreen(pCfgFile);
    } else {
      printf("Touchscreen calibration loaded from a config file.\n");
      D4D_SetTouchScreenCalibration(tchCalibration);
    }
    lwcfg_close(pCfgFile);
  }


  screenSaverTimeout = 50;                                                      // to start screensaver after 5 seconds

  
  // The print screen button inicialization
#ifdef BSP_BUTTON2
  /* opening pins for input */
  if (!lwgpio_init(&button, BSP_BUTTON2, LWGPIO_DIR_INPUT, LWGPIO_VALUE_NOCHANGE))
  {
      printf("Initializing button GPIO as input failed.\n");
      _task_block();
  }
  lwgpio_set_functionality(&button, BSP_BUTTON2_MUX_GPIO);
  lwgpio_set_attribute(&button, LWGPIO_ATTR_PULL_UP, LWGPIO_AVAL_ENABLE);
#endif
  
  // Change used color scheme
  D4D_SetDefaultScheme((D4D_CLR_SCHEME*)&d4d_clr_scheme_navy);

  // Screenshot variables
  gbGetScreenshot = 0;
  gucScreenshotsCnt = 0;
  

  while(1)
  {
    D4D_Poll();
    _time_delay(10);
    
    // The print screen handler
#ifdef BSP_BUTTON2    
    if (LWGPIO_VALUE_LOW == lwgpio_get_value(&button))
      getScreenshot();  
#endif
    // Set gbGetScreenshot to 1 via debugger or httpd to get printscreen
    if(gbGetScreenshot) {
      getScreenshot();
      gbGetScreenshot = !gbGetScreenshot;
    }
  }
  
}

static void getScreenshot(void) {
  char filename[PATHNAME_SIZE];
  sprintf(filename, "A:\\IMAGES\\SCRNSHTS\\SCSHT%03d.BMP", gucScreenshotsCnt++);
  D4D_PrintScreenToBMP(filename);
}

/** Invokes the screen calibration and saves it to the config file.
* @param[in]  pCfgFile  Pointer to an intialized config file structure.
*                       If NULL, TS calibrates, the result just is not written
*                       to any config file. */
void main_calibrateTouchscreen(LWCFG_FILE_PTR pCfgFile) {
  D4D_TOUCHSCREEN_CALIB tchCalibration;
  
  // calibrate the touchscreen
  D4D_CalibrateTouchScreen();
  printf("Calibration successful!\n");
  
  if(!pCfgFile) return;
  
  // store the calibration to the config file
  tchCalibration = D4D_GetTouchScreenCalibration();
  lwcfg_setInteger(pCfgFile, CONFIG_KEY_TS_CALIBRATED,      (long)tchCalibration.ScreenCalibrated);
  lwcfg_setInteger(pCfgFile, CONFIG_KEY_TS_OFFSET_X,        (long)tchCalibration.TouchScreenXoffset);
  lwcfg_setInteger(pCfgFile, CONFIG_KEY_TS_OFFSET_Y,        (long)tchCalibration.TouchScreenYoffset);
  lwcfg_setInteger(pCfgFile, CONFIG_KEY_TS_BITSPERPIXEL_X,  (long)tchCalibration.TouchScreenXBitsPerPixelx16);
  lwcfg_setInteger(pCfgFile, CONFIG_KEY_TS_BITSPERPIXEL_Y,  (long)tchCalibration.TouchScreenYBitsPerPixelx16);
}

/* EOF */
