

#include "USB_MSD_Disk.h"
#include "..\main.h"

#include <mfs.h>
#include <shell.h>
#include "..\tasks.h"
#include"..\format\format.h"
#include "..\File_management\file_management.h"
#include "..\SQM4K70_EB\IAR\D4D_Configuration\d4dtchhw_kinetis_adc_cfg.h"

USB_MSD_CONFIG_STRUCT  msd_config;

void USB_MSD_disk(uint_32 temp);

extern USB_ENDPOINTS usb_desc_ep;
extern DESC_CALLBACK_FUNCTIONS_STRUCT  desc_callback;
USB_CLASS_CALLBACK_STRUCT msc_class_cb;
USB_REQ_CALLBACK_STRUCT  vend_req_cb;
USB_CLASS_CALLBACK_STRUCT param_cb;
USB_CLASS_MSC_ENDPOINT *endpoint_array;

DISK_STRUCT g_disk;
boolean USB_Disk_block;
MQX_FILE_PTR  usbdrive_filesystem_handle;

#if HIGH_SPEED_DEVICE
#if PSP_MQX_CPU_IS_MPC512x || PSP_MQX_CPU_IS_MPC830x
   static uint_8 usb_recv_buff[MSD_RECV_BUFFER_SIZE] = {0};
   static uint_8 usb_send_buff[MSD_SEND_BUFFER_SIZE] = {0};
#else
    #pragma define_section hs_buffer ".usb_hs_buffer" RW
    __declspec(hs_buffer) static uint_8 usb_recv_buff[MSD_RECV_BUFFER_SIZE] = {0};
    __declspec(hs_buffer) static uint_8 usb_send_buff[MSD_SEND_BUFFER_SIZE] = {0};
#endif
#endif

#define WAIT_BLOCKED 0xF1


void USB_App_Callback(uint_8 event_type, void* val,pointer arg);
void Bulk_Transaction_Callback(uint_8 event_type,void* val,pointer arg);


void USB_App_Callback(uint_8 event_type, void* val,pointer arg) 
{
    UNUSED_ARGUMENT (arg)
    UNUSED_ARGUMENT (val)    
    if(event_type == USB_APP_BUS_RESET) 
    {
        g_disk.start_app=FALSE;    
    }
    else if(event_type == USB_APP_ENUM_COMPLETE) 
    {
        g_disk.start_app=TRUE;        
    }
    else if(event_type == USB_APP_ERROR)
    {
    }
    return;
}

void Bulk_Transaction_Callback(uint_8 event_type, void* val, pointer arg) 
{
    PTR_LBA_APP_STRUCT lba_data_ptr;
    uint_8_ptr prevent_removal_ptr; 
    //uint_8_ptr load_eject_start_ptr;   
    PTR_DEVICE_LBA_INFO_STRUCT device_lba_info_ptr;

    UNUSED_ARGUMENT (arg)
            
    switch(event_type)
    {
        case USB_APP_DATA_RECEIVED :
            break;
        case USB_APP_SEND_COMPLETE :
            break;
        case USB_MSC_START_STOP_EJECT_MEDIA :
            //load_eject_start_ptr = (uint_8_ptr)val;
            break;
        case USB_MSC_DEVICE_READ_REQUEST :          
            /* copy data from storage device before sending it on USB Bus 
               (Called before calling send_data on BULK IN endpoints)*/
            lba_data_ptr = (PTR_LBA_APP_STRUCT)val;
            /* read data from mass storage device to driver buffer */
            #if RAM_DISK_APP
                USB_mem_copy(g_disk.storage_disk + lba_data_ptr->offset,
                    lba_data_ptr->buff_ptr, 
                    lba_data_ptr->size);
            #endif         
            break;
        case USB_MSC_DEVICE_WRITE_REQUEST :
            /* copy data from USb buffer to Storage device 
               (Called before after recv_data on BULK OUT endpoints)*/
            lba_data_ptr = (PTR_LBA_APP_STRUCT)val;
            /* read data from driver buffer to mass storage device */
            #if RAM_DISK_APP
                USB_mem_copy(lba_data_ptr->buff_ptr,
                    g_disk.storage_disk + lba_data_ptr->offset,
                    lba_data_ptr->size);
            #endif                          
            break;
        case USB_MSC_DEVICE_FORMAT_COMPLETE :
            break;
        case USB_MSC_DEVICE_REMOVAL_REQUEST :
            prevent_removal_ptr = (uint_8_ptr) val;
            if(SUPPORT_DISK_LOCKING_MECHANISM)
            {                
                g_disk.disk_lock = *prevent_removal_ptr;
            }
            else if((!SUPPORT_DISK_LOCKING_MECHANISM)&&(!(*prevent_removal_ptr)))
            {
                /*there is no support for disk locking and removal of medium is enabled*/
                /* code to be added here for this condition, if required */ 
            }
            break;
        case USB_MSC_DEVICE_GET_INFO :
            device_lba_info_ptr = (PTR_DEVICE_LBA_INFO_STRUCT)val;
            #if RAM_DISK_APP
                        device_lba_info_ptr->total_lba_device_supports = USB_TOTAL_LOGICAL_ADDRESS_BLOCKS;  
                        device_lba_info_ptr->length_of_each_lab_of_device = LENGTH_OF_EACH_LAB; 
            #endif
            device_lba_info_ptr->num_lun_supported = LOGICAL_UNIT_SUPPORTED;
            break;          
        default : 
            break;
    }
        
    return;
}
 
void USB_MSD_disk(uint_32 temp)
{    
  UNUSED_ARGUMENT(temp);
  volatile char bRepeat = 1;
  
  USB_Disk_block = FALSE;
  
  g_disk.storage_disk=(uint_8*)USB_MSD_DISK_BASE;
  
    endpoint_array = USB_mem_alloc_zero(sizeof(USB_CLASS_MSC_ENDPOINT)*MSC_DESC_ENDPOINT_COUNT);
    if(endpoint_array == NULL)
    { 
        return ;
    }
    USB_mem_zero(&g_disk, sizeof(DISK_STRUCT));
    USB_mem_zero(&msd_config,sizeof(USB_MSD_CONFIG_STRUCT)); 

    msc_class_cb.callback = USB_App_Callback;
    msc_class_cb.arg = &g_disk.app_handle;
    
    vend_req_cb.callback = (USB_REQ_FUNC)NULL;
    vend_req_cb.arg = NULL;
    
    param_cb.callback = Bulk_Transaction_Callback;
    param_cb.arg = &g_disk.app_handle;

    _int_disable();

    /*Pass the configuration of the MASS STORAGE DEVICE to lower layers*/
#if RAM_DISK_APP
    msd_config.device_info.total_lba_device_supports = USB_TOTAL_LOGICAL_ADDRESS_BLOCKS;    
    msd_config.device_info.length_of_each_lab_of_device = LENGTH_OF_EACH_LAB; 
#else
    #error "UNSUPPORTED MASS STOARGE DEVICE"        
#endif
    msd_config.device_info.num_lun_supported = LOGICAL_UNIT_SUPPORTED;  
    msd_config.implementing_disk_drive = IMPLEMENTING_DISK_DRIVE;
    /* Pass USB Interface settings to lower layer */    
    msd_config.usb_max_suported_interfaces = USB_MAX_SUPPORTED_INTERFACES;
    msd_config.bulk_in_endpoint = BULK_IN_ENDPOINT;
    msd_config.bulk_in_endpoint_packet_size =BULK_IN_ENDP_PACKET_SIZE;
    msd_config.bulk_out_endpoint = BULK_OUT_ENDPOINT;   
    msd_config.ep_desc_data = &usb_desc_ep;
    msd_config.desc_endpoint_cnt = MSC_DESC_ENDPOINT_COUNT;
    msd_config.ep = endpoint_array;
    /* Configure and Pass App buffers for send and receive to lower layers*/        
    msd_config.msd_buff.msc_lba_send_buff_size = MSD_SEND_BUFFER_SIZE;

#if HIGH_SPEED_DEVICE 
    msd_config.msd_buff.msc_lba_send_ptr = usb_send_buff;
#else
    msd_config.msd_buff.msc_lba_send_ptr = USB_mem_alloc_zero(MSD_SEND_BUFFER_SIZE);
    if(NULL == msd_config.msd_buff.msc_lba_send_ptr)
    {
        return ;
    }
#endif  
    msd_config.msd_buff.msc_lba_recv_buff_size = MSD_RECV_BUFFER_SIZE;
#if HIGH_SPEED_DEVICE
    msd_config.msd_buff.msc_lba_recv_ptr = usb_recv_buff;
#else   
    msd_config.msd_buff.msc_lba_recv_ptr = USB_mem_alloc_zero(MSD_RECV_BUFFER_SIZE);    
    if(NULL == msd_config.msd_buff.msc_lba_recv_ptr)
    {
        return ;
    }
#endif  
    /* Register the callbacks to lower layers */
    msd_config.msc_class_callback = &msc_class_cb;
    msd_config.vendor_req_callback = &vend_req_cb;
    msd_config.param_callback = &param_cb;
    msd_config.desc_callback_ptr = &desc_callback;
    
    if (MQX_OK != _usb_device_driver_install(USBCFG_DEFAULT_DEVICE_CONTROLLER)) {
#if _DEBUG
      printf("4: Driver could not be installed\n");
#endif
      return;
    }
    
    // USB DISK
    RAM_disk_format(&usbdrive_filesystem_handle, (pointer)USB_MSD_DISK_BASE, (_file_size)USB_MSD_DISK_SIZE, USB_DRIVE, "mfs_ramdisk_b:", "SQM4_EB");
    
#if defined(FRD7040TPT) || defined(FRD5040TPT)
    _time_delay(8000);
#else
    _time_delay(5500);
#endif
    
    MQX_FILE_PTR fs_ptr;
    fs_ptr = fopen("B:y.txt", "w");
    fclose(fs_ptr);    
    MFS_delete_file(USB_DRIVE,"", "Y.TXT");
    
    
    g_disk.storage_disk=(uint_8*)USB_MSD_DISK_BASE;
            
    g_disk.app_handle = USB_Class_MSC_Init(&msd_config);        //inicializace MSC a USB Stack vrstvy                
    _int_enable();
    
    while (bRepeat) 
    {      
      USB_MSC_Periodic_Task();        //funkce se musi volat periodicky

      _time_delay(10);
      while(USB_Disk_block) {
        _time_delay(1);
      }
//      if(g_disk.start_app==TRUE&&(copyid<10))
//      {
//      }    
      
    }   
}

boolean USB_MSD_Disk_init(void)
{
  USB_Disk_block = FALSE;
  
    if (MQX_OK != _usb_device_driver_install(USBCFG_DEFAULT_DEVICE_CONTROLLER)) {
  #if _DEBUG
      printf("4: Driver could not be installed\n");
  #endif
      return FALSE;
    }
//  _task_id id_USB_Disk;
//  _mqx_uint error;
//    
//  id_USB_Disk = _task_get_id_from_name("USB_MSD_disk"); 
//  error = _task_restart(id_USB_Disk,0,0);
//  
//  if(error==MQX_INVALID_TASK_ID) {
//    return FALSE;
//  }
  return TRUE; 
}

boolean USB_MSD_Disk_deinit(void)
{
  USB_Disk_block = TRUE;
  
  if (MQX_OK != _usb_device_driver_uninstall()) {
#if _DEBUG
    printf("4: Driver could not be installed\n");
#endif
    return FALSE;
  }
  return TRUE;  
}


