/******************************************************************************
* 
* Copyright (c) 2012 Elnico s.r.o.
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY ELNICO "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL ELNICO OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
****************************************************************************//*!
*
* @file      utils.h
*
* @author    Petr Kubiznak
* 
* @version   1.0
* 
* @date      Sep-04-2012
* 
* @brief     Utility functions and macro definitions.
*
*******************************************************************************/

#ifndef __UTILS_H__970460443216547401321547
#define __UTILS_H__970460443216547401321547


/*******************************************************************************
*       Includes header files
*******************************************************************************/
#include "d4d.h"

/*******************************************************************************
*       Functions definitions
*******************************************************************************/
#if defined(__MQX__) || defined(D4D_OS_MQX)
  //void * memcpy ( void * destination, const void * source, size_t num );
  #define memcpy(destination,source,num)  _mem_copy(source,destination,num)
#else
  //void * memcpy ( void * destination, const void * source, size_t num );
  #define memcpy(destination,source,num)  memcpy(destination,source,num)
#endif


/*******************************************************************************
*       D4D objects instantiation macros
*******************************************************************************/

#define D4D_DECLARE_STD_CHILD_LABEL(name, text, x, y, cx, cy, font, pRelations) \
      _D4D_DECLARE_LABEL(D4D_CONST, name, text, x, y, cx, cy, 0, NULL, pRelations, (D4D_OBJECT_F_VISIBLE | D4D_OBJECT_F_ENABLED), NULL, font, NULL, NULL)

#define D4D_DECLARE_TXT_CHILD_BUTTON(name, text, x, y, cx, cy, fontId, pRelations, onclick) \
      _D4D_DECLARE_BUTTON(D4D_CONST, name, text, x, y, cx, cy, 0, NULL, pRelations, (D4D_BTN_F_DEFAULT), NULL, NULL, NULL, fontId, NULL, onclick, NULL)


/*******************************************************************************
*       Functions declarations
*******************************************************************************/

/*******************************************************************************
* Reads data from a file to the memory.
* @param[out] dst         Destination pointer. If NULL, function allocates new
*                         memory (of exactly the needed size). Else it assumes
*                         there is enough space allocated to read the required
*                         length of data.
* @param[in]  sFilename   Path to the file to read.
* @param[in]  maxlen      Maximum number of bytes to read. If 0, function reads
*                         the whole file.
* @return                 Number of bytes read or 0 in case of failure.
*******************************************************************************/
LWord readFile(void **dst, D4D_CHAR *sFilename, unsigned int maxlen);

/*******************************************************************************
* Loads a bitmap from a file to RAM.
* @param[out] dst         Destination pointer. If dst->pData is NULL, function
*                         allocates new memory (of exactly the needed size).
*                         Else it assumes there is enough space allocated
*                         to read the whole bitmap.
* @param[in]  sFilename   Path to the file to read.
* @return                 0 on success, 1 on invalid argument, 2 on file read
*                         error.
*******************************************************************************/
Byte loadBitmap(D4D_BMP *dst, D4D_CHAR *sFilename);

/*******************************************************************************
* Frees the memory allocated for a bitmap by the loadBitmap() call.
* @param[out] bmp         Bitmap to free its pData member variable.
*******************************************************************************/
void freeBitmap(D4D_BMP *bmp);

#endif /* __UTILS_H__970460443216547401321547 */
